OBJECTS_TMP =$(SOURCES:%.cpp=$(PROJECT_OBJ_PATH)/%.o)
OBJECTS     =$(OBJECTS_TMP:%.c=$(PROJECT_OBJ_PATH)/%.o)

EXPORT_HEADERS = $(HEADERS:%=$(PROJECT_INC_PATH)/%)

DEPFILES_TMP=$(SOURCES:%.cpp=$(PROJECT_DEP_PATH)/%.P)
DEPFILES    =$(DEPFILES_TMP:%.c=$(PROJECT_DEP_PATH)/%.P)
DEPFILES    +=$(PLUGINS:%.cpp=$(PROJECT_DEP_PATH)/%PLUG.P)

# Be aware: with insure option -MD puts the dependency file in the same
# directory than the source file.  With classic compilators, -MD puts the
# dependency file in the target directory

ifeq ($(SUFFIX), _insure)
DEP_PATH	= 
PLUGIN_DEP_PATH = 
else
DEP_PATH	= $(PROJECT_OBJ_PATH)/
PLUGIN_DEP_PATH = $(PROJECT_PLUGIN_OBJ_PATH)/
endif
DEP_FILE	= $(DEP_PATH)$*.d
PLUGIN_DEP_FILE	= $(PLUGIN_DEP_PATH)$*PLUG.d

export-header: $(EXPORT_HEADERS) 

$(LIB) :  export-header $(OBJECTS)
	$(AR) $@ $(PROJECT_OBJ_PATH)/*.o

# Rule to generate moc files for Qt
m%.cpp: %.h
	@echo "*"
	@echo "* Construct MOC file $@ "
	@echo "*"
	$(MOC) -o $@ $<

# Rule for combining compilation and dependency generation:
# - put objects in $(PROJECT_OBJ_PATH)
# - put dependencies in $(PROJECT_DEP_PATH)

# ---> OBJECTS 
$(PROJECT_OBJ_PATH)/%.o : %.cpp
	@echo "*"
	@echo "* Build $*.o from $*.cpp "
	@echo "*"
	$(CXXALL) -MD -o $@ -c $< $($*_CXXFLAGS)
	@cp $(DEP_FILE) $(PROJECT_DEP_PATH)/$*.P; \
	sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\$$//' \
	    -e '/^$$/ d' -e 's/$$/ :/' < $(DEP_FILE)\
	    >> $(PROJECT_DEP_PATH)/$*.P; \
	rm -f $(DEP_FILE)


$(PROJECT_OBJ_PATH)/%.o : %.c
	@echo "*"
	@echo "* Build $@ from $<  "
	@echo "*"
	$(CXXALL) -MD -o $@ -c $< $($*_CXXFLAGS)
	@cp $(DEP_FILE) $(PROJECT_DEP_PATH)/$*.P; \
	sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\$$//' \
	    -e '/^$$/ d' -e 's/$$/ :/' < $(DEP_FILE)\
	    >> $(PROJECT_DEP_PATH)/$*.P; \
	rm -f $(DEP_FILE)

# ---> PLUGINS 
$(PROJECT_PLUGIN_OBJ_PATH)/%PLUG.o : %.cpp
	@echo "*"
	@echo "* Build $*PLUG.o from $*.cpp  "
	@echo "*"
	$(CXXALL) -fPIC -MD -o $@ -c $< $($*_CXXFLAGS)
	@cp $(PLUGIN_DEP_FILE) $(PROJECT_DEP_PATH)/$*PLUG.P; \
	sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\$$//' \
	    -e '/^$$/ d' -e 's/$$/ :/' < $(PLUGIN_DEP_FILE)\
	    >> $(PROJECT_DEP_PATH)/$*PLUG.P; \
	rm -f $(PLUGIN_DEP_FILE)

define plugin_template
$(1)_PLUGIN_PATH ?= $$(PROJECT_PLUGIN_PATH)
$(1)_OBJS = $($(1)_OBJ_FILES) $(1:%=$(PROJECT_PLUGIN_OBJ_PATH)/%PLUG.o)
PLUGINSO += $$($(1)_PLUGIN_PATH)/$(1).so
PLUGIN_OBJS += $$($(1)_OBJS)
$$($(1)_PLUGIN_PATH)/$(1).so: $$($(1)_OBJS)
	@echo "*"
	@echo "* Linking $$(@F)"
	@echo "*"
	$$(CXXALL) -shared -o $$@ $$^ $($(1)_EXT_OBJ) $($(1)_LDFLAGS) $(LDFLAGS) $(LIBS)
endef

$(foreach TARGET,$(PLUGINS:%.cpp=%),$(eval $(call plugin_template,$(TARGET))))

plugin: $(PLUGIN_OBJS) $(PLUGINSO) 

# ---> HEADERS
$(PROJECT_INC_PATH)/%.h : %.h
	@ln -snf $(shell pwd)/$< $@

$(PROJECT_INC_PATH)/%.hpp : %.hpp
	@ln -snf $(shell pwd)/$< $@

$(PROJECT_INC_PATH)/%.t.cpp : %.t.cpp
	@ln -snf $(shell pwd)/$< $@

