############################################################################
# Usage:
#   AC_HAVE_ARP_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the ARP development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_arp_desired     true | false (defaults to true)
# < $ac_arp_avail       true | false
# < $ac_arp_cxxflags    (extra flags the C++ compiler needs)
# < $ac_arp_ldflags     (extra flags the linker needs)
# < $ac_arp_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
# Modifs:
#   Nicolas Mansard, <Nicolas.Mansard@irisa.fr> 3-NOV-5
#
# TODO:
#



AC_DEFUN([AC_HAVE_ARP_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_arp_avail=false
ac_arp_cppflags=
ac_arp_cflags=
ac_arp_cxxflags=
ac_arp_ldflags=
ac_arp_libs=

# internal variables
ac_arp_desired=true
ac_arp_extrapath="/local/soft/ViSP/ARP"

AC_ARG_WITH([arp],
AC_HELP_STRING([--with-arp], [enable use of Arp [[default=yes]]])
AC_HELP_STRING([--with-arp=DIR], [give prefix location of ARP]),
  [ case $withval in
    no)  ac_arp_desired=false ;;
    yes) ac_arp_desired=true ;;
    *)   ac_arp_desired=true
         ac_arp_extrapath=$withval ;;
    esac],
  [])

if $ac_arp_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the Arp library],
      cv_arp_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_arp_cxxflags="-I$ac_arp_extrapath/src"
      ac_arp_ldflags="-L$ac_arp_extrapath/lib/$OS/$CXX_VERSION"
      ac_arp_libs="-lmodel -lutils -lrobust -lfeature -lsite -lcalcpose"

      CXXFLAGS="$ac_arp_cxxflags $ac_visp1_cxxflags "
      LDFLAGS="$ac_arp_ldflags $ac_visp1_ldflags"
      LIBS="$ac_arp_libs $ac_visp1_libs"



      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [/*#include <robust/CRobust.h>*/],
        [/*CRobust rob(1);*/],
        [cv_arp_avail=true],
        [cv_arp_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_arp_avail=$cv_arp_avail
fi

if $ac_arp_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_ARP_IFELSE()

