############################################################################
# Usage:
#   AC_HAVE_BOOST_THREAD_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Boost_thread development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_boost_thread_desired     true | false (defaults to true)
# < $ac_boost_thread_avail       true | false
# < $ac_boost_thread_cxxflags    (extra flags the C++ compiler needs)
# < $ac_boost_thread_ldflags     (extra flags the linker needs)
# < $ac_boost_thread_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_BOOST_THREAD_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_boost_thread_avail=false
ac_boost_thread_cppflags=
ac_boost_thread_cflags=
ac_boost_thread_cxxflags=
ac_boost_thread_ldflags=
ac_boost_thread_libs=

# internal variables
ac_boost_thread_desired=true
ac_boost_thread_extrapath="/usr"

AC_ARG_WITH([boost_thread],
AC_HELP_STRING([--with-boost_thread], [enable use of Boost_thread [[default=yes]]])
AC_HELP_STRING([--with-boost_thread=DIR], [give prefix location of Boost_thread]),
  [ case $withval in
    no)  ac_boost_thread_desired=false ;;
    yes) ac_boost_thread_desired=true ;;
    *)   ac_boost_thread_desired=true
         ac_boost_thread_extrapath=$withval ;;
    esac],
  [])
AC_ARG_WITH([boost_thread-version],
AC_HELP_STRING([--with-boost_thread-version], [enable use of Boost_thread [[default=yes]]])
AC_HELP_STRING([--with-boost_thread-version=VERSION], [give suffix location of Boost_thread]),
  [ case $withval in
    no)  ac_boost_thread_version_needed=false ;;
    *) ac_boost_thread_version_needed=true 
       ac_boost_thread_version=$withval ;;
    esac],
  [ac_boost_thread_version_needed=false])


if $ac_boost_thread_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the Boost_thread library],
      cv_boost_thread_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_boost_thread_cxxflags="-I$ac_boost_thread_extrapath/include -I$ac_boost_thread_extrapath/include/boost_thread-sandbox"
      ac_boost_thread_ldflags="-L$ac_boost_thread_extrapath/lib"

      if $ac_boost_thread_version_needed; then
	ac_boost_thread_cxxflags=$ac_boost_thread_cxxflags/boost_thread-$ac_boost_thread_version
      	ac_boost_thread_libs="-lboost_thread_thread-gcc-mt -lpthread -pthread"
      else		
      	ac_boost_thread_libs="-lboost_thread_thread -lpthread -pthread"
      fi

      CXXFLAGS="$CXXFLAGS $ac_boost_thread_cxxflags"
      LDFLAGS="$LDFLAGS $ac_boost_thread_ldflags"
      LIBS="$ac_boost_thread_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [#include <boost_thread/thread.hpp>],
        [boost_thread::mutex m;],
        [cv_boost_thread_avail=true],
        [cv_boost_thread_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_boost_thread_avail=$cv_boost_thread_avail
fi

if $ac_boost_thread_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_BOOST_THREAD_IFELSE()

