############################################################################
# Usage:
#   AC_HAVE_BOOST_LAPACK_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Boost_lapack development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_boost_lapack_desired     true | false (defaults to true)
# < $ac_boost_lapack_avail       true | false
# < $ac_boost_lapack_cxxflags    (extra flags the C++ compiler needs)
# < $ac_boost_lapack_ldflags     (extra flags the linker needs)
# < $ac_boost_lapack_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_BOOST_LAPACK_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_boost_lapack_avail=false
ac_boost_lapack_cppflags=
ac_boost_lapack_cflags=
ac_boost_lapack_cxxflags=
ac_boost_lapack_ldflags=
ac_boost_lapack_libs=

# internal variables
ac_boost_lapack_desired=true
ac_boost_lapack_extrapath="."
ac_lapack_extrapath=/usr/lib

AC_ARG_WITH([boost_lapack],
AC_HELP_STRING([--with-boost_lapack], [enable use of Boost_lapack [[default=yes]]])
AC_HELP_STRING([--with-boost_lapack=DIR], [give prefix location of Boost_lapack]),
  [ case $withval in
    no)  ac_boost_lapack_desired=false ;;
    yes) ac_boost_lapack_desired=true ;;
    *)   ac_boost_lapack_desired=true
         ac_boost_lapack_extrapath=$withval ;;
    esac],
  [])
AC_ARG_WITH([lapack],
AC_HELP_STRING([--with-lapack], [enable use of lapack [[default=yes]]])
AC_HELP_STRING([--with-lapack=DIR], [give lib location of lapack]),
  [ case $withval in
    no)  ac_boost_lapack_desired=false ;;
    yes) ac_lapack_desired=true ;;
    *)   ac_lapack_desired=true
         ac_lapack_extrapath=$withval ;;
    esac],
  [])


if $ac_boost_lapack_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the Boost_lapack library],
      cv_boost_lapack_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_boost_lapack_cxxflags="-I$ac_boost_lapack_extrapath"
      ac_boost_lapack_ldflags="-L$ac_lapack_extrapath/lib"

      ac_boost_lapack_libs="-llapack -lblas"

      CXXFLAGS="$CXXFLAGS $ac_boost_lapack_cxxflags"
      LDFLAGS="$LDFLAGS $ac_boost_lapack_ldflags"
      LIBS="$ac_boost_lapack_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [# include <boost/numeric/ublas/matrix.hpp>
	#include <boost/numeric/bindings/lapack/gesvd.hpp>],
        [boost::numeric::ublas::matrix<double> M;],
        [cv_boost_lapack_avail=true],
        [cv_boost_lapack_avail=false])
      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_boost_lapack_avail=$cv_boost_lapack_avail
fi

if $ac_boost_lapack_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_BOOST_LAPACK_IFELSE()

### # include <boost/numeric/ublas/matrix.hpp>]
