############################################################################
# Usage:
#   AC_HAVE_DYNAMICSJRL_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Dynamicsjrl development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_dynamicsjrl_desired     true | false (defaults to true)
# < $ac_dynamicsjrl_avail       true | false
# < $ac_dynamicsjrl_cxxflags    (extra flags the C++ compiler needs)
# < $ac_dynamicsjrl_ldflags     (extra flags the linker needs)
# < $ac_dynamicsjrl_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_DYNAMICSHRPO_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_dynamicshrpo_avail=false
ac_dynamicshrpo_cppflags=
ac_dynamicshrpo_cflags=
ac_dynamicshrpo_cxxflags=
ac_dynamicshrpo_ldflags=
ac_dynamicshrpo_libs=

# internal variables
ac_dynamicshrpo_desired=true
ac_dynamicshrpo_extrapath="/usr/local"

AC_ARG_WITH([dynamicshrpo],
AC_HELP_STRING([--with-dynamicshrpo], [enable use of Dynamicshrpo [[default=yes]]])
AC_HELP_STRING([--with-dynamicshrpo=DIR], [give prefix location of Dynamicshrpo]),
  [ case $withval in
    no)  ac_dynamicshrpo_desired=false ;;
    yes) ac_dynamicshrpo_desired=true ;;
    *)   ac_dynamicshrpo_desired=true
         ac_dynamicshrpo_extrapath=$withval ;;
    esac],
  [])
AC_ARG_WITH([dynamicshrpo-version],
AC_HELP_STRING([--with-dynamicshrpo-version], [enable use of Dynamicshrpo [[default=yes]]])
AC_HELP_STRING([--with-dynamicshrpo-version=VERSION], [give suffix location of Dynamicshrpo]),
  [ case $withval in
    no)  ac_dynamicshrpo_version_needed=false ;;
    *) ac_dynamicshrpo_version_needed=true 
       ac_dynamicshrpo_version=$withval ;;
    esac],
  [ac_dynamicshrpo_version_needed=false])


if $ac_dynamicshrpo_desired; then

    AC_CACHE_CHECK(
      [if we can link with the dynamicshrpo],
      cv_dynamicshrpo_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_dynamicshrpo_cxxflags="-I$ac_dynamicshrpo_extrapath/include"
      ac_dynamicshrpo_ldflags="-L$ac_dynamicshrpo_extrapath/lib"

      if $ac_dynamicshrpo_version_needed; then
	ac_dynamicshrpo_cxxflags=$ac_dynamicshrpo_cxxflags/dynamicshrpo-$ac_dynamicshrpo_version
      	ac_dynamicshrpo_libs="-ldynamicshrpoJapan"
      else		
      	ac_dynamicshrpo_libs="-ldynamicshrpoJapan"
      fi

      CXXFLAGS="$CXXFLAGS $ac_dynamicshrpo_cxxflags"
      LDFLAGS="$LDFLAGS $ac_dynamicshrpo_ldflags"
      LIBS="$ac_dynamicshrpo_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [#include <dynamicshrpoJapan/Body.h>],
        [dynamicshrpoJapan::Body body(18.5);],
        [cv_dynamicshrpo_avail=true],
        [cv_dynamicshrpo_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_dynamicshrpo_avail=$cv_dynamicshrpo_avail
fi

if $ac_dynamicshrpo_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_DYNAMICSHRPO_IFELSE()

