############################################################################
# Usage:
#   AC_HAVE_DYNAMICSJRL_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Dynamicsjrl development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_dynamicsjrl_desired     true | false (defaults to true)
# < $ac_dynamicsjrl_avail       true | false
# < $ac_dynamicsjrl_cxxflags    (extra flags the C++ compiler needs)
# < $ac_dynamicsjrl_ldflags     (extra flags the linker needs)
# < $ac_dynamicsjrl_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_DYNAMICSJRL_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_dynamicsjrl_avail=false
ac_dynamicsjrl_cppflags=
ac_dynamicsjrl_cflags=
ac_dynamicsjrl_cxxflags=
ac_dynamicsjrl_ldflags=
ac_dynamicsjrl_libs=

# internal variables
ac_dynamicsjrl_desired=true
ac_dynamicsjrl_extrapath="/usr/local"

AC_ARG_WITH([dynamicsjrl],
AC_HELP_STRING([--with-dynamicsjrl], [enable use of Dynamicsjrl [[default=yes]]])
AC_HELP_STRING([--with-dynamicsjrl=DIR], [give prefix location of Dynamicsjrl]),
  [ case $withval in
    no)  ac_dynamicsjrl_desired=false ;;
    yes) ac_dynamicsjrl_desired=true ;;
    *)   ac_dynamicsjrl_desired=true
         ac_dynamicsjrl_extrapath=$withval ;;
    esac],
  [])
AC_ARG_WITH([dynamicsjrl-version],
AC_HELP_STRING([--with-dynamicsjrl-version], [enable use of Dynamicsjrl [[default=yes]]])
AC_HELP_STRING([--with-dynamicsjrl-version=VERSION], [give suffix location of Dynamicsjrl]),
  [ case $withval in
    no)  ac_dynamicsjrl_version_needed=false ;;
    *) ac_dynamicsjrl_version_needed=true 
       ac_dynamicsjrl_version=$withval ;;
    esac],
  [ac_dynamicsjrl_version_needed=false])


if $ac_dynamicsjrl_desired; then

    AC_CACHE_CHECK(
      [if we can link with the dynamicsJRL],
      cv_dynamicsjrl_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_dynamicsjrl_cxxflags="-I$ac_dynamicsjrl_extrapath/include"
      ac_dynamicsjrl_ldflags="-L$ac_dynamicsjrl_extrapath/lib"

      if $ac_dynamicsjrl_version_needed; then
	ac_dynamicsjrl_cxxflags=$ac_dynamicsjrl_cxxflags/dynamicsjrl-$ac_dynamicsjrl_version
      	ac_dynamicsjrl_libs="-ldynamicsJRLJapan"
      else		
      	ac_dynamicsjrl_libs="-ldynamicsJRLJapan"
      fi

      CXXFLAGS="$CXXFLAGS $ac_dynamicsjrl_cxxflags"
      LDFLAGS="$LDFLAGS $ac_dynamicsjrl_ldflags"
      LIBS="$ac_dynamicsjrl_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [#include <dynamicsJRLJapan/Body.h>],
        [dynamicsJRLJapan::Body body(18.5);],
        [cv_dynamicsjrl_avail=true],
        [cv_dynamicsjrl_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_dynamicsjrl_avail=$cv_dynamicsjrl_avail
fi

if $ac_dynamicsjrl_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_DYNAMICSJRL_IFELSE()

