############################################################################
# Usage:
#   AC_HAVE_LIBTRACES_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Libtraces development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_libtraces_desired     true | false (defaults to true)
# < $ac_libtraces_avail       true | false
# < $ac_libtraces_cxxflags    (extra flags the C++ compiler needs)
# < $ac_libtraces_ldflags     (extra flags the linker needs)
# < $ac_libtraces_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_LIBTRACES_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_libtraces_avail=false
ac_libtraces_cppflags=
ac_libtraces_cflags=
ac_libtraces_cxxflags=
ac_libtraces_ldflags=
ac_libtraces_libs=

# internal variables
ac_libtraces_desired=true
ac_libtraces_extrapath="/udd/nmansard/nmansard/these/code/Traces"

AC_ARG_WITH([libtraces],
AC_HELP_STRING([--with-libtraces], [enable use of Libtraces [[default=yes]]])
AC_HELP_STRING([--with-libtraces=DIR], [give prefix location of Libtraces]),
  [ case $withval in
    no)  ac_libtraces_desired=false ;;
    yes) ac_libtraces_desired=true ;;
    *)   ac_libtraces_desired=true
         ac_libtraces_extrapath=$withval ;;
    esac],
  [])

if $ac_libtraces_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the Libtraces library],
      cv_libtraces_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_libtraces_cxxflags="-I$ac_libtraces_extrapath/src"
      ac_libtraces_ldflags="-L$ac_libtraces_extrapath/lib"
      ac_libtraces_libs="-lTraces"

      CXXFLAGS="$CXXFLAGS $ac_libtraces_cxxflags"
      LDFLAGS="$LDFLAGS $ac_libtraces_ldflags"
      LIBS="$ac_libtraces_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [#include <Traces.hh>],
        [Traces traces;],
        [cv_libtraces_avail=true],
        [cv_libtraces_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_libtraces_avail=$cv_libtraces_avail
fi

if $ac_libtraces_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_LIBTRACES_IFELSE()

