############################################################################
# Usage:
#   AC_HAVE_OPENHRP_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the Openhrp development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_openhrp_desired     true | false (defaults to true)
# < $ac_openhrp_avail       true | false
# < $ac_openhrp_cxxflags    (extra flags the C++ compiler needs)
# < $ac_openhrp_ldflags     (extra flags the linker needs)
# < $ac_openhrp_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
#
# TODO:
#



AC_DEFUN([AC_HAVE_OPENHRP_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_openhrp_avail=false
ac_openhrp_cppflags=
ac_openhrp_cflags=
ac_openhrp_cxxflags=
ac_openhrp_ldflags=
ac_openhrp_libs=

# internal variables
ac_openhrp_desired=true
ac_openhrp_extrapath="/home/stasse/src/OpenHRP"
ac_openhrp_libpath="Controller/IOserver/robot/HRP2JRL/bin/"
ac_openhrp_includepath="Controller/IOserver/sys/plugin/"

AC_ARG_WITH([openhrp],
AC_HELP_STRING([--with-openhrp], [enable use of Openhrp [[default=yes]]])
AC_HELP_STRING([--with-openhrp=DIR], [give prefix location of Openhrp]),
  [ case $withval in
    no)  ac_openhrp_desired=false ;;
    yes) ac_openhrp_desired=true ;;
    *)   ac_openhrp_desired=true
         ac_openhrp_extrapath=$withval ;;
    esac],
  [])

if $ac_openhrp_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the Openhrp library],
      cv_openhrp_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_openhrp_cxxflags="-I$ac_openhrp_extrapath/ac_openhrp_includepath"
      ac_openhrp_ldflags="-L$ac_openhrp_extrapath/ac_openhrp_libpath"
      ac_openhrp_libs=""

      CXXFLAGS="$CXXFLAGS $ac_openhrp_cxxflags"
      LDFLAGS="$LDFLAGS $ac_openhrp_ldflags"
      LIBS="$ac_openhrp_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [],
        [],
        [cv_openhrp_avail=true],
        [cv_openhrp_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_openhrp_avail=$cv_openhrp_avail
fi

if $ac_openhrp_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_OPENHRP_IFELSE()

