# Usage:
#   AC_HAVE_VISP1_IFELSE( IF-FOUND, IF-NOT-FOUND )
#
# Description:
#   This macro locates the ViSP1 development system.  If it is found,
#   the set of variables listed below are set up as described and made
#   available to the configure script.
#
# Autoconf Variables:
# > $ac_visp1_desired     true | false (defaults to true)
# < $ac_visp1_avail       true | false
# < $ac_visp1_cxxflags    (extra flags the C++ compiler needs)
# < $ac_visp1_ldflags     (extra flags the linker needs)
# < $ac_visp1_libs        (link library flags the linker needs)
#
# Author:
#   Fabien Spindler, <Fabien.Spindler@irisa.fr>
# Modifie: 
#   Christophe Renaudin 1-AUG-5
# TODO:
#



AC_DEFUN([AC_HAVE_VISP1_IFELSE], [
AC_PREREQ([2.14a])

# official variables
ac_visp1_avail=false
ac_visp1_cppflags=
ac_visp1_cflags=
ac_visp1_cxxflags=
ac_visp1_ldflags=
ac_visp1_libs=

# internal variables
ac_visp1_desired=true
ac_visp1_extrapath="/local/soft/ViSP/ViSP-1"

AC_ARG_WITH([visp1],
AC_HELP_STRING([--with-visp1], [enable use of ViSP1 [[default=yes]]])
AC_HELP_STRING([--with-visp1=DIR], [give prefix location of ViSP1]),
  [ case $withval in
    no)  ac_visp1_desired=false ;;
    yes) ac_visp1_desired=true ;;
    *)   ac_visp1_desired=true
         ac_visp1_extrapath=$withval ;;
    esac],
  [])

if $ac_visp1_desired; then

    AC_CACHE_CHECK(
      [if we can compile and link with the ViSP1 library],
      cv_visp1_avail,
      [ac_save_cxxflags=$CXXFLAGS
      ac_save_ldflags=$LDFLAGS
      ac_save_libs=$LIBS

      ac_visp1_cxxflags="-I$ac_visp1_extrapath/src"
      ac_visp1_ldflags="-L$ac_visp1_extrapath/lib"
      ac_visp1_libs="-lvisp-1.6.9"

      CXXFLAGS="$CXXFLAGS $ac_visp1_cxxflags"
      LDFLAGS="$LDFLAGS $ac_visp1_ldflags"
      LIBS="$ac_visp1_libs $LIBS"

      AC_LANG_PUSH(C++)

      AC_TRY_LINK(
        [#include <math/simple-math.h>],
        [Fact(2);],
        [cv_visp1_avail=true],
        [cv_visp1_avail=false])

      AC_LANG_POP

      CXXFLAGS=$ac_save_cxxflags
      LDFLAGS=$ac_save_ldflags
      LIBS=$ac_save_libs
    ])
    ac_visp1_avail=$cv_visp1_avail

fi

if $ac_visp1_avail; then
  ifelse([$1], , :, [$1])
else
  ifelse([$2], , :, [$2])
fi
]) # AC_HAVE_VISP1_IFELSE()


