function [nerr] = calibForce( fs,Rqs,params,opts );

% compute the error between the force red by the sensor and the force predicted
% knwing the position of the hand and the parameters.
% The parameters are the position of the sensor in the end (3 param, uteta)
% the value of the real mg on the z axis in the world frame.
% The norm is 0 if the model fits perfectly.

% flu is the vector of force : flu = [ flu1; flu2 ... ; flun ]
% where flu_k = hRs f_return 
% and f_return is given by OpenHRP.
% Rqs is the stack of all hand homogenous matrix:
% Rqs = [ m1 ; m2 ... ; mn ], 
% with Mk = reshape(mk,4,4)
% where Mk = worldMhand

% The mg used in the compensation of the sensor value during 
% the calibration is let free (on the three axis).
% The force red by the sensor is made
% proportional to a 3D gain matrix depending on the force value:
% f_red = K f_effective + offset(mg)

idxForce=[ 1:3 7 8:10 14:22 ];
params(idxForce(1:length(params)))=params;

narg=size(params);
if (narg<3)  uteta=[0 0 -pi/2]; else uteta=params(1:3);  end;
if (narg<6)  sc=[0 0 -0.05];    else sc=params(4:6);  end;
if (narg<7)  mg=[ 0;0;-8];      else mg=[0;0;params(7)]; end;
if (narg<10) gamma_f=mg;        else gamma_f=params(8:10)'; end;
if (narg<13) gamma_tau=0;       else gamma_tau=params(11:13)'; end;
if (narg<16) % 3 params
   gains=zeros(6);
else if (narg<19)% 6 params
   gains=[ diag(params(14:16)) zeros(3); zeros(3,6)];
else if (narg<22)
   gains=diag(params(14:19)); 
else if (narg<31)
   gains=[ reshape(params(14:22),3,3) zeros(3); zeros(3,6)];
else if (narg<49)
   Kf=reshape(params(14:22),3,3); Ktau=reshape(params(23:31),3,3);   
   gains=[Kf zeros(3); zeros(3) Ktau];
else 
   gains=reshape(params(14:49),6,6); 
end; end; end; end; end;

sRh=tomatrix(uteta);
K=gains+eye(6);
Sk_sc=[ 0 -sc(3) sc(2); sc(3) 0 -sc(1); -sc(2) sc(1) 0];
if(gamma_tau==0) gamma_tau=-Sk_sc*mg; end;

Kf=K(1:3,1:3);

for i=1:length(Rqs) 
    hRw=reshape(Rqs(i,[1:3 5:7 9:11]),3,3); % reshape transpose the res.
    fred=fs(i,1:3)';
    err(i,:)=Kf*(-fred-sRh*gamma_f)+sRh*hRw*mg;
end;

nerr=norm(err)+norm(gains)*2.50;

if (4==nargin)
    if opts=='plot' 
        subplot(2,1,1); plot(fs); title('F non comp');
        subplot(2,1,2); plot(err); title('F comp');
    end;
end;
