#!/bin/bash

# This script aims at downloading and compile all the packages needed to 
# build the SoT.
# Please change the following variables to adapt to your setup.

export USERNAME=`whoami`
export SOURCE_DIR=/home/nmansard/src
echo "User name : ${USERNAME}"
if [ ${USERNAME}=="nmansard" ]; then
  echo "nmansard configuration"
  export BINARY_INSTALL_PREFIX=/usr/local/4.2.4
else
  export BINARY_INSTALL_PREFIX=${HOME}/devel/openrobots
fi 
export GITUSERNAME=${USERNAME}
export GITREPOSITORY=${GITUSERNAME}@git.laas.fr

export MAKE=make

# This script assumes that a ssh-agent is working.
echo ${BINARY_INSTALL_PREFIX}

function PullPackage() {
if [ -d $1 ]; then 
  cd $1
  git pull
  cd ..
else
  git clone $2 $3
fi

}

function BuildPackage() {
  cd $1
  if ! [ -d $SOURCE_DIR/$1/build ]; then
    mkdir build
  fi 
  cd build
  v=( $( cmake -DCMAKE_INSTALL_PREFIX=${BINARY_INSTALL_PREFIX} -DCMAKE_BUILD_TYPE=RELEASE ..) );
  if [ ! $v ]; then
    echo "Failure on cmake " $1
    exit -1
  fi 
  v=( $( ${MAKE} ) )
  if [ ! $v ]; then
    echo "Failure on trying to make " $1
  fi
  v=( $( ${MAKE} install ) ) 
  if [ ! $v ]; then
    echo "Failure on trying to install " $1
  fi
  cd ../..
}

function ErrorJRLPMT() {
  echo "Error while Pulling JRL MathTools" 
  exit -1
}

# Go to root.
cd ${SOURCE_DIR}
echo "Now in directory src: " ${SOURCE_DIR}

# Install Math Tools
echo "Pulling jrlMathTools"
PullPackage jrlMathTools git+ssh://${GITREPOSITORY}/git/jrl/algo/jrlMathTools
echo "Building jrlMathTools"
BuildPackage jrlMathTools

# Install Matrix Abstract Layer
echo "Pulling MatrixAbstractLayer"
PullPackage MatrixAbstractLayer git+ssh://${GITREPOSITORY}/git/jrl/MALJRL MatrixAbstractLayer
echo "Building MatrixAbstractLayer"
BuildPackage MatrixAbstractLayer

# Install Abstract robot dynamics
echo "Pulling Abstract robot dynamics"
PullPackage abstractRobotDynamics git+ssh://${GITREPOSITORY}/git/jrl/abstractRobotDynamics
echo "Building Abstract Robot dynamics"
BuildPackage abstractRobotDynamics

# Install dynamicsJRLJapan
echo "Pulling dynamics JRL Japan"
PullPackage dynamicsJRLJapan git+ssh://${GITREPOSITORY}/git/jrl/dynamicsJRLJapan 
echo "Building dynamics JRL Japan"
BuildPackage dynamicsJRLJapan 

# Install hrp2Dynamics
echo "Pulling hrp2Dynamics"
PullPackage hrp2Dynamics  git+ssh://${GITREPOSITORY}/git/jrl/hrp2Dynamics
echo "Building hrp2Dynamics"
BuildPackage hrp2Dynamics 

# Install walkGenJrl
echo "Pulling walkGenJrl"
PullPackage walkGenJrl git+ssh://${GITREPOSITORY}/git/jrl/walkGenJrl
echo "Building walkGenJrl"
BuildPackage walkGenJrl

# Install Stack Of Tasks.
echo "Pulling Stack Of Tasks"
PullPackage StackOfTasks git+ssh://${GITREPOSITORY}/git/jrl/StackOfTasks
echo "Building Stack Of Tasks"
# BuildPackage StackOfTasks


