/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Copyright Projet Laas, 2009
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* File:      SoTGenericRobot.hh
* Project:   AF_SOT
* Author:    Nicolas Mansard from Franois Keith 
*
* Version control
* ===============
*
*  $Id$
*
* Description
* ============
*
*
* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_GENERICROBOT_AMELIF_PLUGIN_H__
#define __SOT_GENERICROBOT_AMELIF_PLUGIN_H__

/* -------------------------------------------------------------------------- */
/* --- INCLUDE -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <VNL/vector.h>

/* --- AMELIF --- */
#include "AF_API.hh"
#include "AFSoT/SoTMultiBody.hh"

/* --- SOT --- */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotPeriodicCall.h>



namespace AFScene
{
	class MultiBody;
}
/* -------------------------------------------------------------------------- */
/* --- CLASS ---------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

namespace AFSoT
 {
	class AF_API SoTGenericRobot: public SoTMultiBody
	{
	public : 
		/*! \brief constructor
		*/
		SoTGenericRobot(
			AFScene::MultiBody* mb,
			const std::string & entityName
			);
		virtual ~SoTGenericRobot();

		virtual void initialize();
		virtual void newStep();
		virtual void runStep(unsigned iter);

		//Accessors
		/*! \brief Access to the last control law computed from controlSIN. 
		 */
		virtual void getControlLaw(VNL::Vector<double> &);

		//Public functions related to the SoT
	public:
		virtual void commandLine( const std::string& cmdLine,
					  std::istringstream& cmdArgs,
					  std::ostream& os );
		void play( void );
		void pause( void );


		
	private: /* --- Communication with the SoT --- */
		
		/*! \brief Get the state from the sot.*/
		void readControlLaw( );
		/*! \brief Set the state to the sot. */
		void writeState(unsigned iter);

		
	protected: /* --- ATTRIBUTES --- */

		/* --- SIGNALS OF THE ENTITY --- */
		sotSignalPtr<ml::Vector,int> _controlSIN;
		sotSignalPtr<ml::Vector,int> _attitudeSIN;

		sotSignal<ml::Vector,int> _stateSOUT;
		sotSignal<sotMatrixRotation,int> _attitudeSOUT;

		/* --- PERIODIC CALL --- */
		/*! Peridic call that is called /before/ the MC computation
		* (ie for precomputation needed by the main graph, like 
		* event FSM-based computations). */
		sotPeriodicCall _periodicCallBefore;
		/*! Peridic call that is called /after/ the MC computation (eg
		* for tracing). */
		sotPeriodicCall _periodicCallAfter;

	private:
		bool _suspend;

	private : 
		/* Parameters of the multi body. */
		AFScene::MultiBody* _mb;
		unsigned int _numParam;
		VNL::Vector<double> _mbState;
		VNL::Vector<double> _initQ;

		/* Control law. */
		ml::Vector _controlLaw;

		/* Current iteration. */
		unsigned _iter;
	} ;
}
#endif
