/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Copyright Projet JRL-Japan, 2007
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* File:      SoTHRP2.h
* Project:   SOT
* Author:    Franois Keith and Nicolas Mansard from Paul Evrard
*
* Version control
* ===============
*
*  $Id$
*
* Description
* ============
*
*
* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_HRP2_AMELIF_PLUGIN_H__
#define __SOT_HRP2_AMELIF_PLUGIN_H__

/* -------------------------------------------------------------------------- */
/* --- INCLUDE -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <VNL/vector.h>

/* --- AMELIF --- */
#include "AF_API.hh"
#include "AFSoT/SoTMultiBody.hh"

/* --- SOT --- */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotPeriodicCall.h>



namespace AFScene
{
	class MultiBody;
}
/* -------------------------------------------------------------------------- */
/* --- CLASS ---------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

namespace AFSoT
{
	class AF_API SoTHRP2: public SoTMultiBody
	{
	public:
		enum ForceSignalSource
		{
			FORCE_SIGNAL_RLEG,
			FORCE_SIGNAL_LLEG,
			FORCE_SIGNAL_RARM,
			FORCE_SIGNAL_LARM
		};

	public : 
		/*! \brief constructor
		*/
		SoTHRP2(
			AFScene::MultiBody* mb,
			const std::string & entityName
			);
		~SoTHRP2();


		void initialize();
		void newStep();
		void runStep(unsigned iter);

		//Accessors
		/*! \brief
		*/
		void getControlLaw(VNL::Vector<double> &);

		//Public functions related to the SoT
	public:
		void commandLine( const std::string& cmdLine,
			std::istringstream& cmdArgs,
			std::ostream& os );

		void play( void );
		void pause( void );


		/* --- Communication with the SoT --- */
	private:
		/*! \brief get the state from the sot
		*/
		void readMC( );

		/*! \brief set the state to the sot
		*/
		void writeState(unsigned iter);

		/* --- ATTRIBUTES --- */
	private:
		/* --- SIGNALS OF THE ENTITY --- */
		sotSignalPtr<ml::Vector,int> _controlSIN;
		sotSignalPtr<ml::Vector,int> _attitudeSIN;
		sotSignalPtr<ml::Vector,int> _zmpSIN;

		sotSignal<ml::Vector,int> _stateSOUT;
		std::vector< sotSignal<ml::Vector,int> * > _forcesSOUT;
		sotSignal<sotMatrixRotation,int> _attitudeSOUT;

		sotSignal<ml::Vector,int> _pseudoTorqueSOUT;
		sotSignal<ml::Vector,int> _previousControlSOUT;

		/* --- PERIODIC CALL --- */
		/*! Peridic call that is called /before/ the MC computation
		* (ie for precomputation needed by the main graph, like 
		* event FSM-based computations). */
		sotPeriodicCall _periodicCallBefore;
		/*! Peridic call that is called /after/ the MC computation (eg
		* for tracing). */
		sotPeriodicCall _periodicCallAfter;

	private:
		bool _suspend;
		bool _withForceSignals[4];
		bool _activatePseudoTorqueSignal;
		bool _activatePreviousControlSignal;

	private : 
		//parameters of the mb
		AFScene::MultiBody* _mb;
		unsigned int _numParam;
		VNL::Vector<double>	_mbState;
		VNL::Vector<double> _initQ;

		//velocity
		ml::Vector _mlmc;
		VNL::Vector<double>	 _mlmcVNL;

		//Current iteration
		unsigned _iter;
	} ;
}
#endif
