/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Copyright Projet JRL-Japan, 2007
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* File:      SoTHRP2.h
* Project:   SOT
* Author:    Franois Keith and Nicolas Mansard from Paul Evrard
*
* Version control
* ===============
*
*  $Id$
*
* Description
* ============
*
*
* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_MULTIBODY_AMELIF_PLUGIN_H__
#define __SOT_MULTIBODY_AMELIF_PLUGIN_H__

/* -------------------------------------------------------------------------- */
/* --- INCLUDE -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <VNL/vector.h>

/* --- SOT --- */
#include <sot/sotEntity.h>

/* --- AMELIF --- */
#include "AF_API.hh"


namespace AFScene
{
	class MultiBody;
}
/* -------------------------------------------------------------------------- */
/* --- CLASS ---------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

namespace AFSoT
{
	class AF_API SoTMultiBody: public sotEntity
	{
	public : 
		/*! \brief constructor
		*/
		SoTMultiBody(
			AFScene::MultiBody* mb,
			const std::string & entityName
			);
		virtual ~SoTMultiBody() = 0;

		/*! \brief Initialization after construction. 
		 */
		virtual void initialize() = 0;
		/* \brief Useless.
		 *
		 * This function is supposed to be empty. Just q bit of waste of
		 * computation time (normally, not that much). If you find any 
		 * use to this, please feel free to implement it.
		 */
		virtual void newStep() = 0;
		/* \brief Update the sot and compute the control law. */
		virtual void runStep(unsigned iter) = 0;

		/*! \brief Access to the control law.
		 * 
		 * Fill the param vector with the last control law computed by the SOT,
		 * namely a desired velocity (as not suggested by the function name, 
		 * the vector can be any reference control law, torque, velocity or position.
		 * The interpretation is then done by the Controller instantiated in 
		 * the main programm). 
		 */
		virtual void getControlLaw(VNL::Vector<double> &) = 0;
		virtual void getDesiredSpeed(VNL::Vector<double> &v) { getControlLaw(v); }

		//Public functions related to the SoT
	public:

		/*! \brief Shell command.
		 *
		 * The shell command is a no-space word cmdLine. The args are
		 * separated by spaces. The only return is through the output stream.
		 */
		virtual void commandLine( 
			const std::string& cmdLine,
			std::istringstream& cmdArgs,
			std::ostream& os ) = 0;

		/*! \brief Play the control.
		 *
		 * Modify the state of the robot. After play, the robot is supposed 
		 * to move according to the control law computed by the sot. 
		 */
		virtual void play( void ) = 0;
		/*! \brief Pause the control.
		 *
		 * Modify the state of the robot. After pause, the robot is supposed 
		 * to stop moving according to the SOT control (sometimes, this could means
		 * falling...).
		 */
		virtual void pause( void ) = 0;
	} ;
}
#endif
