/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Copyright Projet JRL-Japan, 2007
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
* File:      SotMainProgram.h
* Project:   SOT
* Author:    Franois Keith and Nicolas Mansard from Paul Evrard
*
* Version control
* ===============
*
*  $Id$
*
* Description
* ============
*
*
* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __STACK_OF_TASKS_AMELIF_PLUGIN_H__
#define __STACK_OF_TASKS_AMELIF_PLUGIN_H__

/* -------------------------------------------------------------------------- */
/* --- INCLUDE -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <vector>

#include "AF_API.hh"

#include <sot/sotPluginLoader.h>
#include "AFSoT/SoTMultiBody.hh"
#include <AFScene/MultiBody.hh>

namespace AFScene
{
	class Universe;
}

/* -------------------------------------------------------------------------- */
/* --- CLASS ---------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
namespace AFSoT
{

	class AF_API SoTWrapper
	{
	public:
		SoTWrapper( void );
		~SoTWrapper();

	public:
		void initialize(void);
		void newStep( );
		void runStep( double dt );

	public:

		/*! \brief add a multibody to the SoT
		*/
		template< class SoTMultiBodyConstructor >
		AFSoT::SoTMultiBody* addMultiBody(AFScene::MultiBody*,
						  const std::string & name);

		/*! \brief returns the SoTMultibody whose multibody's id is id
		*/
		AFSoT::SoTMultiBody* getMultiBodySot(int id) const;

		/*! \brief fills the vector v with all the SoTMultiBody registered
		*/
		void getMultiBodiesSot(std::vector<AFSoT::SoTMultiBody*> & v) const;
		
	public:
		/*! \brief Set the delay corresponding to an iteration.
		* Useful when precising times with the sot 
		* 1 iter <=> 1 dt 
		*/
		void setMinTimeStep(double dt);
		double getMinTimeStep() const;

	public: /* --- SHELL SOT --- */
		void scriptPath( const std::string &strm );
		void script( const std::string &strm );
		void script( std::istringstream &strm );
		void runShellPromptThread(  const bool & catchPlease=true );

	private:
		sotPluginLoader pluginLoader;
		std::vector<SoTMultiBody *> _multiBodiesSot;
		std::vector<SoTMultiBody *> _multiBodiesSotFull;

		unsigned int _iter;

		//
		double _minTimeStep;
	public:
# ifdef SOT_CHECK_TIME 
		static const int TIME_ARRAY_SIZE = 100000;
		double timeArray[TIME_ARRAY_SIZE];
		int stateArray[TIME_ARRAY_SIZE];
		int timeIndex; 
# endif  // #ifdef SOT_CHECK_TIME 
	};




  template< class SoTMultiBodyConstructor >
  SoTMultiBody* 
  SoTWrapper::addMultiBody( AFScene::MultiBody* mb, const std::string & name )
  {	
    /* Pointer to the robot. */
    SoTMultiBody* sotMultiBody = new SoTMultiBodyConstructor(mb, name);
    
    //initial Posture of the robot
    sotMultiBody->initialize();
    
    // _multiBodiesSot is ordered by the AF_MB index. 
    if( (const int&)_multiBodiesSot.size() < mb->getId()+1 )
      _multiBodiesSot.resize(mb->getId()+1,0x0);
    _multiBodiesSot[mb->getId()] = sotMultiBody;
    
    // _multiBodiesSotFull is not ordered.
    _multiBodiesSotFull.push_back(sotMultiBody);
    
    return sotMultiBody;
  }


}

#endif
