/* -------------------------------------------------------------------------- */
/* --- INCLUDE -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include "TestSotMainProgram.hh"
#include <AFScene/DefaultSceneLoader.hh>
#include <AFScene/Universe.hh>
#include <AFScene/MultiBody.hh>

/* --- SOT --- */
#include "AFSoT/SoTWrapper.hh"
#include "AFSoT/SoTHRP2.hh"

TestSotMainProgram::TestSotMainProgram( void )
: _sceneLoader( NULL )
,_universe( NULL )
,_SoT( NULL )
,_DefaultTimeStep(0.005)
{
}

TestSotMainProgram::~TestSotMainProgram( void )
{
}

std::string TestSotMainProgram::getName( void )
{
	return "SotMainProgram"; 
} 

void TestSotMainProgram::initUniverse( void )
{
	// le premier te créer le scene loader.
	_sceneLoader = new AFScene::DefaultSceneLoader(_appInterface);

	// la seconde te permet de recuperer l'universe = le xml.

	_universe = _sceneLoader->createUniverse("AFResources/xml/UniverseSoT.xml", true);

	//Le multibody
	_hrp2 = _universe->getMultiBody(0);

	// Create and fill the sot
	_SoT = new AFSoT::SoTWrapper ();
	_SoTHRP2 = _SoT->addMultiBody<AFSoT::SoTHRP2>(_hrp2, "OpenHRP");
}

#include <AFUtils/Configuration.hh>

void TestSotMainProgram::initProgram( void )
{
	//initialization of the state of the Hrp2
	std::ifstream fileInitialConfig ("AFResources/newTaskData/RobotInitialConfig.txt");
	double numParam = _hrp2->getNumParameters( );
	VNL::Vector<double> initQ (numParam);
	initQ.Fill(0.0);
	if (fileInitialConfig)
	{
		for (unsigned i = 0; i < 40; ++i)
			fileInitialConfig >> initQ[i];
		fileInitialConfig.close();
	}

	_hrp2->setStateVector(initQ);

	//initialize the sot
	_SoT->initialize();

	//launch default script
	std::string path; const std::string labelname="path";
	std::pair<AFUtils::Configuration::tree_iterator,AFUtils::Configuration::tree_iterator> snodes =
		_appInterface->getConfiguration()->pathNodes("/Configuration/Control/sot/scripts");

	for( AFUtils::Configuration::tree_iterator it=snodes.first;it!=snodes.second;++it )
	{
		if( 0==it->second->getString(path,labelname) )
		{
			_SoT->scriptPath(path);
		}
	}
}

/* -------------------------------------------------------------------------- */
/* --- LOOP ----------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


int TestSotMainProgram::run(double dt)
{
	//
	_SoT->newStep();

	// One run execution correspond to one step of the SoT whatever the time dt is
	_SoT->setMinTimeStep(_DefaultTimeStep);
	_SoT->runStep(_DefaultTimeStep);
		
	//integration of the multibody
	int numParam = _hrp2->getNumParameters();
	VNL::Vector<double> mbState (numParam);
	_hrp2->getStateVector( mbState );

	VNL::Vector<double> mbVelocity (numParam);
	_SoTHRP2->getDesiredSpeed(mbVelocity);

	mbState+=mbVelocity * _DefaultTimeStep;

	_hrp2->setStateVector(mbState);

	return 0;
}

void TestSotMainProgram::quitProgram( void )
{
	delete _sceneLoader;
	delete _SoT;
}




/* -------------------------------------------------------------------------- */
/* --- Main Program Handler ------------------------------------------------- */
/* -------------------------------------------------------------------------- */

/* This is for the application to know the name of the main program class.  */
extern "C"
{
	AFMP_EXPORTS AFApplication::MainProgramHandle* getMainPrograms(int& num)
	{
		num = 1;
		AFApplication::MainProgramHandle* result = new AFApplication::MainProgramHandle[1];
		result[0] = new TestSotMainProgram;

		return result;
	}
}


