#include <fstream>
#include <string>
#include <sstream>

#include "PDController.hh"
#include "AFScene/MultiBody.hh"

PDController::PDController(): _multiBody(0x0)
{
}

PDController::PDController(int dim): _multiBody(0x0)
{
	setSize(dim);
}

PDController::~PDController()
{
}

void PDController::setSize(int dimension)
{
	_dimension = dimension;

	_q.Resize(_dimension);
	_q.Fill(0.0);

	_dq.Resize(_dimension);
	_dq.Fill(0.0);

	_qdes.Resize(_dimension);
	_qdes.Fill(0.0);

	_dqdes.Resize(_dimension);
	_dqdes.Fill(0.0);

	_Kp.Resize(_dimension);
	_Kp.Fill(0.0);
	
	_Kd.Resize(_dimension);
	_Kd.Fill(0.0);

	_tau.Resize(_dimension);
	_tau.Fill(0.0);
}

void PDController::setTimeStep(double dt)
{
	_dt = dt;
}

VNL::Vector<double> PDController::control()
{

	//assuming that _dqdes, _q, _dq are set
	

	// Computation of the new desired position
	// given the dq computed in SoT
	// newq = q + dq*dt
	
	_qdes += _dqdes*_dt;

	unsigned size = _Kp.Size();
	// Kp(qdes - q)
	for(unsigned i = 0u; i < size; ++i)
	{
		_tau[i] = _qdes[i];
		_tau[i] -= _q[i];
		_tau[i] *= _Kp[i];
	}

	size = _Kd.Size();
	double de = 0.0;
	// Kp(qdes - q) + Kd(dqdes - dq)
	for(unsigned i = 0u; i < size; ++i)
	{
		de = _dqdes[i];
		de -= _dq[i];
		de *= _Kd[i];
		_tau[i] += de;
	}
	//_tau is now computed \o/

	
	return _tau;
}

void PDController::setDesiredPosition(const VNL::Vector<double>& qdes)
{
	_qdes = qdes;
}

void PDController::setDesiredSpeed(const VNL::Vector<double>& dqdes)
{
	_dqdes = dqdes;
}

void PDController::setKp(const VNL::Vector<double>& Kp)
{
	_Kp = Kp;
}

void PDController::setKd(const VNL::Vector<double>& Kd)
{
	_Kd = Kd;
}

void PDController::getDesiredPosition(VNL::Vector<double>& qdes) const
{
	qdes = _qdes;
}

void PDController::getDesiredSpeed(VNL::Vector<double>& dqdes) const
{
	dqdes = _dqdes;
}

void PDController::getKp(VNL::Vector<double>& Kp) const
{
	Kp = _Kp;
}

void PDController::getKd(VNL::Vector<double>& Kd) const
{
	Kd = _Kd;
}

void PDController::update()
{
	_multiBody->getStateVector(_qdes);
}

void PDController::setMultiBody(AFScene::MultiBody* mb, const char* params)
{
	_multiBody = mb;

	// set the actual configuration
	_multiBody->getStateVector(_q);
	_multiBody->getVelocityVector(_dq);

	// the desired configuration first initialisation
	// it is the same as the actual configuration
	//_multiBody->getStateVector(_qdes);
	update();

	_dqdes.Fill(0.0);

	std::string s(params);
	if(s.length() == 0)
		return;  // params invalid

	std::ifstream in(params);
	if(in.is_open() == false)
		return; // params invalid

	std::string line;
	int id = -1;
	double kp = 0.0;
	double kd = 0.0;
	std::string dummy;

	std::getline(in, line); // first line is title line; ignore

	//simple parsor for Kp and Kd; the file template:
	//jointId Kp[jointId] Kd[jointId] jointName
	while(in.eof() == false)
	{
		in >> id >> kp >> kd >> dummy;
		_Kp[id] = kp;
		_Kd[id] = kd;
	}
}

void PDController::setSoTHRP2(AFSoT::SoTMultiBody* sotHRP2)
{
	_sotHRP2 = sotHRP2;
}

VNL::Vector<double> PDController::computeTorque()
{
	//Get the current configuration
	_multiBody->getStateVector(_q);
	_multiBody->getVelocityVector(_dq);

	//Get the desired configuration from the sotMultibody
	_sotHRP2->getDesiredSpeed(_dqdes);  

	//then compute torque
	return control();
}
