////////////////////////////////////////////////////////////////////////////////////
// stack of tasks and dynamic in AMELIF. 
////////////////////////////////////////////////////////////////////////////////////

#ifndef SOT_AMELIF_DYNAMIC_HH
#define SOT_AMELIF_DYNAMIC_HH

#include "Tools.hh"
//#include "trajectoryComposer.hh"

#include "AFApplication/MainProgram.hh"

#include "AFScene/DefaultSceneLoader.hh"
#include "AFScene/Universe.hh"
#include "AFScene/MultiBody.hh"

#include "AFCollision/CollisionDetection.hh"
#include "AFCollision/CollisionPublisher.hh"
#include "AFCollision/DefaultPointCollision.hh"

#include "AFDynamics/InteractionListener.hh"
#include "AFDynamics/DefaultDynamicsSimulatorLoader.hh"
#include "AFDynamics/DynamicsSimulator.hh"

#include "AFUtils/Timer.hh" 

//#include "AFControl/PDJointController.hh"
#include "sadPDController.hh"

#include "AFSoT/SoTHRP2.hh"
#include "AFSoT/SoTWrapper.hh"
//#include "AFSoT/SoTEntity.hh"

//#include "AFSoT/trajectoryComposer.hh"

//HRP-2 standing the object
namespace SotAmelifDynamicNamespace {
	class SotAmelifDynamic : public AFApplication::MainProgram
	{
	public:

		/* constructor and destructor */
		SotAmelifDynamic();
		virtual ~SotAmelifDynamic();

	public:

		/* initialisation of amelif universe */
		virtual void initUniverse();

		/* initialisation of the program that will use amelif's universe */
		virtual void initProgram();

		/* main function of the program, which is looped */
		virtual int run(double step);

		/* cleaning */
		virtual void quitProgram();
		
		/* get name of the program */
		virtual std::string getName();

	private:

		double _DefaultTimeStep;

		int _numStep; 
		double _moyDet;
		double _moySim;

		/* ??? */
		double _simulationTimeElapsed;
		
 		sotSignal<ml::Matrix,int> * signalTestSOUT; 

		//The scene
		AFScene::SceneLoader* _sceneLoader;
		AFScene::Universe* _universe;
		AFSoT::SoTWrapper * _SoT;

		//The HRP2 and data associated to him
		AFScene::MultiBody* _hrp2;

		//sotHRP2
		AFSoT::SoTMultiBody* _sotHRP2;
		//AFSoT::SoTEntity* _entity;

		//The collision module
		AFCollision::CollisionDetection* _collisionDetection;

		//The dynamic module
		AFDynamics::DynamicsSimulatorLoader* _simLoader;
		AFDynamics::DynamicsSimulator* _simulator;

		//the control
	        sadPDController* _controller;

		InteractionDisplayer * _interactionDisplayer;
	};
}

#endif

