////////////////////////////////////////////////////////////////////////////////////
// stack of tasks and dynamic in AMELIF. 
////////////////////////////////////////////////////////////////////////////////////

#ifndef SAD_MP_ABSTRACT_HH
#define SAD_MP_ABSTRACT_HH

#include "Tools.hh"

#include "AFApplication/MainProgram.hh"

#include "AFScene/DefaultSceneLoader.hh"
#include "AFScene/Universe.hh"
#include "AFScene/MultiBody.hh"

#include "AFUtils/Timer.hh" 
#include "AFSoT/SoTHRP2.hh"
#include "AFSoT/SoTWrapper.hh"

//HRP-2 standing the object
namespace sadMainProgram {
  class sadMainProgramAbstract : public AFApplication::MainProgram
  {
  public:
    
    /* constructor and destructor */
    sadMainProgramAbstract();
    virtual ~sadMainProgramAbstract();
    
  public:
    
    /* initialisation of amelif universe */
    virtual void initUniverse();
    
    /* initialisation of the program that will use amelif's universe */
    virtual void initProgram();
    
    /* main function of the program, which is looped */
    virtual int run(double step);
    
    /* cleaning */
    virtual void quitProgram();
    
    /* get name of the program */
    virtual std::string getName();
    
  protected:

    bool _verbose;
    /* Simulation time step. */
    double _timeStep;
    std::string _referenceName;

    /* The AF scene. */
    AFScene::SceneLoader* _sceneLoader;
    AFScene::Universe* _universe;
    AFSoT::SoTWrapper * _SoT;

    /* The multi-body robot and its associated datas. */
    AFScene::MultiBody* _mainMultiBody;

    /* The SOT structure around the multibody. */
    AFSoT::SoTMultiBody* _sotMainMultiBody;

  private: /* Main-program generic paramaters. */
    static const std::string _UNIVERSE_XML_MARKUP;
    static const std::string _SOT_SCRIPT_PATH;
    static const std::string _INITIAL_POSITION_XML_MARKUP;
    static const double  _DEFAULT_TIME_STEP;

  public:  /* Access to the main-program generic paramaters. */
    virtual const std::string getUniverseXmlMarkup( void ) const;
    virtual const std::string getSotScriptPath( void ) const;
    virtual const std::string getInitialPositionXmlMarkup( void ) const;
    virtual const double getDefaultTimeStep( void ) const;
    virtual const int getMultiBodyId(void) ;

    virtual const std::string getUniverseFileName( void ) const;

  protected: /* Construction of the scene and modules. */
    virtual void constructSot( void );
    virtual void constructSadUniverse( void );
    virtual void constructSimulation( void );
    virtual void constructController( void ) = 0;

    virtual void selecSotMultiBodies( void );
    virtual void selecSimulatedMultiBodies( void );
    virtual void setReferenceName( void );

  protected: /* Initialization of the scene and modules. */
    virtual void initSot( void );
    virtual void initSadUniverse( void );
    virtual void initSimulation( void ) = 0;
    virtual void initController( void ) = 0;

    virtual void initMultiBodiesSpecificParams( void );
    virtual void initMultiBodiesConfiguration( void );
    virtual void initTimeFactor( void );
    virtual void runInitialSotScripts( void );
	  
  protected: /* Run-loop virtual functions. */
    virtual const double computeTimeStep( const double& AFTimeStep ) const;
    virtual void runSot( const double& timeStep );
    virtual void runSadUniverse( const double& timeStep );
    virtual void runSimulation( const double& timeStep ) = 0;
    virtual void runController( const double& timeStep ) = 0;

  protected: /* Destruction */
    virtual void destructSot( void );
    virtual void destructSadUniverse( void );
    virtual void destructSimulation( void ) = 0;
    virtual void destructController( void ) = 0;
  
  };
};

#endif

