////////////////////////////////////////////////////////////////////////////////////
// stack of tasks and dynamic in AMELIF. 
////////////////////////////////////////////////////////////////////////////////////

#ifndef SADMAINPROGRAMDYNAMICABSTRACT_HH
#define SADMAINPROGRAMDYNAMICABSTRACT_HH

#include "sadMainProgramAbstract.hh"

#include "AFCollision/CollisionDetection.hh"
#include "AFCollision/CollisionPublisher.hh"
#include "AFCollision/DefaultPointCollision.hh"

#include "AFDynamics/InteractionListener.hh"
#include "AFDynamics/DefaultDynamicsSimulatorLoader.hh"
#include "AFDynamics/DynamicsSimulator.hh"

#include "AFUtils/Timer.hh" 

#include "sadAbstractController.hh"


//HRP-2 standing the object
namespace sadMainProgram {
  class sadMainProgramDynamicAbstract : public sadMainProgramAbstract
  {
  public:
    
    /* constructor and destructor */
    sadMainProgramDynamicAbstract();
    virtual ~sadMainProgramDynamicAbstract();
    
  public:
    
    /* get name of the program */
    virtual std::string getName();

  protected: 
    using sadMainProgramAbstract::_verbose;
    using sadMainProgramAbstract::_timeStep;
    using sadMainProgramAbstract::_sceneLoader;
    using sadMainProgramAbstract::_universe;
    using sadMainProgramAbstract::_SoT;
    using sadMainProgramAbstract::_mainMultiBody;
    using sadMainProgramAbstract::_sotMainMultiBody;

    virtual const std::string getActuatorConfigXmlMarkup( void ) const;
    virtual const std::string getFixedRootXmlMarkup( void ) const;
    virtual const bool getFixedRootOption( void ) const;
 
  private:
    static const std::string _ACTUATOR_CONFIG_XML_MARKUP;
    static const std::string _FIXED_ROOT_XML_MARKUP;
 
  protected:

    /* The collision module. */
    AFCollision::CollisionDetection* _collisionDetection;

    /* The dynamic module. */
    AFDynamics::DynamicsSimulatorLoader* _simLoader;
    AFDynamics::DynamicsSimulator* _simulator;

    /* The multi-body controller. */
    sadAbstractController* _controller;

    /* A tool to visualize interactions (forces ...). */
    InteractionDisplayer * _interactionDisplayer;
	  
  private: /* Main-program generic paramaters. */

  protected: /* Construction of the scene and modules. */
    virtual void constructSimulation(void);

    virtual void selecSimulatedMultiBodies(void);
  
  protected: /* Initialization of the scene and modules. */
    virtual void initSimulation( void );

    virtual void initMultiBodiesSpecificParams( void );
	  
  protected: /* Run-loop virtual functions. */
    virtual void runSimulation( const double& timeStep );
    virtual void runController( const double& timeStep );

  protected: /* Destruction */
    virtual void destructSimulation( void );
    virtual void destructController( void );
  
  };
};

#endif

