#include "sadMainProgramDynamicTorqueControl.hh"
#include "AFUtils/Configuration.hh"

namespace sadMainProgram {

  sadMainProgramDynamicTorqueControl::sadMainProgramDynamicTorqueControl()
  {
  }
  
  sadMainProgramDynamicTorqueControl::~sadMainProgramDynamicTorqueControl()
  {
  }

  const std::string sadMainProgramDynamicTorqueControl::_TORQUE_CONTROLLER_CONFIG_XML_MARKUP
  = "TorqueControllerData";

  const std::string sadMainProgramDynamicTorqueControl::
  getTorqueControllerConfigXmlMarkup( void ) const
  { 
    std::string res;
    if( 0==_referenceName.length() ) res = "";
    else { res = _referenceName + "_"; }
    res += sadMainProgramDynamicTorqueControl::_TORQUE_CONTROLLER_CONFIG_XML_MARKUP; 
    return res;
  }


  /* --- CONSTRUCTION ------------------------------------------------------- */

  void sadMainProgramDynamicTorqueControl::constructController( void )
  {
    _torqueController = new sadTorqueController(); 

    _controller = _torqueController;
    _torqueController->setSize(_mainMultiBody->getNumDoF());
    _torqueController->setTimeStep(sadMainProgramAbstract::getDefaultTimeStep());
    _torqueController->setSotMultiBody(_sotMainMultiBody);

    /* Build a controller for the SOT MultiBody. */
    std::string controllerFile;
    AFUtils::ConfigNode* config 
      = _appInterface->getConfiguration()->findNode("/Configuration",
						    "Control");
    if(config != 0x0)
      {
	config->getString(controllerFile,getTorqueControllerConfigXmlMarkup()); 
      }
    _torqueController->setMultiBody(_mainMultiBody,controllerFile.c_str());

    if(_verbose) 
      { std::cout << "Parsed TORQUEController config file '" << controllerFile;
	std::cout << std::flush << "'" << std::endl; }
  }


  /* --- INITIALIZATION ----------------------------------------------------- */
  void  sadMainProgramDynamicTorqueControl::initTimeFactor( void )
  {
    sadMainProgramDynamicAbstract::initTimeFactor();
    _torqueController->setTimeStep(_timeStep);
  }
  void sadMainProgramDynamicTorqueControl::initController( void )
  {
    _torqueController->update();
  }


 std::string sadMainProgramDynamicTorqueControl::getName()
  {
    return "AfSotTorqueControlDyn";
  }
}


