#include "sadMainProgramDynamicVelocityControl.hh"
#include "AFUtils/Configuration.hh"

namespace sadMainProgram {

  sadMainProgramDynamicVelocityControl::sadMainProgramDynamicVelocityControl()
  {
  }
  
  sadMainProgramDynamicVelocityControl::~sadMainProgramDynamicVelocityControl()
  {
  }

  const std::string sadMainProgramDynamicVelocityControl::_CONTROLLER_CONFIG_XML_MARKUP
  = "PDControllerData";

  const std::string sadMainProgramDynamicVelocityControl::
  getControllerConfigXmlMarkup( void ) const
  { 
    std::string res;
    if( 0==_referenceName.length() ) res = "";
    else { res = _referenceName + "_"; }
    res += sadMainProgramDynamicVelocityControl::_CONTROLLER_CONFIG_XML_MARKUP; 
    return res;
  }


  /* --- CONSTRUCTION ------------------------------------------------------- */

  void sadMainProgramDynamicVelocityControl::constructController( void )
  {
    _pdcontroller = new sadPDController(); 

    _controller = _pdcontroller;
    _pdcontroller->setSize(_mainMultiBody->getNumDoF());
    _pdcontroller->setTimeStep(sadMainProgramAbstract::getDefaultTimeStep());
    _pdcontroller->setSotMultiBody(_sotMainMultiBody);

    /* Build a controller for the SOT MultiBody. */
    std::string controllerFile;
    AFUtils::ConfigNode* config 
      = _appInterface->getConfiguration()->findNode("/Configuration",
						    "Control");
    if(config != 0x0)
      {
	config->getString(controllerFile,getControllerConfigXmlMarkup()); 
      }
    _pdcontroller->setMultiBody(_mainMultiBody,controllerFile.c_str());

    if(_verbose) 
      { std::cout << "Parsed PDController config file '" << controllerFile;
	std::cout << std::flush << "'" << std::endl; }
  }


  /* --- INITIALIZATION ----------------------------------------------------- */
  void  sadMainProgramDynamicVelocityControl::initTimeFactor( void )
  {
    sadMainProgramDynamicAbstract::initTimeFactor();
    _pdcontroller->setTimeStep(_timeStep);
  }
  void sadMainProgramDynamicVelocityControl::initController( void )
  {
    _pdcontroller->update();
  }


 std::string sadMainProgramDynamicVelocityControl::getName()
  {
    return "AfSotVelocityControlDyn";
  }
}


