#include <fstream>
#include <string>
#include <sstream>

#include "sadPDController.hh"
#include "AFScene/MultiBody.hh"

sadPDController::
sadPDController(): _multiBody(0x0)
{
}

sadPDController::
sadPDController(int dim): _multiBody(0x0)
{
  setSize(dim);
}

sadPDController::
~sadPDController()
{
}

void sadPDController::
setSize(int dimension)
{
  _dimension = dimension;

  _q.Resize(_dimension);
  _q.Fill(0.0);

  _dq.Resize(_dimension);
  _dq.Fill(0.0);

  _qdes.Resize(_dimension);
  _qdes.Fill(0.0);

  _dqdes.Resize(_dimension);
  _dqdes.Fill(0.0);

  _Kp.Resize(_dimension);
  _Kp.Fill(0.0);
	
  _Kd.Resize(_dimension);
  _Kd.Fill(0.0);

  _tau.Resize(_dimension);
  _tau.Fill(0.0);
}

void sadPDController::
setTimeStep(double dt)
{
  _dt = dt;
}

#include <fstream>

int itersadPD = 0;
// class sadPD_intializator {
// public: 
//   sadPD_intializator(void) 
//   {
//     std::ofstream fdebug("/tmp/nmansard/jl_pd.dat"); fdebug << std::endl;
//     std::cout << "Init sadPD" << std::endl;
//   } 
// };
// sadPD_intializator sadPD_init;

VNL::Vector<double> sadPDController::
control()
{
  std::ofstream fdebug("/tmp/nmansard/jl_pd.dat",ios_base::app);

  //assuming that _dqdes, _q, _dq are set
	

  // Computation of the new desired position
  // given the dq computed in SoT
  // newq = q + dq*dt
	
  _qdes += _dqdes*_dt;
  //_qdes.Fill(0); _qdes[0]=.5;
  //if( itersadPD>2000) _qdes[1]=-.5;

  unsigned size = _Kp.Size();
  // Kp(qdes - q)
  for(unsigned i = 0u; i < size; ++i)
    {
      _tau[i] = _qdes[i];
      _tau[i] -= _q[i];
      _tau[i] *= _Kp[i];
    }
  fdebug << itersadPD++ <<" " << _q[0] << " " << _dqdes[0] << " " << _qdes[0] << " " 
	 << _Kp[0] << " " << _tau[0] << " ";
  fdebug << _tau[1] << " " << _tau[2] << " ";

  size = _Kd.Size();
  double de = 0.0;
  // Kp(qdes - q) + Kd(dqdes - dq)
  for(unsigned i = 0u; i < size; ++i)
    {
      de = 0; //_dqdes[i];
      de -= _dq[i];
      de *= _Kd[i];
      _tau[i] += de;
    }
  //_tau is now computed \o/

  fdebug << _dq[0] << " " << _Kd[0] << " " << _tau[0] << " ";
  fdebug << _tau[1] << " " << _tau[2] << " " << std::endl;

  return _tau;
}

void sadPDController::
setDesiredPosition(const VNL::Vector<double>& qdes)
{
  _qdes = qdes;
}

void sadPDController::
setDesiredSpeed(const VNL::Vector<double>& dqdes)
{
  _dqdes = dqdes;
}

void sadPDController::
setKp(const VNL::Vector<double>& Kp)
{
  _Kp = Kp;
}

void sadPDController::
setKd(const VNL::Vector<double>& Kd)
{
  _Kd = Kd;
}

void sadPDController::
getDesiredPosition(VNL::Vector<double>& qdes) const
{
  qdes = _qdes;
}

void sadPDController::
getDesiredSpeed(VNL::Vector<double>& dqdes) const
{
  dqdes = _dqdes;
}

void sadPDController::
getKp(VNL::Vector<double>& Kp) const
{
  Kp = _Kp;
}

void sadPDController::
getKd(VNL::Vector<double>& Kd) const
{
  Kd = _Kd;
}

void sadPDController::
update()
{
  _multiBody->getStateVector(_qdes);
}

bool sadPDController::
parseConfigFile( const std::string& paramsFilename )
{
  if(paramsFilename.length() == 0)
    return false;  // params invalid

  std::ifstream in(paramsFilename.c_str());
  if(in.is_open() == false)
    return false; // params invalid

  std::string line;
  int id = -1;
  double kp = 0.0;
  double kd = 0.0;
  std::string dummy;

  std::getline(in, line); // first line is title line; ignore

  //simple parsor for Kp and Kd; the file template:
  //jointId Kp[jointId] Kd[jointId] jointName
  while(in.eof() == false)
    {
      in >> id >> kp >> kd >> dummy;
      _Kp[id] = kp;
      _Kd[id] = kd;
    }

  return true;
}


void sadPDController::
setMultiBody(AFScene::MultiBody* mb, const char* params)
{
  _multiBody = mb;

  // set the actual configuration
  _multiBody->getStateVector(_q);
  _multiBody->getVelocityVector(_dq);

  parseConfigFile( params );

  // the desired configuration first initialisation
  // it is the same as the actual configuration
  //_multiBody->getStateVector(_qdes);
  update();

  _dqdes.Fill(0.0);

}

void sadPDController::
setSotMultiBody(AFSoT::SoTMultiBody* sotMB)
{
  _sotMultiBody = sotMB;
}

VNL::Vector<double> sadPDController::
computeTorque()
{
  //Get the current configuration
  _multiBody->getStateVector(_q);
  _multiBody->getVelocityVector(_dq);

  //Get the desired configuration from the sotMultibody
  _sotMultiBody->getDesiredSpeed(_dqdes);  

  //then compute torque
  return control();
}
