#include <fstream>
#include <string>
#include <sstream>

#include "sadTorqueController.hh"
#include "AFScene/MultiBody.hh"

const double sadTorqueController::_EPSILON_STATIC = .1;

sadTorqueController::sadTorqueController(): _multiBody(0x0)
{
}

sadTorqueController::sadTorqueController(int dim): _multiBody(0x0)
{
	setSize(dim);
}

sadTorqueController::~sadTorqueController()
{
}

void sadTorqueController::setSize(int dimension)
{
	_dimension = dimension;

	_dq.Resize(_dimension);
	_dq.Fill(0.0);

	_fstatic.Resize(_dimension);
	_fstatic.Fill(0.0);
	
	_ffluid.Resize(_dimension);
	_ffluid.Fill(0.0);

	_tau.Resize(_dimension);
	_tau.Fill(0.0);
}

void sadTorqueController::setTimeStep(double dt)
{
	_dt = dt;
}

void sadTorqueController::setFrictionStatic(const VNL::Vector<double>& f)
{  _fstatic = f; }
void sadTorqueController::setFrictionFluid(const VNL::Vector<double>& f)
{  _ffluid = f; }
void sadTorqueController::getFrictionStatic(VNL::Vector<double>& f)
{ f = _fstatic; }
void sadTorqueController::getFrictionFluid(VNL::Vector<double>& f)
{ f = _ffluid; }

void sadTorqueController::update()
{
  _multiBody->getVelocityVector(_dq);
}

bool sadTorqueController::parseConfigFile( const std::string& paramsFilename )
{
  if(paramsFilename.length() == 0)
    { return false;  /* Params invalid. */ }

      std::ifstream in(paramsFilename.c_str());
  if(in.is_open() == false)
    { return false; /* Params invalid. */ }
  
  std::string line;
  int id = -1;
  double fst = 0.0;
  double ffl = 0.0;
  std::string dummy;

  std::getline(in, line); // first line is title line; ignore
	
  //simple parsor for Kp and Kd; the file template:
  //jointId Kp[jointId] Kd[jointId] jointName
  while(in.eof() == false)
    {
      in >> id >> fst >> ffl >> dummy;
      _fstatic[id] = fst;
      _ffluid[id] = ffl;
    }

  return true;
}
void sadTorqueController::setMultiBody(AFScene::MultiBody* mb, 
				       const std::string& params)
{
	_multiBody = mb;

	/* The desired configuration first initialisation
	 * it is the same as the actual configuration. */
	update();

	if( parseConfigFile( params ) )
	  { std::cerr << "Error while parsing '" << params << "'" << std::endl; }
}

void sadTorqueController::setSotMultiBody(AFSoT::SoTMultiBody* sotMB)
{
	_sotMultiBody = sotMB;
}


int itertorque = 0;
 class torque_intializator {
 public: 
   torque_intializator(void) 
   {
     std::ofstream fdebug("/tmp/nmansard/jl_torq.dat"); fdebug << std::endl;
     std::cout << "Init torque" << std::endl;
   } 
 };
 torque_intializator torque_init;

VNL::Vector<double> sadTorqueController::computeTorque()
{
  /* Get the current configuration. */
  update();
  
  /* Get the reference control from the sotMultibody. */
  _sotMultiBody->getControlLaw(_tau);  

  /* Applied static friction. */
  for( unsigned int i=0;i<_dq.Size();++i )
    {
      if( _dq[i]<-_EPSILON_STATIC ) _tau[i] += _fstatic[i];
      else if( _dq[i]>_EPSILON_STATIC ) _tau[i] -= _fstatic[i];
    }

  /* Applied fluid friction. */
  for( unsigned int i=0;i<_dq.Size();++i )
    {
      _tau[i] -= _dq[i]*_ffluid[i];
    }
  
  //then compute torque
  
  std::ofstream fdebug("/tmp/nmansard/jl_torq.dat",ios_base::app); 
  fdebug << itertorque++ << " " << _tau << "\t" << _dq << std::endl;


  return _tau;
}
