/** @doc This object implements a Corba client for the teleoperation of HRP from TUM

CVS Information:
$Id$
$Author$
$Date$
$Revision$
$Source$
$Log$

Copyright (c) 2003-2006, 
@author Olivier Stasse, Paul Evrard, Sylvain Miossec
   
JRL-Japan, CNRS/AIST

All rights reserved.
   
Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:
   
* Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
* Neither the name of the CNRS and AIST nor the names of its contributors 
may be used to endorse or promote products derived from this software without specific prior written permission.
   
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cmath>

#include <ConnectionToCorba.h>
//#include <Types.h>
#include <pthread.h>

#define ODEBUG2(x)
#define ODEBUG3(x) fprintf(stderr,"CorbaConnection: %p \n",x);

#if 1
#define ODEBUG(x) fprintf(stderr,"CorbaConnection: %p \n",x);
#else
#define ODEBUG(x) 
#endif


#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#define usleep(duration) Sleep(duration/1000)
#endif



using namespace std;

TeleopWithTUM_Server::SeqMeasureData Measures_from_HRP_Threads_Exchange;  
TeleopWithTUM_Server::ControlData Control_to_HRP_Threads_Exchange;

extern pthread_mutex_t UDP_CORBA_Exchange_Mutex;

ConnectionToCorba::ConnectionToCorba(istringstream &istrm)
{
  m_NbOfProcessWithoutTrialConnection = 10;
  m_IntervalBetweenConnectionTrials =0;

  istrm >> m_ServerID;
  istrm >> m_ServerKind;
  
  TryConnectionToStackOfTasks();
}

void show_chunk(const CosNaming::BindingList &bl)
{
  for (CORBA::ULong i=0;i<bl.length();i++)
    {
      cout << bl[i].binding_name[0].id;
      if (bl[i].binding_name[0].kind[0] != '\0')
	cout << "(" << bl[i].binding_name[0].kind << ")";
      if (bl[i].binding_type == CosNaming::ncontext)
	cout << ": context" << endl;
      else
	cout << ": reference" << endl;
    }
}

void list_context(CosNaming::NamingContext_ptr nc)
{
  CosNaming::BindingIterator_var it;
  CosNaming::BindingList_var bl;

  const CORBA::ULong CHUNK=100;

  nc->list(CHUNK, bl ,it);
  show_chunk(bl);

  if (!CORBA::is_nil(it))
    {
      while(it->next_n(CHUNK,bl))
	show_chunk(bl);
      it->destroy();
    }
}

CORBA::Object_ptr ConnectionToCorba::getObjectReference(string ServerID, string ServerKind)
{
  CORBA::ORB_ptr orb=0;
  try 
    {
      int largc = 3;
      char *largv[] = { "grabdma","-ORBInitRef","NameService=corbaloc:iiop:172.16.59.2:5005/NameService"}; //hrp2010c
      //char *largv[] = { "ExchangeData","-ORBInitRef","NameService=corbaloc:iiop:jrlhrp01.a02.aist.go.jp:5005/NameService"};
      //char *largv[] = { "ExchangeData","-ORBInitRef","NameService=corbaloc:iiop:dhcpt1228.a02.aist.go.jp:5005/NameService"}; // Paul
      //char *largv[] = { "ExchangeData","-ORBInitRef","NameService=corbaloc:iiop:dhcpt1252.a02.aist.go.jp:5005/NameService"};  // Sylvain
      
      orb = CORBA::ORB_init(largc, largv);
    }
  
  catch(CORBA::SystemException&) 
    {
      fprintf(stderr, "Caught CORBA::SystemException.\n");
    }

  catch(CORBA::Exception&) 
    {
      fprintf(stderr,"Caught CORBA::Exception.\n");
    }

  catch(...) 
    {
      fprintf(stderr,"Caught unknown exception.\n");
    }

  CosNaming::NamingContext_var rootContext;
  CORBA::Object_var obj;
  try 
    {
      // Obtain a reference to the root context of the Name service:
 
      obj = orb->resolve_initial_references("NameService");
      
    }
  catch(...)
    {
      cout << "Phase 1 error"<<endl;
    }

  try
    {
      // Narrow the reference returned.
      rootContext = CosNaming::NamingContext::_narrow(obj);
    }
  catch(CosNaming::NamingContext::NotFound& ex) {
    // This should not happen!
    fprintf(stderr,"NameService not found.\n");
    return CORBA::Object::_nil();
  }
  catch(CosNaming::NamingContext::InvalidName& ex) {
    // This should not happen!
    fprintf(stderr,"Invalid Name.\n");
    return CORBA::Object::_nil();
  }
  catch(CosNaming::NamingContext::CannotProceed& ex) {
    // This should not happen!
    fprintf(stderr,"Cannot Proceed.\n");
    return CORBA::Object::_nil();
  }
  catch(CosNaming::NamingContext::AlreadyBound& ex) {
    // This should not happen!
    fprintf(stderr,"Already Bound.\n");
    return CORBA::Object::_nil();
  }
  catch(CosNaming::NamingContext::NotEmpty& ex) {
    // This should not happen!
    fprintf(stderr,"Not empty.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::BAD_INV_ORDER) {
    fprintf(stderr,"Bad inv order.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::BAD_OPERATION) {
    fprintf(stderr,"BAD_OPERATION.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::BAD_PARAM) {
    fprintf(stderr,"BAD_PARAM.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::BAD_TYPECODE) {
    fprintf(stderr,"BAD_TYPECODE.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::COMM_FAILURE) {
    fprintf(stderr,"COMM_FAILURE.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::DATA_CONVERSION) {
    fprintf(stderr,"DATA_CONVERSION.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::IMP_LIMIT) {
    fprintf(stderr,"IMP_LIMIT.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::INITIALIZE) {
    fprintf(stderr,"INITIALIZE .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::INTERNAL) {
    fprintf(stderr,"INTERNAL.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::INV_OBJREF) {
    fprintf(stderr,"INV_OBJREF.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::INV_POLICY) {
    fprintf(stderr,"INV_POLICY .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::MARSHAL) {
    fprintf(stderr," MARSHAL.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::NO_IMPLEMENT) {
    fprintf(stderr,"NO_IMPLEMENT .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::NO_MEMORY) {
    fprintf(stderr,"NO_MEMORY .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::NO_PERMISSION) {
    fprintf(stderr,"NO_PERMISSION .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::NO_RESOURCES) {
    fprintf(stderr," NO_RESOURCES.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::NO_RESPONSE) {
    fprintf(stderr," NO_RESPONSE.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::OBJECT_NOT_EXIST) {
    fprintf(stderr,"OBJECT_NOT_EXIST .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::OBJ_ADAPTER) {
    fprintf(stderr,"OBJ_ADAPTER .\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::TRANSIENT) {
    fprintf(stderr,"Could not reach the NameService server.\n");
    return CORBA::Object::_nil();
  }
  catch(CORBA::UNKNOWN) {
    fprintf(stderr,"UNKNOWN .\n");
    return CORBA::Object::_nil();
  }
  
  catch(...)
    { 
      cout << "Phase 2 error" << endl;
    }
  try
    {
      //  list_context(rootContext);
      
      if( CORBA::is_nil(rootContext) ) 
	{
	  fprintf(stderr,"Failed to narrow the root naming context.\n");
	  return CORBA::Object::_nil();
	}
    }
  
  catch(CosNaming::NamingContext::NotFound& ex) {
    // This should not happen!
    fprintf(stderr,"NameService not found.\n");
    return CORBA::Object::_nil();
  }

  catch(CORBA::ORB::InvalidName& ex) {
    // This should not happen!
    fprintf(stderr,"Service required is invalid [does not exist].\n");
    return CORBA::Object::_nil();
  }

  catch(...)
    {
      fprintf(stderr,"Unknown Error\n");
      return CORBA::Object::_nil();
    }
  // Create a name object, containing the name test/context:
  CosNaming::Name name;
  name.length(1);

  name[0].id   = ServerID.c_str();
  name[0].kind = ServerKind.c_str();


  // Note on kind: The kind field is used to indicate the type
  // of the object. This is to avoid conventions such as that used
  // by files (name.type -- e.g. test.ps = postscript etc.)
 
  try {
    // Resolve the name to an object reference.
    return rootContext->resolve(name);
  }
  catch(CosNaming::NamingContext::NotFound& ex) {
    // This exception is thrown if any of the components of the
    // path [contexts or the object] aren't found:
    fprintf(stderr,"Context not found.\n");
  }
  catch(CORBA::COMM_FAILURE& ex) {
    fprintf(stderr,"Caught system exception COMM_FAILURE -- unable to contact the \n naming service.\n");
  }
  catch(CORBA::SystemException&) {
    fprintf(stderr,"Caught a CORBA::SystemException while using the naming service.\n");
  }
  catch(...)
    {
      fprintf(stderr,"I caught you AHAHHAHAHA !\n");
    }
  cout << "Here...failed to find the Server " << ServerID << endl;
  return CORBA::Object::_nil();
}




int ConnectionToCorba::TryConnectionToStackOfTasks()
{
  CORBA::Object_ptr anObject=0;
  
  try 
    {
      anObject = getObjectReference(m_ServerID,m_ServerKind);
    } 
  catch(...)
    {
      ODEBUG("Visual Servoing not found");
    }

  //   if( CORBA::is_nil(anObject) ) 
  //{
  //	fprintf(stderr,"Object is null.\n");
  //	m_StackOfTasks = CORBA::Object::_nil();
  //	return 0;
  //}

  try
    {
      m_StackOfTasks = TeleopWithTUM_Server::_narrow(anObject);
    }
  catch(...)
    {
      ODEBUG("Visual Servoing not narrowed");
    }
  
  
  return 0;
}

void ConnectionToCorba::SetControlData()
{
  if (!CORBA::is_nil(m_StackOfTasks))
    {
      try 
	{
	  // Get data from udp thread
	  TeleopWithTUM_Server::ControlData Control_to_HRP = Control_to_HRP_Threads_Exchange;	  

	  // Send some data to the robot...
	  m_StackOfTasks->SetControlData(Control_to_HRP);
	}
  
  
      catch(...)
	{
      
	  ODEBUG("Sorry the data was not send to the visual servoing plugin" );
	  if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	    {
	      CORBA::release(m_StackOfTasks);
	      TryConnectionToStackOfTasks();
	      m_NbOfProcessWithoutTrialConnection=0;
	    }
	  else
	    m_NbOfProcessWithoutTrialConnection++;
      
	}
    }
  else 
    {
      if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	{
	  TryConnectionToStackOfTasks();
	  m_NbOfProcessWithoutTrialConnection=0;
	}
      else
	m_NbOfProcessWithoutTrialConnection++;
    }
  return;
}

void ConnectionToCorba::GetMeasureData()
{
  if (!CORBA::is_nil(m_StackOfTasks))
    {
      try 
	{
	  // Get data from udp thread
	  TeleopWithTUM_Server::SeqMeasureData_var Measures_from_HRP;  
	      
	  // Send some data to the robot...
	  m_StackOfTasks->GetMeasureData(Measures_from_HRP);

	  // Take data from ROBOT !
	  Measures_from_HRP_Threads_Exchange.length(Measures_from_HRP->length());

	  for( unsigned int i =0;i<Measures_from_HRP->length();++i )
	    {
	      Measures_from_HRP_Threads_Exchange[i] = Measures_from_HRP[i];
	    }	  
	}
	
  
      catch(...)
	{
      
	  ODEBUG("Sorry the data was not send to the visual servoing plugin" );
	  if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	    {
	      CORBA::release(m_StackOfTasks);
	      TryConnectionToStackOfTasks();
	      m_NbOfProcessWithoutTrialConnection=0;
	    }
	  else
	    m_NbOfProcessWithoutTrialConnection++;
      
	}
    }
  else 
    {
      if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	{
	  TryConnectionToStackOfTasks();
	  m_NbOfProcessWithoutTrialConnection=0;
	}
      else
	m_NbOfProcessWithoutTrialConnection++;
    }
  return;
}

void ConnectionToCorba::ExchangeData()
{
  if (!CORBA::is_nil(m_StackOfTasks))
    {
      try 
	{
	  // Get data from udp thread
	  TeleopWithTUM_Server::SeqMeasureData_var Measures_from_HRP;  
	  TeleopWithTUM_Server::ControlData Control_to_HRP;
	      
	  Control_to_HRP = Control_to_HRP_Threads_Exchange;	  

	  // Send some data to the robot...
	  m_StackOfTasks->GetMeasureData(Measures_from_HRP);
	  m_StackOfTasks->SetControlData(Control_to_HRP);

	  // Take data from ROBOT !
	  Measures_from_HRP_Threads_Exchange.length(Measures_from_HRP->length());

	  for( unsigned int i =0;i<Measures_from_HRP->length();++i )
	    {
	      Measures_from_HRP_Threads_Exchange[i] = Measures_from_HRP[i];
	    }
	  
	}
	
  
      catch(...)
	{
      
	  ODEBUG("Sorry the data was not send to the visual servoing plugin" );
	  if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	    {
	      CORBA::release(m_StackOfTasks);
	      TryConnectionToStackOfTasks();
	      m_NbOfProcessWithoutTrialConnection=0;
	    }
	  else
	    m_NbOfProcessWithoutTrialConnection++;
      
	}
    }
  else 
    {
      if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	{
	  TryConnectionToStackOfTasks();
	  m_NbOfProcessWithoutTrialConnection=0;
	}
      else
	m_NbOfProcessWithoutTrialConnection++;
    }
  return;
}


void ConnectionToCorba::ExchangeData(TeleopWithTUM_Server::ControlData &Control_to_HRP, 
				     TeleopWithTUM_Server::SeqMeasureData_var &Measures_from_HRP)
{
  if (!CORBA::is_nil(m_StackOfTasks))
    {
      try 
	{
	  // Send some data to the robot...

	  m_StackOfTasks->SetControlData(Control_to_HRP);
	  m_StackOfTasks->GetMeasureData(Measures_from_HRP);

	}

      catch(...)
	{

	  ODEBUG("Sorry the data was not send to the visual servoing plugin" );
	  if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	    {
	      CORBA::release(m_StackOfTasks);
	      TryConnectionToStackOfTasks();
	      m_NbOfProcessWithoutTrialConnection=0;
	    }
	  else
	    m_NbOfProcessWithoutTrialConnection++;

	}
    }
  else 
    {
      if (m_NbOfProcessWithoutTrialConnection>m_IntervalBetweenConnectionTrials)
	{
	  TryConnectionToStackOfTasks();
	  m_NbOfProcessWithoutTrialConnection=0;
	}
      else
	m_NbOfProcessWithoutTrialConnection++;
    }

}


void * CorbaThread(void *)
{
  string arguments("tele  ");
  istringstream istrm(arguments);

  ConnectionToCorba *aConnectionToCorba = new ConnectionToCorba(istrm);
  
  while(true)
    {
      usleep(4500);
      aConnectionToCorba->ExchangeData();
    }
  pthread_exit(NULL);
}
