#ifndef _CONNECTION_CLIENT_H_
#define _CONNECTION_CLIENT_H_

#include<string>
#include<sstream>

#ifdef __OMNIORB4__
#include <omniORB4/CORBA.h>
#endif

#ifdef __ORBIX__
#include <OBE/CORBA.h>
#include <OBE/CosNaming.h>
#endif 

#include <TeleopWithTUM.h>

class ConnectionToCorba
{
 private:
  /*! CORBA object for accessing the visual servoing plugin. */
  TeleopWithTUM_Server * m_StackOfTasks;

  /*! Save the name of the server for the stack of tasks. */
  std::string m_ServerID;
  
  /*! Save the kind of the server for the stask of tasks. */
  std::string m_ServerKind;
  
  /*! Number of unsuccessful call to ExchangeData 
    without trying to reconnect to the stack of tasks server. */
  int m_NbOfProcessWithoutTrialConnection;

  /*! Interval between two reconnection trials 
    to the stack of tasks server. */
  int m_IntervalBetweenConnectionTrials;
  
 public:
  
  /*! Constructor default */
  ConnectionToCorba(std::istringstream &istrm);
  
  /*! Exchange data with the stack of task:
   */
  void SetControlData();
  void GetMeasureData();

  void ExchangeData();

  void ExchangeData(TeleopWithTUM_Server::ControlData &Control_to_HRP, 
		    TeleopWithTUM_Server::SeqMeasureData_var &Measures_from_HRP);

  /*! Try to realize the connection with the stack of tasks,
    in case we are deconnected during experiments.
  */
  int TryConnectionToStackOfTasks();

  /*! Connect to the name service and build the 
    reference to the stack of tasks. 
   */
  CORBA::Object_ptr getObjectReference(std::string ServerID, std::string ServerKind);

  
};

void * measureThread(void *);

#endif /* _CONNECTION_CLIENT_H_*/
