/** @doc This object implements the necessary mecanisms
    to connect to a corba server.

   CVS Information:
   $Id$
   $Author$
   $Date$
   $Revision$
   $Source$
   $Log$

   Copyright (c) 2003-2008, 
   @author Olivier Stasse
   
   JRL-Japan, CNRS/AIST

   All rights reserved.
   
   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:
   
   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   * Neither the name of the CNRS and AIST nor the names of its contributors 
   may be used to endorse or promote products derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <sotCorbaConnection.h>

#include <iostream>
#include <fstream>
#include <string>
#include <math.h>


#define ODEBUG2(x)
#define ODEBUG3(x) cerr << "ColorDetection:" << x << endl

#if 0
#define ODEBUG(x) cerr << "ColorDetection:" <<  x << endl
#else
#define ODEBUG(x) 
#endif

using namespace std;

sotCorbaConnection::sotCorbaConnection(string &ServerName,
				 string &ServerKind,
				 string &HostnameNameservice)
{
  m_ServerName = ServerName;
  m_ServerKind = ServerKind;
  m_HostNameServer = HostnameNameservice;
}

sotCorbaConnection::sotCorbaConnection()
{
}

sotCorbaConnection::~sotCorbaConnection()
{
}

CORBA::Object_ptr sotCorbaConnection::getObjectReference()
{
	CosNaming::NamingContext_var rootContext;

	cout<<"sotCorbaConnection::getObjectReference() 0\n";
  try {
      int largc = 3;
      string NS="NameService=corbaloc:iiop:";
      NS+= m_HostNameServer;
      //NS+= ":5005/NameService";
      NS+= ":2809/NameService";
      const char *largv[] = { "progname","-ORBInitRef",(char *)NS.c_str()};
      
      m_orb = CORBA::ORB_init(largc, (char**)largv);

      cout<<"sotCorbaConnection::getObjectReference() 1\n";
      if( CORBA::is_nil(m_orb) ) return CORBA::Object::_nil();
    }
  catch(CORBA::SystemException&) {
      cerr << "Caught CORBA::SystemException." << endl;
		return CORBA::Object::_nil();
    }
  catch(CORBA::Exception&) {
      cerr << "Caught CORBA::Exception." << endl;
		return CORBA::Object::_nil();
    }
  catch(...) {
      cerr << "Caught unknown exception." << endl;
	  return CORBA::Object::_nil();
    }

  try 
    {
      // Obtain a reference to the root context of the Name service:
      CORBA::Object_var obj;
      obj = m_orb->resolve_initial_references("NameService");

	cout<<"sotCorbaConnection::getObjectReference() 2\n";
	if( CORBA::is_nil(obj) ) return CORBA::Object::_nil();

            
      // Narrow the reference returned.
      rootContext = CosNaming::NamingContext::_narrow(obj);
      
	cout<<"sotCorbaConnection::getObjectReference() 3\n";
      
      if( CORBA::is_nil(rootContext) ){
	  	cerr << "Failed to narrow the root naming context." << endl;
	  	return CORBA::Object::_nil();
	  }
    }
  catch(CORBA::ORB::InvalidName& ex) {
    // This should not happen!
    cerr << "Service required is invalid [does not exist]." << endl;
    return CORBA::Object::_nil();
  }
  catch(...) {
	cerr << "Unknown Error" << endl;
	return CORBA::Object::_nil();
  }

  // Create a name object, containing the name test/context:
  CosNaming::Name name;
  name.length(1);

  name[0].id   = m_ServerName.c_str();
  name[0].kind = m_ServerKind.c_str();

  cout<<"sotCorbaConnection::getObjectReference() 4 \n" << 
    m_ServerName.c_str() << " " << m_ServerKind.c_str() <<endl;

  /*  name[1].id = (const char*) "Echo";
      name[1].kind = (const char*) "Object";*/

  // Note on kind: The kind field is used to indicate the type
  // of the object. This is to avoid conventions such as that used
  // by files (name.type -- e.g. test.ps = postscript etc.)
 
  try {
    // Resolve the name to an object reference.
    cout<<"sotCorbaConnection::getObjectReference() 5 finishes OK\n";
    return rootContext->resolve(name);
  }
  catch(CosNaming::NamingContext::NotFound& ex) {
    // This exception is thrown if any of the components of the
    // path [contexts or the object] aren't found:
    cerr << "Context not found." << endl;
  }
  catch(CORBA::COMM_FAILURE& ex) {
    cerr << "Caught system exception COMM_FAILURE -- unable to contact the naming service." << endl;
  }
  catch(CORBA::SystemException&) {
    cerr << "Caught a CORBA::SystemException while using the naming service."
	 << endl;
  }
  cout << "Here...failed to find the Server " << name[0].id << endl;
	cout<<"sotCorbaConnection::getObjectReference() 5 finishes BAD\n";
  return CORBA::Object::_nil();
}






