/** @doc This object implements a simple color detection based on
    histogram.

   CVS Information:
   $Id$
   $Author$
   $Date$
   $Revision$
   $Source$
   $Log$

   Copyright (c) 2006, 
   @author Olivier Stasse
   
   JRL-Japan, CNRS/AIST

   All rights reserved.
   
   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:
   
   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   * Neither the name of the CNRS and AIST nor the names of its contributors 
   may be used to endorse or promote products derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef _HRP2_COLOR_DETECTION_H_
#define _HRP2_COLOR_DETECTION_H_

/*! This object defines the abstract class defining a basic
 * vision process
 * 
 * Copyright (c) 2004 Olivier Stasse, JRL, CNRS/AIST
 *
 *
 */

#include <string>
#include <vector>

#ifdef __OMNIORB4__
#include <omniORB4/CORBA.h>
#endif

#ifdef __ORBIX__
#include <OBE/CORBA.h>
#include <OBE/CosNaming.h>
#endif 



/*! Simple client for a Corba Server
 */
class sotCorbaConnection {

 public:

  /*! Constructor */
  sotCorbaConnection(std::string & ServerName, std::string & ServerKind, std::string & HostnameNameservice);
  sotCorbaConnection();

  /*! Destructor */
  virtual ~sotCorbaConnection();


  /*! Try connection to visual servoing. */
  int TryConnectionToCorbaServer();

  /*! Set name of the server to be connected to.*/
  void SetServerName( std::string &ServerName ) { m_ServerName = ServerName; };

  /*! Set kind of the server to be connected to. */
  void SetServerKind( std::string & ServerKind ) { m_ServerKind = ServerKind; };

  /*! Set the name of the host where is located the name service. */
  void SetHostnameNameService( std::string &TheHostnameNameService ) { m_HostNameServer = TheHostnameNameService; };

  /*! Established the object reference */
  CORBA::Object_ptr getObjectReference();
  
 protected:
  

  /*! Corba fields 
    @{
   */

  /*! CORBA ORB */
  CORBA::ORB_ptr m_orb;

  /*! Server name. */
  std::string m_ServerName;

  /*! @} */
  /*! Hostname for the nameservice */
  std::string m_HostNameServer;

  /*! Kind of the server. */
  std::string m_ServerKind;

};

#endif /* _HRP2_COLOR_DETECTION_H_ */
