/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      coshell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard 
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *  Connection to a sotServerCommand, used as a relay to transmitt
 * shell instruction to the sotInterpretor running inside OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <string>
#include <sstream>
#include <iostream>
#include <vector>
#include <fstream>
#include <assert.h>

#include "sotCorbaConnection.h"
/* IDL automatically generated file. */
#include <ServerCommand.h>

#include <ctime>
#include <fstream>
#include "connection.h"
#include "sotDebug.h"

//using namespace std;
//using namespace sfbcomm;


/* --- IP AND CONFIG --------------------------------------------------------- */
/* --- IP AND CONFIG --------------------------------------------------------- */
/* --- IP AND CONFIG --------------------------------------------------------- */
#define IPADDR "129.187.147.174"      // IP adress of the remote computer TUM
//#define IPADDR "172.16.59.1"        // IP adress of Vision server
//#define IPADDR "150.29.191.252"     // IP adress of Sylvain's comp.
//#define IPADDR "150.29.190.242"     // IP adress of jrlhrp01
#define RECVPORT 4446                 // receive port
#define SENDPORT 4445                 // send port
     

/* -------------------------------------------------------------------------- */
/* --- OPTIONS -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class OptArgs 
{
public:
  std::string corbaServerName;
  std::string remoteServerName;
  std::string serverKind;
  bool receive,emit;

  std::string udpIp;
  unsigned int recvPort,sendPort;

  OptArgs( void ) 
    : corbaServerName( "coshell" )
      ,remoteServerName("localhost")
      ,serverKind("")  
      ,receive(true),emit(true)
      ,udpIp(IPADDR)
      ,recvPort(RECVPORT)
      ,sendPort(SENDPORT)
  {}

  static const char optlist [];

public:
  
  void parseOptions(  int argc,char** argv )
  {
    int ch;
    while( (ch=getopt( argc,argv,optlist ))!=-1 ) 
    {
      switch (ch) 
	{
	case 'c':
	  corbaServerName = optarg;
	  break;
	case 'r':
	  remoteServerName = optarg;
	  break;
	case 'k':
	  serverKind = optarg;
	  break;
	case 'R':
	  receive=false;
	  break;
	case 'E':
	  emit=false;
	  break;
	case 'U':
	  udpIp = optarg;
	  break;
	case 's':
	  {
	    std::istringstream iss( optarg ); 
	    iss >> sendPort;
	    break; 
	  }
	case 'l':
	  {
	    std::istringstream iss( optarg ); 
	    iss >> recvPort;
	    break; 
	  }
	case '?':
	default:
	  usage();
	}
    }
  }
  void usage( void ) 
  {
    std::cout << "coshell [-c pluginName] [-r remoteServerName] [-k serverKind] -[RE] "
	      << "[-U udpIP] [-s sendPort] [-l recvPort]" 
	      << std::endl; 
  }
};

const char OptArgs::optlist [] = "c:r:k:REU:s:l:";


/* --- INIT SIGNAL ----------------------------------------------------------- */
/* --- INIT SIGNAL ----------------------------------------------------------- */
/* --- INIT SIGNAL ----------------------------------------------------------- */
 
static SOT_Server_Command::CharSeq&
setSigName( const std::string& buffer,
	    SOT_Server_Command::CharSeq& signame,
	    const std::string & IO )
{
  const string sigref = "Teleop-Distant::"+IO+"put(vector)::";
  string sigfullname = sigref + buffer;
  const unsigned char nbCar = sigfullname.length(); 
  signame.length(nbCar);
  for( unsigned int i=0;i<nbCar;++i ){ signame[i]=sigfullname[i]; }
  return signame;
}
 
static SOT_Server_Command::DoubleSeq&
setInitVector( SOT_Server_Command::DoubleSeq &force,
	       const unsigned int & dim )
{
  force.length(dim);
  for( unsigned i=0;i<dim;++i ) force[i]=0;
  return force;
}

#define INIT_SOUT(name,dim) \
  name  \
    = serverPtr->createOutputVectorSignal( setSigName( #name,signame,"out" ) ); \
  serverPtr->writeOutputVectorSignal( name,setInitVector(initVector,dim) ); 
#define INIT_SIN(name) \
  name  \
    = serverPtr->createInputVectorSignal( setSigName( #name,signame,"in" ) ); 


const std::string cmd = "run /home/nmansard/src/StackOfTasks/script/debug";

static std::string sendSotCommand( SOT_Server_Command_ptr serverPtr,
				   const std::string& cmd )
{
  SOT_Server_Command::CharSeq signame; 
  signame.length( cmd.size() ); 
  for( unsigned int i=0;i<cmd.size();++i ) signame[i]=cmd[i];
  SOT_Server_Command::StringStreamer_var outputShell;
  serverPtr->runAndRead( signame,outputShell );

  std::ostringstream ans; 
  for( unsigned int i=0;i<outputShell->length();i++ )	
    { ans << outputShell[i]; }
  usleep( 1000*20 );
  return ans.str();
}

/* --- GLOBAL VARIABLES ------------------------------------------------------ */
/* --- GLOBAL VARIABLES ------------------------------------------------------ */
/* --- GLOBAL VARIABLES ------------------------------------------------------ */

ofstream data_send_TUM;
ofstream data_recv_TUM;

SOT_Server_Command_ptr serverPtr = NULL;
long headSOUT,forceRSOUT,forceLSOUT,posRSOUT,posLSOUT,gripSOUT,refSOUT;
long forceRSIN,forceLSIN,posRSIN,posLSIN;

/* --- RECEIVE --------------------------------------------------------------- */
/* --- RECEIVE --------------------------------------------------------------- */
/* --- RECEIVE --------------------------------------------------------------- */

void 
recvData( sfbcomm::connection* recvConn )
{
  sotDEBUGIN(15);

  int             err;
  const int       NB_DATA = 10; // 14
  double          dataBuffer[NB_DATA];   // Temp variable.
  double          latency = 0;  // Used to measure the latency.
  bool            setToNone = false;
  timeval         last_recv_time;
  timeval         curr_time;
  unsigned long   diff_time_micros;

  gettimeofday(&last_recv_time, NULL);
  gettimeofday(&curr_time, NULL);

  /* Create the data structure. */
  sotDEBUG(15) << "Init the data. " << std::endl;
  sfbcomm::datatype *recv_data = new sfbcomm::datatype();
  recv_data->createbuffer("A", (long) sizeof(dataBuffer));
  recv_data->setbuffer("A", (unsigned char *)dataBuffer);

  recv_data->createbuffer("latency", (long) sizeof(double));
  recv_data->setbuffer("latency", (unsigned char *)&latency);
    
  sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
  for(int i=0; i< NB_DATA; i++)   { dataBuffer[i]=-10.0; }

  /* Receive data from the network. */
  sotDEBUG(15) << "Start the recv loop. " << std::endl;
  while (true) 
    {
      sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
      /* Read the data. */
      sotDEBUG(15) << "connect ... " << std::endl;
      err = recvConn->receivedata(recv_data,1);
      gettimeofday(&curr_time, NULL);
      sotDEBUG(15) << "connect done!" << std::endl;

      if( err < 0 ) /* If we did not get a measure.... */
	{
	  diff_time_micros 
	    = ((unsigned long)(curr_time.tv_sec-last_recv_time.tv_sec)* 1000000L 
	       +(unsigned long)(curr_time.tv_usec - last_recv_time.tv_usec));
            
	  if (diff_time_micros > 200000L)
            { if(! setToNone ) {cerr << " Error while UDP recv. " << endl;  } }
	  continue;
	} 

      /* else we get a measure ... */
      setToNone = false;	      
      last_recv_time = curr_time;

#define SOT_UDP_TO_CORBA(signame,init,dim)				\
      if( 0!=signame##SOUT ){						\
	sotDEBUG(45) << "Copy to Corba. "<< #signame << std::endl;	\
	SOT_Server_Command::DoubleSeq signame##VECT; signame##VECT.length(dim); \
	for( unsigned i=0;i<dim;++i ) signame##VECT[i]=dataBuffer[init+i]; \
	serverPtr->writeOutputVectorSignal( signame##SOUT,signame##VECT ); \
      }
      
      sotDEBUG(35) <<"Head signal = " << headSOUT <<std::endl;
      SOT_UDP_TO_CORBA(forceR,0,6);
      SOT_UDP_TO_CORBA(head,7,2);
      //SOT_UDP_TO_CORBA(grip,9,1);
      //SOT_UDP_TO_CORBA(posR,6,6);

    
      /* Get left arm torque and gripper distance. */
      /* TODO ... */

      /* Get package number. */
      if( 0!=refSOUT ){
	SOT_Server_Command::DoubleSeq ref; ref.length(1);
	ref[0]=(unsigned long)dataBuffer[9+0];
	serverPtr->writeOutputVectorSignal( refSOUT,ref );
      }

      //       if( x[12]==1.0 ) 
      // 	{ Control_to_HRP_Threads_Exchange.controlScheme 
      // 	    = TeleopWithTUM_Server::F_V; }
      //       else
      // 	{ Control_to_HRP_Threads_Exchange.controlScheme 
      // 	    = TeleopWithTUM_Server::NONE; }
      
      
      ////////////// Debug: display received data ////////////
      sotDEBUG(45) << "Trace. "<<std::endl;
      data_recv_TUM << curr_time.tv_sec << " " << curr_time.tv_usec << " " ;
      for(int i=0;i<NB_DATA;i++) { data_recv_TUM << dataBuffer[i] << " " ; }
      data_recv_TUM << endl;
      ////////////////////////////////////////////////////////
    }

  sotDEBUGOUT(15);
}

/* --- SEND ------------------------------------------------------------------ */
/* --- SEND ------------------------------------------------------------------ */
/* --- SEND ------------------------------------------------------------------ */
void sendData( sfbcomm::connection & sendConn )
{
  sotDEBUGOUT(15);

  data_send_TUM << "Input" << std::endl;

  /* Local variables. */
  int                  err = 0;
  sfbcomm::datatype *  send_data = new sfbcomm::datatype();
  static unsigned long packn = 0ul;
  const int            NB_DATA = 8;  
  double               dataBuffer[NB_DATA];   
  double               latency;      // Used to measure the latency
  timeval              last_recv_time;
  timeval              curr_time;
  //unsigned long        diff_time_micros;
  unsigned long        waitTime;

  /* Init time counters. */
  gettimeofday(&last_recv_time, NULL);
  gettimeofday(&curr_time, NULL);

  /* Create the data structure. */
  send_data->createbuffer("B", (long) sizeof(dataBuffer));
  send_data->setbuffer("B",(unsigned char *)dataBuffer);
  send_data->createbuffer("latency", (long) sizeof(double));
  send_data->setbuffer("latency", (unsigned char *) &latency);

  while(true)
    {
      data_send_TUM << "Loop" << std::endl;

      gettimeofday(&curr_time, NULL);
      //waitTime = 20000L - 
      // ( (unsigned long)(curr_time.tv_sec - last_recv_time.tv_sec) 
      // * 1000000L 
      // + (unsigned long)(curr_time.tv_usec - last_recv_time.tv_usec) );
      waitTime = 18000L;  // Seems to be the best value to get 20ms period
      if(waitTime > 0)
	{ usleep(waitTime); }
      gettimeofday(&last_recv_time, NULL);
      sotDEBUG(45) << curr_time.tv_usec << endl;

      data_send_TUM << "Loop" << std::endl;
      //TODO LATER aConnectionToCorba->GetMeasureData();

      /* Try to read measure data. As soon as there is some, copy it into
       * buffers that will be sent to TUM. */
      {
	data_send_TUM << "Loop" << std::endl;
	SOT_Server_Command::DoubleSeq_var posR; 
	data_send_TUM << "Avant" << std::endl;
	//posR->length(6);
	data_send_TUM << "Apres" << std::endl;
	serverPtr->readInputVectorSignal( posRSIN,posR );

      data_send_TUM << "Loop" << std::endl;
	if( posR->length()!=6 ) 
	  {
	    sotERROR << "Get signal pos of wrong dim: " << posR->length()
		     <<"." << std::endl;
	    continue;
	  }
      data_send_TUM << "Loop" << std::endl;
	
	/* Data to transmit: 7  (Trans=3+Rot=4). */
	for( int i=0;i<7;++i ) 
	  { dataBuffer[i]=posR[i];} 
      }

      dataBuffer[7] = (double) (++packn);

//       if( Measures_from_HRP_Threads_Exchange.length()>0 )
// 	{
// 	  TeleopWithTUM_Server::MeasureData& Measure_from_HRP =
// 	    Measures_from_HRP_Threads_Exchange
// 	    [Measures_from_HRP_Threads_Exchange.length()-1];
	  
// 	  /* Send positions and quaternion of the hand. */
// 	  for(int i = 0; i < 7; ++i) //put 7 to send the quaternion
// 	    x[i] = Measure_from_HRP.Rarm[i];
// 	  x[7] = (double) (++packn);
// 	}
//       else
// 	continue; // --- sorry for that
            
      ////////////// Debug: display received data ////////////
      // display received data
      //sotDEBUG(25) 
      data_send_TUM<< "Sent data: " << std::endl;
      for(int i=0;i<3;i++)  data_send_TUM << dataBuffer[i] << " " << std::endl;
      sotDEBUG(25) << latency << endl;
      //data_send_TUM << curr_time.tv_sec << " " << curr_time.tv_usec << " " ;
      //for(int i=0;i<NB_DATA;i++)
      //data_send_TUM << x[i] << " " ;
      //data_send_TUM << endl;
      ////////////////////////////////////////////////////////  

      /* Should we keep this ? */
      //do { err = sendConn.senddata(send_data,1); }
      //while (err < 0);
      err = sendConn.senddata(send_data,1);

      
    }
  sotDEBUGOUT(15);
}

/* --- CONTROL --------------------------------------------------------------- */
/* --- CONTROL --------------------------------------------------------------- */
/* --- CONTROL --------------------------------------------------------------- */
void *
controlThread( void* dummy ) // TODOOOOOO!
{
  sotDEBUGIN(15);
  /* Setup a udp connection to receive from the network. */
  sotDEBUG(15) << "Connect to " << IPADDR
	       << " on port " << RECVPORT << std::endl;
  sfbcomm::connection* recvConn 
    = new sfbcomm::connection(RECVPORT, (char*)IPADDR, RECVPORT,1);

  sotDEBUG(1) << "Did I start the control thread ? "<<endl;
  // check to ensure that the udp connection was setup correctly
  switch (recvConn->geterror()) 
    {
    case HOST_NOT_FOUND: 
      {
	cerr << "Please enter a correct hostname or IP-Address!" 
	     << endl;
	exit(-1);
      }
    case SOCK_FAILED: 
      {
	cerr << "Could not create Socket!" << endl; 
	exit(-1);
      }
    case BIND_FAILED: 
      {
	cerr << "Could not bind Port!" << endl;
	exit(-1);
      }
    default:             
      break; 
    }
  
  /* Start receiving control data. */
  sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
  recvData(recvConn);
  
  /* Exit the thread. */
  sotDEBUGOUT(15);
  //pthread_exit(NULL);
  return NULL;
}

/* --- MEASURE --------------------------------------------------------------- */
/* --- MEASURE --------------------------------------------------------------- */
/* --- MEASURE --------------------------------------------------------------- */
void *
measureThread( void* argsVoid )
{
  OptArgs* args = (OptArgs*) argsVoid;

  sotDEBUGIN(15);
  data_send_TUM << "thread entered" << std::endl;
  /* Create a udp connection to send data over the network. */
  sfbcomm::connection sendConn(args->sendPort,(char*)args->udpIp.c_str(),
			       args->sendPort,1 );
 
  /* Check to ensure that the udp connection was setup correctly. */
  switch( sendConn.geterror() )
    {
    case HOST_NOT_FOUND: 
      {
	sotERROR << "Please enter a correct hostname or IP-Address!" << endl;
	sotDEBUGOUT(1);
	exit(-1);
      }
    case SOCK_FAILED: 
      {
	sotERROR << "Could not create Socket!" << endl; 
	sotDEBUGOUT(1);
	exit(-1);
      }
    case BIND_FAILED:
      sotERROR << "Could not bind Port!" << endl; 
      sotDEBUGOUT(1);
      exit(-1);
    default:             
      break; 
    }

  sotDEBUG(15) << "Did the check succeded ? "<<endl;

  /* Start communication. */
  sendData( sendConn );

  /* Exit the thread. */
  sotDEBUGOUT(15);
  pthread_exit(NULL);
}

/* --- MAIN ------------------------------------------------------------------ */
/* --- MAIN ------------------------------------------------------------------ */
/* --- MAIN ------------------------------------------------------------------ */

int main( int argc,char ** argv )
{
  /* --- INIT --- */
  /* --- INIT --- */
  /* --- INIT --- */
  sotDebugTrace::openFile();
  sotCorbaConnection * corbaConnectionHandler = NULL;
  OptArgs args; args.parseOptions(argc,argv);


  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  std::cout << " Corba Server Name : " << args.corbaServerName << std::endl
	    << " Server Kind : " << args.serverKind << std::endl
	    << " Remote Server Name : " << args.remoteServerName << std::endl;
  try
    {
      corbaConnectionHandler = new sotCorbaConnection( args.corbaServerName, 
						       args.serverKind,
						       args.remoteServerName );
      sotDEBUG(15) << "Connection ..." << endl;
      CORBA::Object_ptr corba_obj 
	= corbaConnectionHandler ->getObjectReference(); 
      
      sotDEBUG(15) << "Connection [OK]. Cast ... " << endl;
      serverPtr = SOT_Server_Command::_narrow( corba_obj );
      
      if(CORBA::is_nil( serverPtr ) )
	{
	  sotERROR <<"Server ptr is null: connection is not obtained. " <<std::endl;
	  sotERROR << "Fatal error (die die)." <<std::endl;
	  exit(0);
	}
      sotDEBUG(1) << "Cast [OK]. Run the shell." << endl;
    }
  catch(...)
    {
      sotERROR<<"Exception when trying to connect to the corba server \"serverCorbaSot\""
	      <<"the program continues but cannot communicate.."<<std::endl;
      serverPtr = NULL;
      exit(-1);
    }


  /* --- CONFIG --- */
  /* --- CONFIG --- */
  /* --- CONFIG --- */
  sotDEBUG(15) << serverPtr << " " << CORBA::is_nil( serverPtr ) <<std::endl;

  SOT_Server_Command::CharSeq signame; 
  SOT_Server_Command::DoubleSeq initVector; 
  
  headSOUT=0;  forceRSOUT=0; forceLSOUT=0; posRSOUT=0; 
  posLSOUT=0;  gripSOUT=0;   refSOUT=0; 
  forceRSIN=0; forceLSIN=0;  posRSIN=-1;    posLSIN=0;
  
  try
    {
      INIT_SOUT(forceRSOUT,6);
      INIT_SOUT(headSOUT,2);
      INIT_SOUT(gripSOUT,2);
      INIT_SOUT(refSOUT,1);
      
      //INIT_SIN(forceRSIN);
      INIT_SIN(posRSIN);

      sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
      sotDEBUG(5) <<"Pos in signal = " << posRSIN <<std::endl;
    }
  catch( ... )
    {
      sotERROR <<"Signals already exist." <<std::endl;
    }

        sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;

  try
    {
      usleep( 1000*1000 );
      //   std::cout << sendSotCommand(  serverPtr,"run /home/nmansard/"
      // 				"src/StackOfTasks/script/initConnec" );
      sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
      std::cout << sendSotCommand(  serverPtr,"help" );
      sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
    } catch( ... ) {
      sotERROR << "Cannot run the config script. " << std::endl; 
    }
  
  /* --- DATA LOOP --- */
  /* --- DATA LOOP --- */
  /* --- DATA LOOP --- */
  
  data_send_TUM.open("/tmp/nmansard/Data_send_TUM.sav");
  data_recv_TUM.open("/tmp/nmansard/Data_recv_TUM.sav");
  data_send_TUM << "Beg" << std::endl;

  sotDEBUG(5) <<"Head signal = " << headSOUT <<std::endl;
  //controlThread(&args);

  //if(0)
    {
      pthread_t controlThreadHandle; // local variable
      pthread_create(&controlThreadHandle, NULL, controlThread, &args);
      pthread_t measureThreadHandle; // local variable
      data_send_TUM << "thread" << std::endl;
      pthread_create(&measureThreadHandle, NULL, measureThread, &args); 

      pthread_join(controlThreadHandle,0);
      pthread_join(measureThreadHandle,0);
    }

  sotDEBUG(1)<<"Exchange done! (see you soon...)"<<std::endl;
  delete  corbaConnectionHandler;
}
