/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      coshell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard 
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *  Connection to a sotServerCommand, used as a relay to transmitt
 * shell instruction to the sotInterpretor running inside OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <string>
#include <sstream>
#include <iostream>
#include <vector>
#include <fstream>
#include <assert.h>

#include "LocalVariables.h"

#include "sotCorbaConnection.h"
/* IDL automatically generated file. */
#include <ServerCommand.h>

#include <ctime>
#include <fstream>
#include <signal.h>
#include "connection.h"
#include "sotDebug.h"
#include <sys/types.h>
#include <unistd.h>


#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>

//using namespace std;
//using namespace sfbcomm;


/* --- IP AND CONFIG --------------------------------------------------------- */
/* --- IP AND CONFIG --------------------------------------------------------- */
/* --- IP AND CONFIG --------------------------------------------------------- */
#define IPADDR "129.187.147.174"      // IP adress of the remote computer TUM
//#define IPADDR "129.187.147.242"      // IP adress of the remote computer TUM
//#define IPADDR "172.16.59.1"        // IP adress of Vision server
//#define IPADDR "150.29.191.252"     // IP adress of Sylvain's comp.
//#define IPADDR "150.29.190.242"     // IP adress of jrlhrp01
#define RECVPORT 4446                 // receive port
#define SENDPORT 4445                 // send port
     

/* -------------------------------------------------------------------------- */
/* --- OPTIONS -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class OptArgs 
{
public:
  std::string corbaServerName;
  std::string remoteServerName;
  std::string serverKind;
  bool receive,emit;

  std::string udpIp;
  unsigned int recvPort,sendPort;

  OptArgs( void ) 
    : corbaServerName( "coshell" )
      ,remoteServerName("localhost")
      ,serverKind("")  
      ,receive(true),emit(true)
      ,udpIp(IPADDR)
      ,recvPort(RECVPORT)
      ,sendPort(SENDPORT)
  {}

  static const char optlist [];

public:
  
  void parseOptions(  int argc,char** argv )
  {
    int ch;
    while( (ch=getopt( argc,argv,optlist ))!=-1 ) 
    {
      switch (ch) 
	{
	case 'c':
	  corbaServerName = optarg;
	  break;
	case 'r':
	  remoteServerName = optarg;
	  break;
	case 'k':
	  serverKind = optarg;
	  break;
	case 'R':
	  receive=false;
	  break;
	case 'E':
	  emit=false;
	  break;
	case 'U':
	  udpIp = optarg;
	  break;
	case 's':
	  {
	    std::istringstream iss( optarg ); 
	    iss >> sendPort;
	    break; 
	  }
	case 'l':
	  {
	    std::istringstream iss( optarg ); 
	    iss >> recvPort;
	    break; 
	  }
	case '?':
	default:
	  usage();
	}
    }
  }
  void usage( void ) 
  {
    std::cout << "coshell [-c pluginName] [-r remoteServerName] [-k serverKind] -[RE] "
	      << "[-U udpIP] [-s sendPort] [-l recvPort]" 
	      << std::endl; 
  }
};

const char OptArgs::optlist [] = "c:r:k:REU:s:l:";


/* --- INIT SIGNAL ----------------------------------------------------------- */
/* --- INIT SIGNAL ----------------------------------------------------------- */
/* --- INIT SIGNAL ----------------------------------------------------------- */
 
static SOT_Server_Command::CharSeq&
setSigName( const std::string& buffer,
	    SOT_Server_Command::CharSeq& signame,
	    const std::string & IO )
{
  const string sigref = "Teleop-Distant::"+IO+"put(vector)::";
  string sigfullname = sigref + buffer;
  const char nbCar = sigfullname.length(); 
  signame.length(nbCar);
  for( int i=0;i<nbCar;++i ){ signame[i]=sigfullname[i]; }
  return signame;
}
 
static SOT_Server_Command::DoubleSeq&
setInitVector( SOT_Server_Command::DoubleSeq &force,
	       const unsigned int & dim )
{
  force.length(dim);
  for( unsigned i=0;i<dim;++i ) force[i]=0;
  return force;
}

#define INIT_SOUT(name,dim) \
  name  \
    = serverPtr->createOutputVectorSignal( setSigName( #name,signame,"out" ) ); \
  serverPtr->writeOutputVectorSignal( name,setInitVector(initVector,dim) );  \
  sotDEBUG(45) << #name"sig = " << name << std::endl;
#define INIT_SIN(name) \
  name  \
    = serverPtr->createInputVectorSignal( setSigName( #name,signame,"in" ) ); 


const std::string cmd = "run " SCRIPTDIRECTORY "/debug";

static std::string sendSotCommand( SOT_Server_Command_ptr serverPtr,
				   const std::string& cmd )
{
  SOT_Server_Command::CharSeq signame; 
  signame.length( cmd.size() ); 
  for( unsigned int i=0;i<cmd.size();++i ) signame[i]=cmd[i];
  SOT_Server_Command::StringStreamer_var outputShell;
  serverPtr->runAndRead( signame,outputShell );

  std::ostringstream ans; 
  for( unsigned int i=0;i<outputShell->length();i++ )	
    { ans << outputShell[i]; }
  usleep( 1000*20 );
  return ans.str();
}

#define SOT_UDP_TO_CORBA(signame,init,dim)                               \
   if( -1!=signame##SOUT ){                                               \
   SOT_Server_Command::DoubleSeq signame##VECT; signame##VECT.length(dim); \
   for( unsigned i=0;i<dim;++i ) signame##VECT[i]=dataBuffer[init+i];    \
   if( serverPtr )                                                       \
     serverPtr->writeOutputVectorSignal( signame##SOUT,signame##VECT );  \
  }

#define SOT_CORBA_TO_UDP(signame,init,dim)                         \
if( (-1!=signame##SIN)&&(serverPtr) ) {                            \
  SOT_Server_Command::DoubleSeq_var signame##VECT;                 \
    serverPtr->readInputVectorSignal( signame##SIN,signame##VECT );\
  if( signame##VECT->length()!=dim )                               \
    {sotERROR << "Get signal pos of wrong dim: "                   \
              << signame##VECT->length() <<"." << std::endl;}      \
  for( int i=0;i<dim;++i ) { dataBuffer[init+i]=signame##VECT[i];} \
}


/* --- GLOBAL VARIABLES ------------------------------------------------------ */
/* --- GLOBAL VARIABLES ------------------------------------------------------ */
/* --- GLOBAL VARIABLES ------------------------------------------------------ */

ofstream data_send_TUM;
ofstream data_recv_TUM;

SOT_Server_Command_ptr serverPtr = NULL;
long headSOUT,forceRSOUT,posRSOUT,refSOUT,connectSOUT;
long forceLSOUT,posLSOUT;
long gripSOUT;
long forceRSIN,forceLSIN,posRSIN,posLSIN;

boost::try_mutex recvConnectionMutex;
boost::try_mutex sendConnectionMutex;
boost::condition resetCorba;
boost::mutex resetCorbaMutex;
 
//DEBUG int ping=0;

/* --- RECEIVE --------------------------------------------------------------- */
/* --- RECEIVE --------------------------------------------------------------- */
/* --- RECEIVE --------------------------------------------------------------- */
void 
recvData( sfbcomm::connection& recvConn )
{
  sotDEBUGIN(15);
  sotDEBUG(25) << "Entering recvData !" << endl;

  int             err; 
  const int       NB_DATA = 17; // 14
  double          dataBuffer[NB_DATA];  
  double          latency = 0;  // Used to measure the latency.
  //bool            noDataReceived = false;
  timeval         last_recv_time;
  timeval         curr_time;
  unsigned long   diff_time_micros;

  gettimeofday(&last_recv_time, NULL);
  gettimeofday(&curr_time, NULL);

  /* Create the data structure. */
  sfbcomm::datatype *recv_data = new sfbcomm::datatype();
  recv_data->createbuffer("A", (long) sizeof(dataBuffer));
  recv_data->setbuffer("A", (unsigned char *)dataBuffer);

  recv_data->createbuffer("latency", (long) sizeof(double));
  recv_data->setbuffer("latency", (unsigned char *)&latency);
    
  for(int i=0; i< NB_DATA; i++)   { dataBuffer[i]=0.0; }

  /* Receive data from the network. */

  for( unsigned int connecIter=0;;++connecIter )
    {
      /* Read the data. */
      err = recvConn.receivedataunbuf(recv_data,1);
      gettimeofday(&curr_time, NULL);

      if( err < 0 ) /* If we did not get a measure.... */
	{
	  diff_time_micros 
	    = (unsigned long)(curr_time.tv_sec-last_recv_time.tv_sec)*1000000L 
	    + (unsigned long)(curr_time.tv_usec-last_recv_time.tv_usec);
	  if (diff_time_micros > 200000L)
            { 
	      cerr << " Error while UDP recv. " << std::endl;  
	      bzero(dataBuffer,sizeof(dataBuffer));
	      dataBuffer[9]=0;
	      /* In that case, we don't continue to send an update to the
	       * corba. */
	    }
	  else continue;
	} 

      /* else we get a measure ... */
      last_recv_time = curr_time;
      
      try
	{
 	  boost::try_mutex::scoped_try_lock lock(recvConnectionMutex);

	  sotDEBUG(45) << "Head sig = " << headSOUT << std::endl;

// 	  dataBuffer[0] = curr_time.tv_sec ; 
// 	  dataBuffer[1] = curr_time.tv_usec ; 


/*--- TODO: replace this hard-coded ugly thing --- */
/*--- TODO: replace this hard-coded ugly thing --- */
/*--- TODO: replace this hard-coded ugly thing --- */

// Conversion gripper distance to angle

	  const double pi = 3.14159256;
	  double gripperDistance[2] = {dataBuffer[12], dataBuffer[13]};

	  for(int i = 0; i < 2; ++i) {
	    double Ld = 0.5 * gripperDistance[i] / (sqrt(2.0) * 0.045);
	    double desiredAngle = 0;

	    if (Ld < 1.) {
	      if (Ld > -1.){ desiredAngle = asin(Ld); }
	      else{ desiredAngle = - pi/2.; }
	    } 
	    else{ desiredAngle = pi/2.; }

	    if (desiredAngle > (pi/4.))
	      desiredAngle = pi/4.;
	    if (desiredAngle < 0.)
	      desiredAngle = 0.;

	    dataBuffer[12+i] = desiredAngle;
	  }

/*--- END TODO: replace this hard-coded ugly thing --- */
/*--- END TODO: replace this hard-coded ugly thing --- */
/*--- END TODO: replace this hard-coded ugly thing --- */


	  SOT_UDP_TO_CORBA(forceR,0,6);
	  SOT_UDP_TO_CORBA(forceL,6,6);
	  SOT_UDP_TO_CORBA(grip,12,2);
	  SOT_UDP_TO_CORBA(head,14,2); 
	  //SOT_UDP_TO_CORBA(posR,6,6);
	  
	  //DEBUG	  //if( dataBuffer[0]>1e-4 ) ping=1; 

   	  /* Get package number. */
	  /*if( 0!=refSOUT ){
	   *SOT_Server_Command::DoubleSeq ref; ref.length(1);
	   *ref[0]=(unsigned long)dataBuffer[13+0];
	   *serverPtr->writeOutputVectorSignal( refSOUT,ref ); 
	   */
	
	  /* Get package number. */
	  if( 0!=connectSOUT )
	    {
	    SOT_Server_Command::DoubleSeq connectVECT; connectVECT.length(1);
	    connectVECT[0]=(unsigned long)dataBuffer[NB_DATA-1];
	    if( serverPtr )
	      serverPtr->writeOutputVectorSignal( connectSOUT,connectVECT );
	  }
	} 
      catch( boost::lock_error& le ) {}
      catch( ... ) { sotDEBUG(15) << "Other error. " << std::endl; }
          
      /* ----------- Debug: display received data ------------------- *
       * display received data
       * sotERROR << "Received data: ";
       * control scheme, package number, and head orientation
       * sotERROR << x[12] << " " << x[13] << " " << x[10] << " " << x[11] << endl;  */
      if( data_recv_TUM.is_open() ) if(! (connecIter%10) )
	{
	  data_recv_TUM << "sot2-" << curr_time.tv_sec << " " << curr_time.tv_usec << " " ;
	  for(int i=0;i<NB_DATA;i++) { data_recv_TUM << dataBuffer[i] << " " ; }
	  data_recv_TUM << endl;
	}
      /* ---------------------------------------------------------- */
  
    }

  sotDEBUGOUT(15);
}

/* --- SEND ------------------------------------------------------------------ */
/* --- SEND ------------------------------------------------------------------ */
/* --- SEND ------------------------------------------------------------------ */
void sendData( sfbcomm::connection & sendConn )
{
  sotDEBUGIN(15);

  /* Local variables. */
  int                  err = 0;
  sfbcomm::datatype *  send_data = new sfbcomm::datatype();
  static unsigned long packn = 0ul;
  const int            NB_DATA = 14+1;  
  double               dataBuffer[NB_DATA];   
  double               latency;      // Used to measure the latency
  timeval              last_recv_time;
  timeval              curr_time;
  unsigned long        waitTime;

  /* Init time counters. */
  gettimeofday(&last_recv_time, NULL);
  gettimeofday(&curr_time, NULL);

  /* Create the data structure. */
  send_data->createbuffer("B", (long) sizeof(dataBuffer));
  send_data->setbuffer("B",(unsigned char *)dataBuffer);
  send_data->createbuffer("latency", (long) sizeof(double));
  send_data->setbuffer("latency", (unsigned char *) &latency);

  for( unsigned int connecIter=0;;++connecIter )
    {
      gettimeofday(&curr_time, NULL);
      waitTime = 18000L;  // Seems to be the best value to get 20ms period
      if(waitTime > 0) { usleep(waitTime); }
      gettimeofday(&last_recv_time, NULL);

      /* Try to read measure data. As soon as there is some, copy it into
       * buffers that will be sent to TUM. */
      try
	{
	  boost::try_mutex::scoped_try_lock lock(sendConnectionMutex);
	  sotDEBUG(15) << "Convert corba to UDP. " <<std::endl;

	  //DEBUG	  	  dataBuffer[0]=ping;
	  SOT_CORBA_TO_UDP(posR,0,7);
	  SOT_CORBA_TO_UDP(posL,7,7);
	  
	}
      catch( boost::lock_error& le ) {}
      catch( ... ) { sotDEBUG(15) << "Other error. " << std::endl; }

      dataBuffer[NB_DATA-1] = (double) (++packn);

      ////////////// Debug: display received data ////////////
      if( data_send_TUM.is_open() ) if(! (connecIter%10) )
	{
	  data_send_TUM << curr_time.tv_sec << " " << curr_time.tv_usec << " " ;
	  for(int i=0;i<NB_DATA;i++) data_send_TUM << dataBuffer[i] << " " ;
	  data_send_TUM << endl;
	}
      ////////////////////////////////////////////////////////  

      /* Should we keep this ? */
      //do { err = sendConn.senddata(send_data,1); }
      //while (err < 0);
      err = sendConn.senddata(send_data,1);
      if( 0!=err ) { sotDEBUG(15) << "Error in sendConn. " << std::endl;} 
    }
  sotDEBUGOUT(15);
}

/* --- CONTROL --------------------------------------------------------------- */
/* --- CONTROL --------------------------------------------------------------- */
/* --- CONTROL --------------------------------------------------------------- */
void *
controlThread( void* argsVoid )
{
  OptArgs* args = (OptArgs*) argsVoid;

  sotDEBUGIN(15);
  /* Setup a udp connection to receive from the network:
   * UDP connection for receiving data from realtime prozess. */
  std::cout << "Send to " << args->udpIp.c_str() << " on port " 
	    << args->recvPort << std::endl;
  sfbcomm::connection recvConn(args->recvPort,(char*)args->udpIp.c_str(),
			       args->recvPort,1 );

  sotDEBUG(1) << "Did I start the control thread ? "<<endl;
  // check to ensure that the udp connection was setup correctly
  switch (recvConn.geterror()) 
    {
    case HOST_NOT_FOUND: 
      {
	sotERROR << "Please enter a correct hostname or IP-Address!" 
	     << endl;
	exit(-1);
      }
    case SOCK_FAILED: 
      {
	sotERROR << "Could not create Socket!" << endl; 
	exit(-1);
      }
    case BIND_FAILED: 
      {
	sotERROR << "Could not bind Port!" << endl;
	exit(-1);
      }
    default:             
      break; 
    }
  
  /* Start receiving control data. */
  recvData( recvConn );
  
  /* Exit the thread. */
  sotDEBUGOUT(15);
  pthread_exit(NULL);
}

/* --- MEASURE --------------------------------------------------------------- */
/* --- MEASURE --------------------------------------------------------------- */
/* --- MEASURE --------------------------------------------------------------- */
void *
measureThread( void* argsVoid )
{
  OptArgs* args = (OptArgs*) argsVoid;

  sotDEBUGIN(15);
  /* Create a udp connection to send data over the network. */
  std::cout << "Send to " << args->udpIp.c_str() << " on port " 
	    << args->sendPort << std::endl;
  sfbcomm::connection sendConn(args->sendPort,(char*)args->udpIp.c_str(),
			       args->sendPort,1 );
 
  /* Check to ensure that the udp connection was setup correctly. */
  switch( sendConn.geterror() )
    {
    case HOST_NOT_FOUND: 
      {
	sotERROR << "Please enter a correct hostname or IP-Address!" << endl;
	sotDEBUGOUT(1);
	exit(-1);
      }
    case SOCK_FAILED: 
      {
	sotERROR << "Could not create Socket!" << endl; 
	sotDEBUGOUT(1);
	exit(-1);
      }
    case BIND_FAILED:
      sotERROR << "Could not bind Port!" << endl; 
      sotDEBUGOUT(1);
      exit(-1);
    default:             
      break; 
    }

  sotDEBUG(15) << "Did the check succeded ? "<<endl;

  /* Start communication. */
  sendData( sendConn );

  /* Exit the thread. */
  sotDEBUGOUT(15);
  pthread_exit(NULL);
}

/* --- CORBA ----------------------------------------------------------------- */
/* --- CORBA ----------------------------------------------------------------- */
/* --- CORBA ----------------------------------------------------------------- */

sotCorbaConnection *
initCorbaConnection( OptArgs& args,
		     SOT_Server_Command_ptr& sot_serverPtr )
{
  std::cout << " Corba Server Name : " << args.corbaServerName << std::endl
	    << " Server Kind : " << args.serverKind << std::endl
	    << " Remote Server Name : " << args.remoteServerName << std::endl;
  sotCorbaConnection * corbaConnectionHandler = NULL;
  sot_serverPtr =NULL;

  try
    {
      corbaConnectionHandler = new sotCorbaConnection( args.corbaServerName, 
						       args.serverKind,
						       args.remoteServerName );
      sotDEBUG(15) << "Connection ..." << endl;
      CORBA::Object_ptr corba_obj 
	= corbaConnectionHandler ->getObjectReference(); 
	  
      sotDEBUG(15) << "Connection [OK]. Cast ... " << endl;
      sot_serverPtr = SOT_Server_Command::_narrow( corba_obj );
	  
      if(CORBA::is_nil( sot_serverPtr ) )
	{
	  sotERROR <<"Server ptr is null: connection is not obtained. " <<std::endl;
// 	  sotERROR << "Fatal error (die die)." <<std::endl;
// 	  exit(0);
	  return NULL;
	}
      sotDEBUG(1) << "Cast [OK]. Run the shell." << endl;
    }
  catch(...)
    {
      sotERROR <<"Exception when trying to connect to the corba server "
	       <<"\"serverCorbaSot\" the program continues "
	       << "but cannot communicate..."<<std::endl;
      sot_serverPtr = NULL;
      //exit(-1);
      return NULL;
    }
      
  /* --- CONFIG --- */
  /* --- CONFIG --- */
  /* --- CONFIG --- */
  sotDEBUG(15) << serverPtr << " " << CORBA::is_nil( serverPtr ) <<std::endl;
      
  SOT_Server_Command::CharSeq signame; 
  SOT_Server_Command::DoubleSeq initVector; 
      
  headSOUT=-1;  forceRSOUT=-1; forceLSOUT=-1; posRSOUT=-1; 
  posLSOUT=-1;  gripSOUT=-1;   refSOUT=-1;    connectSOUT=-1;
  forceRSIN=-1; forceLSIN=-1;  posRSIN=-1;    posLSIN=-1;
      
  try
    {
      INIT_SOUT(forceRSOUT,6);
      INIT_SOUT(forceLSOUT,6);
      INIT_SOUT(headSOUT,2);
      INIT_SOUT(gripSOUT,2);  // 0.082 Maximal value
      INIT_SOUT(refSOUT,1);
      INIT_SOUT(connectSOUT,1);
	  
      //INIT_SIN(forceRSIN);
      INIT_SIN(posRSIN);
      INIT_SIN(posLSIN);
    }
  catch( ... )
    {
      sotERROR <<"Signals already exist." <<std::endl;
    }
      
      
  try
    {
      usleep( 1000*1000 );
      //std::cout << sendSotCommand(  serverPtr,"coshell.signals" );
      
      std::cout << sendSotCommand(  serverPtr,"run " SCRIPTDIRECTORY "/teleoperation/initConnec" );
      std::cout << sendSotCommand(  serverPtr,"coshell.signals" );
      //std::cout << sendSotCommand(  serverPtr,"help" );
    } catch( ... ) 
      {   sotERROR << "Cannot run the config script. " << std::endl; }

  return corbaConnectionHandler;
}

void destroyCorbaConnection( sotCorbaConnection * corbaPtr,
			     SOT_Server_Command_ptr& narrow_server )
{
  if( corbaPtr ) delete corbaPtr; 
  narrow_server = NULL;
}


void killCtlC( int signNum )
{
  kill(getpid(),-9); // pan dans la tete

}
//sighandler_t signal(int signum, sighandler_t handler);
void handleCtlC( int signNum )
{
  sotDEBUG(1) << "SIGINT." << std::endl;
  usleep(1000*100);
  signal( SIGINT,&killCtlC );
  usleep(1000*100);
  resetCorba.notify_one();
}
//sighandler_t signal(int signum, sighandler_t handler);

void * corbaManager( void * argsV )
{
  OptArgs & args = *(OptArgs*) argsV;
  sotDEBUGIN(1);

  /* --- INIT --- */
  /* --- INIT --- */
  /* --- INIT --- */

  sotCorbaConnection * corbaConnectionHandler = NULL;

  /* --- CONNECTION --- */
  boost::mutex::scoped_lock locker(resetCorbaMutex);
  while( 1 )
    {
      sotDEBUG(1) << "(re)Init corba." << std::endl;
      {
	boost::try_mutex::scoped_lock lockerSend(sendConnectionMutex);
	boost::try_mutex::scoped_lock lockerRecv(recvConnectionMutex);
	corbaConnectionHandler = initCorbaConnection( args,serverPtr );
      }
      //signal( SIGINT,&handleCtlC );
      resetCorba.wait( locker );
      {
	boost::try_mutex::scoped_lock lockerSend(sendConnectionMutex);
	boost::try_mutex::scoped_lock lockerRecv(recvConnectionMutex);
	destroyCorbaConnection(corbaConnectionHandler,serverPtr);
      }
    } // end while (1)
  sotDEBUGOUT(1);
}

/* --- MAIN ------------------------------------------------------------------ */
/* --- MAIN ------------------------------------------------------------------ */
/* --- MAIN ------------------------------------------------------------------ */

int main( int argc,char ** argv )
{
  sotDebugTrace::openFile();
  OptArgs args; args.parseOptions(argc,argv);
  
  /* --- DATA LOOP --- */
  /* --- DATA LOOP --- */
  /* --- DATA LOOP --- */
  string Path(LOGDIRECTORY);
  Path+="/Data_send_TUM.sav";
  data_send_TUM.open((char *)Path.c_str());
  Path =LOGDIRECTORY;
  Path+="/Data_recv_TUM.sav";
  data_recv_TUM.open((char *)Path.c_str());

  //signal( SIGINT,&handleCtlC );

//   corbaManager(&args);

  //controlThread(&args);

  //measureThread(&args);
  

  pthread_t corbaThreadHandle; // local variable
  pthread_create(&corbaThreadHandle, NULL, corbaManager, &args); 

  usleep(1000*2000);

  pthread_t controlThreadHandle; // local variable
  pthread_create(&controlThreadHandle, NULL, controlThread, &args);

  pthread_t measureThreadHandle; // local variable
  pthread_create(&measureThreadHandle, NULL, measureThread, &args); 
  
  pthread_join(measureThreadHandle,0);
  pthread_join(corbaThreadHandle,0);
  pthread_join(controlThreadHandle,0);
  
  if(0)
    {

    }

  sotDEBUG(1)<<"Exchange done! (see you soon...)"<<std::endl;
  
}
