/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      coshell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard 
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *  Connection to a sotServerCommand, used as a relay to transmitt
 * shell instruction to the sotInterpretor running inside OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <string>
#include <sstream>
#include <iostream>
#include <vector>
#include <fstream>
#include <assert.h>

#include "sotCorbaConnection.h"
/* IDL automatically generated file. */
#include <ServerCommand.h>

#include <sotDebug.h>

using namespace std;


/* -------------------------------------------------------------------------- */
/* --- OPTIONS -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class OptArgs 
{
public:
  std::string corbaServerName;
  std::string remoteServerName;
  std::string serverKind;

  OptArgs( void ) 
    : corbaServerName( "coshell" )
    ,remoteServerName("localhost")
    ,serverKind("")  
  {}

  static const char optlist [];

public:
  
  void parseOptions(  int argc,char** argv )
  {
    int ch;
    while( (ch=getopt( argc,argv,optlist ))!=-1 ) 
    {
      switch (ch) 
	{
	case 'c':
	  corbaServerName = optarg;
	  break;
	case 'r':
	  remoteServerName = optarg;
	  break;
	case 'k':
	  serverKind = optarg;
	  break;
	case '?':
	default:
	  usage();
	}
    }
  }
  void usage( void ) 
  { cout << "coshell [-c pluginName] [-r remoteServerName] [-k serverKind]"<<endl; }
};

const char OptArgs::optlist [] = "c:r:k:";

static SOT_Server_Command::CharSeq&
setSigName( const std::string& buffer,
	    SOT_Server_Command::CharSeq& signame,
	    const std::string & IO )
{
  const string sigref = "Teleop-Distant::"+IO+"put(vector)::";
  string sigfullname = sigref + buffer;
  const char nbCar = sigfullname.length(); 
  signame.length(nbCar);
  for( unsigned int i=0;i<nbCar;++i ){ signame[i]=sigfullname[i]; }
  return signame;
}

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


int main( int argc,char ** argv )
{
  /* --- INIT --- */
  /* --- INIT --- */
  /* --- INIT --- */

  sotCorbaConnection * corbaConnectionHandler = NULL;
  SOT_Server_Command_ptr serverPtr = NULL;
  OptArgs args; args.parseOptions(argc,argv);

  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  cout << " Corba Server Name : " << args.corbaServerName << endl
       << " Server Kind : " << args.serverKind << endl
       << " Remote Server Name : " << args.remoteServerName << endl;
  try
    {
      corbaConnectionHandler = new sotCorbaConnection( args.corbaServerName, 
						    args.serverKind,
						    args.remoteServerName );
      sotDEBUG(15) << "Connection ..." << endl;
      CORBA::Object_ptr corba_obj 
	= corbaConnectionHandler ->getObjectReference(); 
      
      sotDEBUG(15) << "Connection [OK]. Cast ... " << endl;
      serverPtr = SOT_Server_Command::_narrow( corba_obj );
      
      if(CORBA::is_nil( serverPtr ) )
	{
	  std::cout <<"Wrong server " <<std::endl;
	  exit(0);
	}
      sotDEBUG(1) << "Cast [OK]. Run the shell." << endl;
    }
  catch(...)
    {
      cerr<<"Exception when trying to connect to the corba server \"serverCorbaSot\"";
      cerr<<"the program continues but cannot communicate.."<<endl;
      serverPtr = NULL;
      exit(-1);
    }


  /* --- CONFIG --- */
  std::cout << serverPtr << " " << CORBA::is_nil( serverPtr ) <<std::endl;

  SOT_Server_Command::CharSeq signame; 
  
  CORBA::Long forceSOUT 
    = serverPtr->createOutputVectorSignal( setSigName( "forceOUT",signame,"out" ) );
  {
    SOT_Server_Command::DoubleSeq force; force.length(6);
    for( unsigned i=0;i<6;++i ) force[i]=0;
    serverPtr->writeOutputVectorSignal( forceSOUT,force );
  }
  CORBA::Long headSOUT
    = serverPtr->createOutputVectorSignal( setSigName( "head",signame,"out" ) );

  CORBA::Long forceSIN
    = serverPtr->createInputVectorSignal( setSigName( "forceIN",signame,"in" ) );
  
  usleep( 1000*20 );
  const std::string cmd = "run /home/nmansard/src/StackOfTasks/script/debug";
  signame.length( cmd.size() ); 
  for( unsigned int i=0;i<cmd.size();++i ) signame[i]=cmd[i];
  SOT_Server_Command::StringStreamer_var outputShell;
  serverPtr->runAndRead( signame,outputShell );
  for( unsigned int i=0;i<outputShell->length();i++ )	
    { cout << outputShell[i]; }
  cout << endl;
  usleep( 1000*20 );

  /* --- DATA LOOP --- */
  SOT_Server_Command::CharSeq input;
  SOT_Server_Command::StringStreamer_var output;
  const unsigned int BUFFER_SIZE = 1024;
  char buffer [ BUFFER_SIZE ] ;

  for( unsigned int iter=0;;++iter )
    {
      SOT_Server_Command::DoubleSeq force; force.length(6);
      for( unsigned i=0;i<6;++i ) force[i]=iter+i;
      serverPtr->writeOutputVectorSignal( forceSOUT,force );

      SOT_Server_Command::DoubleSeq_var forceVar;
      serverPtr->readInputVectorSignal( forceSIN,forceVar );
      std::cout << "Iter: " << iter << ":  ";
      for( unsigned i=0;i<6;++i ) std::cout << forceVar[i] << "   ";
      std::cout << std::endl;
      usleep( 1000*20 );
    }
  
  cout<<"Shell done! (see you soon...)"<<endl;

}
