/*
 * Improved version of the Corba client connection provided
 * in the SoT (ORB handled as a singleton).
 */


#ifndef CORBA_CONNECTION_H___
#define CORBA_CONNECTION_H___

// STL
#include <string>

// OmniORB
#include <omniORB4/CORBA.h>


/* ------------------------------------------------------- */
/* --- ORB Singleton ------------------------------------- */
/* ------------------------------------------------------- */

/*
 * Beware! The creation of the singleton instance can not
 * be made thread-safe!
 */

class TheORB
{
public:

  static void set_ns_ip(const char* ip);
  static void set_ns_port(const char* port);
  static void set_ns_name(const char* ns);
  static CORBA::ORB_ptr get();

private:

  TheORB();

private:

  TheORB(const TheORB&);
  TheORB& operator=(const TheORB&);

private:

  static CORBA::ORB_var orb;
  static std::string    ip;
  static std::string    port;
  static std::string    ns;
};


/* ------------------------------------------------------- */
/* --- CORBA client -------------------------------------- */
/* ------------------------------------------------------- */

class CorbaConnection
{
public:

  void set_server_name(const char* name);
  void set_server_kind(const char* kind);
  CORBA::Object_ptr get_server();

public:

  CorbaConnection();

private:

  int try_corba_connection();
  
  CORBA::Object_ptr
  get_object_reference(const char* serverId, const char* serverKind);

private:

  CORBA::Object_ptr server;
  std::string       server_id;
  std::string       server_kind;
  int               trials;
  bool              connection_ok;

private:

  static const int  trials_before_connection;
};


/* --- EXCEPTIONS ------------------------------------------------- */

class DS_ServerConnectionException
{
public:
  DS_ServerConnectionException() { std::cerr << "CORBA COMM_FAILURE" << std::endl; }
};

class DS_SystemException
{
public:
  DS_SystemException() { std::cerr << "CORBA Exception" << std::endl; }
};

class DS_FatalException
{
public:
  DS_FatalException() { std::cerr << "CORBA Fatal Exception" << std::endl; }
};

class DS_Exception
{
public:
  DS_Exception() { std::cerr << "Exception" << std::endl; }
};


/* --- HELPERS ---------------------------------------------------- */

template<class Tptr, class T>
inline void initServer(CorbaConnection& c, Tptr& server, const char* name)
{
  c.set_server_name(name);
  CORBA::Object_ptr corba_obj = c.get_server();
  assert(!CORBA::is_nil(corba_obj) && "Could not get server");
  server = T::_narrow(corba_obj);
  assert(!CORBA::is_nil(server) && "Could not narrow server");
}

#endif
