/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotNextStepPgSot.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <algorithm>

#include "sotNextStepPgSot.h"

#include <sot/sotDebug.h>
#include <sot/sotFactory.h>
#include <sot/sotMacrosSignal.h>
#include <sot/sotExceptionPatternGenerator.h>
#include <sot/sotPool.h>

SOT_FACTORY_ENTITY_PLUGIN(sotNextStepPgSot,"NextStepPgSot");
 

/* --- CONSTRUCT ------------------------------------------------------------- */
/* --- CONSTRUCT ------------------------------------------------------------- */
/* --- CONSTRUCT ------------------------------------------------------------- */

sotNextStepPgSot::
sotNextStepPgSot( const std::string & name ) 
  :sotNextStep(name)
{
  sotDEBUGIN(5);
  m_StepModificationMode = sotNextStepPgSot::ADDING_STEP;
  m_NextStepTime = -1.0;
  m_NbOfFirstSteps = 0;
  m_PGI = 0;
  sotDEBUGOUT(5);
}


/* --- FUNCTIONS ------------------------------------------------------------ */
/* --- FUNCTIONS ------------------------------------------------------------ */
/* --- FUNCTIONS ------------------------------------------------------------ */


void sotNextStepPgSot::
starter( const int & timeCurr )
{
  sotDEBUGIN(15);

  sotNextStep::starter(timeCurr);

  if(pgEntity) {
    std::ostringstream cmdstd; std::ostringstream os;
    cmdstd << ":StartOnLineStepSequencing ";
    for( std::deque< sotFootPrint >::const_iterator iter = footPrintList.begin();
	   iter!=footPrintList.end();++iter )
    {
      cmdstd <<  iter->x << " " << iter->y << " " << iter->theta << " ";
    }
    std::istringstream cmdArg( cmdstd.str() );
    std::istringstream emptyArg;
    pgEntity->commandLine( "initState", emptyArg, os );
    pgEntity->commandLine( "parsecmd", cmdArg, os );
    sotDEBUG(15) << "Cmd: " << cmdstd.str() << std::endl;
  } else { sotERROR <<"PG not set" << std::endl; }

  state = STATE_STARTED;
  
  sotDEBUGOUT(15); 
  return;
}

void sotNextStepPgSot::
stoper( const int & timeCurr )
{
  sotDEBUGIN(15);

  if(pgEntity) {
    std::ostringstream cmdstd; std::ostringstream os;
    cmdstd << ":StopOnLineStepSequencing";
    std::istringstream cmdArg( cmdstd.str() );
    pgEntity->commandLine( "parsecmd", cmdArg, os );
  } else { sotERROR <<"PG not set" << std::endl; }

  state = STATE_STOPED;
  
  sotDEBUGOUT(15); 
  return;
}

void sotNextStepPgSot::
introductionCallBack( const int & timeCurr ) 
{
  sotDEBUGIN(15); 

  if( state==STATE_STARTED ) 
    {
      
      sotFootPrint & lastStep =  footPrintList.back();
      if( pgEntity ) 
	{
	  if (m_StepModificationMode == sotNextStepPgSot::ADDING_STEP)
	    {
	      std::string cmdLine = "addStep";
	      std::ostringstream cmdArgIn; std::ostringstream os;
	      cmdArgIn << lastStep.x << " " << lastStep.y << " " << lastStep.theta;
	      std::istringstream cmdArg( cmdArgIn.str() );
	      pgEntity->commandLine(cmdLine,cmdArg,os); 
	    }
	  else if (m_StepModificationMode == sotNextStepPgSot::CHANGING_STEP)
	    {
	      if (m_NbOfFirstSteps<3)
		m_NbOfFirstSteps++;
	      else
	      {
		sotDEBUG(15) << "m_NextStepTime: "<< m_NextStepTime << std::endl;

		PatternGeneratorJRL::FootAbsolutePosition aFAP;

		unsigned int lSupportFoot=0;

		lSupportFoot = m_sPG->SupportFootSOUT(timeCurr);
		
		double NextFAPx = lastStep.x;
		double NextFAPy = lastStep.y;
		double NextFAPtheta = lastStep.theta;
		if (0)
		  {
		    NextFAPx = 0.3*(double)rand()/(double)RAND_MAX -0.15;
		    NextFAPy = 0.58*(double)rand()/(double)RAND_MAX -0.24;
		    NextFAPtheta = 10.0*(double)rand()/(double)RAND_MAX - 5.0;
		  }
		/* Warning : HARDCODED VALUES */

		if (NextFAPx<-0.10) 
		  NextFAPx = -0.10;
		if (NextFAPx>0.10) 
		  NextFAPx = 0.10;
		
		if (NextFAPtheta<-5.0) 
		  NextFAPtheta = -5.0;
		if (NextFAPtheta>5.0) 
		  NextFAPtheta = 5.0;
		
		if (lSupportFoot==1)
		  { 
		    // This is the right foot support.
		    // Make sure this is relative to the 
		    // next stepping value.
		    NextFAPy= NextFAPy - 0.19;
		    
		    // Test boundaries relative to this
		    // stepping value.
		    if (NextFAPy<-0.05) 
		      NextFAPy = -0.05;
		    if (NextFAPy>0.19)
		      NextFAPy = 0.19;
		  }
		else 
		  {
		    // This is the left foot support.
		    // Make sure this is relative to the 
		    // next stepping value.
		    NextFAPy= NextFAPy + 0.19;

		    // Test boundaries relative to this
		    // stepping value.
		    if (NextFAPy<-0.19) 
		      NextFAPy = -0.19;
		    if (NextFAPy>0.05)
		      NextFAPy = 0.05;		    		    
		  }

		/* End of Warning : HARDCODED VALUES */
		aFAP.x = NextFAPx;
		aFAP.y = NextFAPy;
		aFAP.theta= NextFAPtheta;
		std::cout << lSupportFoot << std::endl;
		std::cout << "FAP = ( "<< aFAP.x << " " << aFAP.y << " " << aFAP.z << " ) " << std::endl;
		m_NextStepTime = 0.0;

		m_PGI->ChangeOnLineStep(0.805,aFAP,m_NextStepTime);

		if (m_NextStepTime<0.0)
		  period =sotNextStep::PERIOD_DEFAULT;
		else
		  period = (unsigned int)((m_NextStepTime+0.025)*200.0);
		
	      }
	    }
	} else {
	  sotERROR << "Walk plugin not found. " << std::endl; 
	  /* TODO Error: PG not set. */ 
	}
    }

  sotDEBUGOUT(15); 
  return;

}



void sotNextStepPgSot::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  if( "help"==cmdLine )
    {
      os << "sotNextStepPgSot: " << std::endl 
	 << " - initPg [<pg_name>]" << std::endl
	 << " - stepmodificationmode [add|change]" <<std::endl;
    }
  else if( "initPg" == cmdLine )
    {
      std::string name = "pg";
      cmdArgs >> std::ws; if( cmdArgs.good()) cmdArgs >> name;
      pgEntity = &sotPool.getEntity( name );
      m_sPG = dynamic_cast<sotPatternGenerator *>(pgEntity);
      if (m_sPG!=0)
	m_PGI = m_sPG->GetPatternGeneratorInterface();
    }
  else if( "stepmodificationmode" == cmdLine)
    {
      std::string stgmode;
      cmdArgs >> std::ws; 
      if( cmdArgs.good()) 
	{
	  cmdArgs >> stgmode;
	  if (stgmode=="add")
	    m_StepModificationMode=sotNextStepPgSot::ADDING_STEP;
	  else if (stgmode=="change")
	    m_StepModificationMode=sotNextStepPgSot::CHANGING_STEP;
	}
      else
	{
	  if (m_StepModificationMode==sotNextStepPgSot::ADDING_STEP)
	    os << "add";
	  else if (m_StepModificationMode==sotNextStepPgSot::CHANGING_STEP)
	    os << "change";
	}
    }
  else { sotNextStep::commandLine( cmdLine,cmdArgs,os ); }

}

