/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2008
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotPatternGenerator.cpp
 * Project:   SOT
 * Author:    Olivier Stasse
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <iostream>

#include <sot/sotPatternGenerator.h>
#include <sot/sotDebug.h>
#include <sot/sotFactory.h>
#include <sot/sotMatrixHomogeneous.h>

using namespace std;


SOT_FACTORY_ENTITY_PLUGIN(sotPatternGenerator,"PatternGenerator");

sotPatternGenerator::
sotPatternGenerator( const std::string & name ) 
  :sotEntity(name)
		   //  ,m_PGI(NULL)
  ,m_PreviewControlParametersFile()
  ,m_vrmlDirectory()
  ,m_vrmlMainFile()
  ,m_xmlSpecificityFile()
  ,m_xmlRankFile()

  ,m_init(false)

  ,firstSINTERN( boost::bind(&sotPatternGenerator::InitOneStepOfControl,this,_1,_2),
		 sotNOSIGNAL,"sotPatternGenerator("+name+")::intern(dummy)::init" ) 

  ,OneStepOfControlS( boost::bind(&sotPatternGenerator::OneStepOfControl,this,_1,_2),
		      firstSINTERN << jointPositionSIN ,"sotPatternGenerator("+name+")::onestepofcontrol" ) 

   ,m_dataInProcess(0)
  ,jointPositionSIN(NULL,"sotPatternGenerator("+name+")::input(vector)::position")

  ,motorControlJointPositionSIN(NULL,"sotPatternGenerator("+name+")::input(vector)::motorcontrol")

  ,ZMPPreviousControllerSIN(NULL,"sotPatternGenerator("+name+")::input(vector)::zmppreviouscontroller")

  ,ZMPRefSOUT( boost::bind(&sotPatternGenerator::getZMPRef,this,_1,_2),
	       OneStepOfControlS,
	    "sotPatternGenerator("+name+")::output(vector)::zmpref" ) 

  ,CoMRefSOUT( boost::bind(&sotPatternGenerator::getCoMRef,this,_1,_2),
	     OneStepOfControlS,
	       "sotPatternGenerator("+name+")::output(matrix)::comref" ) 

  ,dCoMRefSOUT( boost::bind(&sotPatternGenerator::getdCoMRef,this,_1,_2),
		OneStepOfControlS,
	       "sotPatternGenerator("+name+")::output(matrix)::dcomref" ) 

  ,comSIN(NULL,"sotPatternGenerator("+name+")::input(vector)::com")

  ,LeftFootCurrentPosSIN(NULL,"sotPatternGenerator("+name+")::input(homogeneousmatrix)::leftfootcurrentpos")

  ,RightFootCurrentPosSIN(NULL,"sotPatternGenerator("+name+")::input(homogeneousmatrix)::rightfootcurrentpos")

  ,LeftFootRefSOUT( boost::bind(&sotPatternGenerator::getLeftFootRef,this,_1,_2),
	    OneStepOfControlS,
	    "sotPatternGenerator("+name+")::output(homogeneousmatrix)::leftfootref" ) 

  ,RightFootRefSOUT( boost::bind(&sotPatternGenerator::getRightFootRef,this,_1,_2),
		     OneStepOfControlS,
		   "sotPatternGenerator("+name+")::output(homogeneousmatrix)::rightfootref" ) 
  ,dotLeftFootRefSOUT( boost::bind(&sotPatternGenerator::getdotLeftFootRef,this,_1,_2),
	    OneStepOfControlS,
	    "sotPatternGenerator("+name+")::output(homogeneousmatrix)::dotleftfootref" ) 

  ,dotRightFootRefSOUT( boost::bind(&sotPatternGenerator::getdotRightFootRef,this,_1,_2),
		     OneStepOfControlS,
		   "sotPatternGenerator("+name+")::output(homogeneousmatrix)::dotrightfootref" ) 

  ,FlyingFootRefSOUT( boost::bind(&sotPatternGenerator::getFlyingFootRef,this,_1,_2),
		   OneStepOfControlS,
		   "sotPatternGenerator("+name+")::output(homogeneousmatrix)::flyingfootref" ) 


   ,SupportFootSOUT( boost::bind(&sotPatternGenerator::getSupportFoot,this,_1,_2),
		     OneStepOfControlS,
		     "sotPatternGenerator("+name+")::output(uint)::SupportFoot" )
   ,jointWalkingErrorPositionSOUT(boost::bind(&sotPatternGenerator::getjointWalkingErrorPosition,this,_1,_2),
				  OneStepOfControlS,
				  "sotPatternGenerator("+name+")::output(vector)::walkingerrorposition")
  
  ,waistattitudeSOUT(boost::bind(&sotPatternGenerator::getWaistAttitude,this,_1,_2),
		     OneStepOfControlS,
		     "sotPatternGenerator("+name+")::output(vectorRPY)::waistattitude")
  ,waistattitudeabsoluteSOUT(boost::bind(&sotPatternGenerator::getWaistAttitudeAbsolute,this,_1,_2),
			     OneStepOfControlS,
			     "sotPatternGenerator("+name+")::output(vectorRPY)::waistattitudeabsolute")
   
  ,waistpositionSOUT(boost::bind(&sotPatternGenerator::getWaistPosition,this,_1,_2),
		     OneStepOfControlS,
		     "sotPatternGenerator("+name+")::output(vector)::waistposition")
   ,waistpositionabsoluteSOUT(boost::bind(&sotPatternGenerator::getWaistPositionAbsolute,this,_1,_2),
			      OneStepOfControlS,
			      "sotPatternGenerator("+name+")::output(vector)::waistpositionabsolute")

  ,dataInProcessSOUT(boost::bind(&sotPatternGenerator::getDataInProcess, this, _1, _2),
		     OneStepOfControlS,
		      "sotPatternGenerator("+name+")::output(bool)::inprocess")
  ,InitZMPRefSOUT( boost::bind(&sotPatternGenerator::getInitZMPRef,this,_1,_2),
	       OneStepOfControlS,
	    "sotPatternGenerator("+name+")::output(vector)::initzmpref" ) 

  ,InitCoMRefSOUT( boost::bind(&sotPatternGenerator::getInitCoMRef,this,_1,_2),
	     OneStepOfControlS,
	       "sotPatternGenerator("+name+")::output(matrix)::initcomref" ) 

  ,InitWaistPosRefSOUT( boost::bind(&sotPatternGenerator::getInitWaistPosRef,this,_1,_2),
			OneStepOfControlS,
			"sotPatternGenerator("+name+")::output(vector)::initwaistposref" ) 

  ,InitWaistAttRefSOUT( boost::bind(&sotPatternGenerator::getInitWaistAttRef,this,_1,_2),
			OneStepOfControlS,
			"sotPatternGenerator("+name+")::output(vectorRPY)::initwaistattref" ) 

  ,InitLeftFootRefSOUT( boost::bind(&sotPatternGenerator::getInitLeftFootRef,this,_1,_2),
			OneStepOfControlS,
			"sotPatternGenerator("+name+")::output(homogeneousmatrix)::initleftfootref" ) 
   
  ,InitRightFootRefSOUT( boost::bind(&sotPatternGenerator::getInitRightFootRef,this,_1,_2),
			 OneStepOfControlS,
			 "sotPatternGenerator("+name+")::output(homogeneousmatrix)::initrightfootref" ) 


{
  m_LocalTime = 0;
  m_TimeStep = 0.005;
  m_DoubleSupportPhaseState = false;

  m_ZMPRefPos.resize(4);
  m_ZMPRefPos.fill(0.0);
  m_ZMPRefPos(3) = 1.0;
  m_ZMPPrevious.resize(4);
  m_ZMPPrevious(3) = 1.0;
  m_COMRefPos.resize(3);
  m_dCOMRefPos.resize(3);
  m_dCOMRefPos.fill(0.0);
  m_InitZMPRefPos.resize(3);
  m_InitZMPRefPos.fill(0);
  m_InitCOMRefPos.resize(3);
  m_InitCOMRefPos.fill(0);
  m_InitWaistRefPos.resize(3);
  m_InitWaistRefPos.fill(0);
  m_InitWaistRefAtt.resize(3);
  m_InitWaistRefAtt.fill(0);

  m_WaistAttitude.resize(3);
  m_WaistAttitude.fill(0);
  m_WaistPosition.resize(3);
  m_WaistPosition.fill(0);
  m_WaistAttitudeAbsolute.resize(3);
  m_WaistAttitudeAbsolute.fill(0);
  m_WaistPositionAbsolute.resize(3);
  m_WaistPositionAbsolute.fill(0);

  m_SupportFoot = 1; // Means that we do not know which support foot it is.
  m_ReferenceFrame = WORLD_FRAME;
  
  /* TODO: To be removed and extract from 
     Humanoid specificities.
   */
  m_AnkleSoilDistance = 0.105; 
  sotDEBUGIN(5);

  firstSINTERN.setDependancyType(sotTimeDependancy<int>::BOOL_DEPENDANT);
  // TODO: here, the 'setConstant' destroy the pointer toward
  // function initOneStepOfControl. By calling firstSINTERN(t), whatever t,
  // nothing will happen (well, it will just return 0).
  // To initialize firstSINTERN (without destroying the pointer), use
  // firstSINTERN.setReady() instead.
  // TODO: Remove the next line: // firstSINTERN.setConstant(0);
  firstSINTERN.setReady(true);

  //OneStepOfControlS.setDependancyType(sotTimeDependancy<int>::ALWAYS_READY);
  //  OneStepOfControlS.setConstant(0);

  signalRegistration( jointPositionSIN <<
		      motorControlJointPositionSIN <<
		      ZMPPreviousControllerSIN <<
		      ZMPRefSOUT <<
		      CoMRefSOUT <<
		      dCoMRefSOUT);

  signalRegistration( dataInProcessSOUT <<
		      LeftFootCurrentPosSIN <<
		      RightFootCurrentPosSIN << 
		      LeftFootRefSOUT <<
		      RightFootRefSOUT);

  signalRegistration( SupportFootSOUT <<
		      jointWalkingErrorPositionSOUT <<
		      waistattitudeSOUT <<
		      waistpositionSOUT <<
		      waistattitudeabsoluteSOUT <<
		      waistpositionabsoluteSOUT);

  signalRegistration( dotLeftFootRefSOUT <<
		      dotRightFootRefSOUT);
		      
  signalRegistration( InitZMPRefSOUT <<
		      InitCoMRefSOUT <<
		      InitWaistPosRefSOUT << 
		      InitWaistAttRefSOUT << 
		      InitLeftFootRefSOUT <<
		      InitRightFootRefSOUT <<		      
		      comSIN  );

  dataInProcessSOUT.setReference( &m_dataInProcess );

  sotDEBUGOUT(5);
}

bool sotPatternGenerator::InitState(void)
{
  sotDEBUGIN(15); 
  // TODO
  // Instead of (0) ie .access(0), it could be rather used:
  // .accessCopy()
  // Instead of copy value (ml::Vector pos) it could be rather
  // used reference (const ml::Vector & post)
  ml::Vector pos = jointPositionSIN(m_LocalTime); 
  ml::Vector com = comSIN(m_LocalTime); 
  ml::Vector lZMPPrevious = ZMPPreviousControllerSIN(m_LocalTime);
  for(unsigned int i=0;i<3;i++)
    m_ZMPPrevious[i] = lZMPPrevious(i);
  //  m_ZMPPrevious[2] =m_AnkleSoilDistance; // Changed the reference frame.

  ml::Vector res;
  res.resize( pos.size()-6);

  for(unsigned i=0;i<res.size();i++)
    res(i) = pos(i+6);

  m_JointErrorValuesForWalking.resize(res.size());

  sotDEBUG(25) << "Joint Values:" << res << endl;
  sotDEBUG(25) << "ZMPRef:" << lZMPPrevious << endl;

  try 
    {
      m_PGI->SetCurrentJointValues(res.accessToMotherLib());
      for(unsigned int i=0;i<3;i++)
	m_ZMPPrevious[i] = lZMPPrevious(i);
      //      m_ZMPPrevious[2] = -m_AnkleSoilDistance; // Changed the reference frame.
      
      {
	PatternGeneratorJRL::COMPosition lStartingCOMPosition;
	MAL_S3_VECTOR(,double) lStartingZMPPosition;
	MAL_VECTOR(,double) lWaistPosition;
	PatternGeneratorJRL::FootAbsolutePosition InitLeftFootAbsPos;
	PatternGeneratorJRL::FootAbsolutePosition InitRightFootAbsPos;

	m_PGI->EvaluateStartingState(lStartingCOMPosition,
				     lStartingZMPPosition,
				     lWaistPosition,
				     InitLeftFootAbsPos,
				     InitRightFootAbsPos);

	
	m_InitCOMRefPos(0) = lStartingCOMPosition.x[0];
	m_InitCOMRefPos(1) = lStartingCOMPosition.y[0];
	m_InitCOMRefPos(2) = lStartingCOMPosition.z[0];

	m_InitZMPRefPos(0) = m_ZMPPrevious[0] = lStartingCOMPosition.x[0];
	m_InitZMPRefPos(1) = m_ZMPPrevious[1] = lStartingCOMPosition.y[0];
	m_InitZMPRefPos(2) = m_ZMPPrevious[2] = 0;

	sotDEBUG(25) << "InitZMPRefPos :" <<m_InitZMPRefPos<< endl;
	
	m_InitWaistRefPos(0) = 
	  m_WaistPositionAbsolute(0) = lWaistPosition(0);
	m_InitWaistRefPos(1) = 
	  m_WaistPositionAbsolute(1) = lWaistPosition(1);
	m_InitWaistRefPos(2) = 
	  m_WaistPositionAbsolute(2) = lWaistPosition(2);

	m_InitWaistRefAtt(0) = 
	  m_WaistAttitudeAbsolute(0) = lWaistPosition(3);
	m_InitWaistRefAtt(1) = 
	  m_WaistAttitudeAbsolute(1) = lWaistPosition(4);
	m_InitWaistRefAtt(2) = 
	  m_WaistAttitudeAbsolute(2) = lWaistPosition(5);

	FromAbsoluteFootPosToDotHomogeneous(InitRightFootAbsPos,
					    m_InitRightFootPosition,
					    m_dotRightFootPosition);
	FromAbsoluteFootPosToDotHomogeneous(InitLeftFootAbsPos,
					    m_InitLeftFootPosition,
					    m_dotLeftFootPosition);
		  
	m_LeftFootPosition = m_InitLeftFootPosition;
	m_RightFootPosition = m_InitRightFootPosition;
	
	sotDEBUG(25) << "ILF :" <<m_InitLeftFootPosition << " "
		     << "LRF :" <<m_InitRightFootPosition << endl;
      }

      if (0)
      {
	MAL_S3_VECTOR(lZMPPrevious,double);
	for(unsigned int i=0;i<3;i++)
	  lZMPPrevious(i) = 0.0;

	m_PGI->setZMPInitialPoint(lZMPPrevious);
	for(unsigned int i=0;i<3;i++)
	  m_ZMPPrevious(i) = lZMPPrevious(i);
      }
    }
  catch(...)
    {
      SOT_THROW sotExceptionPatternGenerator( sotExceptionPatternGenerator::PATTERN_GENERATOR_JRL,
					      "Error while setting the current joint values of the WPG.");      
      return false;
    }
  sotDEBUGOUT(15);
  return true;
}
bool sotPatternGenerator::buildModel( void )
{
  
  string GlobalInitialization = m_PreviewControlParametersFile;
  GlobalInitialization += " ";
  GlobalInitialization += m_vrmlDirectory; 
  GlobalInitialization += " ";
  GlobalInitialization += m_vrmlMainFile; 
  GlobalInitialization += " ";
  GlobalInitialization += m_xmlSpecificityFile; 
  GlobalInitialization += " ";
  GlobalInitialization += m_xmlRankFile; 

  ofstream aof("out.dat");
  aof << "GlobalInitialization :" << GlobalInitialization <<std::endl;
  aof.close();

  std::istringstream strm(GlobalInitialization);
  try
    {
      m_PGI = new pg::PatternGeneratorInterface(strm);
    }

  catch (...)
    {
      SOT_THROW sotExceptionPatternGenerator( sotExceptionPatternGenerator::PATTERN_GENERATOR_JRL,
					      "Error while allocating the Pattern Generator.",
					      "(PG creation process for object %s).",
					      getName().c_str());
    }

  m_init = true;
  return false;
}

sotPatternGenerator::
~sotPatternGenerator( void )
{
  sotDEBUGIN(5);
  if( 0!=m_PGI )
    {
      delete m_PGI;
      m_PGI = 0;
    }

  sotDEBUGOUT(5);
  
  return;
}


/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
void sotPatternGenerator::
setVrmlDirectory( const std::string& filename )
{
  m_vrmlDirectory = filename;
}
void sotPatternGenerator::
setVrmlMainFile( const std::string& filename )
{
  m_vrmlMainFile = filename;
}
void sotPatternGenerator::
setXmlSpecificityFile( const std::string& filename )
{
  m_xmlSpecificityFile = filename;
}
void sotPatternGenerator::
setXmlRankFile( const std::string& filename )
{
  m_xmlRankFile = filename;
}

void sotPatternGenerator::
setParamPreviewFile( const std::string& filename )
{
  m_PreviewControlParametersFile = filename;
}


/* --- COMPUTE -------------------------------------------------------------- */
/* --- COMPUTE -------------------------------------------------------------- */
/* --- COMPUTE -------------------------------------------------------------- */

#include <MatrixAbstractLayer/boostspecific.h>

ml::Vector & sotPatternGenerator::
getZMPRef(ml::Vector & ZMPRefval, int time)
{
  sotDEBUGIN(5);
 
  OneStepOfControlS(time);

  sotDEBUG(5) << "ZMPRefPos transmitted" << m_ZMPRefPos << endl;
  ZMPRefval = m_ZMPRefPos;

  sotDEBUGOUT(5);
  return ZMPRefval;
}

ml::Vector & sotPatternGenerator::
getCoMRef(ml::Vector & CoMRefval, int time)
{
  sotDEBUGIN(25);
  
  OneStepOfControlS(time);
  CoMRefval = m_COMRefPos;

  sotDEBUGOUT(25);
  return CoMRefval;
}

ml::Vector & sotPatternGenerator::
getdCoMRef(ml::Vector & CoMRefval, int time)
{
  sotDEBUGIN(25);
  
  OneStepOfControlS(time);
  CoMRefval = m_dCOMRefPos;

  sotDEBUGOUT(25);
  return CoMRefval;
}

ml::Vector & sotPatternGenerator::
getInitZMPRef(ml::Vector & InitZMPRefval, int time)
{
  sotDEBUGIN(5);
 
  sotDEBUG(5) << "InitZMPRefPos transmitted" << m_InitZMPRefPos << endl;
  InitZMPRefval = m_InitZMPRefPos;

  sotDEBUGOUT(5);
  return InitZMPRefval;
}

ml::Vector & sotPatternGenerator::
getInitCoMRef(ml::Vector & InitCoMRefval, int time)
{
  sotDEBUGIN(25);
  
  InitCoMRefval = m_InitCOMRefPos;

  sotDEBUGOUT(25);
  return InitCoMRefval;
}

ml::Vector & sotPatternGenerator::
getInitWaistPosRef(ml::Vector & InitWaistRefval, int time)
{
  sotDEBUGIN(25);
  
  InitWaistRefval = m_InitWaistRefPos;

  sotDEBUGOUT(25);
  return InitWaistRefval;
}
sotVectorRollPitchYaw & sotPatternGenerator::
getInitWaistAttRef(sotVectorRollPitchYaw & InitWaistRefval, int time)
{
  sotDEBUGIN(25);
  
  for(unsigned int i=0;i<3;++i)
    InitWaistRefval(i) = m_InitWaistRefAtt(i);

  sotDEBUGOUT(25);
  return InitWaistRefval;
}



sotMatrixHomogeneous & sotPatternGenerator::
getLeftFootRef(sotMatrixHomogeneous & LeftFootRefVal, int time)
{
  sotDEBUGIN(25);

  OneStepOfControlS(time);
  LeftFootRefVal = m_LeftFootPosition;
  sotDEBUGOUT(25) ;
  return LeftFootRefVal;
}
sotMatrixHomogeneous & sotPatternGenerator::
getRightFootRef(sotMatrixHomogeneous & RightFootRefval, int time)
{
  sotDEBUGIN(25);

  OneStepOfControlS(time);

  RightFootRefval = m_RightFootPosition;
  sotDEBUGOUT(25);
  return RightFootRefval;
}
sotMatrixHomogeneous & sotPatternGenerator::
getdotLeftFootRef(sotMatrixHomogeneous & LeftFootRefVal, int time)
{
  sotDEBUGIN(25);

  OneStepOfControlS(time);
  LeftFootRefVal = m_dotLeftFootPosition;
  sotDEBUGOUT(25) ;
  return LeftFootRefVal;
}
sotMatrixHomogeneous & sotPatternGenerator::
getdotRightFootRef(sotMatrixHomogeneous & RightFootRefval, int time)
{
  sotDEBUGIN(25);

  OneStepOfControlS(time);

  RightFootRefval = m_dotRightFootPosition;
  sotDEBUGOUT(25);
  return RightFootRefval;
}

sotMatrixHomogeneous & sotPatternGenerator::
getInitLeftFootRef(sotMatrixHomogeneous & LeftFootRefVal, int time)
{
  sotDEBUGIN(25);

  LeftFootRefVal = m_InitLeftFootPosition;
  sotDEBUGOUT(25) ;
  return LeftFootRefVal;
}
sotMatrixHomogeneous & sotPatternGenerator::
getInitRightFootRef(sotMatrixHomogeneous & RightFootRefval, int time)
{
  sotDEBUGIN(25);

  RightFootRefval = m_InitRightFootPosition;
  sotDEBUGOUT(25);
  return RightFootRefval;
}

sotMatrixHomogeneous & sotPatternGenerator::
getFlyingFootRef(sotMatrixHomogeneous & FlyingFootRefval, int time)
{
  sotDEBUGIN(25);
  OneStepOfControlS(time);
  FlyingFootRefval = m_FlyingFootPosition;
  sotDEBUGOUT(25);
  return FlyingFootRefval;
}

int &sotPatternGenerator::
InitOneStepOfControl(int &dummy, int time)
{
  sotDEBUGIN(15);
  // TODO: modified first to avoid the loop.
  firstSINTERN.setReady(false);
  //  buildModel();
  // Todo: modified the order of the calls
  //OneStepOfControlS(time);
  sotDEBUGIN(15);
  return dummy;
}

void sotPatternGenerator::getAbsoluteWaistPosAttHomogeneousMatrix(sotMatrixHomogeneous &aWaistMH)
{
  
  const double cr = cos(m_WaistAttitudeAbsolute(0)); // ROLL
  const double sr = sin(m_WaistAttitudeAbsolute(0));
  const double cp = cos(m_WaistAttitudeAbsolute(1)); // PITCH
  const double sp = sin(m_WaistAttitudeAbsolute(1));
  const double cy = cos(m_WaistAttitudeAbsolute(2)); // YAW
  const double sy = sin(m_WaistAttitudeAbsolute(2));
  
  aWaistMH.fill(0.0);

  aWaistMH(0,0) = cy*cp;
  aWaistMH(0,1) = cy*sp*sr-sy*cr;
  aWaistMH(0,2) = cy*sp*cr+sy*sr;
  aWaistMH(0,3) = m_WaistPositionAbsolute(0);

  aWaistMH(1,0) = sy*cp;
  aWaistMH(1,1) = sy*sp*sr+cy*cr;
  aWaistMH(1,2) = sy*sp*cr-cy*sr;
  aWaistMH(1,3) = m_WaistPositionAbsolute(1);

  aWaistMH(2,0) = -sp;
  aWaistMH(2,1) = cp*sr;
  aWaistMH(2,2) = cp*cr;
  aWaistMH(2,3) = m_WaistPositionAbsolute(2);

  aWaistMH(3,3) = 1.0;
  
}

void sotPatternGenerator::FromAbsoluteFootPosToDotHomogeneous(pg::FootAbsolutePosition aFootPosition,
							      sotMatrixHomogeneous &aFootMH,
							      sotMatrixHomogeneous &adotFootMH)
{
  sotMatrixRotation dRot,Twist,Rot;
  adotFootMH.setIdentity();
  FromAbsoluteFootPosToHomogeneous(aFootPosition,aFootMH);

  for(unsigned int i=0;i<3;i++)
    for(unsigned int j=0;j<3;j++)
      Rot(i,j) = aFootMH(i,j);
  
  Twist(0,0)=0.0; Twist(0,1)= -aFootPosition.dtheta; Twist(0,2) = aFootPosition.domega;
  Twist(1,0)= aFootPosition.dtheta; Twist(1,1)= 0.0; Twist(1,2) = aFootPosition.domega2;
  Twist(2,0)= -aFootPosition.domega; Twist(2,1)= -aFootPosition.domega2; Twist(2,2) = 0.0;

  Twist.multiply(Rot,dRot);
  
  for(unsigned int i=0;i<3;i++)
    for(unsigned int j=0;j<3;j++)
      adotFootMH(i,j) = dRot(i,j);

  adotFootMH(0,3) = aFootPosition.dx;
  adotFootMH(1,3) = aFootPosition.dy;
  adotFootMH(2,3) = aFootPosition.dz;
  
}

void sotPatternGenerator::FromAbsoluteFootPosToHomogeneous(pg::FootAbsolutePosition aFootPosition,
							   sotMatrixHomogeneous &aFootMH)
{
  double c,s,co,so;
  c = cos(aFootPosition.theta*M_PI/180.0);
  s = sin(aFootPosition.theta*M_PI/180.0);

  co = cos(aFootPosition.omega*M_PI/180.0);
  so = sin(aFootPosition.omega*M_PI/180.0);
  
  aFootMH(0,0) = c*co;        aFootMH(0,1) = -s;       aFootMH(0,2) = c*so; 
  aFootMH(1,0) = s*co;        aFootMH(1,1) =  c;       aFootMH(1,2) = s*so; 
  aFootMH(2,0) = -so ;        aFootMH(2,1) =  0;       aFootMH(2,2) =   co; 
  aFootMH(3,0) = 0 ;          aFootMH(3,1) =  0;       aFootMH(3,2) =   0; 

  aFootMH(0,3) = aFootPosition.x + m_AnkleSoilDistance*so;
  aFootMH(1,3) = aFootPosition.y;
  aFootMH(2,3) = aFootPosition.z + m_AnkleSoilDistance*co;
  aFootMH(3,3) = 1.0;
}

int &sotPatternGenerator::
OneStepOfControl(int &dummy, int time)
{
  m_LocalTime=time;
  int lSupportFoot; // Local support foot.
  // Default value
  m_JointErrorValuesForWalking.fill(0.0);
  try
    {
      for(unsigned int i=0;i<3;i++)
	m_ZMPRefPos(i) = m_ZMPPrevious[i];
    }
  catch(...) 
    { m_ZMPRefPos(0) = m_ZMPRefPos(1) = m_ZMPRefPos(2) = 0.0;
      m_ZMPRefPos(3) = 1.0;};
  //  m_WaistAttitudeAbsolute.fill(0);
  //  m_WaistPositionAbsolute.fill(0);

  try 
    {
      m_LeftFootPosition = LeftFootCurrentPosSIN(time);
      m_RightFootPosition = RightFootCurrentPosSIN(time);
    }
  catch (...) 
    { };

  sotDEBUG(25) << "LeftFootCurrentPos:  " << m_LeftFootPosition << endl;
  sotDEBUG(25) << "RightFootCurrentPos:  " << m_RightFootPosition << endl;


  sotDEBUGIN(15);
  if (m_PGI!=0)
    {
      // TODO: Calling firstSINTERN may cause an infinite loop
      // since the function initonestepofcontrol calls without
      // control this actual function. 'Hopefully', the function
      // pointer of firstSINTERN has been earlier destroyed
      // by setconstant(0). 
      firstSINTERN(time);
      ml::Vector CurrentState = motorControlJointPositionSIN(time);
      /*! \brief Absolute Position for the left and right feet. */
      pg::FootAbsolutePosition lLeftFootPosition,lRightFootPosition;
      /*! \brief Absolute position of the reference CoM. */
      pg::COMPosition lCOMRefPos;
  
      sotDEBUG(45) << "mc = " << CurrentState << std::endl;

      // TODO: hummmm, nice hard-coding '46' ... :)
      MAL_VECTOR_DIM(CurrentConfiguration,double,46);
      MAL_VECTOR_DIM(CurrentVelocity,double,46);
      MAL_VECTOR_DIM(CurrentAcceleration,double,46);
      /*! \brief Position of the reference ZMP. */
      MAL_VECTOR_DIM(ZMPTarget,double,3);
      MAL_VECTOR_FILL(ZMPTarget,0);

      sotDEBUG(25) << "Before One Step of control " << ZMPTarget[0] << " " 
		   << ZMPTarget[1] << " " 
		   << ZMPTarget[2] << endl;
      

      // Test if the pattern value has some value to provide.
      if (m_PGI->RunOneStepOfTheControlLoop(CurrentConfiguration,
					    CurrentVelocity,
					    CurrentAcceleration,
					    ZMPTarget,
					    lCOMRefPos,
					    lLeftFootPosition,
					    lRightFootPosition))
	{
	  sotDEBUG(25) << "After One Step of control " << endl 
		       << "CurrentState:" << CurrentState << endl
		       << "CurrentConfiguration:" << CurrentConfiguration << endl;

	  m_ZMPRefPos(0) = ZMPTarget[0];
	  m_ZMPRefPos(1) = ZMPTarget[1];
	  m_ZMPRefPos(2) = ZMPTarget[2];
	  m_ZMPRefPos(3) = 1.0;
	  sotDEBUG(25) << "ZMPTarget return by the PG: "<< m_ZMPRefPos <<endl;
	  for(int i=0;i<3;i++)
	    {
	      m_WaistPositionAbsolute(i) = CurrentConfiguration(i);
	      m_WaistAttitudeAbsolute(i) = CurrentConfiguration(i+3);
	    }
	  //std::cout <<"sotPG: " <<m_WaistPositionAbsolute <<std::endl;
	  m_COMRefPos(0) = lCOMRefPos.x[0];
	  m_COMRefPos(1) = lCOMRefPos.y[0];
	  m_COMRefPos(2) = lCOMRefPos.z[0];

	  m_dCOMRefPos(0) = lCOMRefPos.x[1];
	  m_dCOMRefPos(1) = lCOMRefPos.y[1];
	  m_dCOMRefPos(2) = lCOMRefPos.z[1];


  	  // If so compute the difference between the current state and the next one.
	  if (CurrentState.size()==m_JointErrorValuesForWalking.size())
	    {
	      for(unsigned int li=0;li<m_JointErrorValuesForWalking.size();li++)
		m_JointErrorValuesForWalking(li)= (CurrentConfiguration(li+6)- CurrentState(li) )/m_TimeStep;
	      
	    }
	  else
	    {
	      sotDEBUG(25) << "Size not coherent between CurrentState and m_JointErrorValuesForWalking: " 
			   << CurrentState.size()<< " " 
			   << m_JointErrorValuesForWalking.size()<< " " 
			   << endl;
	    }
	  
	  sotDEBUG(25) << "lLeftFootPosition : " 
		       << lLeftFootPosition.x << " " 
		       << lLeftFootPosition.y << " " 
		       << lLeftFootPosition.z << " " 
		       << lLeftFootPosition.theta << endl;
	  sotDEBUG(25) << "lRightFootPosition : " 
		       << lRightFootPosition.x << " " 
		       << lRightFootPosition.y << " " 
		       << lRightFootPosition.z << " " 
		       << lRightFootPosition.theta << endl;
	  /* Fill in the homogeneous matrix using the world reference frame*/
	  FromAbsoluteFootPosToDotHomogeneous(lLeftFootPosition,
					      m_LeftFootPosition,
					      m_dotLeftFootPosition);
	  FromAbsoluteFootPosToDotHomogeneous(lRightFootPosition,
					      m_RightFootPosition,
					      m_dotRightFootPosition);
	  sotDEBUG(25) << "lLeftFootPosition.stepType: " << lLeftFootPosition.stepType
		       << " lRightFootPosition.stepType: " << lRightFootPosition.stepType <<endl;
	  // Find the support foot feet. 
	  if (lLeftFootPosition.stepType==-1)
	    {
	      lSupportFoot=1;
	      m_DoubleSupportPhaseState = 0;
	    }
	  else if (lRightFootPosition.stepType==-1)
	    {
	      lSupportFoot=0;
	      m_DoubleSupportPhaseState = 0;
	    }
	  else /* m_LeftFootPosition.z ==m_RightFootPosition.z 
		  We keep the previous support foot half the time of the double phase..
	       */
	    {
	      lSupportFoot=m_SupportFoot;
	    }
	  
	  /* Update the class related member. */
	  m_SupportFoot = lSupportFoot;

	  if ((m_ReferenceFrame==EGOCENTERED_FRAME) ||
	      (m_ReferenceFrame==LEFT_FOOT_CENTERED_FRAME) || 
	      (m_ReferenceFrame==WAIST_CENTERED_FRAME))
	    {
	      sotDEBUG(25) << "Inside egocentered frame " <<endl;
	      sotMatrixHomogeneous PoseOrigin,iPoseOrigin, WaistPoseAbsolute;

	      getAbsoluteWaistPosAttHomogeneousMatrix(WaistPoseAbsolute);

	      if (m_ReferenceFrame==EGOCENTERED_FRAME)
		{
		  if (m_SupportFoot==1)
		    PoseOrigin = m_LeftFootPosition;
		  else 
		    PoseOrigin = m_RightFootPosition;	    
		}
	      else if (m_ReferenceFrame==LEFT_FOOT_CENTERED_FRAME) 
		{
		  PoseOrigin = m_LeftFootPosition;
		}
	      else if (m_ReferenceFrame==WAIST_CENTERED_FRAME)
		{
		  PoseOrigin = WaistPoseAbsolute;
		}
	      PoseOrigin.inverse(iPoseOrigin);
	      
	      sotDEBUG(25) << "Old ComRef:  " << m_COMRefPos << endl;
	      sotDEBUG(25) << "Old LeftFootRef:  " << m_LeftFootPosition << endl;
	      sotDEBUG(25) << "Old RightFootRef:  " << m_RightFootPosition << endl;
	      sotDEBUG(25) << "Old PoseOrigin:  " << PoseOrigin << endl;


	      ml::Vector lVZMPRefPos(4), lV2ZMPRefPos(4);
	      ml::Vector lVCOMRefPos(4), lV2COMRefPos(4);
	      
	      for(unsigned int li=0;li<3;li++)
		{
		  lVZMPRefPos(li) = m_ZMPRefPos(li);
		  lVCOMRefPos(li) = m_COMRefPos(li);
		}
	      lVZMPRefPos(3) = lVCOMRefPos(3) = 1.0;

	      // We do not touch to ZMP. 
	      lV2ZMPRefPos = iPoseOrigin * (WaistPoseAbsolute * lVZMPRefPos);
	      
	      // Put the CoM reference pos in the Pos Origin reference frame.
	      lV2COMRefPos = iPoseOrigin * lVCOMRefPos;
	  
	      sotMatrixHomogeneous lMLeftFootPosition = m_LeftFootPosition;
	      sotMatrixHomogeneous lMRightFootPosition = m_RightFootPosition;

	      m_LeftFootPosition = iPoseOrigin * lMLeftFootPosition;
	      m_RightFootPosition = iPoseOrigin * lMRightFootPosition;

	      for(unsigned int i=0;i<3;i++)
		{
		  m_ZMPRefPos(i) = lV2ZMPRefPos(i);
		  m_COMRefPos(i) = lV2COMRefPos(i);
		}
	      
	      sotMatrixHomogeneous lWaistPoseAbsoluste = WaistPoseAbsolute;
	      WaistPoseAbsolute = iPoseOrigin * WaistPoseAbsolute;

	      sotMatrixRotation newWaistRot;
	      WaistPoseAbsolute.extract(newWaistRot);
	      sotVectorRollPitchYaw newWaistRPY;
	      newWaistRPY.fromMatrix(newWaistRot);
	      m_WaistAttitude = newWaistRPY;

	      WaistPoseAbsolute.extract(m_WaistPosition);

	      sotDEBUG(25) << "ComRef:  " << m_COMRefPos << endl;
	      sotDEBUG(25) << "iPoseOrigin:  " << iPoseOrigin << endl;
	    }
	  sotDEBUG(25) << "After egocentered frame " << endl;
	  
	  for(unsigned int i=0;i<3;i++)
	    m_ZMPPrevious[i] = m_ZMPRefPos(i);

	  m_dataInProcess = 1;
	}
      else 
	{
	  sotDEBUG(1) << "Error while compute one step of PG." << std::endl;
	  // TODO: SOT_THROW 
	  m_dataInProcess = 0;

	
    
	}
      sotDEBUG(25) << "After computing error " << m_JointErrorValuesForWalking << endl;
    }
  else
  {
    m_COMRefPos = comSIN.access(time);
    m_ZMPRefPos(0) = m_COMRefPos(0);
    m_ZMPRefPos(1) = m_COMRefPos(1);
    m_ZMPRefPos(2) = 0.0;
    m_ZMPRefPos(3) = 1.0;
  }
  sotDEBUG(25) << "LeftFootRef:  " << m_LeftFootPosition << endl;
  sotDEBUG(25) << "RightFootRef:  " << m_RightFootPosition << endl;

  sotDEBUGOUT(15);
  return dummy;
}


/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */

void sotPatternGenerator::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUG(25) << "# In { Cmd " << cmdLine <<endl;

  std::string filename;
  if( cmdLine == "setVrmlDir" )
    {  cmdArgs>>filename; setVrmlDirectory( filename );  }
  else if( cmdLine == "setVrml" )
    {  cmdArgs>>filename; setVrmlMainFile( filename );  }
  else if( cmdLine == "setXmlSpec" )
    {  cmdArgs>>filename; setXmlSpecificityFile( filename );  }
  else if( cmdLine == "setXmlRank" )
    {  cmdArgs>>filename; setXmlRankFile( filename );  }
  else if( cmdLine == "setParamPreview")
    {cmdArgs>>filename; setParamPreviewFile( filename );  }
  else if( cmdLine == "setFiles" )
    {
      cmdArgs>>filename; setParamPreviewFile( filename ); 
      cmdArgs>>filename; setVrmlDirectory( filename ); 
      cmdArgs>>filename; setVrmlMainFile( filename );  
      cmdArgs>>filename; setXmlSpecificityFile( filename );  
      cmdArgs>>filename; setXmlRankFile( filename );  
    }
  else if( cmdLine == "displayFiles" )
    {
      cmdArgs >> ws; bool filespecified = false; 
      if( cmdArgs.good() )
	{
	  filespecified = true; 
	  std::string filetype; cmdArgs >> filetype;
	  sotDEBUG(15) << " Request: " << filetype << std::endl;
	  if( "vrmldir" == filetype ) { os << m_vrmlDirectory << std::endl; }
	  else if( "xmlspecificity" == filetype ) { os << m_xmlSpecificityFile << std::endl; }
	  else if( "xmlrank" == filetype ) { os << m_xmlRankFile << std::endl; }
	  else if( "vrmlmain" == filetype ) { os << m_vrmlMainFile << std::endl; }
	  else filespecified = false;
	}
      if( ! filespecified ) 
	    {
	      os << "  - VRML Directory:\t\t\t" << m_vrmlDirectory <<endl
		 << "  - XML Specificity File:\t\t" << m_xmlSpecificityFile <<endl
		 << "  - XML Rank File:\t\t\t" << m_xmlRankFile <<endl
		 << "  - VRML Main File:\t\t\t" << m_vrmlMainFile <<endl;
	    }
    }
  else if( cmdLine == "buildModel" )
    {
      if(! m_init )buildModel(); else os << "  !! Already parsed." <<endl;
    }
  else if( cmdLine == "initState" )
    {
      InitState();
    }
  else if (cmdLine == "FrameReference" )
    {
      string FrameReference;
      cmdArgs >> FrameReference;
      if (FrameReference=="World")
	{
	  m_ReferenceFrame = WORLD_FRAME;
	}
      else if (FrameReference=="Egocentered")
	{
	  m_ReferenceFrame = EGOCENTERED_FRAME;
	}
      else if (FrameReference=="LeftFootcentered")
	{
	  m_ReferenceFrame = LEFT_FOOT_CENTERED_FRAME;
	}
      else if (FrameReference=="Waistcentered")
	{
	  m_ReferenceFrame = WAIST_CENTERED_FRAME;
	}
      else 
	{
	  if (m_ReferenceFrame == EGOCENTERED_FRAME)
	    os << "Egocentered" <<endl;
	  else if (m_ReferenceFrame == WORLD_FRAME)
	    os << "World" <<endl;
	  else if (m_ReferenceFrame == LEFT_FOOT_CENTERED_FRAME)
	    os << "LeftFootcentered" <<endl;
	  else if (m_ReferenceFrame == WAIST_CENTERED_FRAME)
	    os << "Waistcentered" <<endl;
	  else 
	    os << "Something wrong reference frame ill-defined."<< endl;
	}

    }
  else if (cmdLine == "timestep")
    {
      if (cmdArgs.eof())
	{
	  os << "Timestep: " << m_TimeStep << endl;
	}
      else
	{
	  double ldt;
	  cmdArgs >> ldt;
	  if (ldt<0.0)
	    {
	      os << "\tNot a valid value for timestep.\n\tIt should be positive." <<endl;
	    }
	}
    }
  else if( cmdLine == "help" )
    {
      os << "PatternGenerator:"<<endl
	 << "  - setVrmlDir - setVrml - setXmlSpec - setXmlRank - setParamPreview <file>" <<endl
	 << "\t\t\t\t:set the config files" <<endl
	 << "  - setFiles <%1> ... <%5>\t:set files in the order cited above" <<endl
	 << "  - displayFiles\t\t\t:display the 5 config files" <<endl
	 << "  - buildModel\t\t\t:parse the files set unsing the set{Xml|Vrml} \
                  commands and create internal models." << endl
	 << "  - parsecmd\t\t\t: Command send directly to the Humanoid Walking Pattern Generator framework" << endl
	 << "  - timestep\t\t\t: without arguments display the internal time step, " << endl
	 << "     with a double dt set timestep to dt."
	 << "  - FrameReference\t\t\t: Change the reference frame used to compute the " << endl
         << "                  \t\t\t  features reference trajectories. Possible values are:"  << endl
	 << "                  \t\t\t  World, LeftFootcentered, Waistcentered, Egocentered" 
 	 << endl;

      sotEntity::commandLine(cmdLine,cmdArgs,os);

    }
  else if( cmdLine == "parsecmd")
    {
      if (m_PGI!=NULL)
	m_PGI->ParseCmd(cmdArgs);
	
    }
  else if( cmdLine == "addStep" )
    { 
      double x,y,th; 
      cmdArgs >> std::ws >> x  >> std::ws >> y  >> std::ws; 
      if( cmdArgs.good() ) { cmdArgs  >> th;  m_PGI->AddStepInStack(x,y,th); } 
      else { os << "!! Error while introducing step " << x  << " " << y << " " << th << std::endl; }
    }
  else if( cmdLine == "ChangeNextStep" )
    { 
      sotDEBUG(15) << "ChangeNextStep :begin"<<std::endl;
      PatternGeneratorJRL::FootAbsolutePosition aFAP;
      double NextStepTime,NextNextStepTime;
      cmdArgs >> std::ws >> NextStepTime >> std::ws >> aFAP.x  >> std::ws >> aFAP.y  >> std::ws; 
      if( cmdArgs.good() ) 
	{ 
	  cmdArgs  >> aFAP.theta;  m_PGI->ChangeOnLineStep(NextStepTime,aFAP,NextNextStepTime); 
	  os << NextNextStepTime << std::endl; ;
	} 
      else { sotDEBUG(15) << "!! Error while introducing step " << aFAP.x  
			  << " " << aFAP.y << " " << aFAP.theta << std::endl; }
      sotDEBUG(15) << "ChangeNextStep : end"<<std::endl;
    }
  else { 
    sotEntity::commandLine( cmdLine,cmdArgs,os); 
  }

  sotDEBUGOUT(15) ;

}

ml::Vector & sotPatternGenerator::getjointWalkingErrorPosition(ml::Vector &res,int time)
{
  sotDEBUGIN(5);

  OneStepOfControlS(time);

  res=m_JointErrorValuesForWalking;

  sotDEBUGOUT(5);
  
  return res;
}

unsigned int & sotPatternGenerator::
getSupportFoot(unsigned int &res, int time)
{
  res = m_SupportFoot;
  return res;
}

sotVectorRollPitchYaw & sotPatternGenerator::
getWaistAttitude( sotVectorRollPitchYaw&res, int time)
{
  sotDEBUGIN(5);
  OneStepOfControlS(time);
  for( unsigned int i=0;i<3;++i ) { res(i) = m_WaistAttitude(i); }
  sotDEBUG(5) << "WaistAttitude: " << m_WaistAttitude << endl;
  sotDEBUGOUT(5);
  return res;
}

sotVectorRollPitchYaw & sotPatternGenerator::
getWaistAttitudeAbsolute(sotVectorRollPitchYaw &res, int time)
{
  sotDEBUGIN(5);
  OneStepOfControlS(time);
  sotDEBUG(15) << "I survived one step of control" << std::endl;
  for( unsigned int i=0;i<3;++i ) { res(i) = m_WaistAttitudeAbsolute(i); }
  sotDEBUG(5) << "WaistAttitude: " << m_WaistAttitudeAbsolute << endl;
  sotDEBUGOUT(5);
  return res;
}

ml::Vector & sotPatternGenerator::
getWaistPosition(ml::Vector &res, int time)
{
  sotDEBUGIN(5);
  OneStepOfControlS(time);
  res = m_WaistPosition;
  sotDEBUG(5) << "WaistPosition: " << m_WaistPosition << endl;
  sotDEBUGOUT(5);
  return res;
}
ml::Vector & sotPatternGenerator::
getWaistPositionAbsolute(ml::Vector &res, int time)
{
  sotDEBUGIN(5);
  OneStepOfControlS(time);
  res = m_WaistPositionAbsolute; 
  /* ARGH ! ->  res(2) =0*/ 
  sotDEBUG(5) << "WaistPosition: " << m_WaistPositionAbsolute << endl;
  sotDEBUGOUT(5);
  return res;
}

unsigned & sotPatternGenerator::
getDataInProcess(unsigned &res, int time)
{
  sotDEBUGIN(5);
  OneStepOfControlS(time);
  res = m_dataInProcess;
  sotDEBUG(5) << "DataInProcess: " << m_dataInProcess << endl;
  sotDEBUGOUT(5);
  return res;
}
