/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotWhichFootUpper.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTWHICHFOOTUPPER_H__
#define __SOT_SOTWHICHFOOTUPPER_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotMatrixRotation.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotWhichFootUpper_EXPORTS) 
#    define SOTWHICHFOOTUPPER_EXPORT __declspec(dllexport)
#  else  
#    define SOTWHICHFOOTUPPER_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTWHICHFOOTUPPER_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTWHICHFOOTUPPER_EXPORT sotWhichFootUpper
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;

 protected:

  static const unsigned int INDEX_LEFT_FOOT_DEFAULT;
  static const unsigned int INDEX_RIGHT_FOOT_DEFAULT;
  unsigned int indexLeftFoot, indexRightFoot;

  static const double TRIGGER_THRESHOLD_DEFAULT;
  double triggerThreshold;

  unsigned int lastFoot;
  
 public: /* --- CONSTRUCTION --- */

  sotWhichFootUpper( const std::string& name );
  virtual ~sotWhichFootUpper( void );

 public: /* --- SIGNAL --- */

  sotSignalPtr<sotMatrixRotation,int> waistRsensorSIN; 
  sotSignalPtr<sotMatrixRotation,int> worldRsensorSIN; 
  sotSignalPtr<sotMatrixHomogeneous,int> waistMlfootSIN; 
  sotSignalPtr<sotMatrixHomogeneous,int> waistMrfootSIN; 
  
  sotSignalTimeDependant<sotMatrixHomogeneous,int> worldMlfootSOUT;
  sotSignalTimeDependant<sotMatrixHomogeneous,int> worldMrfootSOUT;
  sotSignalTimeDependant<unsigned int,int> whichFootSOUT;

  sotSignalPtr<sotMatrixHomogeneous,int> waistMsensorSIN; 
  sotSignalTimeDependant<sotMatrixRotation,int> waistRsensorSOUT; 

 public: /* --- FUNCTIONS --- */

  static sotMatrixHomogeneous &
    computeFootPosition( const sotMatrixHomogeneous& waistMfoot,
			 const sotMatrixRotation& waistRsensor,			 
			 const sotMatrixRotation& worldRsensor,			 
			 sotMatrixHomogeneous& res );

  unsigned int & whichFoot( const sotMatrixHomogeneous& waistMlfoot,
			    const sotMatrixHomogeneous& waistMrfoot,
			    unsigned int& res );

  
 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};





#endif // #ifndef __SOT_SOTWHICHFOOTUPPER_H__

