/* SOT */
#include <sot/sotControlPD.h>

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotControlPD,"ControlPD");

const double sotControlPD::
TIME_STEP_DEFAULT = .001;

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */


#define __SOT_ControlPD_INIT \

sotControlPD::
sotControlPD( const std::string & name )
 :sotEntity(name)				
 ,TimeStep(0)
 ,KpSIN(NULL,"sotControlPD("+name+")::input(vector)::Kp")
 ,KdSIN(NULL,"sotControlPD("+name+")::input(vector)::Kd")
 ,positionSIN(NULL,"sotControlPD("+name+")::input(vector)::position")
 ,desiredpositionSIN(NULL,"sotControlPD("+name+")::input(vector)::desiredposition")
 ,velocitySIN(NULL,"sotControlPD("+name+")::input(vector)::velocity")
 ,desiredvelocitySIN(NULL,"sotControlPD("+name+")::input(vector)::desiredvelocity")
 ,controlSOUT( boost::bind(&sotControlPD::computeControl,this,_1,_2),
	       KpSIN << KdSIN << positionSIN << desiredpositionSIN
	       << velocitySIN << desiredvelocitySIN,
	      "sotControlPD("+name+")::output(vector)::control" ) 
{
  init(TimeStep);
  sotEntity::signalRegistration( KpSIN << KdSIN << positionSIN << desiredpositionSIN << velocitySIN << desiredvelocitySIN << controlSOUT );
}

void sotControlPD::
init(const double& Stept)
{
  TimeStep = Stept;

  return;
}

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

void sotControlPD::
display( std::ostream& os ) const
{
  os << "ControlPD "<<getName();
  try{
    os <<"control = "<<controlSOUT; 
  }
  catch (sotExceptionSignal e) {}
  os <<" ("<<TimeStep<<") ";
}

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

double& sotControlPD::setsize(int dimension)

{
	_dimension = dimension;
        return _dimension;
}

ml::Vector& sotControlPD::
computeControl( ml::Vector &tau, int t ) 
{
  sotDEBUGIN(15);		  
  const ml::Vector& Kp = KpSIN(t);
  const ml::Vector& Kd = KdSIN(t);		  
  const ml::Vector& position = positionSIN(t);
  const ml::Vector& desiredposition = desiredpositionSIN(t);
  const ml::Vector& velocity = velocitySIN(t);		  
  const ml::Vector& desiredvelocity = desiredvelocitySIN(t);
		  
  unsigned size = Kp.size();		
  tau.resize(size);
  for(unsigned i = 0u; i < size; ++i)
    {
      tau(i) = desiredposition(i);
      tau(i) -= position(i);
      tau(i) *= Kp(i);
     // std::cout << " position " << position << std::endl;
     // std::cout << " tau1 " << tau << std::endl;
    }
  
  size = Kd.size();
  double de = 0.0;
  for(unsigned i = 0u; i < size; ++i)
    {
      de = desiredvelocity(i);
      de -= velocity(i);
      de *= Kd(i);
      tau(i) += de;
    }
  
  sotDEBUGOUT(15);
 // std::cout << " tau " << tau << std::endl;
 // std::cout << "velocity " << velocity << std::endl;
  return tau;

}



