#ifndef __SOT_Control_PD_HH__
#define __SOT_Control_PD_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotEntity.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotControlPD_EXPORTS)
#    define SOTControlPD_EXPORT __declspec(dllexport)
#  else  
#    define SOTControlPD_EXPORT  __declspec(dllimport)
#  endif 
#else
#  define SOTControlPD_EXPORT 
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTControlPD_EXPORT sotControlPD
: public sotEntity
{

 public: /* --- CONSTRUCTOR ---- */

  sotControlPD( const std::string & name );

 public: /* --- INIT --- */

  void init( const double& step);

 public: /* --- CONSTANTS --- */

  /* Default values. */
  static const double TIME_STEP_DEFAULT;   // = 0.001

 public: /* --- ENTITY INHERITANCE --- */
  static const std::string CLASS_NAME;
  virtual void display( std::ostream& os ) const; 
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }


 protected: 
  
  /* Parameters of the torque-control function: 
   * tau = kp * (qd-q) + kd* (dqd-dq) */
  double TimeStep;
  double _dimension;

 public:  /* --- SIGNALS --- */

  sotSignalPtr<ml::Vector,int> KpSIN;
  sotSignalPtr<ml::Vector,int> KdSIN;
  sotSignalPtr<ml::Vector,int> positionSIN;
  sotSignalPtr<ml::Vector,int> desiredpositionSIN;
  sotSignalPtr<ml::Vector,int> velocitySIN;
  sotSignalPtr<ml::Vector,int> desiredvelocitySIN;
  sotSignalTimeDependant<ml::Vector,int> controlSOUT;

 protected:

  double& setsize(int dimension);
  ml::Vector& computeControl( ml::Vector& tau,int t );

};





#endif // #ifndef __SOT_Control_PD_HH__
