/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2008
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      CORBAReference.h (from DecisionByHFSM)
 * Project:   SOT
 * Author:    Olivier Stasse
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 * Provide basic connection to a CORBA/ORB.
 * (omniorb at least)
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <iostream>
#include <sot/CORBAReference.h>

#define ODEBUG2(x)
#define ODEBUG3(x) cerr << "CORBAReference:" << x << endl

#if 0
#define ODEBUG(x) cerr << "CORBAReference:" <<  x << endl
#else
#define ODEBUG(x)
#endif

using namespace std;

CORBAReference::CORBAReference()
{
  m_HostName = "";
  m_PortNumber = 5005;  
}

CORBAReference::~CORBAReference()
{
}

bool CORBAReference::InitORB()
{
  int largc = 3;
  char largv[3][1024] = { "AFPTWP","-ORBInitRef","NameService=corbaloc:iiop:localhost:5005/NameService"};
  char *lpargv[3];

  if (m_HostName.length()>0) 
    {
      sprintf(largv[2],"NameService=corbaloc:iiop:%s:%d/NameService",
	      (char *)m_HostName.c_str(), m_PortNumber);
      
      lpargv[0] = largv[0];
      lpargv[1] = largv[1];
      lpargv[2] = largv[2];


      ODEBUG3("Flag 0 " << largc << " " << lpargv[0] << " " << lpargv[1] << " " << lpargv[2]);
      m_orb = CORBA::ORB_init(largc, lpargv);
    }
  else
    {
      cout << "Using OMNIORB_CONFIG" << endl;
      int largc=0;
      char *lpargv[0]= {};
      m_orb = CORBA::ORB_init(largc, lpargv);
    }

  if (CORBA::is_nil(m_orb))
    return false;

  return true;

}

void CORBAReference::InitRootContext()
{
  
  try 
    {
      ODEBUG("Try to find the name context." );
      // Obtain a reference to the root context of the Name service:

	    
      CORBA::Object_var obj;
      obj = m_orb->resolve_initial_references("NameService");
      
      // Narrow the reference returned.
      m_rootContext = CosNaming::NamingContext::_narrow(obj);
      
      
      if( CORBA::is_nil(m_rootContext) ) 
	{
	  cerr << "Failed to narrow the root naming context." << endl;
	}
    }

  catch(CORBA::ORB::InvalidName& ex) 
    {
      // This should not happen!
      cerr << "Name Service required is said to be invalid." << endl;
    }

  catch(...)
    {
      cout << "Unknown Error during resolving the name service" << endl;
    }

}


CORBA::Object_ptr CORBAReference::getObjectReference(string ServerID, string ServerKind)
{
  try {
    // Create a name object, containing the name test/context:
    CosNaming::Name name;
    name.length(1);
    
    name[0].id   = ServerID.c_str();
    name[0].kind = ServerKind.c_str();
    
    /*  name[1].id = (const char*) "Echo";
	name[1].kind = (const char*) "Object";*/
    
    // Note on kind: The kind field is used to indicate the type
    // of the object. This is to avoid conventions such as that used
    // by files (name.type -- e.g. test.ps = postscript etc.)
    ODEBUG("Try to resolve the name.");
    
    
    // Resolve the name to an object reference.
    return m_rootContext->resolve(name);
  }
  catch(CosNaming::NamingContext::NotFound& ex) {
    // This exception is thrown if any of the components of the
    // path [contexts or the object] aren't found:
    cerr << "Context not found." << endl;
  }
  catch(CORBA::COMM_FAILURE& ex) {
    cerr << "Caught system exception COMM_FAILURE -- unable to contact the "
	 << "naming service." << endl;
  }
  catch(CORBA::BAD_PARAM)
    {
      cout << "Bad Param Try with name component " << endl;
    }      
  
  catch(CORBA::SystemException&) 
    {
      cerr << "Caught a CORBA::SystemException while using the naming service."
	   << endl;
    }

  cout << "Here...failed to find the Server " << ServerID << endl;
  return CORBA::Object::_nil();
}


bool CORBAReference::SetNameServiceHostname(string &TheHostName)
{
  m_HostName = TheHostName;
  return true;
}

bool CORBAReference::SetPortNumber(unsigned short &PortNumber)
{
  m_PortNumber = PortNumber;
  return true;
}

const string & CORBAReference::GetNameServiceHostname() const
{
  return m_HostName;
}

const unsigned short & CORBAReference::GetPortNumber() const
{
  return m_PortNumber;
}

const CosNaming::NamingContext_var & CORBAReference::getRootContext() const
{
  return m_rootContext;
}

const CORBA::ORB_var & CORBAReference::getORB() const
{
  return m_orb;
}
