/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotServerAbstract.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#define __SOT_SERVER_ABSTRACT_CC__

#include <sot/sotConfig.h>


#include <omniORB4/CORBA.h>

#if HAVE_OPENHRP == 1 
#if HAVE_OPENHRP_V2 == 1
#include <plugin.h>
#endif /* HAVE_OPENHRP_V2 */
#endif /* HAVE_OPENHRP */

/* SOT */
#include <sot/sotServerAbstract.h>
#include <sot/sotDebug.h>
#include <sot/sotExceptionTools.h>




/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

sotServerAbstract::
sotServerAbstract(const std::string &name)
:sotEntity(name)
{
  // Specify a name and a port if needed
  // by uncommenting the following block:
  
  // string NameServiceHostname("hrp2010c");
  // anORBRef->SetNameServiceHostname(NameServiceHostname);
  // unsigned short NameServicePortNumber=5005;
  // anORBRef->SetPortNumber(NameServicePortNumber);
  
  // Otherwise simply make sure that OMNIORB_CONFIG is set to 
  // the right path.

  m_ORBRef.InitORB();
  m_ORBRef.InitRootContext();    
  initialBinding();
}

void sotServerAbstract::
initialBinding()
{
  /*
  PortableServer::POA_var poa;
  poa = PortableServer::POA::_narrow(m_ORBRef.getORB());
  m_sotAbstractServerID = poa->activate_object(this);
  _remove_ref();
  */
  
}

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
#if HAVE_OPENHRP
#if HAVE_OPENHRP_V2
void sotServerAbstract::
bindCorbaServer( plugin_manager* OpenHRP_manager )
{
  sotDEBUGIN(15);

  if(! OpenHRP_manager ) 
    {       SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Manager reference is null. " );     }

  CosNaming::NamingContext_var cxt = OpenHRP_manager->cxt;
  CosNaming::Name corbaCosName;

  std::cout << "bindCorbaServer - getName:"<< getName() <<endl;

  corbaCosName.length(1);
  corbaCosName[0].id = CORBA::string_dup(getName().c_str());
  corbaCosName[0].kind = CORBA::string_dup("");

  try
    {  cxt->rebind(corbaCosName,this->_this() ); }
  catch( const CosNaming::NamingContext::AlreadyBound & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Already bound." ); }
  catch( const CosNaming::NamingContext::NotFound & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Not found." ); }
  catch( const CosNaming::NamingContext::InvalidName & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Invalid name." ); }
  catch( const CosNaming::NamingContext::CannotProceed & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Cannot Proceed." ); }
  catch( ... )
    {  SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				    "Unknwon error." ); }

  sotDEBUGOUT(15);
}
#endif /* HAVE_OPENHRP_V2 */
#endif /* HAVE_OPENHRP */

void sotServerAbstract::
rebind(  CosNaming::Name &corbaCosName)
{
  //  cxt->rebind(corbaCosName, this->_this());
}
void sotServerAbstract::
bindCorbaServer( )
{
  sotDEBUGIN(15);
  
  CosNaming::NamingContext_var cxt = m_ORBRef.getRootContext();
  CosNaming::Name corbaCosName;
  corbaCosName.length(1);
  corbaCosName[0].id = CORBA::string_dup(getName().c_str());
  corbaCosName[0].kind = CORBA::string_dup("");

  try
    { rebind(corbaCosName);  }
  catch( const CosNaming::NamingContext::AlreadyBound & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Already bound." ); }
  catch( const CosNaming::NamingContext::NotFound & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Not found." ); }
  catch( const CosNaming::NamingContext::InvalidName & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Invalid name." ); }
  catch( const CosNaming::NamingContext::CannotProceed & )
    { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				   "Cannot Proceed." ); }
  catch( ... )
    {  SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
				    "Unknwon error." ); }

  sotDEBUGOUT(15);
}



/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <sot/sotPool.h>

void sotServerAbstract::
commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,std::ostream& os )
{
  if( cmdLine == "help" )
    {
      os << "ServerAbstract:"<<std::endl
	 << "  - register \t\t\t:register into the OpenHRP entity nameservice." << std::endl;

      sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
  else if( cmdLine == "register" )
    {
#if HAVE_OPENHRP & HAVE_OPENHRP_V2
      plugin * openhrp = dynamic_cast<plugin*>( &sotPool.getEntity( "OpenHRP" ) );
      if(! openhrp ) { SOT_THROW sotExceptionTools( sotExceptionTools::CORBA,
						    "Cannot find OpenHRP." ); }
      bindCorbaServer( openhrp->manager );
#else
      bindCorbaServer();
#endif
    }
  else
    {      sotEntity::commandLine( cmdLine,cmdArgs,os );    }
}




