/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotServerCommand.cpp
 * Project:   SOT
 * Author:    Torea Foissotte and Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *    Generic server to launch command and get signal data from the
 *  the interpretor shell.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* --- CORBA --- */
#include <omniORB4/CORBA.h>

/* --- SOT --- */
#include <sot/sotServerCommand.h>
#include <sot/sotDebug.h>
#include <sot/sotExceptionTools.h>

#include <sot/sotInterpretor.h>
#include <sot/sotDebug.h>

/* --- SOT FACTORY ----------------------------------------------------- */
#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN( sotServerCommand, "ServerCommand" );

/* --- CONSTRUCTION  --------------------------------------------------- */
sotServerCommand::
sotServerCommand( const std::string& name )
  :sotServerAbstract( name )
{
  sotDEBUGIN(15);

  sotDEBUGOUT(15);
}

sotServerCommand::
~sotServerCommand( void ) 
{
  sotDEBUGIN(15);

  sotDEBUGOUT(15);
}
 

/* -------------------------------------------------------------------------- */
/* --- IMPLEMENTATION CORBA ------------------------------------------------- */
/* -------------------------------------------------------------------------- */

using namespace std;

void sotServerCommand::
initialBinding()
{
  PortableServer::POA_var poa;
  poa = PortableServer::POA::_narrow(getORBRef().getORB());
  poa->activate_object(this);
  _remove_ref();
  bindCorbaServer();
}	

void sotServerCommand::
rebind(CosNaming::Name & corbaCosName)
{
  getORBRef().getRootContext()->rebind(corbaCosName, this->_this());
}

void sotServerCommand::
run( const SOT_Server_Command::CharSeq& cmdCorba ) throw(CORBA::SystemException)
{
  sotDEBUGIN(15);
  sotDEBUG(15) <<  "received "<<cmdCorba.length()<<" char"<< endl;

  string cmd;
  for( unsigned int i=0;i<cmdCorba.length();++i )
    { cmd += cmdCorba[i]; }
  sotDEBUG(15) <<  "Run: <" << cmd <<">"<< endl;

  istringstream dummy; ostringstream dummydummy;
  try
    {sotShell.cmd( cmd,dummy,dummydummy );}
  catch( sotExceptionAbstract& e ) 
    { sotDEBUG(1) << "Catch exeption: " << e << endl; }
  catch( ... ) 
    { sotDEBUG(1) << "Catch unknown exception." << endl ; }

  sotDEBUGOUT(15);
}

void sotServerCommand::
getStringStream( SOT_Server_Command::StringStreamer_out StringStreamer )
{
}


void sotServerCommand::
runAndRead( const SOT_Server_Command::CharSeq& cmdCorba,
	    SOT_Server_Command::StringStreamer_out os )
  throw(CORBA::SystemException)
{
  sotDEBUGIN(15);
  sotDEBUG(15) << "received "<<cmdCorba.length()<<" char"<< endl;

  string line,cmd;
  for( unsigned int i=0;i<cmdCorba.length();++i )
    {
      line += cmdCorba[i]; 
      sotDEBUG(45) << "C[" << i << "] = '" << cmdCorba[i] <<"'"<<endl;
    }

  sotDEBUG(15) <<  "Received: <" << line << ">" << endl;

  istringstream iss( line ); 
  iss >> cmd;  
  sotDEBUG(15) <<  "Run: ["<<cmd.length()<<"] <" << cmd << ">" << endl;

  stringstream ioss;
  try
    {sotShell.cmd( cmd,iss,ioss );}
  catch( sotExceptionAbstract& e ) 
    { 
      sotDEBUG(1) << "Catch exeption: " << e << endl; 
      ioss << "Catch exeption: " << e << endl; 
    }
  catch( ... ) 
    { 
      sotDEBUG(1) << "Catch unknown exception." << endl ; 
      ioss << "Catch unknown exception." << endl ; 
    }

  sotDEBUG(15) << "Copy the result." << endl;
  sotDEBUGF(15,"Data = %p.",os._data);
  SOT_Server_Command::StringStreamer_var resCorba( new SOT_Server_Command::StringStreamer );
  string res = ioss.str();
  resCorba->length( res.length() );
  for( unsigned int i=0;i<res.length();++i )
    { resCorba[i] = res[i]; }
  os=resCorba._retn();

  /* No memory to free in resCorba, free is done by the caller. */
  sotDEBUG(15) <<  "Return: " << res <<endl;

  sotDEBUGOUT(15);
}


void sotServerCommand::
readVector( const SOT_Server_Command::CharSeq& signalNameCorba,
	    SOT_Server_Command::DoubleSeq_out value ) 
  throw(CORBA::SystemException)
{
  sotDEBUGIN(15);

  std::string buffer;
  for( unsigned int i=0;i<signalNameCorba.length();++i ) 
    { buffer += signalNameCorba[i]; }
  std::istringstream sigName(buffer);
  
  try
    {
      sotSignalAbstract<int>& sigA = sotPool.getSignal( sigName );
      sotSignal<ml::Vector,int>& signal 
	= dynamic_cast< sotSignal<ml::Vector,int>& >( sigA ); 

      const ml::Vector& data = signal.accessCopy();
      SOT_Server_Command::DoubleSeq_var resCorba( new SOT_Server_Command::DoubleSeq );
      resCorba->length( data.size() );
      for( unsigned int i=0;i<data.size();++i ){ resCorba[i]=data(i); }
      value=resCorba._retn();
    }
  catch( sotExceptionFactory& e ) 
    { 
      sotDEBUG(1) << "Catch exception " << e << std::endl; 
      sotDEBUG(1) << "Signal "<<buffer<<" does not exist." << endl; 
    }
  catch( std::bad_cast& e )  
    { sotDEBUG(1) << "STD::bad_cast: signal "<<buffer
		  <<" is not of type Vector." << endl; }
  catch( ... )  
    { sotDEBUG(1) << "Signal "<<buffer
		  <<" is not define of not of type Vector." << endl; }

  sotDEBUGOUT(15);
}

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

long int sotServerCommand::
createOutputVectorSignal( const SOT_Server_Command::CharSeq& signalNameCorba )
  throw(CORBA::SystemException)
{
  sotDEBUGIN(15);
  
  std::string signalName;
  for( unsigned int i=0;i<signalNameCorba.length();++i ) 
    { signalName += signalNameCorba[i]; }

  // Does the signal exist?
  typedef sotSignalTimeDependant< ml::Vector,int > sig_t;
  std::vector<sig_t*>::iterator its =
    std::find_if(vectorSOUT.begin(), vectorSOUT.end(),
		 boost::bind(std::equal_to<std::string>(),
			     boost::bind(&sig_t::getName, _1),
			     boost::ref(signalName))
		 );

  // If the signal already exists, return it.
  if( its!=vectorSOUT.end() ) {
    long i = std::distance(vectorSOUT.begin(), its);
    return i;
  }

  sotDEBUG(15) << "Create output signal: <" << signalName <<">."<< std::endl;
  sotSignalTimeDependant< ml::Vector,int > * newSignal
    = new sotSignalTimeDependant< ml::Vector,int > 
    ( signalName );
  newSignal->setDependancyType( sotTimeDependancy<int>::BOOL_DEPENDANT );
  
  vectorSOUT.push_back( newSignal );
  signalRegistration( *newSignal );

  sotDEBUGOUT(15);
  /* Return the rank of the new signal in the std::vector. */
  return( vectorSOUT.size()-1 );
}

long int sotServerCommand::
createInputVectorSignal( const SOT_Server_Command::CharSeq& signalNameCorba )
  throw(CORBA::SystemException)
{
  sotDEBUGIN(15);
  
  std::string signalName;
  for( unsigned int i=0;i<signalNameCorba.length();++i ) 
    { signalName += signalNameCorba[i]; }

  // Does the signal exist?
  typedef sotSignalPtr< ml::Vector,int > sig_t;
  std::vector<sig_t*>::iterator its =
    std::find_if(vectorSIN.begin(), vectorSIN.end(),
		 boost::bind(std::equal_to<std::string>(),
			     boost::bind(&sig_t::getName, _1),
			     boost::ref(signalName))
		 );

  // If the signal already exists, return it.
  if( its!=vectorSIN.end() ) {
    long i = std::distance(vectorSIN.begin(), its);
    return i;
  }

  sotDEBUG(15) << "Create input signal: <" << signalName <<">."<< std::endl;
  sotSignalPtr< ml::Vector,int > * newSignal
    = new sotSignalPtr< ml::Vector,int > 
    ( NULL,signalName );
  
  vectorSIN.push_back( newSignal );
  signalRegistration( *newSignal );

  sotDEBUGOUT(15);
  /* Return the rank of the new signal in the std::vector. */
  return( vectorSIN.size()-1 );
}

void sotServerCommand::
readInputVectorSignal( long int signalRankCorba,
		       SOT_Server_Command::DoubleSeq_out value ) 
  throw(CORBA::SystemException)
{ 
  sotDEBUGIN(15);
  
  int signalRank = signalRankCorba ;
  sotSignalPtr<ml::Vector,int>& signal = *vectorSIN[signalRank];
  
  const ml::Vector& data = signal.accessCopy();
  SOT_Server_Command::DoubleSeq_var resCorba( new SOT_Server_Command::DoubleSeq );
  resCorba->length( data.size() );
  for( unsigned int i=0;i<data.size();++i ){ resCorba[i]=data(i); }
  value=resCorba._retn();
  
  sotDEBUGOUT(15);
}


void sotServerCommand::
writeOutputVectorSignal( long int signalRankCorba,
			 const SOT_Server_Command::DoubleSeq& value ) 
  throw(CORBA::SystemException)
{ 
  sotDEBUGIN(15);
  
  int signalRank = signalRankCorba ;
  sotSignal<ml::Vector,int>& signal = *vectorSOUT[signalRank];
  
  ml::Vector data( value.length() );
  for( unsigned int i=0;i<value.length();++i ){ data(i)=value[i]; }
  signal = data;
  
  sotDEBUGOUT(15);
}

/* -------------------------------------------------------------------------- */
/* --- SHELL SOT ------------------------------------------------------------ */
/* -------------------------------------------------------------------------- */


void sotServerCommand::
commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUGIN(15);
  if( cmdLine == "help" )
    {
      os << "Class ServerCommand:\n";
      sotServerAbstract::commandLine( cmdLine,cmdArgs,os );
    }
  else
    {
      sotServerAbstract::commandLine( cmdLine,cmdArgs,os );
    }
  sotDEBUGOUT(15);
}
