/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotServerCommand.h
 * Project:   SOT
 * Author:    Torea FOISSOTTE and Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * inspired by sotServerTeleop
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#ifndef __SOT_SERVER_COMMAND_H__
#define __SOT_SERVER_COMMAND_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* --- SOT --- */
#include <sot/sotServerAbstract.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* --- CORBA IDL --- */ 
#include <ServerCommand.h>

/* --- BOOST --- */
#ifdef HAVE_LIBBOOST_THREAD
#include <boost/thread.hpp>
#endif

/* --- MAAL --- */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


class sotServerCommand
:public sotServerAbstract
,virtual public POA_SOT_Server_Command
  ,virtual public PortableServer::RefCountServantBase 
{
 protected:
  std::vector< sotSignalTimeDependant< ml::Vector,int >* > vectorSOUT;
  std::vector< sotSignalPtr< ml::Vector,int >* > vectorSIN;

 public: /* --- CONSTRUCTION --- */
  sotServerCommand( const std::string& name );
  virtual ~sotServerCommand( void ); 

 public:  /* --- sot ENTITY --- */
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 public: /* --- CORBA implementation --- */
  
  virtual void initialBinding();
  virtual void rebind(  CosNaming::Name & corbaCosName);

  void run( const SOT_Server_Command::CharSeq& cmd ) 
    throw(CORBA::SystemException);
  void runAndRead( const SOT_Server_Command::CharSeq& cmd,
		   SOT_Server_Command::StringStreamer_out os )
    throw(CORBA::SystemException);

  void getStringStream( SOT_Server_Command::StringStreamer_out StringStreamer ) ;
  void readVector( const SOT_Server_Command::CharSeq& cmd,
		   SOT_Server_Command::DoubleSeq_out value ) 
    throw(CORBA::SystemException);

  long int createOutputVectorSignal( const SOT_Server_Command::CharSeq& signalNameCorba )
    throw(CORBA::SystemException);
  long int createInputVectorSignal( const SOT_Server_Command::CharSeq& signalNameCorba )
    throw(CORBA::SystemException);

  void readInputVectorSignal( long int signalRank,
			      SOT_Server_Command::DoubleSeq_out value ) 
    throw(CORBA::SystemException);
  void writeOutputVectorSignal( long int signalRank,
				const SOT_Server_Command::DoubleSeq& value ) 
    throw(CORBA::SystemException);
};

#endif // #ifndef __SOT_SERVER_COMMAND_H__
