/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotAngleEstimator.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_ANGLE_ESTIMATOR_H__
#define __SOT_ANGLE_ESTIMATOR_H__
/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotAngleEstimator_EXPORTS)
#    define SOTANGLEESTIMATOR_EXPORT __declspec(dllexport)
#  else  
#    define SOTANGLEESTIMATOR_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTANGLEESTIMATOR_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotMatrixRotation.h>

/* STD */
#include <string>


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTANGLEESTIMATOR_EXPORT sotAngleEstimator
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;

 public: /* --- CONSTRUCTION --- */

  sotAngleEstimator( const std::string& name );
  virtual ~sotAngleEstimator( void );

 public: /* --- SIGNAL --- */

  sotSignalPtr<sotMatrixRotation,int> sensorWorldRotationSIN; // estimate(worldRc)
  sotSignalPtr<sotMatrixHomogeneous,int> sensorEmbeddedPositionSIN; // waistRchest
  sotSignalPtr<sotMatrixHomogeneous,int> contactWorldPositionSIN; // estimate(worldRf)
  sotSignalPtr<sotMatrixHomogeneous,int> contactEmbeddedPositionSIN; // waistRleg
  sotSignalTimeDependant<ml::Vector,int> anglesSOUT;  // [ flex1 flex2 yaw_drift ]
  sotSignalTimeDependant<sotMatrixRotation,int> flexibilitySOUT;  // footRleg
  sotSignalTimeDependant<sotMatrixRotation,int> driftSOUT;  // Ryaw = worldRc est(wRc)^-1
  sotSignalTimeDependant<sotMatrixRotation,int> sensorWorldRotationSOUT;  // worldRc
  sotSignalTimeDependant<sotMatrixRotation,int> waistWorldRotationSOUT;  // worldRwaist
  sotSignalTimeDependant<sotMatrixHomogeneous,int> waistWorldPositionSOUT; // worldMwaist
  sotSignalTimeDependant<ml::Vector,int> waistWorldPoseRPYSOUT; // worldMwaist

 public: /* --- FUNCTIONS --- */
  ml::Vector& computeAngles( ml::Vector& res,
			     const int& time );
  sotMatrixRotation& computeFlexibilityFromAngles( sotMatrixRotation& res,
						   const int& time );
  sotMatrixRotation& computeDriftFromAngles( sotMatrixRotation& res,
					     const int& time );
  sotMatrixRotation& computeSensorWorldRotation( sotMatrixRotation& res,
						 const int& time ); 
  sotMatrixRotation& computeWaistWorldRotation( sotMatrixRotation& res,
						const int& time );
  sotMatrixHomogeneous& computeWaistWorldPosition( sotMatrixHomogeneous& res,
						   const int& time );
  ml::Vector& computeWaistWorldPoseRPY( ml::Vector& res,
					const int& time );
  
 public: /* --- PARAMS --- */
  bool fromSensor;

  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};






#endif // #ifndef __SOT_ANGLE_ESTIMATOR_H__

