/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotDynamic.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotDynamic.h>
#include <sot/sotDebug.h>

#include <robotDynamics/jrlHumanoidDynamicRobot.h>
#include <robotDynamics/jrlRobotDynamicsObjectConstructor.h>

#include <sot/sotFactory.h>


SOT_FACTORY_ENTITY_PLUGIN(sotDynamic,"Dynamic");

sotDynamic::
sotDynamic( const std::string & name, bool build ) 
  :sotEntity(name)
  ,aHDR(NULL)
  ,aHDMB(NULL)
  ,inertia(NULL) 
  ,debugInertia(0)

  ,vrmlDirectory()
  ,vrmlMainFile()
  ,xmlSpecificityFile()
  ,xmlRankFile()

  ,init(false)

  ,jointPositionSIN(NULL,"sotDynamic("+name+")::input(vector)::position")
  ,freeFlyerPositionSIN(NULL,"sotDynamic("+name+")::input(vector)::ffposition")
  ,jointVelocitySIN(NULL,"sotDynamic("+name+")::input(vector)::velocity")
  ,freeFlyerVelocitySIN(NULL,"sotDynamic("+name+")::input(vector)::ffvelocity")
  ,jointAccelerationSIN(NULL,"sotDynamic("+name+")::input(vector)::acceleration")
  ,freeFlyerAccelerationSIN(NULL,"sotDynamic("+name+")::input(vector)::ffacceleration")

  ,firstSINTERN( boost::bind(&sotDynamic::initNewtonEuler,this,_1,_2),
		 sotNOSIGNAL,"sotDynamic("+name+")::intern(dummy)::init" ) 
  ,newtonEulerSINTERN( boost::bind(&sotDynamic::computeNewtonEuler,this,_1,_2),
		       firstSINTERN<<jointPositionSIN<<freeFlyerPositionSIN
		       <<jointVelocitySIN<<freeFlyerVelocitySIN
		       <<jointAccelerationSIN<<freeFlyerAccelerationSIN,
		       "sotDynamic("+name+")::intern(dummy)::newtoneuleur" ) 

  ,zmpSOUT( boost::bind(&sotDynamic::computeZmp,this,_1,_2),
	    newtonEulerSINTERN,
	    "sotDynamic("+name+")::output(vector)::zmp" ) 
  ,JcomSOUT( boost::bind(&sotDynamic::computeJcom,this,_1,_2),
	     newtonEulerSINTERN,
	     "sotDynamic("+name+")::output(matrix)::Jcom" ) 
  ,comSOUT( boost::bind(&sotDynamic::computeCom,this,_1,_2),
	    newtonEulerSINTERN,
	    "sotDynamic("+name+")::output(vector)::com" ) 
  ,inertiaSOUT( boost::bind(&sotDynamic::computeInertia,this,_1,_2),
		newtonEulerSINTERN,
		"sotDynamic("+name+")::output(matrix)::inertia" ) 
  ,footHeightSOUT( boost::bind(&sotDynamic::computeFootHeight,this,_1,_2),
		   newtonEulerSINTERN,
		   "sotDynamic("+name+")::output(double)::footHeight" ) 

  ,upperJlSOUT( boost::bind(&sotDynamic::getUpperJointLimits,this,_1,_2),
		sotNOSIGNAL,
		"sotDynamic("+name+")::output(vector)::upperJl" ) 

  ,lowerJlSOUT( boost::bind(&sotDynamic::getLowerJointLimits,this,_1,_2),
		sotNOSIGNAL,
		"sotDynamic("+name+")::output(vector)::lowerJl" ) 

  ,inertiaRotorSOUT( "sotDynamic("+name+")::output(matrix)::inertiaRotor" )
  ,gearRatioSOUT( "sotDynamic("+name+")::output(matrix)::gearRatio" )
  ,inertiaRealSOUT( boost::bind(&sotDynamic::computeInertiaReal,this,_1,_2),
		    inertiaSOUT << gearRatioSOUT << inertiaRotorSOUT,
		    "sotDynamic("+name+")::output(matrix)::inertiaReal" ) 
  ,MomentaSOUT( boost::bind(&sotDynamic::computeMomenta,this,_1,_2),
		newtonEulerSINTERN,
		"sotDynamic("+name+")::output(vector)::momenta" ) 
  ,AngularMomentumSOUT( boost::bind(&sotDynamic::computeAngularMomentum,this,_1,_2),
			newtonEulerSINTERN,
			"sotDynamic("+name+")::output(vector)::angularmomentum" ) 
  ,JAngularMomentumSOUT(boost::bind(&sotDynamic::computeJAngularMomentum,this,_1,_2),
			newtonEulerSINTERN,
			"sotDynamic("+name+")::output(matrix)::Jangularmomentum")
{
  sotDEBUGIN(5);

  if( build ) buildModel();

  firstSINTERN.setDependancyType(sotTimeDependancy<int>::BOOL_DEPENDANT);
  //DEBUG: Why =0? should be function. firstSINTERN.setConstant(0);

  signalRegistration( jointPositionSIN<<freeFlyerPositionSIN
		      <<jointVelocitySIN<<freeFlyerVelocitySIN
		      <<jointAccelerationSIN<<freeFlyerAccelerationSIN);
  signalRegistration( zmpSOUT<<comSOUT<<JcomSOUT<<footHeightSOUT
		      <<upperJlSOUT<<lowerJlSOUT<<inertiaSOUT 
		      <<inertiaRealSOUT << inertiaRotorSOUT << gearRatioSOUT );
  signalRegistration( MomentaSOUT << AngularMomentumSOUT << JAngularMomentumSOUT);

  sotDEBUGOUT(5);
}

void sotDynamic::
buildModel( void )
{
  sotDEBUGIN(5);

  CjrlRobotDynamicsObjectConstructor<
    dynamicsJRLJapan::DynamicMultiBody, 
    dynamicsJRLJapan::HumanoidDynamicMultiBody, 
    dynamicsJRLJapan::JointFreeflyer, 
    dynamicsJRLJapan::JointRotation,
    dynamicsJRLJapan::JointTranslation,
    dynamicsJRLJapan::Body> aRobotDynamicsObjectConstructor;

  aHDR = aRobotDynamicsObjectConstructor.createhumanoidDynamicRobot();
  aHDMB = dynamic_cast<djj::HumanoidDynamicMultiBody *>(aHDR);
  if( NULL == aHDMB )
    {
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::DYNAMIC_JRL,
				     "Error parsing aHDR to aHDMB.",
				     "(DJJ model creation process for object %s).",
				     getName().c_str());
    }
  //aDMB = (djj::DynamicMultiBody *) aHDMB->getDynamicMultiBody();

  sotDEBUGOUT(5);
}


sotDynamic::
~sotDynamic( void )
{
  sotDEBUGIN(5);
  if( 0!=aHDMB )
    {
      delete aHDMB;
      aHDR = 0;
      aHDMB = 0;
      //aDMB = 0;
    }

  for(  std::list< sotSignalAbstract<int>* >::iterator iter = genericSignalRefs.begin();
	iter != genericSignalRefs.end();
	++iter )
    {
      sotSignalAbstract<int>* sigPtr = *iter;
      delete sigPtr;
    }

  sotDEBUGOUT(5);
  return;
}

/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
/* --- CONFIG --------------------------------------------------------------- */
void sotDynamic::
setVrmlDirectory( const std::string& filename )
{
  vrmlDirectory = filename;
}
void sotDynamic::
setVrmlMainFile( const std::string& filename )
{
  vrmlMainFile = filename;
}
void sotDynamic::
setXmlSpecificityFile( const std::string& filename )
{
  xmlSpecificityFile = filename;
}
void sotDynamic::
setXmlRankFile( const std::string& filename )
{
  xmlRankFile = filename;
}
void sotDynamic::
parseConfigFiles( void )
{
  sotDEBUGIN(15);
  try
    {
      sotDEBUG(35) << "Parse the vrml."<<endl;
      aHDMB->parserVRML(vrmlDirectory.c_str(),vrmlMainFile.c_str(),
			xmlRankFile.c_str());
      sotDEBUG(35) << "Parse the specificity."<<endl;
      aHDMB->SetHumanoidSpecificitiesFile(xmlSpecificityFile);
    }
  catch( ... )
    { SOT_THROW sotExceptionDynamic( sotExceptionDynamic::DYNAMIC_JRL,
				     "Error while parsing." ); }
  inertia.init( aHDMB );
  init = true;
  sotDEBUGOUT(15);
}

/* --- SIGNAL ACTIVATION ---------------------------------------------------- */
/* --- SIGNAL ACTIVATION ---------------------------------------------------- */
/* --- SIGNAL ACTIVATION ---------------------------------------------------- */
sotSignalTimeDependant< ml::Matrix,int > & sotDynamic::
createJacobianSignal( const std::string& signame,const unsigned int& bodyRank )
{

  vector<CjrlJoint *> aVec = aHDMB->jointVector();
  if( bodyRank>=aVec.size() )
    {
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_RANK,
				     "Joint rank is too high.",
				     "(rank=%d, while creating J signal).",
				     bodyRank );
    }
  djj::Joint * aJoint = (djj::Joint *)aVec[bodyRank]; 


  sotSignalTimeDependant< ml::Matrix,int > * sig 
    = new sotSignalTimeDependant< ml::Matrix,int >
    ( boost::bind(&sotDynamic::computeGenericJacobian,this,aJoint,_1,_2),
      newtonEulerSINTERN,
      "sotDynamic("+name+")::output(matrix)::"+signame );

  genericSignalRefs.push_back( sig );
  signalRegistration( *sig );
  return *sig;
}

sotSignalTimeDependant< ml::Matrix,int > & sotDynamic::
createEndeffJacobianSignal( const std::string& signame,const unsigned int& bodyRank )
{
  sotDEBUGIN(15);

  vector<CjrlJoint *> aVec = aHDMB->jointVector();
  if( bodyRank>=aVec.size() )
    {
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_RANK,
				     "Joint rank is too high.",
				     "(rank=%d, while creating J signal).",
				     bodyRank );
    }
  djj::Joint * aJoint = (djj::Joint *)aVec[bodyRank]; 


  sotSignalTimeDependant< ml::Matrix,int > * sig 
    = new sotSignalTimeDependant< ml::Matrix,int >
    ( boost::bind(&sotDynamic::computeGenericEndeffJacobian,this,aJoint,_1,_2),
      newtonEulerSINTERN,
      "sotDynamic("+name+")::output(matrix)::"+signame );

  genericSignalRefs.push_back( sig );
  signalRegistration( *sig );

  sotDEBUGOUT(15);
  return *sig;
}

void sotDynamic::
destroyJacobianSignal( const std::string& signame )
{
  bool deletable = false;
  sotSignalTimeDependant< ml::Matrix,int > * sig = & jacobiansSOUT( signame );
  for(  std::list< sotSignalAbstract<int>* >::iterator iter = genericSignalRefs.begin();
	iter != genericSignalRefs.end();
	++iter )
    {
      if( (*iter) == sig ) { genericSignalRefs.erase(iter); deletable = true; break; }
    }

  if(! deletable )
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::CANT_DESTROY_SIGNAL,
				     "Cannot destroy signal",
				     " (while trying to remove generic jac. signal <%s>).",
				     signame.c_str() );
    }

  signalDeregistration( signame );

  delete sig;
}

/* --- POINT --- */
/* --- POINT --- */
/* --- POINT --- */

sotSignalTimeDependant< sotMatrixHomogeneous,int >& sotDynamic::
createPositionSignal( const std::string& signame,const unsigned int& bodyRank )
{
  sotDEBUGIN(15);

  vector<CjrlJoint *> aVec = aHDMB->jointVector();

  if( bodyRank>=aVec.size() )
    {
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_RANK,
				     "Joint rank is too high.",
				     "(rank=%d, while creating position signal).",
				     bodyRank );
    }
  djj::Joint * aJoint = (djj::Joint *)aVec[bodyRank]; 

  sotSignalTimeDependant< sotMatrixHomogeneous,int > * sig 
    = new sotSignalTimeDependant< sotMatrixHomogeneous,int >
    ( boost::bind(&sotDynamic::computeGenericPosition,this,aJoint,_1,_2),
      newtonEulerSINTERN,
      "sotDynamic("+name+")::output(matrixHomo)::"+signame );

  genericSignalRefs.push_back( sig );
  signalRegistration( *sig );

  sotDEBUGOUT(15);
  return *sig;
}


void sotDynamic::
destroyPositionSignal( const std::string& signame )
{
  bool deletable = false;
  sotSignalTimeDependant< sotMatrixHomogeneous,int > * sig = & positionsSOUT( signame );
  for(  std::list< sotSignalAbstract<int>* >::iterator iter = genericSignalRefs.begin();
	iter != genericSignalRefs.end();
	++iter )
    {
      if( (*iter) == sig ) { genericSignalRefs.erase(iter); deletable = true; break; }
    }

  if(! deletable )
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::CANT_DESTROY_SIGNAL,
				     "Cannot destroy signal",
				     " (while trying to remove generic pos. signal <%s>).",
				     signame.c_str() );
    }

  signalDeregistration( signame );

  delete sig;
}

/* --- ACCELERATION --- */
/* --- ACCELERATION --- */
/* --- ACCELERATION --- */

sotSignalTimeDependant< ml::Vector,int >& sotDynamic::
createAccelerationSignal( const std::string& signame,const unsigned int& bodyRank )
{
  sotDEBUGIN(15);

  vector<CjrlJoint *> aVec = aHDMB->jointVector();

  if( bodyRank>=aVec.size() )
    {
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_RANK,
				     "Joint rank is too high.",
				     "(rank=%d, while creating acceleration signal).",
				     bodyRank );
    }
  djj::Joint * aJoint = (djj::Joint *)aVec[bodyRank]; 

  sotSignalTimeDependant< ml::Vector,int > * sig 
    = new sotSignalTimeDependant< ml::Vector,int >
    ( boost::bind(&sotDynamic::computeGenericAcceleration,this,aJoint,_1,_2),
      newtonEulerSINTERN,
      "sotDynamic("+name+")::output(matrixHomo)::"+signame );

  genericSignalRefs.push_back( sig );
  signalRegistration( *sig );

  sotDEBUGOUT(15);
  return *sig;
}


void sotDynamic::
destroyAccelerationSignal( const std::string& signame )
{
  bool deletable = false;
  sotSignalTimeDependant< ml::Vector,int > * sig = & accelerationsSOUT( signame );
  for(  std::list< sotSignalAbstract<int>* >::iterator iter = genericSignalRefs.begin();
	iter != genericSignalRefs.end();
	++iter )
    {
      if( (*iter) == sig ) { genericSignalRefs.erase(iter); deletable = true; break; }
    }

  if(! deletable )
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::CANT_DESTROY_SIGNAL,
				     "Cannot destroy signal",
				     " (while trying to remove generic acc signal <%s>).",
				     signame.c_str() );
    }

  signalDeregistration( signame );

  delete sig;
}
/* --- COMPUTE -------------------------------------------------------------- */
/* --- COMPUTE -------------------------------------------------------------- */
/* --- COMPUTE -------------------------------------------------------------- */

#include <MatrixAbstractLayer/boostspecific.h>

static void MAAL1_V3d_to_MAAL2( const vector3d& source,
				ml::Vector & res )
{ 
  sotDEBUG(5) << source <<endl;
  res(0) = source[0]; 
  res(1) = source[1]; 
  res(2) = source[2]; 
}

ml::Matrix& sotDynamic::
computeGenericJacobian( djj::Joint * aJoint,ml::Matrix& res,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);

  aJoint->computeJacobianJointWrtConfig();
  res.initFromMotherLib(aJoint->jacobianJointWrtConfig());
  sotDEBUGOUT(25);

  return res;
}

ml::Matrix& sotDynamic::
computeGenericEndeffJacobian( djj::Joint * aJoint,ml::Matrix& res,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);

  aJoint->computeJacobianJointWrtConfig();

  ml::Matrix J,V(6,6);
  J.initFromMotherLib(aJoint->jacobianJointWrtConfig());

  /* --- TODO --- */
  sotMatrixHomogeneous M;
  computeGenericPosition(aJoint,M,time);
  //M=M.inverse();

  for( int i=0;i<3;++i )
    for( int j=0;j<3;++j )
      {
	V(i,j)=M(j,i);
	V(i+3,j+3)=M(j,i);
	V(i+3,j)=0.;
	V(i,j+3)=0.;
      }

  sotDEBUG(25) << "0Jn = "<< J;
  sotDEBUG(25) << "V = "<< V;
  V.multiply(J,res);
  sotDEBUGOUT(25);

  return res;
}

sotMatrixHomogeneous& sotDynamic::
computeGenericPosition( djj::Joint * aJoint,sotMatrixHomogeneous& res,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  const matrix4d & m4 = aJoint->currentTransformation();

  res.resize(4,4);
  for( int i=0;i<4;++i )
    for( int j=0;j<4;++j ) 
      res(i,j) = MAL_S4x4_MATRIX_ACCESS_I_J(m4,i,j);

  //  aJoint->computeJacobianJointWrtConfig();
  //res.initFromMotherLib(aJoint->jacobianJointWrtConfig());

  sotDEBUGOUT(25);
  return res;
}

ml::Vector& sotDynamic::
computeGenericAcceleration( djj::Joint* j,ml::Vector& res,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  CjrlRigidAcceleration aRA = j->jointAcceleration();
  vector3d al= aRA.linearAcceleration();
  vector3d ar= aRA.rotationAcceleration(); // TODO: Dont copy, reference.

  res.resize(6);
  for( int i=0;i<3;++i )
    {
      res(i)=al(i);
      res(i+3)=ar(i);
    }

  sotDEBUGOUT(25);
  return res;
}



ml::Vector& sotDynamic::
computeZmp( ml::Vector& ZMPval,int time )
{
  if (ZMPval.size()!=3)
    ZMPval.resize(3);

  newtonEulerSINTERN(time);
  MAAL1_V3d_to_MAAL2(aHDMB->zeroMomentumPoint(),ZMPval);
  sotDEBUGOUT(25);
  return ZMPval;
}

ml::Matrix& sotDynamic::
computeJAngularMomentum( ml::Matrix& JAngMom,int time )
{
   
  sotDEBUGIN(25) ;
  newtonEulerSINTERN(time);
  matrixNxP lJAngMom;
  aHDMB->getJacobianAngularMomentumWrtCoM(lJAngMom);
  JAngMom.initFromMotherLib(lJAngMom);
  sotDEBUGOUT(25) ;
  return JAngMom;
}

ml::Vector& sotDynamic::
computeMomenta(ml::Vector & Momenta, int time)
{
  vector3d LinearMomentum, AngularMomentum;

  if (Momenta.size()!=6)
    Momenta.resize(6);

  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  LinearMomentum = aHDMB->linearMomentumRobot();
  AngularMomentum = aHDMB->angularMomentumRobot();

  for(unsigned int i=0;i<3;i++)
    {
      Momenta(i)   = LinearMomentum(i);
      Momenta(i+3) = AngularMomentum(i);
    }

  sotDEBUGOUT(25) << "Momenta :" << Momenta ;
  return Momenta;
}

ml::Vector& sotDynamic::
computeAngularMomentum(ml::Vector & Momenta, int time)
{
  vector3d  AngularMomentum;

  if (Momenta.size()!=3)
    Momenta.resize(3);

  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  AngularMomentum = aHDMB->angularMomentumRobot();

  for(unsigned int i=0;i<3;i++)
    {
      Momenta(i) = AngularMomentum(i);
    }

  sotDEBUGOUT(25) << "AngularMomenta :" << Momenta ;
  return Momenta;

}

ml::Matrix& sotDynamic::
computeJcom( ml::Matrix& Jcom,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  aHDMB->computeJacobianCenterOfMass();
  Jcom.initFromMotherLib(aHDMB->jacobianCenterOfMass());
  sotDEBUGOUT(25);
  return Jcom;
}

ml::Vector& sotDynamic::
computeCom( ml::Vector& com,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  com.resize(3);
  MAAL1_V3d_to_MAAL2(aHDMB->positionCenterOfMass(),com);
  sotDEBUGOUT(25);
  return com;
}

ml::Matrix& sotDynamic::
computeInertia( ml::Matrix& A,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);

  inertia.update();
  inertia.computeInertiaMatrix();
  A= inertia.getInertiaMatrix();

  if( 1==debugInertia )
    {
      for( unsigned int i=0;i<18;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
      for( unsigned int i=20;i<22;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
      for( unsigned int i=28;i<36;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
    }
  else if( 2==debugInertia )
    {
      for( unsigned int i=0;i<18;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
      for( unsigned int i=20;i<22;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
      for( unsigned int i=28;i<29;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
      for( unsigned int i=35;i<36;++i )
	for( unsigned int j=0;j<36;++j )
	  if( i==j ) A(i,i)=1; 
	  else {  A(i,j)=A(j,i)=0; }
    }

  sotDEBUGOUT(25);
  return A;
}

ml::Matrix& sotDynamic::
computeInertiaReal( ml::Matrix& res,int time )
{
  sotDEBUGIN(25);

  const ml::Matrix & A = inertiaSOUT(time);
  const ml::Vector & gearRatio = gearRatioSOUT(time);
  const ml::Vector & inertiaRotor = inertiaRotorSOUT(time);

  res = A;
  for( unsigned int i=0;i<gearRatio.size();++i )
    res(i,i) += (gearRatio(i)*gearRatio(i)*inertiaRotor(i)); 

  sotDEBUGOUT(25);
  return res;
}

double& sotDynamic::
computeFootHeight( double& foot,int time )
{
  sotDEBUGIN(25);
  newtonEulerSINTERN(time);
  foot=aHDMB->footHeight();
  sotDEBUGOUT(25);
  return foot;
}


/* --- SIGNAL --------------------------------------------------------------- */
/* --- SIGNAL --------------------------------------------------------------- */
/* --- SIGNAL --------------------------------------------------------------- */

sotSignalTimeDependant<ml::Matrix,int>& sotDynamic::
jacobiansSOUT( const std::string& name )
{
  sotSignalAbstract<int> & sigabs = sotEntity::getSignal(name);

  try { 
    sotSignalTimeDependant<ml::Matrix,int>& res 
      = dynamic_cast< sotSignalTimeDependant<ml::Matrix,int>& >( sigabs );
    return res;
  } catch( std::bad_cast e ) {
    SOT_THROW sotExceptionSignal( sotExceptionSignal::BAD_CAST,
				  "Impossible cast.",
				  " (while getting signal <%s> of type matrix.",
				  name.c_str()); 
  }
}
sotSignalTimeDependant<sotMatrixHomogeneous,int>& sotDynamic::
positionsSOUT( const std::string& name )
{
  sotSignalAbstract<int> & sigabs = sotEntity::getSignal(name);

  try { 
    sotSignalTimeDependant<sotMatrixHomogeneous,int>& res 
      = dynamic_cast< sotSignalTimeDependant<sotMatrixHomogeneous,int>& >( sigabs );
    return res;
  } catch( std::bad_cast e ) {
    SOT_THROW sotExceptionSignal( sotExceptionSignal::BAD_CAST,
				  "Impossible cast.",
				  " (while getting signal <%s> of type matrixHomo.",
				  name.c_str()); 
  }
}
sotSignalTimeDependant<ml::Vector,int>& sotDynamic::
accelerationsSOUT( const std::string& name )
{
  sotSignalAbstract<int> & sigabs = sotEntity::getSignal(name);

  try { 
    sotSignalTimeDependant<ml::Vector,int>& res 
      = dynamic_cast< sotSignalTimeDependant<ml::Vector,int>& >( sigabs );
    return res;
  } catch( std::bad_cast e ) {
    SOT_THROW sotExceptionSignal( sotExceptionSignal::BAD_CAST,
				  "Impossible cast.",
				  " (while getting signal <%s> of type Vector.",
				  name.c_str()); 
  }
}


int& sotDynamic::
computeNewtonEuler( int& dummy,int time )
{
  sotDEBUGIN(15);
  ml::Vector pos = jointPositionSIN(time); // TODO &pos
  ml::Vector vel = jointVelocitySIN(time);
  ml::Vector acc = jointAccelerationSIN(time);

  firstSINTERN(time);
  if( freeFlyerPositionSIN )
    {
      const ml::Vector& ffpos = freeFlyerPositionSIN(time);
      for( int i=0;i<6;++i ) pos(i) = ffpos(i); 
    }
  if( freeFlyerVelocitySIN )
    {
      const ml::Vector& ffvel = freeFlyerVelocitySIN(time);
      for( int i=0;i<6;++i ) vel(i) = ffvel(i); 
    }
  if( freeFlyerAccelerationSIN )
    {
      const ml::Vector& ffacc = freeFlyerAccelerationSIN(time);
      for( int i=0;i<6;++i ) acc(i) = ffacc(i); 
    }
  if(! aHDMB->currentConfiguration(pos.accessToMotherLib()))
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_SIZE,
				     "Position vector size uncorrect",
				     " (Vector size is %d, should be %d).",
				     pos.size(),aHDMB->currentConfiguration().size() );
    }


  if(! aHDMB->currentVelocity(vel.accessToMotherLib()) )
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_SIZE,
				     "Velocity vector size uncorrect",
				     " (Vector size is %d, should be %d).",
				     vel.size(),aHDMB->currentVelocity().size() );
    }

  if(! aHDMB->currentAcceleration(acc.accessToMotherLib()) )
    { 
      SOT_THROW sotExceptionDynamic( sotExceptionDynamic::JOINT_SIZE,
				     "Acceleration vector size uncorrect",
				     " (Vector size is %d, should be %d).",
				     acc.size(),aHDMB->currentAcceleration().size() );
    }

  aHDMB->computeForwardKinematics();
  
  sotDEBUG(1)<< "pos = " <<pos <<endl;
  sotDEBUG(1)<< "vel = " <<vel <<endl;
  sotDEBUG(1)<< "acc = " <<acc <<endl;

  sotDEBUGOUT(15);
  return dummy;
}
int& sotDynamic::
initNewtonEuler( int& dummy,int time )
{
  sotDEBUGIN(15);
  firstSINTERN.setReady(false);
  computeNewtonEuler(dummy,time);
  for( int i=0;i<3;++i )
    aHDMB->computeForwardKinematics();

  sotDEBUGOUT(15);
  return dummy;
}

ml::Vector& sotDynamic::
getUpperJointLimits( ml::Vector& res,const int& time )
{
  sotDEBUGIN(25);
  const unsigned int NBJ = aHDMB->numberDof();
  res.resize( NBJ );
  for( unsigned int i=0;i<NBJ;++i )
    {
      res(i)=aHDMB->upperBoundDof( i );
    }
  sotDEBUGOUT(25);
  return res;
}

ml::Vector& sotDynamic::
getLowerJointLimits( ml::Vector& res,const int& time )
{
  sotDEBUGIN(25);
  const unsigned int NBJ = aHDMB->numberDof();
  res.resize( NBJ );
  for( unsigned int i=0;i<NBJ;++i )
    {
      res(i)=aHDMB->lowerBoundDof( i );
    }
  sotDEBUGOUT(25);
  return res;
}



/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
void sotDynamic::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUG(25) << "# In { Cmd " << cmdLine <<endl;
  std::string filename;
  if( cmdLine == "debugInertia" )
    {  
      cmdArgs>>ws; if(cmdArgs.good()) 
		     { 
		       std::string arg; cmdArgs >> arg; 
		       if( (arg=="true")||(arg=="1") )
			 { debugInertia = 1; }
		       else if( (arg=="2")||(arg=="grip") )
			 { debugInertia = 2; }
		       else debugInertia=0;

		     }
      else { os << "debugInertia = " << debugInertia << std::endl; }
    }
  else if( cmdLine == "setVrmlDir" )
    {  cmdArgs>>filename; setVrmlDirectory( filename );  }
  else if( cmdLine == "setVrml" )
    {  cmdArgs>>filename; setVrmlMainFile( filename );  }
  else if( cmdLine == "setXmlSpec" )
    {  cmdArgs>>filename; setXmlSpecificityFile( filename );  }
  else if( cmdLine == "setXmlRank" )
    {  cmdArgs>>filename; setXmlRankFile( filename );  }
  else if( cmdLine == "setFiles" )
    {
      cmdArgs>>filename; setVrmlDirectory( filename ); 
      cmdArgs>>filename; setVrmlMainFile( filename );  
      cmdArgs>>filename; setXmlSpecificityFile( filename );  
      cmdArgs>>filename; setXmlRankFile( filename );  
    }
  else if( cmdLine == "displayFiles" )
    {
      cmdArgs >> ws; bool filespecified = false; 
      if( cmdArgs.good() )
	{
	  filespecified = true; 
	  std::string filetype; cmdArgs >> filetype;
	  sotDEBUG(15) << " Request: " << filetype << std::endl;
	  if( "vrmldir" == filetype ) { os << vrmlDirectory << std::endl; }
	  else if( "xmlspecificity" == filetype ) { os << xmlSpecificityFile << std::endl; }
	  else if( "xmlrank" == filetype ) { os << xmlRankFile << std::endl; }
	  else if( "vrmlmain" == filetype ) { os << vrmlMainFile << std::endl; }
	  else filespecified = false;
	}
      if( ! filespecified ) 
	{
	  os << "  - VRML Directory:\t\t\t" << vrmlDirectory <<endl
	     << "  - XML Specificity File:\t\t" << xmlSpecificityFile <<endl
	     << "  - XML Rank File:\t\t\t" << xmlRankFile <<endl
	     << "  - VRML Main File:\t\t\t" << vrmlMainFile <<endl;
	}
    }
  else if( cmdLine == "parse" )
    {
      if(! init )parseConfigFiles(); else cout << "  !! Already parsed." <<endl;
    }
  else if( cmdLine == "createJacobian" )
    {
      std::string Jname; cmdArgs >> Jname;
      unsigned int rank; cmdArgs >> rank;
      createJacobianSignal(Jname,rank);
    }
  else if( cmdLine == "destroyJacobian" )
    {
      std::string Jname; cmdArgs >> Jname;
      destroyJacobianSignal(Jname);
    }
  else if( cmdLine == "createPosition" )
    {
      std::string Jname; cmdArgs >> Jname;
      unsigned int rank; cmdArgs >> rank;
      createPositionSignal(Jname,rank);
    }
  else if( cmdLine == "destroyPosition" )
    {
      std::string Jname; cmdArgs >> Jname;
      destroyPositionSignal(Jname);
    }
  else if( cmdLine == "createAcceleration" )
    {
      std::string Jname; cmdArgs >> Jname;
      unsigned int rank; cmdArgs >> rank;
      createAccelerationSignal(Jname,rank);
    }
  else if( cmdLine == "destroyAcceleration" )
    {
      std::string Jname; cmdArgs >> Jname;
      destroyAccelerationSignal(Jname);
    }
  else if( cmdLine == "createEndeffJacobian" )
    {
      std::string Jname; cmdArgs >> Jname;
      unsigned int rank; cmdArgs >> rank;
      createEndeffJacobianSignal(Jname,rank);
    }
  else if( cmdLine == "createOpPoint" )
    {
      std::string Jname; cmdArgs >> Jname;
      unsigned int rank; cmdArgs >> rank;
      createEndeffJacobianSignal(string("J")+Jname,rank);
      createPositionSignal(Jname,rank);
      sotDEBUG(15)<<endl;
    }
  else if( cmdLine == "destroyOpPoint" )
    {
      std::string Jname; cmdArgs >> Jname;
      destroyJacobianSignal(string("J")+Jname);
      destroyPositionSignal(Jname);
    }
  else if( cmdLine == "ndof" ) { os << aHDMB->numberDof() <<endl; return; }
  else if( cmdLine == "setComputeCom" )
    {
      unsigned int b; cmdArgs >> b;
      comActivation(b);
    }
  else if( cmdLine == "setComputeZmp" )
    {
      unsigned int b; cmdArgs >> b;
      zmpActivation(b);
    }
  else if( cmdLine == "setProperty" )
    {
      string prop,val; cmdArgs >> prop; 
      if( cmdArgs.good() ) cmdArgs >> val; else val="true";
      aHDMB->setProperty( prop,val );
    }
  else if( cmdLine == "getProperty" )
    {
      string prop,val; cmdArgs >> prop; 
      aHDMB->getProperty( prop,val );
      os<<val<<endl;
    }
  else if( cmdLine == "displayProperties" )
    {
      std::istringstream iss("ComputeVelocity ComputeCoM ComputeAccelerationCoM ComputeMomentum ComputeZMP ComputeBackwardDynamics");
      string prop,val; const unsigned int STR_SIZE=30;
      while( iss.good() )
	{
	  iss>>prop;
	  aHDMB->getProperty( prop,val );
	  os<<prop;
	  for( unsigned int i=prop.length();i<STR_SIZE;++i ) os<<" ";
	  os<<" -> "<<val <<endl;
	} 
    }
  else if( cmdLine == "help" )
    {
      os << "Dynamics:"<<endl
	 << "  - setVrmlDir - setVrml - setXmlSpec - setXmlRanks <file>" <<endl
	 << "\t\t\t\t:set the config files" <<endl
	 << "  - setFiles <%1> ... <%4>\t:set files in the order cited above" <<endl
	 << "  - displayFiles\t\t\t:display the 5 config files" <<endl
	 << "  - parse\t\t\t:parse the files set unsing the set{Xml|Vrml} commands." <<endl
	 << "  - createJacobian <name> <point>:create a signal named <name> " << endl
	 << "  - createEndeffJacobian <name> <point>:create a signal named <name> "
	 << "forwarding the jacoian computed at <point>." <<endl
	 << "  - destroyJacobian <name>\t:delete the jacobian signal <name>" << endl
	 << "  - {create|destroy}Position\t:handle position signals." <<endl
	 << "  - {create|destroy}OpPoint\t:handle Operation Point (ie pos+jac) signals." <<endl
	 << "  - {create|destroy}Acceleration\t:handle acceleration signals." <<endl
	 << "  - {get|set}Property <name> [<val>]: set/get the property." <<endl
	 << "  - displayProperties: print the prop-val couples list." <<endl
	 << "  - ndof\t\t\t: display the number of DOF of the robot."<< endl;
      
      sotEntity::commandLine(cmdLine,cmdArgs,os);
    }
  else { sotEntity::commandLine( cmdLine,cmdArgs,os); }

  sotDEBUGOUT(15);

}



