/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotForceCompensation.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTFORCECOMPENSATION_H__
#define __SOT_SOTFORCECOMPENSATION_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixRotation.h>
#include <sot/sotMatrixForce.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotForceCompensation_EXPORTS)
#    define SOTFORCECOMPENSATION_EXPORT __declspec(dllexport)
#  else  
#    define SOTFORCECOMPENSATION_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTFORCECOMPENSATION_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTFORCECOMPENSATION_EXPORT sotForceCompensation
{
 private:
  static sotMatrixRotation I3;
 protected:
  bool usingPrecompensation;

 public:
  sotForceCompensation( void );
  static sotMatrixForce& computeHandXworld( 
					   const sotMatrixRotation & worldRhand,
					   const ml::Vector & transSensorCom,
					   sotMatrixForce& res );

  
  static sotMatrixForce& computeHandVsensor( const sotMatrixRotation & sensorRhand,
					     sotMatrixForce& res );
  static sotMatrixForce& computeSensorXhand( const sotMatrixRotation & sensorRhand,
					     const ml::Vector & transSensorCom,
					     sotMatrixForce& res );
/*   static ml::Matrix& computeInertiaSensor( const ml::Matrix& inertiaJoint, */
/* 					   const sotMatrixForce& sensorXhand, */
/* 					   ml::Matrix& res ); */

  static ml::Vector& computeTorsorCompensated( const ml::Vector& torqueInput,
					       const ml::Vector& torquePrecompensation,
					       const ml::Vector& gravity,
					       const sotMatrixForce& handXworld,
					       const sotMatrixForce& handVsensor,
					       const ml::Matrix& gainSensor,
					       const ml::Vector& momentum,
					       ml::Vector& res );

  static ml::Vector& crossProduct_V_F( const ml::Vector& velocity,
				       const ml::Vector& force,
				       ml::Vector& res );
  static ml::Vector& computeMomentum( const ml::Vector& velocity,
				      const ml::Vector& acceleration,
				      const sotMatrixForce& sensorXhand,
				      const ml::Matrix& inertiaJoint,
				      ml::Vector& res );

  static ml::Vector& computeDeadZone( const ml::Vector& torqueInput,
				      const ml::Vector& deadZoneLimit,
				      ml::Vector& res );
  
 public: // CALIBRATION

  std::list<ml::Vector> torsorList;
  std::list<sotMatrixRotation> rotationList;

  void clearCalibration( void );
  void addCalibrationValue( const ml::Vector& torsor,
			    const sotMatrixRotation & worldRhand );
  
  ml::Vector calibrateTransSensorCom( const ml::Vector& gravity,
				      const sotMatrixRotation& handRsensor );
  ml::Vector calibrateGravity( const sotMatrixRotation& handRsensor,
			       bool precompensationCalibration = false,
			       const sotMatrixRotation& hand0Rsensor = I3 );

    
  
};

/* --------------------------------------------------------------------- */
/* --- PLUGIN ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTFORCECOMPENSATION_EXPORT sotForceCompensationPlugin
:public sotEntity, public sotForceCompensation
{
 public:
  static const std::string CLASS_NAME;
  bool calibrationStarted;


 public: /* --- CONSTRUCTION --- */

  sotForceCompensationPlugin( const std::string& name );
  virtual ~sotForceCompensationPlugin( void );

 public: /* --- SIGNAL --- */

  /* --- INPUTS --- */
  sotSignalPtr<ml::Vector,int> torsorSIN; 
  sotSignalPtr<sotMatrixRotation,int> worldRhandSIN; 

  /* --- CONSTANTS --- */
  sotSignalPtr<sotMatrixRotation,int> handRsensorSIN; 
  sotSignalPtr<ml::Vector,int> translationSensorComSIN; 
  sotSignalPtr<ml::Vector,int> gravitySIN; 
  sotSignalPtr<ml::Vector,int> precompensationSIN; 
  sotSignalPtr<ml::Matrix,int> gainSensorSIN; 
  sotSignalPtr<ml::Vector,int> deadZoneLimitSIN; 
  sotSignalPtr<ml::Vector,int> transSensorJointSIN; 
  sotSignalPtr<ml::Matrix,int> inertiaJointSIN; 

  sotSignalPtr<ml::Vector,int> velocitySIN; 
  sotSignalPtr<ml::Vector,int> accelerationSIN; 

  /* --- INTERMEDIATE OUTPUTS --- */
  sotSignalTimeDependant<sotMatrixForce,int> handXworldSOUT; 
  sotSignalTimeDependant<sotMatrixForce,int> handVsensorSOUT; 
  sotSignalPtr<ml::Vector,int> torsorDeadZoneSIN; 

  sotSignalTimeDependant<sotMatrixForce,int> sensorXhandSOUT;
  //sotSignalTimeDependant<ml::Matrix,int> inertiaSensorSOUT;
  sotSignalTimeDependant<ml::Vector,int> momentumSOUT; 
  sotSignalPtr<ml::Vector,int> momentumSIN; 

  /* --- OUTPUTS --- */
  sotSignalTimeDependant<ml::Vector,int> torsorCompensatedSOUT; 
  sotSignalTimeDependant<ml::Vector,int> torsorDeadZoneSOUT;

  typedef int sotDummyType;
  sotSignalTimeDependant<sotDummyType,int> calibrationTrigerSOUT; 

 public: /* --- COMMANDLINE --- */

  sotDummyType& calibrationTriger( sotDummyType& dummy,int time );


  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );



};





#endif // #ifndef __SOT_SOTFORCECOMPENSATION_H__

