/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotIntegratorForce.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTINTEGRATORFORCE_H__
#define __SOT_SOTINTEGRATORFORCE_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotMatrixRotation.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotIntegratorForce_EXPORTS) 
#    define SOTINTEGRATORFORCE_EXPORT __declspec(dllexport)
#  else  
#    define SOTINTEGRATORFORCE_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTINTEGRATORFORCE_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTINTEGRATORFORCE_EXPORT sotIntegratorForce
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;

 protected:
  double timeStep;
  static const double  TIME_STEP_DEFAULT ; // = 5e-3


 public: /* --- CONSTRUCTION --- */

  sotIntegratorForce( const std::string& name );
  virtual ~sotIntegratorForce( void );

 public: /* --- SIGNAL --- */

  sotSignalPtr<ml::Vector,int> forceSIN; 
  sotSignalPtr<ml::Matrix,int> massInverseSIN; 
  sotSignalPtr<ml::Matrix,int> frictionSIN; 

  /* Memory of the previous iteration. The sig is fed by the previous
   * computations. */
  sotSignalPtr<ml::Vector,int> velocityPrecSIN; 
  sotSignalTimeDependant<ml::Vector,int> velocityDerivativeSOUT; 
  sotSignalTimeDependant<ml::Vector,int> velocitySOUT; 

  sotSignalPtr<ml::Matrix,int> massSIN; 
  sotSignalTimeDependant<ml::Matrix,int> massInverseSOUT; 

 public: /* --- FUNCTIONS --- */
  ml::Vector& computeDerivative( ml::Vector& res,
				  const int& time );
  ml::Vector& computeIntegral( ml::Vector& res,
			       const int& time );

  ml::Matrix& computeMassInverse( ml::Matrix& res,
				  const int& time );

  
 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};






#endif // #ifndef __SOT_WAISTATTITUDEFROMSENSOR_H__

