/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMassApparent.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotMassApparent.h>
#include <sot/sotDebug.h>
#include <sot/sotFactory.h>


SOT_FACTORY_ENTITY_PLUGIN(sotMassApparent,"MassApparent");


sotMassApparent::
sotMassApparent( const std::string & name ) 
  :sotEntity(name)
   ,jacobianSIN(NULL,"sotMassApparent("+name+")::input(vector)::jacobian")
   ,inertiaInverseSIN(NULL,"sotMassApparent("+name+")::input(vector)::inertiaInverse")
   ,massInverseSOUT( boost::bind(&sotMassApparent::computeMassInverse,this,_1,_2),
		     jacobianSIN<<inertiaInverseSIN,
		    "sotMassApparent("+name+")::output(Vector)::massInverse" )
   ,massSOUT( boost::bind(&sotMassApparent::computeMass,this,_1,_2),
	      massInverseSOUT,
	      "sotMassApparent("+name+")::output(Vector)::mass" )

   ,inertiaSIN(NULL,"sotMassApparent("+name+")::input(vector)::inertia")
  ,inertiaInverseSOUT( boost::bind(&sotMassApparent::computeInertiaInverse,this,_1,_2),
		       inertiaSIN,
		       "sotMassApparent("+name+")::input(vector)::inertiaInverseOUT")
{
  sotDEBUGIN(5);
  
  signalRegistration( jacobianSIN << inertiaInverseSIN << massInverseSOUT << massSOUT 
		      << inertiaSIN << inertiaInverseSOUT);
  inertiaInverseSIN.plug( &inertiaInverseSOUT );

  sotDEBUGOUT(5);
}


sotMassApparent::
~sotMassApparent( void )
{
  return;
}

/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
ml::Matrix& sotMassApparent::
computeMassInverse( ml::Matrix& res,
		   const int& time )
{
  sotDEBUGIN(15);
  
  const ml::Matrix & J = jacobianSIN( time );
  const ml::Matrix & A = inertiaInverseSIN( time );

  ml::Matrix AJt( J.nbCols(),J.nbRows() );
  A.multiply( J.transpose(),AJt );

  res.resize( J.nbRows(),J.nbRows() );
  J.multiply( AJt,res );

  sotDEBUGOUT(15);
  return res;
}

ml::Matrix& sotMassApparent::
computeMass( ml::Matrix& res,
		   const int& time )
{
  sotDEBUGIN(15);

  const ml::Matrix & omega = massInverseSOUT( time );
  omega.inverse( res );

  sotDEBUGOUT(15);
  return res;
}

ml::Matrix& sotMassApparent::
computeInertiaInverse( ml::Matrix& res,
		       const int& time )
{
  sotDEBUGIN(15);

  const ml::Matrix & A = inertiaSIN( time );
  A.inverse( res );

  sotDEBUGOUT(15);
  return res;
}
