/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMassApparent.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTMASSAPPARENT_H__
#define __SOT_SOTMASSAPPARENT_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotMatrixRotation.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotMassApparent_EXPORTS)
#    define SOTMASSAPPARENT_EXPORT __declspec(dllexport)
#  else  
#    define SOTMASSAPPARENT_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTMASSAPPARENT_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */
class SOTMASSAPPARENT_EXPORT sotMassApparent
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;

 public: /* --- CONSTRUCTION --- */

  sotMassApparent( const std::string& name );
  virtual ~sotMassApparent( void );

 public: /* --- SIGNAL --- */

  sotSignalPtr<ml::Matrix,int> jacobianSIN; 
  sotSignalPtr<ml::Matrix,int> inertiaInverseSIN; 
  sotSignalTimeDependant<ml::Matrix,int> massInverseSOUT; 
  sotSignalTimeDependant<ml::Matrix,int> massSOUT; 

  sotSignalPtr<ml::Matrix,int> inertiaSIN; 
  sotSignalTimeDependant<ml::Matrix,int> inertiaInverseSOUT; 

 public: /* --- FUNCTIONS --- */
  ml::Matrix& computeMassInverse( ml::Matrix& res,const int& time );
  ml::Matrix& computeMass( ml::Matrix& res,const int& time );
  ml::Matrix& computeInertiaInverse( ml::Matrix& res,const int& time );
};





#endif // #ifndef __SOT_SOTMASSAPPARENT_H__

