/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotWaistAttitudeFromSensor.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotWaistAttitudeFromSensor.h>
#include <sot/sotDebug.h>
#include <sot/sotFactory.h>


SOT_FACTORY_ENTITY_PLUGIN(sotWaistAttitudeFromSensor,"WaistAttitudeFromSensor");

sotWaistAttitudeFromSensor::
sotWaistAttitudeFromSensor( const std::string & name ) 
  :sotEntity(name)
  ,attitudeSensorSIN(NULL,"sotWaistAttitudeFromSensor("+name+")::input(MatrixRotation)::attitudeIN")
  ,positionSensorSIN(NULL,"sotWaistAttitudeFromSensor("+name+")::input(matrixHomo)::position")
  ,attitudeWaistSOUT( boost::bind(&sotWaistAttitudeFromSensor::computeAttitudeWaist,this,_1,_2),
		      attitudeSensorSIN<<positionSensorSIN,
		      "sotWaistAttitudeFromSensor("+name+")::output(RPY)::attitude" ) 
{
  sotDEBUGIN(5);

  signalRegistration( attitudeSensorSIN<<positionSensorSIN
		      <<attitudeWaistSOUT );

  sotDEBUGOUT(5);
}


sotWaistAttitudeFromSensor::
~sotWaistAttitudeFromSensor( void )
{
  sotDEBUGIN(5);

  sotDEBUGOUT(5);
  return;
}

/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
sotVectorRollPitchYaw & sotWaistAttitudeFromSensor::
computeAttitudeWaist( sotVectorRollPitchYaw & res,
		      const int& time )
{
  sotDEBUGIN(15);
  
  const sotMatrixHomogeneous & waistMchest = positionSensorSIN( time );
  const sotMatrixRotation & worldRchest = attitudeSensorSIN( time );

  sotMatrixRotation waistRchest; waistMchest.extract(waistRchest);
  sotMatrixRotation chestRwaist; waistRchest.transpose( chestRwaist );

  sotMatrixRotation worldrchest;
  worldRchest.multiply( chestRwaist,worldrchest);
  res.fromMatrix(worldrchest);
  sotDEBUGOUT(15);
  return res;
}



/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
void sotWaistAttitudeFromSensor::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUG(25) << "Cmd " << cmdLine <<std::endl;

  if( cmdLine == "help" )
    {
      sotEntity::commandLine(cmdLine,cmdArgs,os);
    }
  else { sotEntity::commandLine( cmdLine,cmdArgs,os); }
}


/* === sotWaistPoseFromSensorAndContact ===================================== */
/* === sotWaistPoseFromSensorAndContact ===================================== */
/* === sotWaistPoseFromSensorAndContact ===================================== */

SOT_FACTORY_ENTITY_PLUGIN(sotWaistPoseFromSensorAndContact,
			  "WaistPoseFromSensorAndContact");

sotWaistPoseFromSensorAndContact::
sotWaistPoseFromSensorAndContact( const std::string & name ) 
  :sotWaistAttitudeFromSensor(name)
   ,fromSensor(false)
   ,positionContactSIN(NULL,"sotWaistPoseFromSensorAndContact("+name+")::input(matrixHomo)::contact")
   ,positionWaistSOUT( boost::bind(&sotWaistPoseFromSensorAndContact::computePositionWaist,this,_1,_2),
		       attitudeWaistSOUT<<positionContactSIN,
		       "sotWaistPoseFromSensorAndContact("+name+")::output(RPY+T)::positionWaist" ) 
{
  sotDEBUGIN(5);

  signalRegistration( positionContactSIN<<positionWaistSOUT );

  sotDEBUGOUT(5);
}


sotWaistPoseFromSensorAndContact::
~sotWaistPoseFromSensorAndContact( void )
{
  sotDEBUGIN(5);

  sotDEBUGOUT(5);
  return;
}

/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
ml::Vector& sotWaistPoseFromSensorAndContact::
computePositionWaist( ml::Vector& res,
		      const int& time )
{
  sotDEBUGIN(15);
  
  const sotMatrixHomogeneous&  waistMcontact = positionContactSIN( time );
  sotMatrixHomogeneous contactMwaist; waistMcontact.inverse(contactMwaist);

  res.resize(6);
  for( unsigned int i=0;i<3;++i ) 
    { res(i)=contactMwaist(i,3); }

  if(fromSensor)
    {
      const sotVectorRollPitchYaw & worldrwaist = attitudeWaistSOUT( time );
      for( unsigned int i=0;i<3;++i ) 
	{ res(i+3)=worldrwaist(i); }
    }
  else
    {
      sotMatrixRotation contactRwaist; 
      contactMwaist.extract(contactRwaist);
      sotVectorRollPitchYaw contactrwaist; 
      contactrwaist.fromMatrix( contactRwaist );
      
      for( unsigned int i=0;i<3;++i ) 
	{ res(i+3)=contactrwaist(i); }
    }

  


  sotDEBUGOUT(15);
  return res;
}



/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
void sotWaistPoseFromSensorAndContact::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUG(25) << "Cmd " << cmdLine <<std::endl;

  if( cmdLine == "help" )
    {
      os <<"WaistPoseFromSensorAndContact:"<<std::endl
	 <<"  - fromSensor [true|false]: get/set the flag." << std::endl;
      sotWaistAttitudeFromSensor::commandLine(cmdLine,cmdArgs,os);
    }
  else if( cmdLine == "fromSensor" )
    {
      std::string val; cmdArgs>>val; 
      if( ("true"==val)||("false"==val) )
	{
	  fromSensor = ( val=="true" ); 
	} else {
	  os << "fromSensor = " << (fromSensor?"true":"false") << std::endl;
	}
    }
  else { sotWaistAttitudeFromSensor::commandLine( cmdLine,cmdArgs,os); }
}

