/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotWaistAttitudeFromSensor.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_WAISTATTITUDEFROMSENSOR_H__
#define __SOT_WAISTATTITUDEFROMSENSOR_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotVectorRollPitchYaw.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotWaistAttitudeFromSensor_EXPORTS)
#    define SOTWAISTATTITUDEFROMSENSOR_EXPORT __declspec(dllexport)
#  else  
#    define SOTWAISTATTITUDEFROMSENSOR_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTWAISTATTITUDEFROMSENSOR_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTWAISTATTITUDEFROMSENSOR_EXPORT sotWaistAttitudeFromSensor
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;

 public: /* --- CONSTRUCTION --- */

  sotWaistAttitudeFromSensor( const std::string& name );
  virtual ~sotWaistAttitudeFromSensor( void );

 public: /* --- SIGNAL --- */

  sotVectorRollPitchYaw & computeAttitudeWaist( sotVectorRollPitchYaw & res,
					       const int& time );

  sotSignalPtr<sotMatrixRotation,int> attitudeSensorSIN;
  sotSignalPtr<sotMatrixHomogeneous,int> positionSensorSIN;
  sotSignalTimeDependant<sotVectorRollPitchYaw,int> attitudeWaistSOUT;


 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};


class SOTWAISTATTITUDEFROMSENSOR_EXPORT sotWaistPoseFromSensorAndContact
:public sotWaistAttitudeFromSensor
{
 public:
  static const std::string CLASS_NAME;

 protected:
  bool fromSensor;

 public: /* --- CONSTRUCTION --- */

  sotWaistPoseFromSensorAndContact( const std::string& name );
  virtual ~sotWaistPoseFromSensorAndContact( void );

 public: /* --- SIGNAL --- */

  ml::Vector& computePositionWaist( ml::Vector& res,
				    const int& time );

  sotSignalPtr<sotMatrixHomogeneous,int> positionContactSIN;
  sotSignalTimeDependant<ml::Vector,int> positionWaistSOUT;


 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};





#endif // #ifndef __SOT_WAISTATTITUDEFROMSENSOR_H__

