/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotZmprefFromCom.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotZmprefFromCom.h>
#include <sot/sotDebug.h>
#include <sot/sotFactory.h>

SOT_FACTORY_ENTITY_PLUGIN(sotZmprefFromCom,"ZmprefFromCom");


const double sotZmprefFromCom::DT_DEFAULT = 5e-3; 
const double sotZmprefFromCom::FOOT_HEIGHT_DEFAULT = .105; 

sotZmprefFromCom::
sotZmprefFromCom( const std::string & name ) 
  :sotEntity(name)
   ,dt(DT_DEFAULT) 
   ,footHeight(FOOT_HEIGHT_DEFAULT)
   ,waistPositionSIN(NULL,"sotZmprefFromCom("+name+")::input(MatrixHomo)::waist")
   ,comPositionSIN(NULL,"sotZmprefFromCom("+name+")::input(Vector)::com")
  ,dcomSIN(NULL,"sotZmprefFromCom("+name+")::input(Vector)::dcom")
   ,zmprefSOUT( boost::bind(&sotZmprefFromCom::computeZmpref,this,_1,_2),
		waistPositionSIN<<comPositionSIN<<dcomSIN,
		"sotZmprefFromCom("+name+")::output(RPY)::zmpref" ) 
{
  sotDEBUGIN(5);

  signalRegistration(waistPositionSIN<<comPositionSIN<<dcomSIN<<zmprefSOUT );

  sotDEBUGOUT(5);
}


sotZmprefFromCom::
~sotZmprefFromCom( void )
{
  sotDEBUGIN(5);

  sotDEBUGOUT(5);
  return;
}

/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
/* --- SIGNALS -------------------------------------------------------------- */
ml::Vector& sotZmprefFromCom::
computeZmpref( ml::Vector& res,
	       const int& time )
{
  sotDEBUGIN(15);
  
  const ml::Vector& com = comPositionSIN( time );
  const ml::Vector& dcom = dcomSIN( time );
  const sotMatrixHomogeneous& oTw = waistPositionSIN( time );

  sotMatrixHomogeneous wTo = oTw.inverse();
  ml::Vector nextComRef = dcom;  nextComRef*= dt;nextComRef+=com;
  
  
  nextComRef(2) = -footHeight; // projection on the ground.
  wTo.multiply(nextComRef,res);

  sotDEBUGOUT(15);
  return res;
}



/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
void sotZmprefFromCom::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUG(25) << "Cmd " << cmdLine <<std::endl;

  if( cmdLine == "help" )
    {
      os<<"zmprefFromCom:" << std::endl
	<<"  - dt [<value>]: get/set the dt value." << std::endl
	<<"  - footHeight [<value>]: "
	<<"get/set the height of the foot ("
	<< FOOT_HEIGHT_DEFAULT <<" by default)." <<std::endl;
      sotEntity::commandLine(cmdLine,cmdArgs,os);
    }
  if( cmdLine == "dt" )
    {
      cmdArgs>>std::ws;
      if( cmdArgs.good() ) cmdArgs>>dt; else os <<"dt = " << dt<<std::endl;
    }
  if( cmdLine == "footHeight" )
    {
      cmdArgs>>std::ws;
      if( cmdArgs.good() ) cmdArgs>>footHeight; 
      else os <<"footHeight = " << footHeight<<std::endl;
    }
  else { sotEntity::commandLine( cmdLine,cmdArgs,os); }
}

