/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotZmprefFromCom.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_ZMPREFFROMCOM_H__
#define __SOT_ZMPREFFROMCOM_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotMatrixHomogeneous.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotZmprefFromCom_EXPORTS)
#    define SOTZMPREFFROMCOM_EXPORT __declspec(dllexport)
#  else  
#    define SOTZMPREFFROMCOM_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTZMPREFFROMCOM_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTZMPREFFROMCOM_EXPORT sotZmprefFromCom
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  double dt;
  const static double DT_DEFAULT; // = 5e-3; // 5ms
  double footHeight;
  const static double FOOT_HEIGHT_DEFAULT; // = .105; 

 public: /* --- CONSTRUCTION --- */

  sotZmprefFromCom( const std::string& name );
  virtual ~sotZmprefFromCom( void );

 public: /* --- SIGNAL --- */

  ml::Vector& computeZmpref( ml::Vector& res,
					       const int& time );

  sotSignalPtr<sotMatrixHomogeneous,int> waistPositionSIN;
  sotSignalPtr<ml::Vector,int> comPositionSIN;
  sotSignalPtr<ml::Vector,int> dcomSIN;
  sotSignalTimeDependant<ml::Vector,int> zmprefSOUT;


 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    

};




#endif // #ifndef __SOT_ZMPREFFROMCOM_H__

