/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotEntity.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_ENTITY_HH__
#define __SOT_ENTITY_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotSignalAbstract.h>
#include <sot/sotExceptionFactory.h>
#include <sot/sotSignalArray.h>
#include <sot/sot-0API.h>

/* --- STD --- */
#include <string>
#include <map>
#include <sstream>

/* --- BOOST --- */
#include <boost/noncopyable.hpp>

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOT_EXPORT sotEntity
: private boost::noncopyable
{

 protected:
  std::string name;
  static const std::string CLASS_NAME;
  
  void entityRegistration( void );
  void entityDeregistration( void );

 public:
  sotEntity( const std::string& name );
  virtual ~sotEntity( void );

  const std::string& getName( void ) const { return name; }
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

 protected: /* --- SIGNALS --- */
  typedef   std::map< std::string,sotSignalAbstract<int>* > SignalMap;
  SignalMap signalMap;
  void signalRegistration( const sotSignalArray<int>& signals );
  void signalDeregistration( const std::string& name );
 public:
  sotSignalAbstract<int>& getSignal( const std::string & signalName );
  const sotSignalAbstract<int>& getSignal( const std::string & signalName ) const;
  std::ostream& displaySignalList( std::ostream& os ) const;
  virtual std::ostream& writeGraph( std::ostream& os ) const;
  virtual std::ostream& writeCompletionList( std::ostream& os ) const;

 public: /* --- DISPLAY --- */
  virtual void display( std::ostream& os ) const; 
  SOT_EXPORT friend std::ostream& operator<< (std::ostream& os, const sotEntity& ent );


 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
			    std::ostream& os );

  virtual sotSignalAbstract<int>* test(void) { return 0; }
  virtual void test2( sotSignalAbstract<int>* ptr ) { return ; }

  virtual const std::string& getCommandList( void ) const;
};





#endif /* #ifndef __SOT_ENTITY_HH__ */




