/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotFactory.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_FACTORY_HH__
#define __SOT_FACTORY_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* --- STD --- */
#include <map>
#include <string>

/* --- SOT --- */
class sotFeatureAbstract;
class sotEntity;
class sotTaskAbstract;
#include <sot/sotExceptionFactory.h>
#include <sot/sot-0API.h>

/* --------------------------------------------------------------------- */
/* --- FATORY ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOT_EXPORT sotFactoryStorage
{
 public:

  typedef sotEntity* (*EntityConstructor_ptr)( const std::string& );
  typedef sotFeatureAbstract* (*FeatureConstructor_ptr)( const std::string& );
  typedef sotTaskAbstract* (*TaskConstructor_ptr)( const std::string& );
  
 protected:
  typedef std::map< std::string,EntityConstructor_ptr > EntityMap;
  typedef std::map< std::string,TaskConstructor_ptr > TaskMap;
  typedef std::map< std::string,FeatureConstructor_ptr > FeatureMap;

  EntityMap entityMap;
  FeatureMap featureMap;
  TaskMap taskMap;

 public:

  ~sotFactoryStorage( void );

  void registerEntity( const std::string& entname,EntityConstructor_ptr ent );
  void deregisterEntity( const std::string& entname );
  sotEntity* newEntity( const std::string& name,const std::string& objname );
  bool existEntity( const std::string& name, EntityMap::iterator& entPtr );
  bool existEntity( const std::string& name );

  void registerTask( const std::string& entname,TaskConstructor_ptr ent );
  sotTaskAbstract* newTask( const std::string& name,const std::string& objname );
  bool existTask( const std::string& name, TaskMap::iterator& entPtr );
  bool existTask( const std::string& name );

  void registerFeature( const std::string& entname,FeatureConstructor_ptr ent );
  sotFeatureAbstract* newFeature( const std::string& name,const std::string& objname );
  bool existFeature( const std::string& name, FeatureMap::iterator& entPtr );
  bool existFeature( const std::string& name );

  void  commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
		     std::ostream& os );


};

SOT_EXPORT extern sotFactoryStorage sotFactory;

/* --- REGISTERER ----------------------------------------------------------- */
/* --- REGISTERER ----------------------------------------------------------- */
/* --- REGISTERER ----------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
/* --- ENTITY REGISTERER ---------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class SOT_EXPORT sotEntityRegisterer
{
 private:
  sotEntityRegisterer( void );
  std::string entityName;

 public:
  sotEntityRegisterer( const std::string& entityClassName,
		       sotFactoryStorage::EntityConstructor_ptr maker);

  ~sotEntityRegisterer( void );
};

#define SOT_FACTORY_ENTITY_PLUGIN(sotClassType,className) \
  const std::string sotClassType::CLASS_NAME = className; \
  extern "C" {                                            \
    sotEntity *sotEntityMaker##_##sotClassType( const std::string& objname )    \
    {                                                     \
      return new sotClassType( objname );                 \
    }                                                     \
  sotEntityRegisterer reg##_##sotClassType( className,    \
					    &sotEntityMaker##_##sotClassType );   \
  }                                                       \


/* -------------------------------------------------------------------------- */
/* --- FEATURE REGISTERER --------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class SOT_EXPORT sotFeatureRegisterer
{
 private:
  sotFeatureRegisterer( void );

 public:
  sotFeatureRegisterer( const std::string& featureClassName,
			sotFactoryStorage::FeatureConstructor_ptr maker);
};

#define SOT_FACTORY_FEATURE_PLUGIN(sotFeatureType,className) \
  const std::string sotFeatureType::CLASS_NAME = className;  \
  extern "C" {                                               \
    sotFeatureAbstract *sotFeatureMaker##_##sotFeatureType( const std::string& objname )\
    {                                                        \
      return new sotFeatureType( objname );                  \
    }                                                        \
  sotFeatureRegisterer reg##_##sotFeatureType( className,    \
					       &sotFeatureMaker##_##sotFeatureType );   \
  }                                                          \


/* -------------------------------------------------------------------------- */
/* --- TASK REGISTERER ------------------------------------------------------ */
/* -------------------------------------------------------------------------- */

class SOT_EXPORT sotTaskRegisterer
{
 private:
  sotTaskRegisterer( void );

 public:
  sotTaskRegisterer( const std::string& taskClassName,
		     sotFactoryStorage::TaskConstructor_ptr maker);
};


#define SOT_FACTORY_TASK_PLUGIN(sotTaskType,className) \
  const std::string sotTaskType::CLASS_NAME = className; \
  extern "C" {                                            \
    sotTaskAbstract *sotTaskMaker##_##sotTaskType( const std::string& objname )    \
    {                                                     \
      return new sotTaskType( objname );                 \
    }                                                     \
  sotTaskRegisterer reg##_##sotTaskType( className,&sotTaskMaker##_##sotTaskType );   \
  }                                                       \



#endif /* #ifndef __SOT_FACTORY_HH__ */




