/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotPluginLoader.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_PLUGINLOADER_HH__
#define __SOT_PLUGINLOADER_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* --- STD --- */
#include <list>
#include <string>
#include <map>

/* --- SOT --- */
#include <sot/sotExceptionFactory.h>
#include <sot/sot-0API.h>

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class sotPluginRefMap;

/*! @ingroup factory 
  \brief Load plugins using dynamic libraries.
  This class handles the dynamic libraries which implements
  functionnalities inside the stack of tasks.
  
 */
class SOT_EXPORT sotPluginLoader
{
 protected:

  /*! \brief Store the directory from where the dynamic libraries are loaded. */
  std::string pluginDirectory;

  /*! \brief List of plugin names given by their name postfixed to the directory. */
  std::list< std::string > pluginNames;

  /*! \brief If a plugin has been loaded then this map contains
   the name of the plugin. 
   \note This field looks useless.
  */
  std::map< std::string,std::string > loadedPluginNames;
  
  /*! \brief Keeps a reference to the library according to the name of the 
   plugin. */
  sotPluginRefMap* pluginRefs;

 public:

  /*! \brief Default constructor. */
  sotPluginLoader( void );
  ~sotPluginLoader( void );
  
  /*! \brief Set the directory from which to load the dynamic libraries
   containing the plugins. */
  const std::string& setDirectory( const std::string& n );
  /*! \brief Get the directory from which to load the dynamic libraries
    containing the plugins. */
  const std::string& getDirectory( void );

  /*! \brief Load a list of plugins
  It is done by reading the file configFile which contains for each line
  a plugin name. 
  \par[in] configFile: The name of the file which contains the plugins name.
  \par[in] dir: The name of the directory which containes the dynamic libraries. 
  */
  void loadPluginList( const std::string& configFile, const std::string& dir="" );
  void addPlugin( const std::string& name, const std::string& dir="" );

  void loadPlugins( void );
  void unloadPlugin( const std::string& plugname );

  const std::map< std::string,std::string > getLoadedPluginNames( void ) 
  { return loadedPluginNames; }
  const std::string& searchPlugin( const std::string& plugname );

};


#endif /* #ifndef __SOT_PLUGINLOADER_HH__ */




