/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotShellFunctions.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotShellFunctions.h>
#include <sot/sotPluginLoader.h>
#include <sot/sotFactory.h>
#include <sot/sotDebug.h>
#include <sot/sotEntity.h>
#include <sot/sotSignalCast.h>
#include <sot/sotSignal.h>
#include <sot/sotFlags.h>

#include <MatrixAbstractLayer/boost.h>

#ifdef WIN32
#include <Windows.h>
#endif

#include <fstream>
using namespace std;

void sotShellFunctions::
cmdTry( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - try <cmd...>"
	 << "\t\t\t\tTry <cmd> and display the catched exception." <<endl;
      return;
    }
  string cmdLine2;
  cmdArg>>cmdLine2;
  sotDEBUG(5)<<"Try <" <<cmdLine2<<">"<<endl;
  try{
    sotShell.cmd(cmdLine2,cmdArg,os);
  } catch ( const sotExceptionAbstract& e ) { os << "sotError catch: " <<endl<< e<<endl; }
  catch( ... ) { os<<"Unknown error catch." <<endl; }

}
void sotShellFunctions::
cmdLoadPlugins( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - loadPlugins <file.txt> <directory>"
	 << "\t\tLoad the plugins listed in the file." <<endl;
      return;
    }
  if( NULL!=sotShell.dlPtr )
    {
      string pluginName,directory;
      cmdArg >> pluginName;
      cmdArg >> directory;
      sotDEBUG(15) << "Load plugin list <" <<pluginName<<"> from dir <" << directory<<">."<<endl;
      if( directory.length() != 0 ) sotShell.dlPtr->setDirectory( directory );
      sotShell.dlPtr ->loadPluginList( pluginName );
      sotShell.dlPtr->loadPlugins();
    }
  else { os << "!!  Dynamic loading functionalities not accessible through the shell." <<endl; }
}
void sotShellFunctions::
cmdClearPlugin( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - clearPlugin <className>"
	 << "\t\tDestroy all the objects of type <className>." <<endl;
      return;
    }

  string pluginName;
  cmdArg >> pluginName;
  sotPool.clearPlugin( pluginName );

}

void sotShellFunctions::
cmdDisplayPlugins( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
   if( cmdLine == "help" ) 
    {
      os << "  - displayPlugins "
	 << "\t\t\t\tDisplay the name of the loaded plugins." <<endl;
      return;
    }
  if( NULL!=sotShell.dlPtr )
    {
      std::map< std::string,std::string > m = sotShell.dlPtr->getLoadedPluginNames();
      for( std::map< std::string,std::string >::const_iterator iter = m.begin();
	   iter!=m.end(); ++iter )
	{
	  os << "  - <"<<iter->first<<">:\t"<<iter->second<<endl;
	}

    }
  else { os << "!!  Dynamic loading functionalities not accessible through the shell." <<endl; }


}

void sotShellFunctions::
cmdMatrixDisplay( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
   if( cmdLine == "help" ) 
    {
      os << "  - dispmat {simple|complet|matlab}"
	 << "\t\tChange the display of matrix." <<endl;
      return;
    }

   std::string arg;
   cmdArg >> arg;
//    sotDEBUG(15) << "Display matrix: arg = \""<< arg
// 		<<"\" old="<<0+maal::boost::getDisplayType()<<endl;
//    maal::boost::Vector trojan(3); 

//    if( ("simple"==arg)||(arg[0]=='s') ) 
//       { trojan.setDisplayType( maal::boost::SIMPLE ); }
//    else if( ("matlab"==arg)||(arg[0]=='m') ) 
//      { 
//        trojan.setDisplayType( maal::boost::MATLAB ); 
//        sotDEBUG(15) << "Display matrix: curr = \""
// 		    <<0+maal::boost::MATLAB<<endl;
//      }
//    else if( "cpp"==arg ) { trojan.setDisplayType( maal::boost::CPP ); }
//    else if( ("complet"==arg)||(arg[0]=='c') ) 
//      { trojan.setDisplayType( maal::boost::COMPLET ); }
//    else { os << "Arg <" << arg << "> is not a valid option."<<endl; }

//    sotDEBUG(15) << "Display matrix: curr = \""
// 		<<0+maal::boost::getDisplayType()<<endl;
//    sotDEBUGF(15,"Ptr fun: %p.", maal::boost::getDisplayType);
   //   sotDEBUG(15) << trojan<<std::endl;

   if( ("simple"==arg)||(arg[0]=='s') )  
     sotSignalCast<ml::Vector>::displayType = maal::boost::SIMPLE;
   else if( ("matlab"==arg)||(arg[0]=='m') ) 
     sotSignalCast<ml::Vector>::displayType = maal::boost::MATLAB;
   else if( "cpp"==arg ) 
     sotSignalCast<ml::Vector>::displayType = maal::boost::CPP;
   else if( ("complet"==arg)||(arg[0]=='c') ) 
     sotSignalCast<ml::Vector>::displayType = maal::boost::COMPLET;
   else { os << "Arg <" << arg << "> is not a valid option."<<endl; }
   
}



void sotShellFunctions::
cmdDisplayFactory( const std::string cmdLine, std::istringstream& cmdArg, 
		   std::ostream& os )
{
   if( cmdLine == "help" ) 
    {
      os << "  - "; sotFactory.commandLine(cmdLine,cmdArg,os);
      return;
    }
   
   string cmd2; cmdArg >> cmd2;
   sotFactory.commandLine( cmd2,cmdArg,os );
}

void sotShellFunctions::
cmdCommentary( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" )
    { os << "  # comment with '#': ignore the end of the line." <<endl; }
  return;
}


void sotShellFunctions::
cmdUnplug( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - unplug <obj2.sig2>"
	 << "\t\t\t\tPlug on sig2 (consumer) in sig1->sig2." <<endl;
      return;
    }
  string ssig;
  cmdArg>>ssig;
  
  string obj2,fun2;
  istringstream str2(ssig);
  if( !sotInterpretor::objectNameParser(str2,obj2,fun2) )
    {
      SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				     "Plug function: syntax is unplug OBJ2.SIG2.",
				     "(while calling plug %s %s).",ssig.c_str() );
    }

  sotDEBUG(20) << "Get Ent2 <"<<obj2<<"> ."<<endl;
  sotEntity& ent2 = sotPool.getEntity(obj2);
  sotDEBUG(20) << "Get Sig2 <"<<fun2<<"> ."<<endl;
  sotSignalAbstract<int> &sig2 = ent2.getSignal(fun2);
  

  sotDEBUG(25) << "Unplug..."<<endl;
    sig2.unplug();
}

void sotShellFunctions::
cmdSignalTime( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - signalTime <obj.sig>"
	 << "\t\t\t\tDisplay the time of sig." <<endl;
      return;
    }

  string ssig;
  cmdArg>>ssig;
  
  string obj2,fun2;
  istringstream str2(ssig);
  if( !sotInterpretor::objectNameParser(str2,obj2,fun2) )
    {
      SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				     "Plug function: syntax is unplug OBJ2.SIG2.",
				     "(while calling plug %s %s).",ssig.c_str() );
    }

  sotDEBUG(20) << "Get Ent2 <"<<obj2<<"> ."<<endl;
  sotEntity& ent2 = sotPool.getEntity(obj2);
  sotDEBUG(20) << "Get Sig2 <"<<fun2<<"> ."<<endl;
  sotSignalAbstract<int> &sig2 = ent2.getSignal(fun2);
  
  os << sig2.getTime() << endl;
}

void sotShellFunctions::
cmdSynchroSignal( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - synchro <obj.sig> [<period>]"
	 << "\t\t\t\tGet/Set the syncho of a signal <sig>." <<endl;
      return;
    }

  sotSignalAbstract<int> & sig = sotPool.getSignal( cmdArg );
  cmdArg >>ws;
  if( cmdArg.good() ) 
    {
      int period; cmdArg>>period;
      sig.setPeriodTime( period );
      cmdArg >>ws;
      if( cmdArg.good() ) 
	{
	  /* The sig is recomputed at the given period from the 
	   * current time, at any time T so that T%p==0, p the period. 
	   * By modifying the current time, the sig reomputation is done
	   * at T s.t. T%p=d, d the desynchro. */
	  int currTime = sig.getTime ();
	  int desynchro; cmdArg>>desynchro;
	  sig.setTime( currTime+desynchro );
	}
    }
  else
    {
      os << "period = " << sig.getPeriodTime() << std::endl;
    }
}

void sotShellFunctions::
cmdEcho( const std::string cmdLine, std::istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - echo <string>"
	 << "\t\t\t\tPrint <string. on the standard output." <<endl;
      return;
    }
  
  cmdArg >>ws; 
  while( cmdArg.good() ) 
    {
      std::string toPrint; cmdArg>>toPrint;
      os<<toPrint<<" ";
    }
  os<<std::endl;
}

void sotShellFunctions::
cmdCopy( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - copy <obj1.sig1> <obj2.sig2>"
	 << "\t\tCopy the value of sig1 to constant value in sig2." <<endl;
      return;
    }
  string ssig1,ssig2;
  cmdArg>>ssig1>>ssig2;
  istringstream str1(ssig1),str2(ssig2);
  
  try {
    sotSignalAbstract<int> &sig1 = sotPool.getSignal( str1 );
    sotSignalAbstract<int> &sig2 = sotPool.getSignal( str2 );
    
    sotDEBUG(25) << "Copy..."<<endl;
    sig2.plug(&sig1);
    sig2.setConstantDefault();
    sig2.plug(&sig2);
  } catch( sotExceptionAbstract & err ) { throw;  }
  catch( ... ) {
    SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				   "Copy: syntax is copy OBJ1.SIG1 OBJ2.SIG2.",
				   "(while calling copy %s %s).",ssig1.c_str(),
				   ssig2.c_str() );
  }

}

void sotShellFunctions::
cmdFreeze( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - freeze <obj.sig> "
	 << "\t\tOn a ptr-sig: save the current value from the source "
	 << "and unplug the signal." <<endl;
      return;
    }
  string ssig1;
  cmdArg>>ssig1;
  istringstream str1(ssig1);
  
  try {
    sotSignalAbstract<int> &sig1 = sotPool.getSignal( str1 );
    
    sotDEBUG(25) << "Unplug..."<<endl;
    sig1.setConstantDefault();
    sig1.plug(&sig1);
  } catch( ... ) {
    SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				   "Freeze: syntax is freeze OBJ.SIG.",
				   "(while calling freeze %s ).",ssig1.c_str());
  }

}

void sotShellFunctions::
cmdSqueeze( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - squeeze  <mainObj.mainSig> <squeezeObj.sigIn> <squeezeObj.sigOut>"
	 << "\t\tIntercalate squeezeObj between mainObj and its source." <<endl;
      return;
    }

//   bool target;
//   {
//     string source; cmdArg >> ws >> source;
//     if( source=="source" ) { target=false; }
//     else if( source=="target" ) { target=true; }
//     else SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
// 					"Squeeze: first arg should be source or target",
// 					"(while calling squeeze %s ).",source.c_str());
//   }
  
  string ssigMain,ssigIn,ssigOut;
  cmdArg>>ssigMain>>ssigIn>>ssigOut;
  istringstream strMain(ssigMain);
  istringstream strIn(ssigIn);
  istringstream strOut(ssigOut);

  try {
    sotSignalAbstract<int> &sigMain = sotPool.getSignal(strMain  );
    sotSignalAbstract<int> &sigIn = sotPool.getSignal( strIn );
    sotSignalAbstract<int> &sigOut = sotPool.getSignal( strOut );

    sotSignalAbstract<int> *sigMainSource = sigMain.getPluged();
    if( sigMainSource==&sigMain ) 
      { 
	SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				       "The main signal is autopluged (or set constant). ",
				       "(while calling freeze %s ).",ssigMain.c_str());
      }
    
    sigMain.plug( &sigOut );
    sigIn.plug( sigMainSource );
  }
  catch( sotExceptionFactory& exc  ) {
    switch( exc.getCode() )
      {
      case sotExceptionFactory::UNREFERED_SIGNAL:
	SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				       "Sqeeze: unknown signal. ",
				       "(error while searching signal: %s ).",
				       exc.getMessage() );
	break;
      default:
	throw;
      }
  }
	
}


void sotShellFunctions::
cmdEnableTrace( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - debugtrace [{true|false}] [<filename>=" 
	 << sotDebugTrace::DEBUG_FILENAME_DEFAULT <<"]"
	 << "\t\tOpen/close the file <filename> for debug tracing." <<endl;
      return;
    }
  string opt,filename;
  
  cmdArg >> ws; 
  if( cmdArg.good() )
    {
      cmdArg>>opt>>ws;
      if( opt=="true" )
	if( cmdArg.good() ) 
	  { 
	    cmdArg>>filename; 
	    sotDebugTrace::openFile( filename.c_str() );
	  }
	else { sotDebugTrace::openFile(); }
      else sotDebugTrace::closeFile();
    }
  else 
    {
      if( sotDEBUGFLOW.outputbuffer.good() ) os << "true" <<endl; 
      else os << "false" <<endl; 
    }

}

void sotShellFunctions::
cmdSetPrompt( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - prompt [<string>] " 
	 << "\t\tSet/get the default prompt." <<endl;
      return;
    }

  string opt;
  cmdArg >> ws; 
  if( cmdArg. good() )
    {
      char buffer [80]; cmdArg .getline(buffer,80); 
      sotShell .prompt = buffer;
    } else { os << "Current prompt is <" << sotShell. prompt << ">." << endl; }
}

void sotShellFunctions::
cmdSleep( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - sleep [<float> secs] " 
	 << "\t\tSleep (time in secs)." <<endl;
      return;
    }

  string opt;
  cmdArg >> ws; 
  if( cmdArg. good() )
    {
      double secs; cmdArg >> secs;
      if( secs > 0 )
#ifndef WIN32
	usleep( int(secs*1000000) );
#else
	Sleep( int(secs*1000) );
#endif
    } else { /* TODO ERROR */ }
}

void sotShellFunctions::
cmdBeep( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - beep [<float> secs] " 
	 << "\t\tSend a bip to the std::cout." <<endl;
      return;
    }

  os << char(7) << "Beep!" << std::endl;
}

void sotShellFunctions::
cmdFlagSet( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - set <obj1.sig1(flag type)> {#&|}START:END"
	 << "\t\tSet or reset the flag value." <<endl;
      return;
    }
  
  try {
    sotSignal<sotFlags,int> &sig1 
      = dynamic_cast< sotSignal<sotFlags,int>& >( sotPool.getSignal(cmdArg) );
    
    sotDEBUG(25) << "set..."<<endl;
    sotFlags fl; try { fl = sig1.accessCopy(); } catch(...) {} 
    cmdArg >> std::ws >> fl;
    sotDEBUG(15) << "Fl=" << fl <<std::endl; 
    sig1 = fl;

  } catch( sotExceptionAbstract & err ) { throw;  }
  catch( ... ) {
    SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				   "setflag: sig should be of flag type. ",
				   "(while calling setflag).");
  }

}

void sotShellFunctions::
cmdCompletionList( const std::string cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - completion <filename>"
	 << "\t\tGenerate the completion list for current graph." <<endl;
      return;
    }
  
  try {

    std::string aFileName; cmdArg >> aFileName;
    std::ofstream completionFile((char *)aFileName.c_str());
    sotPool.writeCompletionList( completionFile );


  } catch( sotExceptionAbstract & err ) { throw;  }
  catch( ... ) {
    SOT_THROW sotExceptionFactory( sotExceptionFactory::SYNTAX_ERROR,
				   "setflag: sig should be of flag type. ",
				   "(while calling setflag).");
  }

}


extern "C" {
   sotShellFunctionRegisterer regFun2
   ( "try",boost::bind(sotShellFunctions::cmdTry,_1,_2,_3) );
   sotShellFunctionRegisterer regFun1
   ( "loadPlugins",boost::bind(sotShellFunctions::cmdLoadPlugins,_1,_2,_3) );
   sotShellFunctionRegisterer regFun3
    ( "displayPlugins",boost::bind(sotShellFunctions::cmdDisplayPlugins,_1,_2,_3) );
    sotShellFunctionRegisterer regFun4
    ( "factory",boost::bind(sotShellFunctions::cmdDisplayFactory,_1,_2,_3) );
    sotShellFunctionRegisterer regFun5
    ( "#",boost::bind(sotShellFunctions::cmdCommentary,_1,_2,_3) );
    sotShellFunctionRegisterer regFun6
    ( "dispmat",boost::bind(sotShellFunctions::cmdMatrixDisplay,_1,_2,_3) );
    sotShellFunctionRegisterer regFun7
    ( "unplug",boost::bind(sotShellFunctions::cmdUnplug,_1,_2,_3) );
    sotShellFunctionRegisterer regFun8
    ( "clearPlugin",boost::bind(sotShellFunctions::cmdClearPlugin,_1,_2,_3) );
    sotShellFunctionRegisterer regFun9
    ( "signalTime",boost::bind(sotShellFunctions::cmdSignalTime,_1,_2,_3) );
    sotShellFunctionRegisterer regFun10
    ( "synchro",boost::bind(sotShellFunctions::cmdSynchroSignal,_1,_2,_3) );
    sotShellFunctionRegisterer regFun11
    ( "echo",boost::bind(sotShellFunctions::cmdEcho,_1,_2,_3) );
    sotShellFunctionRegisterer regFun12
    ( "copy",boost::bind(sotShellFunctions::cmdCopy,_1,_2,_3) );
    sotShellFunctionRegisterer regFun13
    ( "freeze",boost::bind(sotShellFunctions::cmdFreeze,_1,_2,_3) );
    sotShellFunctionRegisterer regFun13b
    ( "squeeze",boost::bind(sotShellFunctions::cmdSqueeze,_1,_2,_3) );
    sotShellFunctionRegisterer regFun14
    ( "debugtrace",boost::bind(sotShellFunctions::cmdEnableTrace,_1,_2,_3) );
    sotShellFunctionRegisterer regFun15
    ( "prompt",boost::bind(sotShellFunctions::cmdSetPrompt,_1,_2,_3) );
    sotShellFunctionRegisterer regFun16
    ( "sleep",boost::bind(sotShellFunctions::cmdSleep,_1,_2,_3) );
    sotShellFunctionRegisterer regFun17
    ( "beep",boost::bind(sotShellFunctions::cmdBeep,_1,_2,_3) );
    sotShellFunctionRegisterer regFun18
    ( "setflag",boost::bind(sotShellFunctions::cmdFlagSet,_1,_2,_3) );
    sotShellFunctionRegisterer regFun19
    ( "completion",boost::bind(sotShellFunctions::cmdCompletionList,_1,_2,_3) );

}                                                       

