/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotShellFunctions.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SHELL_FUNCTIONS_HH__
#define __SOT_SHELL_FUNCTIONS_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotExceptionFactory.h>
#include <sot/sotInterpretor.h>

/* --- STD --- */
#include <string>
#include <map>
#include <sstream>

/* --- BOOST --- */
#include <boost/function.hpp>
#include <boost/bind.hpp>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotShellFunctions_EXPORTS) 
#    define SOTSHELLFUNCTIONS_EXPORT __declspec(dllexport)
#  else  
#    define SOTSHELLFUNCTIONS_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTSHELLFUNCTIONS_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#define SOT_STATIC_SHELL_FUNCTION( sotName ) \
  static void sotName( const std::string cmd,std::istringstream& args, \
                       std::ostream& os )

class SOTSHELLFUNCTIONS_EXPORT sotShellFunctions
{
 public:

  SOT_STATIC_SHELL_FUNCTION( cmdLoadPlugins );
  SOT_STATIC_SHELL_FUNCTION( cmdTry );
  SOT_STATIC_SHELL_FUNCTION( cmdDisplayPlugins );
  SOT_STATIC_SHELL_FUNCTION( cmdDisplayFactory );
  SOT_STATIC_SHELL_FUNCTION( cmdCommentary );
  SOT_STATIC_SHELL_FUNCTION( cmdMatrixDisplay );
  SOT_STATIC_SHELL_FUNCTION( cmdUnplug );
  SOT_STATIC_SHELL_FUNCTION( cmdClearPlugin );
  SOT_STATIC_SHELL_FUNCTION( cmdSignalTime );
  SOT_STATIC_SHELL_FUNCTION( cmdSynchroSignal );
  SOT_STATIC_SHELL_FUNCTION( cmdEcho );
  SOT_STATIC_SHELL_FUNCTION( cmdCopy );
  SOT_STATIC_SHELL_FUNCTION( cmdFreeze );
  SOT_STATIC_SHELL_FUNCTION( cmdSqueeze );
  SOT_STATIC_SHELL_FUNCTION( cmdEnableTrace );
  SOT_STATIC_SHELL_FUNCTION( cmdSetPrompt );
  SOT_STATIC_SHELL_FUNCTION( cmdSleep );
  SOT_STATIC_SHELL_FUNCTION( cmdBeep );
  SOT_STATIC_SHELL_FUNCTION( cmdFlagSet );
  SOT_STATIC_SHELL_FUNCTION( cmdCompletionList );
};




#endif //#ifndef __SOT_SHELL_FUNCTIONS_HH__
