/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotLeadFollow.cpp
 * Project:   SOT
 * Author:    Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Homotopy parameter for leader/follower switching.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotLeadFollow.h>

using namespace std;

#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotLeadFollow,"LeadFollow");

sotLeadFollow::
sotLeadFollow( const string& fName )
  : sotEntity( fName )
  ,dminSIN( NULL,"sotLeadFollow("+name+")::input(double)::dminIN")
  ,completionSIN( NULL,"sotLeadFollow("+name+")::input(double)::completionIN")
  ,positionSIN( NULL,"sotLeadFollow("+name+")::input(vector)::positionIN" )
  ,velocitySIN( NULL,"sotLeadFollow("+name+")::input(vector)::velocityIN")
  ,forceSIN( NULL,"sotLeadFollow("+name+")::input(vector)::forceIN")

  ,alphaSOUT( boost::bind(&sotLeadFollow::computeOutput,this, _1, _2),
	      completionSIN<<positionSIN<<velocitySIN<<forceSIN,
		"sotLeadFollow("+name+")::output(vector)::alpha" )

  ,beta(1)
  ,scale(0)
  ,dm_min(0.019)
  ,dm_max(0.025)
  ,mode(1)

{
  signalRegistration( dminSIN<<completionSIN<<positionSIN<<velocitySIN<<forceSIN<<alphaSOUT );
}

ml::Vector&
sotLeadFollow::computeOutput( ml::Vector& res,int time )
{
  res.resize(6); res.fill(1);
  double comp = completionSIN.access(time);
  double dm = dminSIN.access(time);
  double Y = (dm - dm_min) / (dm_max - dm_min);
  if(Y < 0){ Y = 0; }
  if(Y > 1){ Y = 1; }
  double a = 0.5 * ( 1 + tanh(1/Y - 1/(1-Y)) );

  for(int i = 1; i < 3; ++i) {
    if(mode == 0){ res(i) = 0; }
    else if(mode == 1){ res(i) = 1; }
    else if(mode == 2) {
      res(i) = a;
    }
    else {
      res(i) = 0.5 + scale * (0.5 * ( 1 + tanh(beta/comp - 1/(1-comp/beta)) ) - 0.5);
    }
  }

  return res;
}

void sotLeadFollow::display( std:: ostream& os ) const
{
  os << "LeadFollow<" << name << ">" << endl;
}

void sotLeadFollow::commandLine( const std::string& cmdLine,
				 std::istringstream& cmdArgs,
				 std::ostream& os )
{
  if (cmdLine == "help") {
    os << "LeadFollow:" << endl;
    os << " - beta" << endl;
    os << " - scale" << endl;
    os << " - dm_min" << endl;
    os << " - dm_max" << endl;
    os << " - mode" << endl;
  }
  else if (cmdLine == "beta") {
    double tmp;
    if( cmdArgs>>tmp ){ beta = tmp; }
    os << "beta = " << beta << endl;
  }
  else if (cmdLine == "scale" ) {
    double tmp;
    if( cmdArgs>>tmp){ scale = tmp; }
    os << "scale = " << scale << endl;
  }
  else if (cmdLine == "dm_min" ) {
    double tmp;
    if( cmdArgs>>tmp){ dm_min = tmp; }
    os << "dm_min = " << dm_min << endl;
  }
  else if (cmdLine == "dm_max" ) {
    double tmp;
    if( cmdArgs>>tmp){ dm_max = tmp; }
    os << "dm_max = " << dm_max << endl;
  }
  else if (cmdLine == "mode" ) {
    int tmp;
    if( cmdArgs>>tmp ){ mode = tmp; }
    os << "mode = " << mode << endl;
  }
  else { sotEntity::commandLine( cmdLine,cmdArgs,os ); }
}
