/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotLeadFollow.h
 * Project:   SOT
 * Author:    Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Homotopy parameter for leader/follower switching.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_LEAD_FOLLOW_HH__
#define __SOT_LEAD_FOLLOW_HH__


/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>


/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotLeadFollow_EXPORTS)
#    define SOTLEADFOLLOW_EXPORT __declspec(dllexport)
#  else  
#    define SOTLEADFOLLOW_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTLEADFOLLOW_EXPORT
#endif


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTLEADFOLLOW_EXPORT sotLeadFollow
  : public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  sotSignalPtr< double,int > dminSIN;
  sotSignalPtr< double,int > completionSIN;
  sotSignalPtr< ml::Vector,int > positionSIN;
  sotSignalPtr< ml::Vector,int > velocitySIN;
  sotSignalPtr< ml::Vector,int > forceSIN;
  sotSignalTimeDependant< ml::Vector,int > alphaSOUT;

 public:

  sotLeadFollow( const std::string& name );
  virtual ~sotLeadFollow( void ) {}

  virtual ml::Vector& computeOutput( ml::Vector& res, int time );

  virtual void display( std::ostream& ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 private:

  double beta;
  double scale;
  double dm_min;
  double dm_max;
  int mode;
};


#endif
