/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMotion.h
 * Project:   SOT
 * Author:    Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Generates a reference velocity to track a trajectory in the
 * Cartesian space
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_MOTION_HH__
#define __SOT_MOTION_HH__


/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* STL */
#include <vector>


/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotMotion_EXPORTS)
#    define SOTMOTION_EXPORT __declspec(dllexport)
#  else  
#    define SOTMOTION_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTMOTION_EXPORT
#endif


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTMOTION_EXPORT sotMotion
  : public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  sotSignalPtr< double,int > durationSIN;
  sotSignalTimeDependant< double,int > completionSOUT;
  sotSignalTimeDependant< ml::Vector,int > positionSOUT;

 public:

  sotMotion( const std::string& name );
  virtual ~sotMotion( void ) {}

  virtual ml::Vector& computeOutput( ml::Vector& res, int time );
  virtual double& computeCompletion( double& res, int time );

  virtual void display( std::ostream& ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 private:

  struct Poly
  {
    int order;
    ml::Vector a0;
    ml::Vector a1;
    ml::Vector a2;
    ml::Vector a3;

    ml::Vector& compute(ml::Vector& res, double t);
    ml::Vector& derivative(ml::Vector& res, double t);
  };

  void quadratic(Poly& Pi, const std::vector< ml::Vector >& b);
  void spline(Poly& Pi, const std::vector< ml::Vector >& b);
  void spline(Poly& Pi, const ml::Vector& b0, const ml::Vector& b1,
		     const ml::Vector& b2, const ml::Vector& df);
  void computeParameters();

 private:

  std::vector< ml::Vector > keys;
  std::vector< Poly > polys;

  static const double dt;
  double xf;
  double x;
  int startTime;
  bool saveStartTime;
  bool started;
};


#endif
