/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMatrixConstant.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotMatrixConstant.h>

#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotMatrixConstant,"MatrixConstant");

using namespace std;


/* --------------------------------------------------------------------- */
/* --- MATRIX ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */
void sotMatrixConstant::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs, 
	     std::ostream& os )
{
  if( cmdLine == "resize" )
    {
      cmdArgs >>rows>>cols;
      ml::Matrix m(rows,cols);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "fill" )
    {
      cmdArgs >>color;
      ml::Matrix m(rows,cols);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "ping")
    { SOUT.sotSignalAbstract<int>::setReady() ;   }
  else if( cmdLine == "eye") 
    {
      ml::Matrix m(rows,cols); m.setIdentity();
      SOUT.setConstant(m);
    }
  else if( cmdLine == "[]" ) 
    {
      unsigned int i,j; double v;
      cmdArgs >> i >> j >> v;
      ml::Matrix m = SOUT.accessCopy();
      m(i,j) = v;
      SOUT.setConstant(m);
    }
  else if( cmdLine == "help" )
    {
      os << "sotMatrixConstant"<<endl
	 << "  - resize i j\t\t:resize the output to a i x j zero matrix." <<endl
	 << "  - fill x\t\t:fill the matrix with <x> value." <<endl
	 << "  - eye\t\t:fill the matrix with 0s and a diag of 1."<<endl
	 << "  - [] i j x\t\t:set matrix[i,j] = x."<<endl;
    }
  else sotEntity::commandLine(cmdLine,cmdArgs,os);
}
    






