/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMatrixConstant.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotMatrixConstant.h>

#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotMatrixConstant,"MatrixConstant");

using namespace std;


/* --------------------------------------------------------------------- */
/* --- VECTOR ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */
void sotVectorConstante::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs, 
	     std::ostream& os )
{
  if( cmdLine == "resize" )
    {
      cmdArgs >>row;
      ml::Vector m(row);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "fill" )
    {
      cmdArgs >>color;
      ml::Vector m(row);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "help" )
    {
      os << "sotVectorConstante"<<endl
	 << "  - resize i \t\t:resize the output to a i vector filled as previously." <<endl
	 << "  - fill x\t\t:fill the vector with <x> value." <<endl;
    }
  else sotEntity::commandLine(cmdLine,cmdArgs,os);
}
/* --------------------------------------------------------------------- */
/* --- MATRIX ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */
void sotMatrixConstant::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs, 
	     std::ostream& os )
{
  if( cmdLine == "resize" )
    {
      cmdArgs >>row>>cols;
      ml::Matrix m(row,cols);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "fill" )
    {
      cmdArgs >>color;
      ml::Matrix m(row,cols);
      m.fill(color);
      SOUT.setConstant(m);
    }
  else if( cmdLine == "help" )
    {
      os << "sotMatrixConstant"<<endl
	 << "  - resize i j\t\t:resize the output to a i x j zero matrix." <<endl
	 << "  - fill x\t\t:fill the matrix with <x> value." <<endl;
    }
  else sotEntity::commandLine(cmdLine,cmdArgs,os);
}
    






