/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignal.t.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SIGNAL_TEMPLATE
#define __SOT_SIGNAL_TEMPLATE


#include <sot/sotSignal.h>
#include <sot/sotSignalCast.h>

#undef VP_TEMPLATE_DEBUG_MODE
#define VP_TEMPLATE_DEBUG_MODE 0
#include <sot/sotDebug.h>

#define __SOT_SIGNAL_INIT(name,Tcpy,Tref,TrefNC,mutex)  \
     sotSignalAbstract<Time>(name)               \
     ,signalType(SIGNAL_TYPE_DEFAULT)            \
     ,Tcopy1(Tcpy)                               \
     ,Tcopy2(Tcpy)                               \
     ,Tcopy(&Tcopy1)                             \
     ,Treference(Tref)                           \
     ,TreferenceNonConst(TrefNC)                 \
     ,Tfunction()                                \
     ,keepReference( KEEP_REFERENCE_DEFAULT )    \
     ,providerMutex(mutex)                    



template< class T,class Time >
sotSignal<T,Time>::
sotSignal( std::string name )
  :__SOT_SIGNAL_INIT(name,,NULL,NULL,NULL)
{
  return;
}

/* -------------------------------------------------------------------------- */


template< class T,class Time >
void sotSignal<T,Time>::
set( std::istringstream& stringValue )
{
  try 
    {
      (*this) = sotSignalCast<T>::cast( stringValue ); 
    }
  catch SOT_RETHROW
    catch (...)
      { SOT_THROW sotExceptionSignal( sotExceptionSignal::SET_IMPOSSIBLE,
				      "Set operation not possible with this signal. ",
				      "(bad cast while setting %s).",this->getName().c_str());
      }
   
}

template< class T,class Time >
void sotSignal<T,Time>::
get( std::ostream& os ) const
{
  try { sotSignalCast<T>::disp( this->accessCopy(),os ); }
  catch SOT_RETHROW
    catch (...) 
      { SOT_THROW sotExceptionSignal( sotExceptionSignal::SET_IMPOSSIBLE,
				      "get operation not possible with this signal. ",
				      "(bad cast while getting value from %s).",
				      sotSignalAbstract<Time>::getName().c_str());
      }
   
}

template< class T,class Time >
void sotSignal<T,Time>::
trace( std::ostream& os ) const
{
  try { sotSignalCast<T>::trace( this->accessCopy(),os ); }
  catch SOT_RETHROW
    catch (...) 
      { SOT_THROW sotExceptionSignal( sotExceptionSignal::SET_IMPOSSIBLE,
				      "TRACE operation not possible with this signal. ",
				      "(bad cast while getting value from %s).",
				      sotSignalAbstract<Time>::getName().c_str());
      }
   
}


/* -------------------------------------------------------------------------- */

template< class T,class Time >
const T& sotSignal<T,Time>::
setTcopy( const T& t ) 
{
  if( Tcopy==&Tcopy1 )
    {
      Tcopy2 = t;
      copyInit = true;
      Tcopy = &Tcopy2; 
      return Tcopy2;
    }
  else 
    {
      Tcopy1 = t;
      copyInit = true;
      Tcopy = &Tcopy1;
      return Tcopy1;
    }
}

template< class T,class Time >
T& sotSignal<T,Time>::
getTwork( void )
{
  if( Tcopy==&Tcopy1 ) return Tcopy2; else return Tcopy1;
}

template< class T,class Time >
const T& sotSignal<T,Time>::
getTwork( void ) const 
{
  if( Tcopy==&Tcopy1 ) return Tcopy2; else return Tcopy1;
}


template< class T,class Time >
const T& sotSignal<T,Time>::
switchTcopy( void )
{
  if( Tcopy==&Tcopy1 ) 
    {
      Tcopy = &Tcopy2;
      return Tcopy2; 
      }
  else
    {
      Tcopy = &Tcopy1;
      return Tcopy1;
    }
}



template< class T,class Time >
void sotSignal<T,Time>::
setConstant( const T& t ) 
{
  signalType = CONSTANT; setTcopy(t);
  setReady();
}


template< class T,class Time >
void sotSignal<T,Time>::
setReference( const T* t,Mutex *mutexref )
{
  signalType = REFERENCE; Treference = t; 
  providerMutex = mutexref;
  copyInit = false;
  setReady();
}

template< class T,class Time >
void sotSignal<T,Time>::
setReferenceNonConstant( T* t,Mutex *mutexref )
{
  signalType = REFERENCE_NON_CONST;
  Treference = t; 
  TreferenceNonConst = t; 
  providerMutex = mutexref;
  copyInit = false;
  setReady();
}

template< class T,class Time >
void sotSignal<T,Time>::
setFunction( boost::function2<T&,T&,Time> t,Mutex *mutexref )
{ 
  signalType = FUNCTION;
  Tfunction = t;
  providerMutex = mutexref;
  copyInit = false;
  setReady();
}

template< class T,class Time >
const T& sotSignal<T,Time>::
accessCopy( void ) const
{
  return *Tcopy; 
}



template< class T,class Time >
const T& sotSignal<T,Time>::
access(const Time& t)
{
  //sotTDEBUG(20) << "# In (" << sotSignalAbstract<Time>::name <<")"<<std::endl;
  switch( signalType )
    {
    case REFERENCE:
    case REFERENCE_NON_CONST:
      {
	if( NULL==providerMutex )
	  {
	    copyInit=true;
	    signalTime = t;
	    return setTcopy( *Treference ); 
	  }
	else
	  {
	    try
	      {
#ifdef HAVE_LIBBOOST_THREAD
		boost::try_mutex::scoped_try_lock lock(*providerMutex);
#endif
		copyInit = true;
		signalTime = t;
		return setTcopy(*Treference);
	      }
	    catch( MutexError le )
	      { return accessCopy(); }
	  }

	break;
      }

    case FUNCTION: 
      {
	if( NULL==providerMutex )
	  {
	    signalTime = t;
	    Tfunction(getTwork(),t); 
	    copyInit=true; 
	    return switchTcopy();
	  }
	else
	  {
	    try
	      {
#ifdef HAVE_LIBBOOST_THREAD
		boost::try_mutex::scoped_try_lock lock(*providerMutex);
#endif
		signalTime = t;
		Tfunction(getTwork(),t); 
		copyInit = true;
		return switchTcopy();
	      }
	    catch( MutexError le )
	      { return accessCopy(); }
	  }
	break;
      }
    case CONSTANT: 
    default:
      if( this->getReady() ) 
	{
	  setReady(false);
	  setTime(t);
	}
      return accessCopy();
    };
}



template< class T,class Time >
sotSignal<T,Time>& sotSignal<T,Time>::operator= ( const T& t )
{
  if( keepReference&&(REFERENCE_NON_CONST==signalType)&&(NULL!=TreferenceNonConst) )
    {
      if( NULL==providerMutex )
	{
	  setTcopy( t );
	  (*TreferenceNonConst)=t; 
	}
      else
	{
	  try
	    {
#ifdef HAVE_LIBBOOST_THREAD
	      boost::try_mutex::scoped_try_lock lock(*providerMutex);
#endif
	      setTcopy(t);
	      (*TreferenceNonConst)=t;
	    }
	  catch( MutexError le )
	    { /* TODO ERROR */ }
	}
    }
  else
    { setConstant( t ); }
  return *this; 
}


template< class T,class Time >
std::ostream& sotSignal<T,Time>::
display (std::ostream& os) const
{
  os<<"Sig:"<<this->name<<" (Type ";
  switch( this->signalType ) 
    {
    case sotSignal<T,Time>::CONSTANT: os<< "Cst";break; 
    case sotSignal<T,Time>::REFERENCE: os<<"Ref"; break;
    case sotSignal<T,Time>::REFERENCE_NON_CONST: os<<"RefNonCst"; break;
    case sotSignal<T,Time>::FUNCTION: os<<"Fun"; break; 
    }
  return os<<")";
}




#endif // #ifndef __SOT_SIGNAL_TEMPLATE

/*
 * Local variables:
 * c-basic-offset: 2
 * End:
 */

