/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalArray.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_SIGNAL_ARRAY_HH
#define __SOT_SIGNAL_ARRAY_HH

#include <sot/sotSignalAbstract.h>
#include <sot/sot-0API.h>

template<class Time>
class sotSignalArray;

template<class Time>
class sotSignalArray_const
{ 
 public:
  static const int DEFAULT_SIZE = 20; 

 protected:
  const sotSignalAbstract<Time> ** const_array;
  unsigned int size,rank; 

 public:
  sotSignalArray_const<Time>( const unsigned int& sizeARG=DEFAULT_SIZE ) 
    : const_array(NULL),size(sizeARG),rank(0)
    { createArray(); }

  sotSignalArray_const<Time>( const sotSignalAbstract<Time>& sig) 
    : const_array(NULL),size(1),rank(0)
    { createArray(); addElmt(&sig); }

   sotSignalArray_const<Time> ( const sotSignalArray<Time>& siga )
    : const_array(NULL),size(DEFAULT_SIZE),rank(siga.getSize())
    {
      createArray();
      for( unsigned int i=0;i<rank;++i ) const_array[i]=&siga[i];
    }

   sotSignalArray_const<Time> ( const sotSignalArray_const<Time>& siga )
    : const_array(NULL),size(DEFAULT_SIZE),rank(siga.getSize())
    {
      createArray();
      for( unsigned int i=0;i<rank;++i ) const_array[i]=&siga[i];
    }

  virtual ~sotSignalArray_const<Time>( void )
    {
      if( NULL!=const_array ) delete [] const_array;
    }

 protected:
  void createArray( void )
    {
      if(0<size) {  const_array = new const sotSignalAbstract<Time>* [size]; }
    }
  void addElmt( const sotSignalAbstract<Time>* el ) 
    { if(rank<size) const_array[rank++] = el;  }

 public: 
  virtual sotSignalArray_const<Time>& operator<< (const sotSignalAbstract<Time>& sig)
    { addElmt( &sig ); return *this; }

 public:
  virtual const sotSignalAbstract<Time>& operator[] ( const unsigned int& idx ) const
    { return *const_array[idx];   }
  virtual unsigned int getSize( void ) const { return rank; }

/*   sotSignalArray_const<Time>& operator= ( sotSignalArray<Time> & siga ) */
/*     { */
/*       for( unsigned int i=0;i<rank;++i ) const_array[i]=&siga[i];  */
/*       rank=siga.getSize(); */
/*     } */
    

  //  friend class sotSignalArray<Time>;
};


template<class Time>
sotSignalArray_const<Time> operator<<( const sotSignalAbstract<Time>& sig1,
				       const sotSignalAbstract<Time>& sig2 )
{ sotSignalArray_const<Time> res(20); res<<sig1; res<<sig2; return res;}


/* ---- */


template<class Time>
class sotSignalArray
:public sotSignalArray_const<Time>
{ 
 public:
  using sotSignalArray_const<Time>::DEFAULT_SIZE;
  using sotSignalArray_const<Time>::size;
  using sotSignalArray_const<Time>::rank;
 protected:
  mutable sotSignalAbstract<Time> ** array;

 public:
  sotSignalArray<Time>( const unsigned int& sizeARG=DEFAULT_SIZE ) 
    :sotSignalArray_const<Time>(0)
    { size=sizeARG; createArray(); }

  sotSignalArray<Time>( sotSignalAbstract<Time>& sig) 
    : sotSignalArray_const<Time>(0)
    { size=1; createArray(); addElmt(&sig); }

   sotSignalArray<Time> ( const sotSignalArray<Time>& siga )
     :sotSignalArray_const<Time>( DEFAULT_SIZE )
     ,array(NULL)
    {
      rank = siga.getSize();
      createArray();
      for( unsigned int i=0;i<rank;++i ) array[i]=&siga[i];
    }
  virtual ~sotSignalArray<Time>( void )
    {
      if( NULL!=array ) delete [] array;
    }

 protected:
  void createArray( void )
    {
      if(0<size) {  array = new sotSignalAbstract<Time>* [size]; }
    }
  void addElmt( sotSignalAbstract<Time>* el ) 
    { if(rank<size) array[rank++] = el;  }

 public: //protected:
  virtual sotSignalArray<Time>& operator<< (sotSignalAbstract<Time>& sig)
    { addElmt( &sig ); return *this; }

  virtual sotSignalArray_const<Time> operator<< (const sotSignalAbstract<Time>& sig) const
    {
      sotSignalArray_const<Time> res(size); 
      res=*this; 
      res<<sig; 
      return res; 
    }


 public:
  virtual sotSignalAbstract<Time>& operator[] ( const unsigned int& idx ) const
    { 
      return *array[idx];
    }
};




template<class Time>
sotSignalArray<Time> operator<<( sotSignalAbstract<Time>& sig1,
				 sotSignalAbstract<Time>& sig2 )
{ sotSignalArray<Time> res(20); res<<sig1; res<<sig2; return res;}






SOT_EXPORT extern sotSignalArray<int> sotNOSIGNAL;


#endif /* #ifndef __SOT_SIGNAL_ARRAY_HH */
