/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalCast.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotPool.h>
#include <sot/sotSignalCast.h>
#include <iomanip>
#include <sot/sotFeatureAbstract.h>

#ifdef WIN32
#include < Windows.h >
#endif

sotSignalCast_sotFeatureAbstractPtr sotSignalCast<sotSignalCast_sotFeatureAbstractPtr>::
cast( std::istringstream& iss )
{
  sotSignalCast_sotFeatureAbstractPtr ref; 
  std::string name; iss >> name; 
  if( name.length())
    {
      ref = &sotPool.getFeature(name); 
    }
  else { ref = NULL; }
  return ref;
}

void sotSignalCast<sotSignalCast_sotFeatureAbstractPtr>::
disp(  const sotSignalCast_sotFeatureAbstractPtr & t,std::ostream& os )
{
  if( t ) { t->display(os); os<<std::endl; }
  else { os << "NULL" << std::endl; }
}



struct timeval sotSignalCast<struct timeval>::
cast( std::istringstream& iss )
{
  int u,s; iss >> s >> u;
  struct timeval t; t.tv_sec = s; t.tv_usec = u;
  return t;
}
void sotSignalCast<struct timeval>::
disp( const struct timeval& t,std::ostream& os )
{
  os << t.tv_sec << "s "<< t.tv_usec << "ms";
}





/* --- VECTOR ---------------------------------------------------------- */
/* --- VECTOR ---------------------------------------------------------- */
/* --- VECTOR ---------------------------------------------------------- */

maal::boost::DisplayType sotSignalCast<ml::Vector>::displayType = maal::boost::MATLAB;

ml::Vector sotSignalCast<ml::Vector>::
cast( std::istringstream& iss )
{
   ml::Vector res; 
   iss >> res; 
   return res;
}
void sotSignalCast<ml::Vector>::
disp( const ml::Vector& v1,std::ostream& os )
{
  switch( displayType )
    {
    case maal::boost::COMPLET:
      for( unsigned int i=0;i<v1.size();++i ) 
	{ if(fabs(v1(i))>1e-6) os << v1(i) << "\t"; else os<<0.00<<"\t"; } 
      break;
    case maal::boost::MATLAB:
      os << "[ ";
      for( unsigned int i=0;i<v1.size();++i )
	{
	  os <<  v1(i);
	  if( v1.size()!=i+1 ) { os << ", "; }
	}
      os << "]" << std::endl; 
      break;
    case maal::boost::SIMPLE:
    default:
      os<<v1.accessToMotherLib() << std::endl;
      break;
    }
}

void sotSignalCast<ml::Vector>::
trace( const ml::Vector& t,std::ostream& os )

{
  for( unsigned int i=0;i<t.size();++i ) 
    { os << std::setprecision(9)<<t(i) << "\t"; } 
}




/* --- MATRIX --------------------------------------------------- */
/* --- MATRIX --------------------------------------------------- */
/* --- MATRIX --------------------------------------------------- */

ml::Matrix sotSignalCast<ml::Matrix>::
cast( std::istringstream& iss )
{
   ml::Matrix res; 
   iss >> res; 
   return res;
}

void sotSignalCast<ml::Matrix>::
disp( const ml::Matrix& m1,std::ostream& os )
{
  switch( sotSignalCast<ml::Vector>::displayType)
    {
    case ml::COMPLET:
      for( unsigned int i=0;i<m1.nbRows();++i ) 
	{
	  for( unsigned int j=0;j<m1.nbCols();++j )
	    { if(fabs(m1(i,j))>1e-6) os << m1(i,j) << "\t"; else os<<0.00<<"\t"; } 
	  os<<std::endl;
	}
      break;
    case ml::MATLAB:
      os << "[ ";
      for( unsigned int i=0;i<m1.nbRows();++i )
	{
	  for( unsigned int j=0;j<m1.nbCols();++j )
	    {
	      os <<  m1(i,j) << ", ";
	    }
	  if( m1.nbRows()!=i+1 ) { os << ";" << std::endl; }
	  else { os << "]" << std::endl; }
	}
      break;
    case ml::SIMPLE:
    default:
      os<<m1.matrix<<std::endl; 
      break;
    }   
}

void sotSignalCast<ml::Matrix>::
trace( const ml::Matrix& t,std::ostream& os )
{  
   for( unsigned int i=0;i<t.nbRows();++i ) 
     for( unsigned int j=0;j<t.nbCols();++j )
       { if(fabs(t(i,j))>1e-6) os << t(i,j) << "\t"; else os<<0.00<<"\t"; } 
}
