/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalCast.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SIGNAL_CAST_HH__
#define __SOT_SIGNAL_CAST_HH__


#include <sot/sotFlags.h>
#include <MatrixAbstractLayer/boost.h>
#include <sot/sotMatrixHomogeneous.h>
#include <sot/sotMatrixTwist.h>
#include <sot/sotVectorUTheta.h>
#include <sot/sotVectorQuaternion.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotMatrixForce.h>
#include <sot/sot-0API.h>
#ifdef WIN32
#include <sot/sotUtilsWindows.h>
#endif

/* -------------------------------------------------------------------------- */
/* --- CLASS ---------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
class sotFeatureAbstract;

template< class T >
class sotSignalCast
{
public:
  SOT_EXPORT static T cast( std::istringstream& stringValue ) { throw 1;}
  SOT_EXPORT static void disp( const T& t,std::ostream& os )  { throw 1;  }
  SOT_EXPORT static void trace( const T& t,std::ostream& os ) { disp(t,os); }
};

/* -------------------------------------------------------------------------- */
/* --- MACROS --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

/* Standard definition macros: the three functions can be specified
 * in the macros. To define then in the cpp, just put ';' in the args.
 */
#define SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,CAST,DISP,TRACE)            \
template<>                                                               \
class sotSignalCast<TYPE>                                                \
{                                                                        \
public:                                                                  \
    SOT_EXPORT static TYPE cast( std::istringstream& iss )         CAST             \
    SOT_EXPORT static void disp( const TYPE& t,std::ostream& os )  DISP             \
    SOT_EXPORT static void trace( const TYPE& t,std::ostream& os ) TRACE            \
}

/* Standard definition macros: the functions <cast> and <disp> have
 * to be implemented in the cpp files. The function <trace> is
 * implemented as a proxy on <disp>.
 */
#define SOT_SIGNAL_CAST_DEFINITION_HPP(TYPE)                             \
 SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,;,;,{ disp(t,os); })

/* Lazy definition: <cast> and <disp> are to proxys on the standard
 * std input (>>) and output (<<). The function <trace> has to be 
 * implemented in the cpp.
 */
#define SOT_SIGNAL_CAST_DEFINITION_TRACE_HPP(TYPE,TRACE)                 \
 SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,                                   \
 {TYPE res; iss >> res; return res; },                                   \
 { os << t <<std::endl; },                                               \
 TRACE )

/* Lazy lazy definition: the three functions are implemented as
 * proxys on std::io operation. 
 */
#define SOT_SIGNAL_CAST_DEFINITION(TYPE)                                 \
 SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,                                   \
 {TYPE res; iss >> res; return res; },                                   \
 { os << t <<std::endl; },                                               \
 { disp(t,os); })

/* Lazy definition of <cast> and <disp> with implementation of
 * <trace> in the cpp. 
 */
#define SOT_SIGNAL_CAST_DEFINITION_TRACE(TYPE)         \
 SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,                                   \
 {TYPE res; iss >> res; return res; },                                   \
 { os << t <<std::endl; },                                               \
 ;)

/* -------------------------------------------------------------------------- */
/* --- MACROS --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

/* --- OTHER --- */
SOT_SIGNAL_CAST_DEFINITION(double);
SOT_SIGNAL_CAST_DEFINITION(int);
SOT_SIGNAL_CAST_DEFINITION(unsigned int);
SOT_SIGNAL_CAST_DEFINITION(sotFlags);

typedef sotFeatureAbstract* sotSignalCast_sotFeatureAbstractPtr  ;

SOT_SIGNAL_CAST_DEFINITION_HPP( sotSignalCast_sotFeatureAbstractPtr );
SOT_SIGNAL_CAST_DEFINITION_HPP( struct timeval );

/* --- Matrices and Vectors --- */
#define SOT_SIGNAL_CAST_DEFINITION_MATRIX(TYPE)         \
SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,{TYPE res; iss >> res; return res; }, \
                                { sotSignalCast<ml::Matrix>::disp(t,os); }, \
                                { sotSignalCast<ml::Matrix>::trace(t,os); })
//SOT_SIGNAL_CAST_DEFINITION_TRACE_HPP(TYPE,{ sotSignalCast<ml::Matrix>::trace(t,os); })

#define SOT_SIGNAL_CAST_DEFINITION_VECTOR(TYPE)         \
SOT_SIGNAL_CAST_FULL_DEFINITION(TYPE,{TYPE res; iss >> res; return res; }, \
                                { sotSignalCast<ml::Vector>::disp(t,os); }, \
                                { sotSignalCast<ml::Vector>::trace(t,os); })
//SOT_SIGNAL_CAST_DEFINITION_TRACE_HPP(TYPE,{ sotSignalCast<ml::Vector>::trace(t,os); })

SOT_SIGNAL_CAST_FULL_DEFINITION(maal::boost::Vector,; SOT_EXPORT static maal::boost::DisplayType displayType;,;,;);
SOT_SIGNAL_CAST_FULL_DEFINITION(maal::boost::Matrix,;,;,;);

/* All the followings are defined with proxys on the equivalent
 * functions ml:: based.
 */
SOT_SIGNAL_CAST_DEFINITION_VECTOR(sotVectorUTheta);
SOT_SIGNAL_CAST_DEFINITION_VECTOR(sotVectorQuaternion);
SOT_SIGNAL_CAST_DEFINITION_VECTOR(sotVectorRollPitchYaw);
SOT_SIGNAL_CAST_DEFINITION_MATRIX(sotMatrixRotation);
SOT_SIGNAL_CAST_DEFINITION_MATRIX(sotMatrixHomogeneous);
SOT_SIGNAL_CAST_DEFINITION_MATRIX(sotMatrixTwist);
SOT_SIGNAL_CAST_DEFINITION_MATRIX(sotMatrixForce);


#endif // #ifndef __SOT_SIGNAL_CAST_HH__



