/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalPtr.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_SIGNAL_PTR_HH
#define __SOT_SIGNAL_PTR_HH

#include <iostream>

#include <sot/sotExceptionSignal.h>
#include <sot/sotSignal.h>

template< class T,class Time >
class sotSignalPtr 
: public sotSignal<T,Time>
{
 public:
    using sotSignalAbstract<Time>::getName;

 protected: 
    sotSignal<T,Time>* signalPtr;
    bool modeNoThrow;
    bool transmitAbstract; 
    sotSignalAbstract<Time> * abstractTransmitter;
    T* transmitAbstractData;

    inline bool autoref( void ) const { return(signalPtr==this); }

 public: /* --- CONSTRUCTORS --- */

    sotSignalPtr( sotSignal<T,Time>* ptr,std::string name="" ) 
	: sotSignal<T,Time>(name)
	,signalPtr(ptr),modeNoThrow(false)
	,transmitAbstract(false)
	,abstractTransmitter(NULL) {}
    virtual ~sotSignalPtr( void ) { signalPtr = NULL; }
    
 public: /* --- PLUG-IN OPERATION --- */

    sotSignal<T,Time>* getPtr ( void ); // throw
    const sotSignal<T,Time>* getPtr ( void ) const; // throw
    sotSignalAbstract<Time>* getAbstractPtr ( void ); // throw
    const sotSignalAbstract<Time>* getAbstractPtr ( void ) const; // throw
    virtual void plug( sotSignalAbstract<Time>* ref );
    virtual void unplug( void ) { plug(NULL); }
    virtual bool isPluged( void ) const { return (NULL!=signalPtr); } 
    virtual sotSignalAbstract<Time>* getPluged( void ) const { return signalPtr; } 
    virtual bool isAbstractPluged( void ) const; 
    virtual const Time& getTime( void ) const;
    
    /* Equivalent operator-like definitions. */
    inline sotSignal<T,Time>* operator->() { return getPtr(); }
    inline const sotSignal<T,Time>* operator->() const { return getPtr(); }
    inline sotSignal<T,Time>& operator*( void ) { return *getPtr(); }
    inline const sotSignal<T,Time>& operator*( void ) const { return *getPtr(); }
    inline operator bool ( void ) const { return isPluged(); }

 public: /* --- INHERITANCE --- */

    virtual bool needUpdate( const Time& t ) const ;
    virtual std::ostream& writeGraph(std::ostream& os) const;
    virtual std::ostream& display( std::ostream& os ) const;

    /* For compatibility, .access() is equivalent to ->access(). For explicite
     * pointer dereference :
     * Prefere ->() to () 
     */
    virtual const T& operator()( const Time & t );
    /* Similarly, Prefere ->access to .access 
     */
    virtual const T& access( const Time & t );
    virtual const T& accessCopy( void ) const;

    inline void setConstantDefault( const T& t )
	{ sotSignal<T,Time>::setConstant(t); modeNoThrow = true; }
    virtual inline void setConstantDefault( void )
	{ setConstantDefault( accessCopy() ); }
    inline void unsetConstantDefault( void ){ modeNoThrow = false; }


 public:  /* --- INHERITANCE --- */
    
    /* SignalPtr could be used as a classical signal, through the normal
     * setting functions. The behavior is to plugged the signalPtr on
     * the classical mother sotSignal layer of the object. 
     */
    virtual void setConstant( const T& t )
	{ plug(this); sotSignal<T,Time>::setConstant(t); }
    virtual void setReference( const T* t,typename sotSignal<T,Time>::Mutex *m=NULL )
	{ plug(this); sotSignal<T,Time>::setReference(t,m); }
    virtual void setFunction( boost::function2<T&,T&,Time> t,typename sotSignal<T,Time>::Mutex *m=NULL )
	{  plug(this);   sotSignal<T,Time>::setFunction(t,m); }

/*     template< class Provider > */
/* 	void setFunction( T& (Provider::*fun)(Time,T&),Provider& obj, */
/* 		  boost::try_mutex *mutexref=NULL ) */
/* 	{ plug(this); sotSignal<T,Time>::setFunction(fun,obj,mutexref); } */

    virtual inline sotSignal<T,Time>& operator= ( const T& t )
	{ setConstant( t ); return *this; }


    virtual std::ostream&  displayDependancies( std::ostream& os,const int depth=-1,
						std::string space="",
						std::string next1="",
						std::string next2="" ) const;
    
 protected: // Interdiction of the rest of the heritage
    
    virtual void addDependancy( const sotSignalAbstract<Time>& signal ) {}
    virtual void removeDependancy( const sotSignalAbstract<Time>& signal ) {}
    virtual void clearDependancies( void ) {}
    
};

#include <sot/sotSignalPtr.t.cpp>


#endif // #ifndef __SOT_SIGNAL_PTR_HH

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */

