/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalPtr.t.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_SIGNAL_PTR_TCPP
#define __SOT_SIGNAL_PTR_TCPP

#include <sot/sotSignalPtr.h>

#undef VP_TEMPLATE_DEBUG_MODE
#define VP_TEMPLATE_DEBUG_MODE 0
#include <sot/sotDebug.h>


template< class T,class Time >
bool sotSignalPtr<T,Time>::
isAbstractPluged( void ) const 
{
  return ( (NULL!=signalPtr)||(abstractTransmitter) ); 
} 


template< class T,class Time >
sotSignal<T,Time>* sotSignalPtr<T,Time>::
getPtr ( void ) 
{
  sotTDEBUGIN(25);
  if(! isPluged() )
    SOT_THROW sotExceptionSignal( sotExceptionSignal::NOT_INITIALIZED,
				  "In sotSignalPtr: SIN ptr not set.",
				  " (in signal <%s>)",getName().c_str());
  sotTDEBUGOUT(25);
  return signalPtr; 
}

template< class T,class Time >
const sotSignal<T,Time>* sotSignalPtr<T,Time>::
getPtr ( void ) const
{
  sotTDEBUGIN(25) << sotSignalAbstract<Time>::name <<"("<< isPluged() <<")"
		  << this << "->"<<signalPtr <<std::endl;
  sotTDEBUGIN(25);
  if(! isPluged() )
    { SOT_THROW sotExceptionSignal( sotExceptionSignal::NOT_INITIALIZED,
				    "In sotSignalPtr: SIN ptr not set.",
				    " (in signal <%s>)",getName().c_str()); }
  sotTDEBUGOUT(25);
  return signalPtr; 
}

template< class T,class Time >
sotSignalAbstract<Time>* sotSignalPtr<T,Time>::
getAbstractPtr ( void ) 
{
  if(! isAbstractPluged() )
    { SOT_THROW sotExceptionSignal( sotExceptionSignal::NOT_INITIALIZED,
				    "In sotSignalPtr: SIN ptr not set.",
				    " (in signal <%s>)",getName().c_str()); }
  if( NULL!=signalPtr ) return signalPtr;
  else return abstractTransmitter;
}

template< class T,class Time >
const sotSignalAbstract<Time>* sotSignalPtr<T,Time>::
getAbstractPtr ( void ) const
{
  if(! isAbstractPluged() )
    { SOT_THROW sotExceptionSignal( sotExceptionSignal::NOT_INITIALIZED,
				    "In sotSignalPtr: SIN ptr not set.",
				    " (in signal <%s>)",getName().c_str()); }
  if( NULL!=signalPtr ) return signalPtr;
  else return abstractTransmitter;
}

template< class T,class Time >
void sotSignalPtr<T,Time>::
plug( sotSignalAbstract<Time>* unknown_ref ) 
{
  sotTDEBUGIN(5);
  if(! unknown_ref )
    { 
      signalPtr = NULL;
      transmitAbstract = false;
      sotTDEBUGOUT(5);
      return;
    }

  sotTDEBUG(5) << "# In  T = "<< getName() <<" =" 
	       <<typeid( sotSignal<T,Time>::Tcopy1 ) .name() << "{ "<<std::endl;

     
  sotSignal<T,Time> * ref = dynamic_cast< sotSignal<T,Time>* > (unknown_ref);
  if( NULL==ref )
    {
      try {
	unknown_ref->checkCompatibility(); 
      }
      catch( T* t )
	{
	  sotTDEBUG(25) << "Cast THROW ok."<< std::endl;
	  sotSignal<T,Time>::setReference( t );
	  transmitAbstract = true; abstractTransmitter = unknown_ref;
	  transmitAbstractData = t;
	}
      catch(...)
	{
	  sotTDEBUG(25) << "FFatal error."<< std::endl;
	  transmitAbstract = false;
	  SOT_THROW sotExceptionSignal( sotExceptionSignal::PLUG_IMPOSSIBLE,
					"Compl. Uncompatible types for plugin.",
					"(while trying to plug <%s> on <%s>).",
					unknown_ref->getName().c_str(),
					this->getName().c_str());
	}
    }
  else
    {
      sotTDEBUG(25) << "Cast ok."<< std::endl;
      transmitAbstract = false;
      signalPtr = ref; 
    }
  sotTDEBUGOUT(5);
}

template< class T,class Time >
bool sotSignalPtr<T,Time>::
needUpdate( const Time& t ) const
{ 
  if( (isAbstractPluged())&&(!autoref()) ){ return getAbstractPtr()->needUpdate(t);} 
  else return sotSignal<T,Time>::needUpdate(t);
}

template< class T,class Time >
const Time& sotSignalPtr<T,Time>::getTime( void ) const
{
  if(  (isAbstractPluged())&&(!autoref()) ){ return getAbstractPtr()->getTime();} 
  return sotSignal<T,Time>::getTime();
}


template< class T,class Time >
const T& sotSignalPtr<T,Time>::
operator()( const Time & t )
{
  return access(t); 
}

template< class T,class Time >
const T& sotSignalPtr<T,Time>::
access( const Time & t ) 
{
  sotTDEBUGIN(15);
  if( modeNoThrow&&(! isPluged())&&sotSignal<T,Time>::copyInit )
    {
      sotTDEBUGOUT(15);
      return sotSignal<T,Time>::accessCopy();
    }
  else if( autoref() )  
    {
      sotTDEBUGOUT(15);
      return sotSignal<T,Time>::access(t);
    }
  else if( transmitAbstract )
    {
      abstractTransmitter->recompute(t);
      sotTDEBUGOUT(15);
      return *transmitAbstractData;
    }
  else
    {
      sotTDEBUGOUT(15);
      return getPtr()->access(t);
    }
}
template< class T,class Time >
const T& sotSignalPtr<T,Time>::
accessCopy( void ) const
{
  if( modeNoThrow&&(! isPluged())&&sotSignal<T,Time>::copyInit )
    return sotSignal<T,Time>::accessCopy();
  else if( autoref() ) return sotSignal<T,Time>::accessCopy();
  else if( transmitAbstract ) return *transmitAbstractData;
  else return getPtr()->accessCopy();
}
template< class T, class Time>
std::ostream & sotSignalPtr<T,Time>::
writeGraph(std::ostream &os) const
{
  std::string LeaderLocalName;
  std::string LeaderNodeName;
  sotSignal<T,Time>::ExtractNodeAndLocalNames(LeaderLocalName,LeaderNodeName);
  if ( isAbstractPluged() && !autoref())
    {
      std::string itLocalName,itNodeName;
      getAbstractPtr()->ExtractNodeAndLocalNames(itLocalName,itNodeName);
      os << "\t\"" << itNodeName << "\" -> \"" << LeaderNodeName << "\"" << std::endl
	 << "\t [ headlabel = \"" << LeaderLocalName 
	 << "\" , taillabel = \"" << itLocalName 
	 << "\", fontsize=7, fontcolor=red ]" << std::endl;
      
    }
  return os;
}

template< class T,class Time >
std::ostream& sotSignalPtr<T,Time>::
display( std::ostream& os ) const
{
  sotTDEBUGIN(25) << sotSignalAbstract<Time>::name << this << "||" << isPluged() << "||"<<signalPtr;
  { sotSignal<T,Time>::display(os); }

  if( (isAbstractPluged())&&(!autoref()) ) 
    {
      // 	    sotTDEBUG(25) << "Display pointed."<<std::endl;
      // 	    getPtr()->display(os<<"PTR->"); 
      os << " -->-- PLUGED";
    }
  else 
    { 
      if(! isAbstractPluged()) os << " UNPLUGED";
      else if(autoref()) os << " AUTOPLUGED";
    }
     
  sotTDEBUGOUT(25);
  return os;
}

template< class T,class Time >
std::ostream& sotSignalPtr<T,Time>::
displayDependancies( std::ostream& os,const int depth,
		     std::string space,
		     std::string next1,std::string next2 ) const
{
  sotTDEBUGIN(25);
  if( (isAbstractPluged())&&(!autoref()) )
    { getAbstractPtr()->displayDependancies(os,depth,space,next1+"-- "
				    +sotSignalAbstract<Time>::name+" -->",next2); }
  else 
    {
      sotSignalAbstract<Time>::displayDependancies(os,depth,space,next1,next2); 
    }
  sotTDEBUGOUT(25);
  return os;
}


#endif // #ifndef __SOT_SIGNAL_PTR_TCPP

/*
 * Local variables:
 * c-basic-offset: 2
 * End:
 */

