/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotSignalTimeDependant.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_DEPENDANT_SIGNAL_HH
#define __SOT_DEPENDANT_SIGNAL_HH

#include <sot/sotSignal.h>

#include <sot/sotTimeDependancy.h>

template< class T,class Time >
class sotSignalTimeDependant  
: public sotSignal<T,Time>
, public sotTimeDependancy<Time> 
{

  //sotTimeDependancy<Time> timeDependancy;

 public:
  sotSignalTimeDependant( std::string name = "" );
  sotSignalTimeDependant( const sotSignalArray_const<Time> & arr
			  ,std::string name = "" );
  sotSignalTimeDependant( boost::function2<T&,T&,Time> t
			  ,const sotSignalArray_const<Time> & sig
			  ,std::string name = "" );
  
  virtual ~sotSignalTimeDependant( void ) {}
  
  inline const T& operator() ( const Time& t1 ){ return access(t1); }
  const T& access( const Time& t1 );
  
  virtual void addDependancy( const sotSignalAbstract<Time>& signal ) ;
  virtual void removeDependancy( const sotSignalAbstract<Time>& signal ) ;
  virtual void clearDependancies( void );

  std::ostream& writeGraph(std::ostream &os) const
    { return os;}

  std::ostream& displayDependancies( std::ostream& os,const int depth=-1,
				     std::string space="",
				     std::string next1="",std::string next2="" ) const
    { return sotTimeDependancy<Time>::displayDependancies( os,depth,space,next1,next2 ); }

  
  virtual bool needUpdate( const Time& t ) const;
  virtual void setPeriodTime( const Time& p ) ;
  virtual Time getPeriodTime( void ) const;

};


/* -------------------------------------------- */


template < class T,class Time >
sotSignalTimeDependant<T,Time>::
  sotSignalTimeDependant( std::string name )
    :sotSignal<T,Time>(name)
     ,sotTimeDependancy<Time>(this)
    {}

template< class T,class Time>
sotSignalTimeDependant<T,Time>::
  sotSignalTimeDependant( const sotSignalArray_const<Time> & arr
			  ,std::string name )
    :sotSignal<T,Time>(name)
    ,sotTimeDependancy<Time>(this,arr)
    {}

template< class T,class Time>
sotSignalTimeDependant<T,Time>::
  sotSignalTimeDependant( boost::function2<T&,T&,Time> t
			  ,const sotSignalArray_const<Time> & sig
			  ,std::string name )
    :sotSignal<T,Time>(name)
    ,sotTimeDependancy<Time>(this,sig)
    { setFunction(t); }
 
template< class T,class Time>
  const T& sotSignalTimeDependant<T,Time>::
access( const Time& t1 )
    {
      const bool up = sotTimeDependancy<Time>::needUpdate(t1); 
      //sotSignalAbstract<Time>::setReady(false);    

/*       std::cout << "Time before: "<< signalTime << " -- "   */
/*   		<< t1<< "  -> Up: "<<up <<std::endl ;   */
      if( up )
	{
	  sotTimeDependancy<Time>::lastAskForUpdate = false;
	  const T& Tres = sotSignal<T,Time>::access(t1); 
	  sotSignalAbstract<Time>::setReady(false);    
	  return Tres;
	}
      else { return sotSignal<T,Time>::accessCopy(); }
    }


template< class T,class Time>
   void sotSignalTimeDependant<T,Time>::
addDependancy( const sotSignalAbstract<Time>& signal ) 
    { sotTimeDependancy<Time>::addDependancy(signal); }

template< class T,class Time>
   void sotSignalTimeDependant<T,Time>::
removeDependancy( const sotSignalAbstract<Time>& signal ) 
    { sotTimeDependancy<Time>::removeDependancy(signal); }

template< class T,class Time>
   void sotSignalTimeDependant<T,Time>::
clearDependancies( void )
    { sotTimeDependancy<Time>::clearDependancy(); }

template< class T,class Time>
  bool sotSignalTimeDependant<T,Time>::
needUpdate( const Time& t ) const
{
  return sotTimeDependancy<Time>::needUpdate(t); 
}

template< class T,class Time>
    void sotSignalTimeDependant<T,Time>::
setPeriodTime( const Time& p ) 
    { sotTimeDependancy<Time>::setPeriodTime( p ); }
template< class T,class Time>
 Time sotSignalTimeDependant<T,Time>::
getPeriodTime( void )  const
    { return sotTimeDependancy<Time>::getPeriodTime();  }


#endif /* #ifndef __SOT_DEPENDANT_SIGNAL_HH */
