/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTimeDependancy.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_TIME_DEPENDANCY_HH
#define __SOT_TIME_DEPENDANCY_HH

#include <list>
#include <iostream> 

#include <sot/sotSignalAbstract.h>
#include <sot/sotSignalArray.h>

template< class Time >
class sotTimeDependancy
{
 public:
  enum DependancyType
    {
      TIME_DEPENDANT, 
      BOOL_DEPENDANT,
      ALWAYS_READY
    };

  mutable Time lastAskForUpdate;

  // protected:
 public:

  sotSignalAbstract< Time >& leader;

  typedef std::list< const sotSignalAbstract<Time> * > Dependancies;
  static const DependancyType DEPENDANCY_TYPE_DEFAULT = TIME_DEPENDANT;

  Dependancies dependancies;
  bool updateFromAllChildren;
  static const bool ALL_READY_DEFAULT = false;

  DependancyType dependancyType;

  Time periodTime;
  static const Time PERIOD_TIME_DEFAULT = 1;

 public:

  sotTimeDependancy( sotSignalAbstract<Time>* sig,
		     const DependancyType dep = DEPENDANCY_TYPE_DEFAULT );
  sotTimeDependancy( sotSignalAbstract<Time>* sig,
		     const sotSignalArray_const<Time>& arr, 
		     const DependancyType dep = DEPENDANCY_TYPE_DEFAULT );
  ~sotTimeDependancy( void ) {}

  void addDependancy( const sotSignalAbstract<Time>& sig );
  void removeDependancy( const sotSignalAbstract<Time>& sig );
  void clearDependancy( void );
  

  virtual std::ostream & writeGraph(std::ostream &os) const;
  std::ostream& displayDependancies( std::ostream& os,const int depth=-1,
				     std::string space="",
				     std::string next1="",std::string next2="" ) const;
  
  bool needUpdate( const Time& t1 ) const;

  void setDependancyType( DependancyType dep ) { dependancyType = dep; }
  
  void setNeedUpdateFromAllChildren( const bool b = true ){ updateFromAllChildren=b; }
  bool getNeedUpdateFromAllChildren( void ) const { return updateFromAllChildren; } 

  void setPeriodTime( const Time& p ) { periodTime = p; }
  Time getPeriodTime( void ) const { return periodTime; }

};


#include <sot/sotTimeDependancy.t.cpp>

#endif /* #ifndef __SOT_TIME_DEPENDANCY_HH */
