 /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTimeDependancy.t.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_TIME_DEPENDANCY_TCPP
#define __SOT_TIME_DEPENDANCY_TCPP

#include <sot/sotTimeDependancy.h>


#undef VP_TEMPLATE_DEBUG_MODE
#define VP_TEMPLATE_DEBUG_MODE 0
#include <sot/sotDebug.h>

#define __SOT_TIME_DEPENDANCY_INIT(sig,dep)     \
    leader(*sig)                                \
    ,dependancies()                             \
    ,updateFromAllChildren(ALL_READY_DEFAULT)   \
    ,dependancyType(dep)                        \
    ,periodTime(PERIOD_TIME_DEFAULT)
                         

template< class Time >
sotTimeDependancy<Time>::
sotTimeDependancy( sotSignalAbstract<Time> *sig
		   ,const DependancyType dep )
  :__SOT_TIME_DEPENDANCY_INIT(sig,dep)
{}

template< class Time >
sotTimeDependancy<Time>::
sotTimeDependancy( sotSignalAbstract<Time> * sig 
		   ,const sotSignalArray_const<Time>& ar
		   ,const DependancyType dep )
  :__SOT_TIME_DEPENDANCY_INIT(sig,dep)
{
  for( unsigned int i=0;i<ar.getSize();++i ) 
    {addDependancy( ar[i] );  }
  
  return ;
}

/* -------------------------------------------------------------------------- */
template< class Time >
void sotTimeDependancy<Time>::
addDependancy( const sotSignalAbstract<Time>& sig )
{
  dependancies.push_front(&sig); 
}

template< class Time >
void sotTimeDependancy<Time>::
removeDependancy( const sotSignalAbstract<Time>& sig )
{
  dependancies.remove(&sig); 
}
template< class Time >
void sotTimeDependancy<Time>::
clearDependancy( void )
{
  dependancies.clear(); 
}

template< class Time >
bool sotTimeDependancy<Time>::
needUpdate( const Time& t1 ) const
{ 
  sotTDEBUG(15) << "# In {" << leader.getName() << " : ("<< leader.getReady()<<") "
		<< t1 << " >? " << leader.getTime() << std::endl;

  if( leader.getReady() ) { sotTDEBUGOUT(15);return true; }
  if( lastAskForUpdate ) { sotTDEBUGOUT(15);return true; }

  switch( dependancyType )
    {
    case ALWAYS_READY: 
      { sotTDEBUGOUT(15);return true; }
    case BOOL_DEPENDANT:  break;
    case TIME_DEPENDANT: 
      {
	if( t1<leader.getTime()+periodTime ) 
	  { sotTDEBUGOUT(15);return false; } 
	break;
      }
    };

  bool res = updateFromAllChildren;
  const typename Dependancies::const_iterator itend=dependancies.end();
  for( typename Dependancies::const_iterator it=dependancies.begin();it!=itend;++it )
    {
      const sotSignalAbstract<Time> &sig = **it;
      sotTDEBUG(15)<< "Ask update for "<< sig <<std::endl; 

      if( (sig.getTime()>leader.getTime()) || (sig.needUpdate(t1)) )
	{ if(updateFromAllChildren) continue; else {res=true; break;} }
      else { if(updateFromAllChildren) {res=false; break;} else continue; }
    }
  lastAskForUpdate = res;

  sotTDEBUGOUT(15);
  return res;
}


template< class Time >
std::ostream& sotTimeDependancy<Time>::
writeGraph(  std::ostream & os) const
{
  std::string LeaderLocalName;
  std::string LeaderNodeName;
  leader.ExtractNodeAndLocalNames(LeaderLocalName,LeaderNodeName);
  if (dependancies.size()!=0)
    {
      const typename Dependancies::const_iterator itend=dependancies.end();
      for( typename Dependancies::const_iterator it=dependancies.begin();it!=itend;++it )
	{
	  std::string itLocalName,itNodeName;
	  (*it)->ExtractNodeAndLocalNames(itLocalName,itNodeName);
	  os << "\"" << itNodeName << "\" -> \"" << LeaderNodeName << "\"" << std::endl
	     << " [ headlabel = \"" << LeaderLocalName << "\" , taillabel = \"" << itLocalName << "\" ]" << std::endl;
	}
    }
  return os;
}

template< class Time >
std::ostream& sotTimeDependancy<Time>::
displayDependancies( std::ostream& os,const int depth,
		     std::string space,
		     std::string next1,std::string next2 ) const
{
  leader.sotSignalAbstract<Time>::displayDependancies(os,depth,space,next1,next2)<<" (";
  switch( dependancyType )
    {
    case ALWAYS_READY: os<<"A"; break;
    case BOOL_DEPENDANT: os << "ready=" << ((leader.getReady())?"TRUE":"FALSE"); break;
    case TIME_DEPENDANT:  
      os  <<"t="<<leader.getTime() <<" (/"<<periodTime<<") " ; 
      break;
    };
  os<<")"; //<<std::endl;
    {
      const typename Dependancies::const_iterator itend=dependancies.end();
      for( typename Dependancies::const_iterator it=dependancies.begin();it!=itend;++it )
	if( depth!=0 )
	  {
	    os<<std::endl;
	    std::string ajout = "|";
	    std::string ajout2 = "|";
	    typename Dependancies::const_iterator it2=it; it2++;
	    if( it2==dependancies.end() ) { ajout = "`"; ajout2= " "; }
	    (*it)->displayDependancies( os,depth-1,space+next2+"   ",ajout,ajout2 );
	  }
	else 
	  { os<<std::endl<<space<<"   `-- ..."; break; }
    }
  return os;
}



#endif /* #ifndef __SOT_TIME_DEPENDANCY_TCPP */


