/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTaskAbstract.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotTaskAbstract.h>
#include <sot/sotPool.h>


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


sotTaskAbstract::
sotTaskAbstract( const std::string& n )
  :sotEntity(n)
   ,taskSOUT( "sotTaskAbstract("+n+")::output(vector)::task" )
   ,jacobianSOUT( "sotTaskAbstract("+n+")::output(matrix)::jacobian" ) 
   ,featureActivationSOUT( "sotTaskAbstract("+n+")::output(vector)::activation" ) 
   ,jacobianInvSINOUT( "sotTaskAbstract("+n+")::inout(matrix)::Jinv" )
   ,jacobianConstrainedSINOUT( "sotTaskAbstract("+n+")::inout(matrix)::JK" )
   ,jacobianProjectedSINOUT( "sotTaskAbstract("+n+")::inout(matrix)::Jt" )
   ,singularBaseImageSINOUT( "sotTaskAbstract("+n+")::inout(matrix)::V" )
   ,rankSINOUT( "sotTaskAbstract("+n+")::inout(matrix)::rank" )
{
 
  taskRegistration();
  signalRegistration( taskSOUT<<jacobianSOUT
		      <<featureActivationSOUT<<jacobianInvSINOUT
		      <<singularBaseImageSINOUT<<rankSINOUT
		      <<jacobianConstrainedSINOUT<<jacobianProjectedSINOUT);

}


void sotTaskAbstract::
taskRegistration( void )
{
  sotPool.registerTask(name,this);
}



void sotTaskAbstract::
 initMemory( const unsigned int & ffsize )
{
  const ml::Matrix & Jac = jacobianSOUT( 0 );
  taskSOUT(0);
  featureActivationSOUT(0);
  const unsigned int nJ = Jac.nbRows();
  const unsigned int mJ = Jac.nbCols() - ffsize;

  sotDEBUG(15) << "Task " << getName() << ": resize " << nJ << "x" << mJ << std::endl;

  Jt.resize( nJ,mJ );
  Jp.resize( mJ,nJ );
  PJp.resize( mJ,nJ );

  Jff.resize( nJ,ffsize );
  Jact.resize( nJ,mJ );

  JK.resize( nJ,mJ );
  U.resize( nJ,nJ );
  V.resize( mJ,mJ );
  S.resize( std::min( nJ,mJ ) );

  JK.fill(0);
  Jt.pseudoInverse(Jp);
}


void sotTaskAbstract::
commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
	     std::ostream& os )
{
  if( cmdLine=="help" )
    {
      os << "TaskAbstract: " << std::endl
	 << " - initMemory [<ffsize>=6]. " << std::endl;
      sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
  else if ( "initMemory"==cmdLine )
    {
      unsigned int ffsize = 6;
      cmdArgs >> std::ws; if( cmdArgs.good() ) cmdArgs >> ffsize;
      initMemory( ffsize );
    }
  else 
    {
      sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
}
	

