/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTaskAbstract.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_TASKABSTRACT_H__
#define __SOT_TASKABSTRACT_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
#include <MatrixAbstractLayer/boostMatrixSvd.h>
namespace ml = maal::boost;

/* STD */
#include <string>

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotSignalPtr.h>
#include <sot/sot-0API.h>
/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */



class SOT_EXPORT sotTaskAbstract
: public sotEntity
{
 protected:
  void taskRegistration( void );
  
 public:
  sotTaskAbstract( const std::string& n );

  
 public: /* --- SIGNALS --- */

  sotSignalTimeDependant< ml::Vector,int > taskSOUT;
  sotSignalTimeDependant< ml::Matrix,int > jacobianSOUT; 
  sotSignalTimeDependant< ml::Vector,int > featureActivationSOUT; 

  sotSignal< ml::Matrix,int > jacobianInvSINOUT;
  sotSignal< ml::Matrix,int > jacobianConstrainedSINOUT;
  sotSignal< ml::Matrix,int > jacobianProjectedSINOUT;
  sotSignal< ml::Matrix,int > singularBaseImageSINOUT;
  sotSignal< unsigned int,int > rankSINOUT;
    
 public: /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine
			    ,std::istringstream& cmdArgs
			    ,std::ostream& os ) ;
 public:
  /* Internal memory to reduce the dynamic allocation at task resolution. */
  mutable ml::MatrixSvd Jt;  //( nJ,mJ );
  mutable ml::Matrix Jp;
  mutable ml::Matrix PJp;

  mutable ml::Matrix Jff; //( nJ,FF_SIZE ); // Free-floating part
  mutable ml::Matrix Jact; //( nJ,mJ );     // Activated part
  mutable ml::Matrix JK; //(nJ,mJ); 

  mutable  ml::Matrix U,V; 
  mutable ml::Vector S;

  virtual void initMemory( const unsigned int & ffsize );
};





#endif /* #ifndef __SOT_TASKABSTRACT_H__ */


