/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotReproConstraints.h
 * Project:   SOT
 * Author:    Paul Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Outputs a freeze signal when velocity / force / workspace
 * constraints are violated.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_REPRO_LIMITATOR__
#define __SOT_REPRO_LIMITATOR__


/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* STL */
#include <string>

/* GMM/GMR */
#include <gmm/Motion.h>


/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotReproConstraints_EXPORTS)
#    define SOTREPROCONSTRAINTS_EXPORT __declspec(dllexport)
#  else  
#    define SOTREPROCONSTRAINTS_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTREPROCONSTRAINTS_EXPORT
#endif


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTREPROCONSTRAINTS_EXPORT sotReproConstraints
  : public sotEntity
{
public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  /*
   * Input and output signals. Add appropriate signals here (force,
   * velocity, ...).
   */

  sotSignalPtr< ml::Vector,int > positionSIN;
  sotSignalPtr< ml::Vector,int > velocitySIN;
  sotSignalPtr< ml::Vector,int > forceSIN;
  sotSignalTimeDependant< int,int > freezeSOUT;

 public:

  sotReproConstraints( const std::string& name );

  virtual int& computeOutput( int& res, int time );

  virtual void display( std::ostream& ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 private:

  /* --- Data --- */

  ml::Vector workspace;
  ml::Vector maxVelocity;
  ml::Vector maxForce;
  bool freezed;
};


#endif
