/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTaskLearning.h
 * Project:   SOT
 * Author:    Paul Evrard, Elena Gribovskaya, Sylvain Calinon
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Wraps the GMM / GMR library from EPFL.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_TASK_LEARNING__
#define __SOT_TASK_LEARNING__


/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotVec.h>

/* STL */
#include <string>

/* GMM/GMR */
#include <gmm/Motion.h>

/* Boost */
#include <boost/shared_ptr.hpp>


/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotTaskLearning_EXPORTS)
#    define SOTTASKLEARNING_EXPORT __declspec(dllexport)
#  else  
#    define SOTTASKLEARNING_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTTASKLEARNING_EXPORT
#endif


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTTASKLEARNING_EXPORT sotTaskLearning
  : public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  sotSignalPtr< int,int > freezeSIN;
  sotSignalPtr< ml::Vector,int > positionSIN;
  sotSignalPtr< ml::Vector,int > velocitySIN;
  sotSignalPtr< ml::Vector,int > forceSIN;
  sotSignalTimeDependant< ml::Vector,int > velocitySOUT;

 public:

  sotTaskLearning( const std::string& name );

  virtual ml::Vector& computeOutput( ml::Vector& res, int time );

  virtual void display( std::ostream& ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 private:

  /* --- State --- */

  Vec<3> pos, dpos, force;
  boost::shared_ptr<Motion> lifting;
  std::string file;
  int nIter;
  bool run;
  bool freezed;

  /* --- Params --- */

  enum {
    MODE_KINEMATIC = 1,
    MODE_FORCE,
    MODE_COMPLETE
  } mode;
  int nIterMax;
  double dt;
  double beta;

  /* --- Implementation of command lines --- */

  void start();
  void stop();
  void setFile( std::istringstream& cmdArgs, std::ostream& os);
  void setMode( std::istringstream& cmdArgs, std::ostream& os);
};

#endif
