/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotBinaryIntToUint.h
 * Project:   SOT
 * Author:    Paul Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Zero out the control signal when collision is detected between
 * two bodies of the controlled robot.
 *
 * !!! WARNING !!! THIS CODE IS CURRENTLY SPECIFIC TO HRP2
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_BINARY_INT_TO_UINT_HH__
#define __SOT_BINARY_INT_TO_UINT_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotBinaryIntToUint_EXPORTS)
#    define SOTBINARYINTTOUINT_EXPORT __declspec(dllexport)
#  else  
#    define SOTBINARYINTTOUINT_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTBINARYINTTOUINT_EXPORT
#endif

class sotBinaryIntToUint
  : public sotEntity
{
public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

  /* --- SIGNALS ------------------------------------------------------------ */
public:

  sotSignalPtr< int,int > binaryIntSIN;
  sotSignalTimeDependant< unsigned,int > binaryUintSOUT;

public:
  sotBinaryIntToUint( const std::string& name );
  virtual ~sotBinaryIntToUint() {}

  virtual unsigned& computeOutput( unsigned& res,int time );

  virtual void display( std::ostream& os ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );
};

#endif
