/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotCollisionDetector.h
 * Project:   SOT
 * Author:    Paul Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * Zero out the control signal when collision is detected between
 * two bodies of the controlled robot.
 *
 * !!! WARNING !!! THIS CODE IS CURRENTLY SPECIFIC TO HRP2
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_FEATURE_COLLISION_DETECTION__
#define __SOT_FEATURE_COLLISION_DETECTION__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* STL */
#include <vector>

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* Boost */
#include <boost/shared_ptr.hpp>

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotCollisionDetector_EXPORTS)
#    define SOTCOLLISIONDETECTOR_EXPORT __declspec(dllexport)
#  else  
#    define SOTCOLLISIONDETECTOR_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTCOLLISIONDETECTOR_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- COLLISION DETECTION --------------------------------------------- */
/* --------------------------------------------------------------------- */

/*
  To decouple entity interface and implementation (Pimpl idiom)
*/
struct CollisionDetectionImpl;

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/*!
  \class sotCollisionDetector
*/
class SOTCOLLISIONDETECTOR_EXPORT sotCollisionDetector 
: public sotEntity
{

 public: 
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }


  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  sotSignalPtr< ml::Vector,int > jointSIN;
  sotSignalPtr< ml::Vector,int > controlSIN;
  sotSignalTimeDependant< ml::Vector,int > controlSOUT;

 public:
  sotCollisionDetector( const std::string& name );
  virtual ~sotCollisionDetector( void ) {}

  virtual ml::Vector& computeControl( ml::Vector& res,int time ); 

  virtual void display( std::ostream& os ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );

 private:
  boost::shared_ptr<CollisionDetectionImpl> cdimpl;
} ;


#endif
