/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotDistantShell.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <sot/sotDistantShell.h>
#include <sot/sotDebug.h>

#ifdef WIN32
#include <Windows.h>
#endif

sotDistantShell::
~sotDistantShell( void )
{
  sotDEBUGIN(5);
  endLoop = true;
  
#ifdef HAVE_LIBBOOST_THREAD
  threadLoop->join();
#endif

  //usleep(1000*1000);
  sotDEBUGOUT(5);
}


void sotDistantShell::
loop( const int lat )
{
  sotDEBUGIN(15);
  while(! endLoop )
    {
      sotDEBUG(45) <<  lat <<std::endl;
      if( filein.loop() )
	{
	  while( filein.ready() )
	    {
	      std::istringstream iss( filein.next() );
	      std::string cmdLine; 
	      sotDEBUG(35) << cmdLine << std::endl;
	      iss>>cmdLine;
	      shell.cmd( cmdLine,iss,os );
	    }
	}
#ifndef WIN32
	usleep( lat*1000 );
#else	  
	Sleep(lat);
#endif
    }
  sotDEBUGOUT(15);
}

void* 
__staticLoop( void* autoref )
{
  sotDistantShell* ref = static_cast<sotDistantShell *>( autoref );
  ref->loop();
  return NULL;
}

void sotDistantShell::
loopInThread( void )
{
  sotDEBUGIN(5);
#ifdef HAVE_LIBBOOST_THREAD
//   pthread_attr_init(&loopThreadAttr);
//   pthread_create(&loopThread,&loopThreadAttr,__staticLoop,this);
//  threadLoop = new boost::thread( boost::bind( &sotDistantShell::loop,this ) );
  threadLoop = new boost::thread (boost::bind( &sotDistantShell::loop,this,
					       latency ));
#else
#ifndef WIN32
#warning Unable to compile with libboost_thread
#endif
  sotDEBUG(5) << "No boost:thread. Unable to launch in thread." <<std::endl;
#endif
  sotDEBUGOUT(5);
}


/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */



#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotDistantShellPlugin,"DShell");

using namespace std;

sotDistantShellPlugin::
sotDistantShellPlugin( const std::string& name )
  : sotEntity(name)
    , dshell( sotShell,"/tmp/nmansard/"+name )
{
  sotDEBUGINOUT(5);
}


sotDistantShellPlugin::
~sotDistantShellPlugin( void )
{
  sotDEBUGINOUT(5);
}


void sotDistantShellPlugin::
commandLine( const std::string& cmdLine,
	     std::istringstream& cmdArgs,
	     std::ostream& os )
{

  sotDEBUGIN(15) << "CMD = " << cmdLine <<endl;

  if( cmdLine == "help") 
    {
      os << "distant shell: "<<endl
	 << "  - open <file>" << endl
	 << "  - start" << endl
	 << "  - stop" << endl
	 << "  - latency <uint>" << endl;
    }
  else if( cmdLine == "open" )
    {
      std::string f; cmdArgs >> f; 
      dshell.inputFile(f);
    }
  else if( cmdLine == "start" )
    {
      dshell.loopInThread();
    }
  else if( cmdLine == "stop" )
    {
      dshell.terminate();
    }
  else if( cmdLine == "latency" )
    {
      unsigned int l; cmdArgs >> l;
      dshell.setLatency(l);
    }
  else
    sotEntity::commandLine( cmdLine,cmdArgs,os );
  
  sotDEBUGOUT(15);

}

