/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotDistantShell.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_DISTANTSHELL_HH__
#define __SOT_DISTANTSHELL_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */


#include <sot/sotInterpretor.h>
#include <sot/sotContiifstream.h>
//#include <pthread.h>

#include <sot/sotConfig.h>
#ifdef HAVE_LIBBOOST_THREAD
#include <boost/thread.hpp>
#endif

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotDistantShell_EXPORTS)
#    define SOTDISTANTSHELL_EXPORT __declspec(dllexport)
#  else  
#    define SOTDISTANTSHELL_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTDISTANTSHELL_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTDISTANTSHELL_EXPORT sotDistantShell
{
 public:
  std::ofstream osOwn;
 protected:
  sotContiifstream filein;
  sotInterpretor& shell;
  std::ostream & os;
/*   pthread_t loopThread; */
/*   pthread_attr_t loopThreadAttr; */

#ifdef HAVE_LIBBOOST_THREAD
  boost::thread* threadLoop;
#else
  int* threadLoop;
#endif 

  bool endLoop;
  
  unsigned int latency;
  static const unsigned int LATENCY_DEFAULT = 10;

 public:
  sotDistantShell(sotInterpretor& shell_,const std::string& fn  ) 
    :osOwn(fn.c_str()),shell(shell_),os(osOwn)
    ,endLoop(false),latency(LATENCY_DEFAULT){ }

  sotDistantShell( const std::string& filename
		   ,sotInterpretor& shell_
		   ,std::ostream& os_ )
    : filein( filename ),shell(shell_),os(os_)
    ,endLoop(false),latency(LATENCY_DEFAULT)
    {}

  sotDistantShell( const std::string& filename
		   ,sotInterpretor& shell_
		   ,const std::string& fn )
    : osOwn(fn.c_str()),filein( filename ),shell(shell_),os(osOwn)
    ,endLoop(false),latency(LATENCY_DEFAULT)
    {}

  ~sotDistantShell( void );

  void inputFile( const std::string& filename ) { filein = filename; }

  void loop( const int latency = LATENCY_DEFAULT ); // in ms
  void loopInThread( void );
  void terminate( void ) { endLoop = true; }
  void setLatency( const unsigned int& l ) { latency = l; }
};



#include <sot/sotEntity.h>

class SOTDISTANTSHELL_EXPORT sotDistantShellPlugin
: public sotEntity
{
 protected:
  sotDistantShell dshell;

 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

 public:
  sotDistantShellPlugin( const std::string& name );
  virtual ~sotDistantShellPlugin( void ); 

 public:
  virtual void commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
			    std::ostream& os );


};


#endif /* #ifndef __SOT_DISTANTSHELL_HH__ */




