/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotGripperControl.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTGRIPPERCONTROL_H__
#define __SOT_SOTGRIPPERCONTROL_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotFlags.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* STD */
#include <string>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotGripperControl_EXPORTS)
#    define SOTGRIPPERCONTROL_EXPORT __declspec(dllexport)
#  else  
#    define SOTGRIPPERCONTROL_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTGRIPPERCONTROL_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTGRIPPERCONTROL_EXPORT sotGripperControl
{
 protected:
  
  double offset;
  static const double OFFSET_DEFAULT;
  ml::Vector factor;

 public:
  sotGripperControl( void );
  
  void computeIncrement( const ml::Vector& torques,
			 const ml::Vector& torqueLimits,
			 const ml::Vector& currentNormPos );
  
  static void computeNormalizedPosition( const ml::Vector& currentPos,
					 const ml::Vector& upperLim,
					 const ml::Vector& lowerLim,
					 ml::Vector& currentNormPos );
  static void computeDenormalizedPosition( const ml::Vector& currentNormPos,
					   const ml::Vector& upperLim,
					   const ml::Vector& lowerLim,
					   ml::Vector& currentPos );
  
  ml::Vector& computeDesiredPosition( const ml::Vector& currentPos,
				      const ml::Vector& torques,
				      const ml::Vector& upperLim,
				      const ml::Vector& lowerLim,
				      const ml::Vector& torqueLimits,
				      ml::Vector& desPos );
					 
  static ml::Vector& selector( const ml::Vector& fullsize,
			       const sotFlags& selec,
			       ml::Vector& desPos );
  
};

/* --------------------------------------------------------------------- */
/* --- PLUGIN ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTGRIPPERCONTROL_EXPORT sotGripperControlPlugin
:public sotEntity, public sotGripperControl
{
 public:
  static const std::string CLASS_NAME;
  bool calibrationStarted;


 public: /* --- CONSTRUCTION --- */

  sotGripperControlPlugin( const std::string& name );
  virtual ~sotGripperControlPlugin( void );

 public: /* --- SIGNAL --- */

  /* --- INPUTS --- */
  sotSignalPtr<ml::Vector,int> positionSIN; 
  sotSignalPtr<ml::Vector,int> upperLimitSIN; 
  sotSignalPtr<ml::Vector,int> lowerLimitSIN; 
  sotSignalPtr<ml::Vector,int> torqueSIN; 
  sotSignalPtr<ml::Vector,int> torqueLimitSIN; 
  sotSignalPtr<sotFlags,int> selectionSIN; 

  /* --- INTERMEDIARY --- */
  sotSignalPtr<ml::Vector,int> positionFullSizeSIN;
  sotSignalTimeDependant<ml::Vector,int> positionReduceSOUT;
  sotSignalPtr<ml::Vector,int> upperLimitFullSizeSIN;
  sotSignalTimeDependant<ml::Vector,int> upperLimitReduceSOUT;
  sotSignalPtr<ml::Vector,int> lowerLimitFullSizeSIN;
  sotSignalTimeDependant<ml::Vector,int> lowerLimitReduceSOUT;
  sotSignalPtr<ml::Vector,int> torqueFullSizeSIN;
  sotSignalTimeDependant<ml::Vector,int> torqueReduceSOUT;
  sotSignalPtr<ml::Vector,int> torqueLimitFullSizeSIN;
  sotSignalTimeDependant<ml::Vector,int> torqueLimitReduceSOUT;
 
  /* --- OUTPUTS --- */
  sotSignalTimeDependant<ml::Vector,int> desiredPositionSOUT; 
  

 public: /* --- COMMANDLINE --- */

  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );

};





#endif // #ifndef __SOT_SOTGRIPPERCONTROL_H__

