/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotJointLimitator.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_FEATURE_JOINTLIMITS_HH__
#define __SOT_FEATURE_JOINTLIMITS_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotExceptionTask.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotJointLimitator_EXPORTS)
#    define SOTJOINTLIMITATOR_EXPORT __declspec(dllexport)
#  else  
#    define SOTJOINTLIMITATOR_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTJOINTLIMITATOR_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/*!
  \class sotJointLimitator
*/
class SOTJOINTLIMITATOR_EXPORT sotJointLimitator 
: public sotEntity
{

 public: 
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

 protected:
  
  double threshold;
  const static double THRESHOLD_DEFAULT; // = .99;

  /* --- SIGNALS ------------------------------------------------------------ */
 public:

  sotSignalPtr< ml::Vector,int > jointSIN;
  sotSignalPtr< ml::Vector,int > upperJlSIN;
  sotSignalPtr< ml::Vector,int > lowerJlSIN;
  sotSignalPtr< ml::Vector,int > controlSIN;
  sotSignalTimeDependant< ml::Vector,int > controlSOUT;
  sotSignalTimeDependant< ml::Vector,int > widthJlSINTERN;

 public:
  sotJointLimitator( const std::string& name );
  virtual ~sotJointLimitator( void ) {}

  virtual ml::Vector& computeControl( ml::Vector& res,int time ); 
  ml::Vector& computeWidthJl( ml::Vector& res,const int& time );

  virtual void display( std::ostream& os ) const;
  void commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );
} ;

#endif // #ifndef __SOT_FEATURE_JOINTLIMITS_HH__

/*
 * Local variables:
 * c-basic-offset: 2
 * End:
 */
