/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMailbox.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_MAILBOX_HH
#define __SOT_MAILBOX_HH

#include <sot/sotConfig.h>
#ifdef  HAVE_LIBBOOST_THREAD


/* --- SOT PLUGIN  --- */
#include <sot/sotEntity.h>
#include <sot/sotSignalTimeDependant.h>

/* --- BOOST --- */
#include <boost/thread/mutex.hpp>
#include <boost/thread/thread.hpp>
#include <boost/thread/xtime.hpp>

/* --- STD --- */
#include <time.h>
#ifndef WIN32
#include <sys/time.h>
#endif /*WIN32*/
//#else
//#include <Winsock2.h>
//#endif
#include <string>

template< class Object >
class sotMailbox
: public sotEntity
{
 public: 
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

 public:
  struct sotTimestampedObject
  {
    Object obj;
    struct timeval timestamp;
  };

public:
  
  sotMailbox( const std::string& name );
  ~sotMailbox( void );
  
  void post( const Object& obj );
  sotTimestampedObject& get( sotTimestampedObject& res,const int& dummy );

  Object& getObject( Object& res,const int& time )
    { 
      const sotTimestampedObject & data = SOUT(time);
      res = data.obj; return res;
    }
  struct timeval& getTimestamp( struct timeval& res,const int& time )
    { 
      const sotTimestampedObject & data = SOUT(time);
      res.tv_sec = data.timestamp.tv_sec ; 
      res.tv_usec = data.timestamp.tv_usec ; 
      return res;
    }

  bool hasBeenUpdated( void );
  
protected:

  boost::timed_mutex mainObjectMutex;
  Object mainObject;
  struct timeval mainTimeStamp;
  bool update;

 public: /* --- SIGNALS --- */

  sotSignalTimeDependant< struct sotTimestampedObject,int > SOUT;
  sotSignalTimeDependant< Object,int > objSOUT;
  sotSignalTimeDependant< struct timeval,int > timeSOUT;

};


#include <sot/sotMailbox.t.cpp>

#endif // #ifdef  HAVE_LIBBOOST_THREAD
#endif // #ifndef  __SOT_MAILBOX_HH





