/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotMailbox.t.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/




#ifndef __SOT_MAILBOX_T_CPP
#define __SOT_MAILBOX_T_CPP



/* --- SOT PLUGIN  --- */
#include <sot/sotMailbox.h>

//#undef VP_TEMPLATE_DEBUG
//#define VP_TEMPLATE_DEBUG_MODE             0
#include <sot/sotDebug.h>

  
/* -------------------------------------------------------------------------- */
/* --- CONSTRUCTION --------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
template< class Object >
sotMailbox<Object>::
sotMailbox( const std::string& name )
  :sotEntity(name)
  ,mainObjectMutex()
  ,mainObject()
  ,update(false)

   ,SOUT( boost::bind(&sotMailbox::get,this,_1,_2),
	  sotNOSIGNAL,
	  "sotMailbox("+name+")::output(Object)::out" )
   ,objSOUT( boost::bind(&sotMailbox::getObject,this,_1,_2),
	     SOUT,
	     "sotMailbox("+name+")::output(Object)::object" )
   ,timeSOUT( boost::bind(&sotMailbox::getTimestamp,this,_1,_2),
	      SOUT,
	      "sotMailbox("+name+")::output(Object)::timestamp" )
{
  signalRegistration( SOUT<<objSOUT<<timeSOUT );
  SOUT.setDependancyType( sotTimeDependancy<int>::BOOL_DEPENDANT );
}

template< class Object >
sotMailbox<Object>::
~sotMailbox( void )
{
  boost::timed_mutex::scoped_lock lockMain( mainObjectMutex );
}

/* -------------------------------------------------------------------------- */
/* --- ACCESS --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
template< class Object >
bool sotMailbox<Object>::
hasBeenUpdated( void )
{
  boost::timed_mutex::scoped_try_lock lockMain( this->mainObjectMutex );
  
  if( lockMain.locked() )
    {
      return update;
    }
  else 
    {
      return false;
    }
}


/* -------------------------------------------------------------------------- */
template< class Object >
typename sotMailbox<Object>::sotTimestampedObject& sotMailbox<Object>::
get( typename sotMailbox<Object>::sotTimestampedObject& res,const int& dummy )
{
  boost::timed_mutex::scoped_try_lock lockMain( this->mainObjectMutex );
  
  if( lockMain.locked() )
    {
      res.timestamp.tv_sec=this->mainTimeStamp.tv_sec;
      res.timestamp.tv_usec=this->mainTimeStamp.tv_usec;
      
      update = false;
      res.obj = this->mainObject;
    }

  return res;
}

/* -------------------------------------------------------------------------- */
template< class Object >
void sotMailbox<Object>::
post( const Object& value )
{
  boost::timed_mutex::scoped_lock lockMain( this->mainObjectMutex );
  mainObject = value;
  gettimeofday( &this->mainTimeStamp, NULL );
  update = true;
  SOUT.setReady();

  return;
}


#endif // #ifdef __SOT_MAILBOX_T_CPP

