/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotNeckLimitation.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_SOTNECKLIMITATION_H__
#define __SOT_SOTNECKLIMITATION_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotTaskAbstract.h>

/* STD */
#include <string>
#include <map>
#include <list>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotNeckLimitation_EXPORTS)
#    define SOTNECKLIMITATION_EXPORT __declspec(dllexport)
#  else  
#    define SOTNECKLIMITATION_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTNECKLIMITATION_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTNECKLIMITATION_EXPORT sotNeckLimitation
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

 protected:

  unsigned int panRank,tiltRank;
  static const unsigned int PAN_RANK_DEFAULT;
  static const unsigned int TILT_RANK_DEFAULT;


  /* The limitation is: sgn.Tilt >= Pan.alpha + beta, with alpha the linear
  * coefficient and beta the affine one, and sgn is +1 or -1. */
  double coeffLinearPan,coeffAffinePan;
  double signTilt; 
  static const double COEFF_LINEAR_DEFAULT;
  static const double COEFF_AFFINE_DEFAULT;
  static const double SIGN_TILT_DEFAULT;

 public: /* --- CONSTRUCTION --- */

  sotNeckLimitation( const std::string& name );
  virtual ~sotNeckLimitation( void );

 public: /* --- SIGNAL --- */
  sotSignalPtr<ml::Vector,int> jointSIN;
  sotSignalTimeDependant<ml::Vector,int> jointSOUT;

 public: /* --- FUNCTIONS --- */
  ml::Vector& computeJointLimitation( ml::Vector& jointLimited,const int& timeSpec );
  
 public: /* --- PARAMS --- */
  virtual void display( std::ostream& os ) const; 
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
};






#endif // #ifndef __SOT_SOTNECKLIMITATION_H__

