/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotPeriodicCall.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef __SOT_PERIODICCALL_HH__
#define __SOT_PERIODICCALL_HH__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/* SOT */
#include <sot/sotSignalAbstract.h>
#include <sot/sotConfig.h>
/* STD */
#include <list>
#include <map>
#include <string>

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/*!
  \class sotPeriodicCall
*/
class SOT_EXPORT sotPeriodicCall 
{
 protected:
  typedef std::map< std::string,sotSignalAbstract<int>* > SignalMapType;
  SignalMapType signalMap;
  
  typedef std::list< std::string > CmdListType;
  CmdListType cmdList;

  int innerTime;

  /* --- FUNCTIONS ------------------------------------------------------------ */
 public:
  sotPeriodicCall( void );
  virtual ~sotPeriodicCall( void ) {}

  void addSignal( const std::string &name, sotSignalAbstract<int>& sig );
  void addSignal( std::istringstream& args );
  void rmSignal( const std::string &name );
  void rmSignal( std::istringstream& args );

  void addCmd( std::istringstream& args );
  void rmCmd( std::istringstream& args );

  void runSignals( const int& t );
  void runCmds( void );
  void run( const int& t );
  
  void clear( void ) { signalMap.clear(); cmdList.clear(); }

  void display( std::ostream& os ) const;
  bool commandLine( const std::string& cmdLine,
		    std::istringstream& cmdArgs,
		    std::ostream& os );
} ;

#endif // #ifndef __SOT_PERIODICCALL_HH__

/*
 * Local variables:
 * c-basic-offset: 2
 * End:
 */
