/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotRobotSimu.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotRobotSimu.h>
#include <sot/sotDebug.h>
using namespace std;



#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotRobotSimu,"RobotSimu");


/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */


sotRobotSimu::
sotRobotSimu( const std::string& n )
  :sotEntity(n)
   ,state(6)
   ,periodicCallBefore( )
   ,periodicCallAfter( )
   ,controlSIN( NULL,"sotRobotSimu("+n+")::input(double)::control" )
  //,attitudeSIN(NULL,"sotRobotSimu::input(matrixRot)::attitudeIN")
   ,attitudeSIN(NULL,"sotRobotSimu::input(vector3)::attitudeIN")
   ,zmpSIN(NULL,"sotRobotSimu::input(vector3)::zmp")
   ,stateSOUT( "sotRobotSimu("+n+")::output(vector)::state" )
   ,attitudeSOUT( "sotRobotSimu("+n+")::output(matrixRot)::attitude" )
   ,pseudoTorqueSOUT( "sotRobotSimu::output(vector)::ptorque" )
   ,previousControlSOUT( "sotRobotSimu("+n+")::output(vector)::previousControl" )
   ,motorcontrolSOUT( "sotRobotSimu("+n+")::output(vector)::motorcontrol" )
   ,ZMPPreviousControllerSOUT( "sotRobotSimu("+n+")::output(vector)::zmppreviouscontroller" )
{
  /* --- FORCES --- */
  for( int i=0;i<4;++i ){ withForceSignals[i] = false; }
  forcesSOUT[0] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceRLEG");
  forcesSOUT[1] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceLLEG");
  forcesSOUT[2] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceRARM");
  forcesSOUT[3] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceLARM");

  signalRegistration( controlSIN<<stateSOUT<<attitudeSOUT<<attitudeSIN<<zmpSIN  
		      <<*forcesSOUT[0]<<*forcesSOUT[1]<<*forcesSOUT[2]<<*forcesSOUT[3] 
		      <<previousControlSOUT <<pseudoTorqueSOUT
		      << motorcontrolSOUT << ZMPPreviousControllerSOUT );
  state.fill(.0); stateSOUT.setConstant( state );
}

void sotRobotSimu::
setStateSize( const unsigned int size )
{
  state.resize(size); state.fill( .0 ); 
  stateSOUT .setConstant( state );
  previousControlSOUT.setConstant( state );
  pseudoTorqueSOUT.setConstant( state );
  motorcontrolSOUT .setConstant( state );
  
  ml::Vector zmp(3); zmp.fill( .0 );
  ZMPPreviousControllerSOUT .setConstant( zmp );
}

void sotRobotSimu::
setState( const ml::Vector st )
{
  state = st; 
  stateSOUT .setConstant( state );
  motorcontrolSOUT .setConstant( state );
}

void sotRobotSimu::
increment( const double dt )
{
  sotDEBUG(25) << "Time : " << controlSIN.getTime()+1 << std::endl;

   periodicCallBefore.runSignals( controlSIN.getTime()+1 );
   periodicCallBefore.runCmds();

   stateSOUT .setConstant( state ); 
  const ml::Vector control = controlSIN( controlSIN.getTime()+1 );

  sotDEBUG(25) << "Cl" <<controlSIN.getTime()<<" = "
	       << control*dt << ": " << control << endl;

  sotDEBUG(25) << "St"<<state.size() << controlSIN.getTime() << ": " << state << endl;
  for( unsigned int i=6;i<state.size();++i )
    { state(i) += (control(i-6)*dt); }

  sotDEBUG(25) << "St"<<state.size() << controlSIN.getTime() << ": " << state << endl;


   ml::Vector forceNull(6); forceNull.fill(0);
   for( int i=0;i<4;++i ){ 
     if(  withForceSignals[i] ) forcesSOUT[i]->setConstant(forceNull); 
   }

  motorcontrolSOUT .setConstant( state );
  
  ml::Vector zmp(3); zmp.fill( .0 );
  ZMPPreviousControllerSOUT .setConstant( zmp );


   periodicCallAfter.runSignals( controlSIN.getTime() );
   periodicCallAfter.runCmds();

}



/* --- DISPLAY ------------------------------------------------------------ */
/* --- DISPLAY ------------------------------------------------------------ */
/* --- DISPLAY ------------------------------------------------------------ */

void sotRobotSimu::display ( std::ostream& os ) const 
{os <<name<<": "<<state<<endl; }


/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
/* --- PARAMS --------------------------------------------------------------- */
#include <sot/sotPool.h>

void sotRobotSimu::
commandLine( const std::string& cmdLine
	     ,std::istringstream& cmdArgs
	     ,std::ostream& os )
{
  if( cmdLine=="help" )
    {
      os << "RobotSimu: "<<endl
	 << "  - resize <uint size>"<<endl
	 << "  - set <vector>"<<endl
	 << "  - inc [<dt>]"<<endl;
	sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
  else if( cmdLine=="resize" )
    {
      unsigned int size; cmdArgs >> size;
      setStateSize( size ); 
    }
  else if( cmdLine=="set" )
    {
      ml::Vector q; cmdArgs >> q;
      setState( q );
    }
  else if( cmdLine == "pause" ) { os << "Not valid in simu" << endl; }
  else if( cmdLine == "play" )  { os << "Not valid in simu" << endl; }
  else if( cmdLine == "withForces" )
  {
    int index;
    cmdArgs >> index;
    if((index >= 0) && (index < 4))
      {
	std::string val;
	cmdArgs >> val;
	if ( ("1"==val)||("true"==val) )
	  {
	    withForceSignals[index] = true;
	    ml::Vector forceNull(6); forceNull.fill(0);
	    forcesSOUT[index]->setConstant(forceNull); 
	  } else withForceSignals[index] = false;
      }
  }
  else if( cmdLine == "whichForces" )
  {
    os << "Force signals: " << endl; 
    for( unsigned int i=0;i<4;++i )
      {
	os << "\t- Force " << i << ": ";
	if( withForceSignals[i] )
	  { os << "Active";	  } else { os << "Inactive"; } 
	os << endl;
      }
  }
  else if( (cmdLine == "withPreviousControl")||(cmdLine == "withPseudoTorque") )
  {
    // Just for compatibility.
  }
  else if( cmdLine=="inc" )
    {
	increment();

    }
  else if( cmdLine=="time" )
    {
      os << "control.time = " << controlSIN.getTime() << endl;
    }
  else if(( cmdLine=="periodicCall" )||( cmdLine=="periodicCallAfter" ))
    {
      string cmd2; cmdArgs >> cmd2;
      periodicCallAfter .commandLine( cmd2,cmdArgs,os );
    }
  else if( cmdLine=="periodicCallBefore" )
    {
      string cmd2; cmdArgs >> cmd2;
      periodicCallBefore .commandLine( cmd2,cmdArgs,os );
    }
  else  
    {
      sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
}
