/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotRobotSimu.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_ROBOT_SIMU_HH
#define __SOT_ROBOT_SIMU_HH

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* -- MaaL --- */
#include <MatrixAbstractLayer/boost.h>
namespace ml= maal::boost;
/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotVectorRollPitchYaw.h>
#include <sot/sotPeriodicCall.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotRobotSimu_EXPORTS)
#    define SOTROBOTSIMU_EXPORT __declspec(dllexport)
#  else  
#    define SOTROBOTSIMU_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTROBOTSIMU_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTROBOTSIMU_EXPORT sotRobotSimu
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

  enum ForceSignalSource
  {
    FORCE_SIGNAL_RLEG,
    FORCE_SIGNAL_LLEG,
    FORCE_SIGNAL_RARM,
    FORCE_SIGNAL_LARM
  };

 protected:
  ml::Vector state;
  
  sotPeriodicCall periodicCallBefore;
  sotPeriodicCall periodicCallAfter;
  bool withForceSignals[4];


 public:
  
  /* --- CONSTRUCTION --- */
  sotRobotSimu( const std::string& name );

  void setStateSize( const unsigned int size );
  void setState( const ml::Vector st );
  void increment( const double dt = 5e-2 );
  


 public: /* --- DISPLAY --- */
  virtual void display( std::ostream& os ) const;
  SOTROBOTSIMU_EXPORT friend std::ostream& operator<< ( std::ostream& os,const sotRobotSimu& r ) 
    { r.display(os); return os;}

 public: /* --- SIGNALS --- */

  sotSignalPtr<ml::Vector,int> controlSIN;
  //sotSignalPtr<sotMatrixRotation,int> attitudeSIN;
  sotSignalPtr<ml::Vector,int> attitudeSIN;
  sotSignalPtr<ml::Vector,int> zmpSIN;

  sotSignal<ml::Vector,int> stateSOUT;
  sotSignal<sotMatrixRotation,int> attitudeSOUT;
  sotSignal<ml::Vector,int>* forcesSOUT[4];

  sotSignal<ml::Vector,int> pseudoTorqueSOUT;
  sotSignal<ml::Vector,int> previousControlSOUT;

  /*! \brief The current state of the robot from the command viewpoint. */
  sotSignal<ml::Vector,int> motorcontrolSOUT;
  /*! \brief The ZMP reference send by the previous controller. */
  sotSignal<ml::Vector,int> ZMPPreviousControllerSOUT;

 public: /* --- COMMANDS --- */
  virtual void commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
			    std::ostream& os );

};



#endif /* #ifndef __SOT_ROBOT_SIMU_HH */




