/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTimeStamp.h
 * Project:   SOT
 * Author:    Paul Evrard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_TIME_STAMP__HH
#define __SOT_TIME_STAMP__HH

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* Classes standards. */
#ifndef WIN32
#include <sys/time.h>
#else /*WIN32*/
#include <Winsock2.h>
#include <sot/sotUtilsWindows.h>
#endif /*WIN32*/

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalTimeDependant.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotTimeStamp_EXPORTS) 
#    define SOTTIMESTAMP_EXPORT __declspec(dllexport)
#  else  
#    define SOTTIMESTAMP_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTTIMESTAMP_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTTIMESTAMP_EXPORT sotTimeStamp
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

 protected:

  struct timeval val;
  unsigned int offsetValue;
  bool offsetSet;
 
 public:
  
  /* --- CONSTRUCTION --- */
  sotTimeStamp( const std::string& name );

 public: /* --- DISPLAY --- */
  virtual void display( std::ostream& os ) const;

 public: /* --- SIGNALS --- */

  /* These signals can be called several time per period, given 
   * each time a different results. Useful for chronos. */
  sotSignal<ml::Vector,int> timeSOUT;
  sotSignal<double,int> timeDoubleSOUT;

  /* These signals can be called several time per period, but give
   * always the same results different results. Useful for synchro. */
  sotSignalTimeDependant<ml::Vector,int> timeOnceSOUT;
  sotSignalTimeDependant<double,int> timeOnceDoubleSOUT;


 protected: /* --- SIGNAL FUNCTIONS --- */
  ml::Vector& getTimeStamp( ml::Vector& res,const int& time );
  double& getTimeStampDouble( const ml::Vector& vect,double& res );
    
 public: /* --- COMMANDS --- */
  virtual void commandLine( const std::string& cmdLine,
			    std::istringstream& cmdArgs,
			    std::ostream& os );
    
};



#endif /* #ifndef __SOT_SOT_HH */
