/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTimer.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_TIMER_HH
#define __SOT_TIMER_HH

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Classes standards. */
#include <list>                    /* Classe std::list   */
#ifndef WIN32
#include <sys/time.h>
#else  /*WIN32*/
// When including Winsock2.h, the MAL must be included first
#include <MatrixAbstractLayer/boost.h>
#include <sot/sotUtilsWindows.h>
#include <Winsock2.h>
#endif /*WIN32*/

/* SOT */
#include <sot/sotEntity.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotInterpretor.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotTimer_EXPORTS) 
#    define SOTTIMER_EXPORT __declspec(dllexport)
#  else  
#    define SOTTIMER_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTTIMER_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- CLASS ----------------------------------------------------------- */
/* --------------------------------------------------------------------- */

template< class T >
class SOTTIMER_EXPORT sotTimer
:public sotEntity
{
 public:
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

 protected:
  
  struct timeval t0,t1;
  double dt;

 public:
  
  /* --- CONSTRUCTION --- */
  sotTimer( const std::string& name );

 public: /* --- DISPLAY --- */
  virtual void display( std::ostream& os ) const;
  SOTTIMER_EXPORT friend std::ostream& operator<< ( std::ostream& os,const sotTimer<T>& timer ) 
    { timer.display(os); return os; }

 public: /* --- SIGNALS --- */

  sotSignalPtr<T,int> sigSIN;
  sotSignalTimeDependant<T,int> sigSOUT;
  sotSignal<double,int> timerSOUT;


 protected: /* --- SIGNAL FUNCTIONS --- */
  void plug( sotSignal<T,int> &sig )
    {
      sigSIN = &sig; dt=0.;
    }

  T& compute( T& t,const int& time )
    {
      sotDEBUGIN(15);
      gettimeofday(&t0,NULL);
      sotDEBUG(15) << "t0: "<< t0.tv_sec << " - " << t0.tv_usec << std::endl;
      
      t = sigSIN( time );

      gettimeofday(&t1,NULL);
      dt = ( (t1.tv_sec-t0.tv_sec) * 1000. 
	     + (t1.tv_usec-t0.tv_usec+0.) / 1000. );
      sotDEBUG(15) << "t1: "<< t1.tv_sec << " - " << t1.tv_usec << std::endl;

      timerSOUT = dt;

      sotDEBUGOUT(15);
      return t;
    }

  double& getDt( double& res,const int& time )
    { 
      res=dt; 
      return res;
    }


 public: /* --- COMMANDS --- */
  virtual void commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
			    std::ostream& os );


};

void cmdChrono( const std::string& cmd,
		std::istringstream& args,
		std::ostream& os );



/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */


/* --- CONSTRUCTION ---------------------------------------------------- */
template< class T >
sotTimer<T>::
sotTimer( const std::string& name )
  :sotEntity(name)
   ,t0(),t1()
   ,dt(0.)
   ,sigSIN( NULL,"sotTimer("+name+")::output(T)::in" )
   ,sigSOUT(  boost::bind(&sotTimer::compute,this,_1,_2),
	      sigSIN,
	      "sotTimer("+name+")::output(T)::out" )
   ,timerSOUT( "sotTimer("+name+")::output(double)::timer" )
{
  sotDEBUGIN(15);
  timerSOUT.setFunction(  boost::bind(&sotTimer::getDt,this,_1,_2) );
	      
  signalRegistration( sigSIN<<sigSOUT<<timerSOUT );
  sotDEBUGOUT(15);
}

/* --- DISPLAY --------------------------------------------------------- */
template< class T >
void sotTimer<T>::
display( std::ostream& os ) const
{
  os << "Timer <"<< sigSIN << "> : " << dt << "ms." << std::endl;
}
/* --- COMMAND --------------------------------------------------------- */
template< class T >
void sotTimer<T>::
commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
	     std::ostream& os )
{
  sotDEBUGIN(15);

  if( cmdLine == "help") 
    {
      os << "Timer: "<<std::endl;
      sotEntity::commandLine( cmdLine,cmdArgs,os );
    }
  else
    sotEntity::commandLine( cmdLine,cmdArgs,os );
  
  
  sotDEBUGOUT(15);
}



#endif /* #ifndef __SOT_SOT_HH */




