/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTracer.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_TRACER_H__
#define __SOT_TRACER_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* Matrix */
#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;

/* STD */
#include <string>
#include <list>
#include <boost/function.hpp>

/* SOT */
//#include <sot/sotExceptionTracer.h>
#include <sot/sotSignalAbstract.h>
#include <sot/sotSignalTimeDependant.h>
#include <sot/sotTimeDependancy.h>
#include <sot/sotEntity.h>
#include <sot/sotExceptionTraces.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotTracer_EXPORTS)
#    define SOTTRACER_EXPORT __declspec(dllexport)
#  else  
#    define SOTTRACER_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTTRACER_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- PARASITE -------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* template< typename Time > */
/* class sotSignalTrigerer */
/* :public sotSignalAbstract<Time>, public sotTimeDependancy<Time> */
/* { */
/*  public: */
/*   mutable boost::function1<void,Time> triger; */
/*   mutable Time timeTriger; */
/*   sotSignalAbstract<Time> * parasiteTarget; */

/*  public: */
  
/*   sotSignalTrigerer( std::string name, boost::function1<void,Time> triger ) */
/*     :sotSignalAbstract<Time>(name),sotTimeDependancy<Time>(this) */
/*     ,triger(triger),timeTriger(-1) {} */

/*   virtual bool needUpdate( const Time& t ) const  */
/*     { */
/*       sotDEBUGIN(15); */
/*       if( t>timeTriger ) triger(t); */
/*       timeTriger = t; */
/*       sotDEBUGOUT(15); */
/*       return false; */
/*     } */

/*   void parasite( sotSignalAbstract<Time>& sig )  */
/*     { sig.addDependancy( *this ); parasiteTarget = &sig;} */

  
/*   virtual void addDependancy( const sotSignalAbstract<Time>& signal ) */
/*     { sotTimeDependancy<Time>::addDependancy(signal); } */
/*   virtual void removeDependancy( const sotSignalAbstract<Time>& signal ) */
/*     { sotTimeDependancy<Time>::removeDependancy(signal); } */
/*   virtual void clearDependancies( void ) */
/*     { sotTimeDependancy<Time>::clearDependancy(); } */
/*   virtual std::ostream& displayDependancies( std::ostream& os,std::string space="", */
/* 					     std::string next1="",std::string next2="" ) const */
/*     { return sotTimeDependancy<Time>::displayDependancies( os,space,next1,next2 ); } */

/* }; */

/* --------------------------------------------------------------------- */
/* --- TRACER ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTTRACER_EXPORT sotTracer
: public sotEntity
{
 protected:
  typedef std::list< const sotSignalAbstract<int>* > SignalList;
  SignalList toTraceSignals;


 public: 
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

 public: 
  enum TraceStyle
    {
      WHEN_SAID   /// Record, then trace to file only when said to.
      ,EACH_TIME  /// Record and trace to file immediately.
      ,FREQUENTLY /// Record X time then trace (X is tuned by setFrenquence() ).
    };
  TraceStyle traceStyle;
  static const TraceStyle TRACE_STYLE_DEFAULT = EACH_TIME;
  double frequency;

  std::string basename;
  std::string suffix;
  std::string rootdir;
  bool namesSet;
  typedef std::list< std::ostream* > FileList;
  FileList files;
  typedef std::list< std::string > NameList;
  NameList names;
  bool play;
  int timeStart;

 public:
  sotTracer( const std::string n );
  virtual ~sotTracer( void ){ closeFiles(); }

  void addSignalToTrace( const sotSignalAbstract<int>& sig,
			 const std::string& filename="" );
  void clearSignalToTrace( void );
  //void parasite( sotSignalAbstract<int>& sig );
  void openFiles( const std::string& rootdir, const std::string& basename,
		  const std::string& suffix );
  virtual void closeFiles( void );

 protected:
  virtual void openFile( const sotSignalAbstract<int> & sig,
			 const std::string& filename );

 public:
  void setTraceStyle( const TraceStyle& style ){ traceStyle = style; }
  TraceStyle getTraceStyle( void ){ return traceStyle; }
  
  void setFrenquency( const double& frqu ){ frequency = frqu; }
  double getFrequency( void ){ return frequency; }

  void record( void );
  virtual void recordSignal( std::ostream& os,
			     const sotSignalAbstract<int>& sig );
  int& recordTrigger( int& dummy, const int& time );

  virtual void trace( void );

 public:
  //sotSignalTrigerer<int> triger;
  sotSignalTimeDependant<int,int> triger; 

  /* --- DISPLAY ------------------------------------------------------------ */
  SOTTRACER_EXPORT friend std::ostream& operator<< ( std::ostream& os,const sotTracer& t );

  /* --- PARAMS --- */
  void display( std::ostream& os ) const;
  virtual void commandLine( const std::string& cmdLine
			    ,std::istringstream& cmdArgs
			    ,std::ostream& os );
};





#endif /* #ifndef __SOT_TRACER_H__ */


