/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTracer.cpp
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* SOT */
#include <sot/sotTracerRealTime.h>
#include <sot/sotDebug.h>
#include <sot/sotPool.h>

#include <boost/bind.hpp>

using namespace std;

#include <sot/sotFactory.h>
SOT_FACTORY_ENTITY_PLUGIN(sotTracerRealTime,"TracerRealTime");

#include <iomanip>

/* --------------------------------------------------------------------- */
/* --- SOTOUTSTRINGSTREAM ---------------------------------------------- */
/* --------------------------------------------------------------------- */

sotOutStringStream::
sotOutStringStream( void )
  : std::ostringstream()
    ,buffer( 0 ),index(0),bufferSize(0),full(false)
{ 
  sotDEBUGINOUT(15);
}

sotOutStringStream::
~sotOutStringStream( void )
{
  sotDEBUGIN(15);
  if( buffer ) delete [] buffer ;
  sotDEBUGOUT(15);
}

void sotOutStringStream::
resize( const unsigned int & size )
{
  sotDEBUGIN(15);
  index=0; bufferSize = size;  full=false;
  if( 0!=buffer ) delete [] buffer;
  buffer = new char [size];
  sotDEBUGOUT(15);
}

bool sotOutStringStream::
addData( const char * data, const unsigned int & size )
{
  sotDEBUGIN(15);
  unsigned int towrite = size; 
  if( index+towrite>bufferSize ) 
    {
      sotDEBUGOUT(15);
      full=true;
      return false;
    }//towrite=bufferSize-index;
  memcpy( buffer+index,data,towrite );
  index+=towrite;
  sotDEBUGOUT(15);
  return true;
  }

void sotOutStringStream::
dump( std::ostream& os )
{
  sotDEBUGIN(15);
  os.write( buffer,index );
  sotDEBUGOUT(15);
}

void sotOutStringStream::
empty( void ) 
{
  sotDEBUGIN(15);
  index=0; full=false;
  sotDEBUGOUT(15);
}



/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */


sotTracerRealTime::sotTracerRealTime( const std::string n )
  :sotTracer(n)
   ,bufferSize( BUFFER_SIZE_DEFAULT )
{
  sotDEBUGINOUT(15);
}  

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */


void sotTracerRealTime::
openFile( const sotSignalAbstract<int> & sig,
	  const std::string& givenname )
{
  sotDEBUGIN(15);
  string signame;
  if( givenname.length() )  
    { signame = givenname;  } else { signame = sig.shortName(); }

  string filename = rootdir + basename + signame + suffix;
  sotDEBUG(5) << "Sig <"<<sig.getName() 
	      << ">: new file "<< filename << endl;
  std::ofstream * newfile = new std::ofstream( filename.c_str() );
  hardFiles.push_back( newfile );

  //std::stringstream * newbuffer = new std::stringstream();
  sotOutStringStream * newbuffer = new sotOutStringStream(); // std::stringstream();
  newbuffer->resize( bufferSize );
  newbuffer->givenname = givenname;
  files.push_back( newbuffer );

  sotDEBUGOUT(15);
}


void sotTracerRealTime::
closeFiles( void )
{
  sotDEBUGIN(15);

  FileList::iterator iter = files.begin();
  HardFileList::iterator hardIter = hardFiles.begin();

  while( files.end()!=iter )
    {
      sotDEBUG(25) << "Close the files." << endl;
    
      std::stringstream * file = dynamic_cast< stringstream* >(*iter);
      std::ofstream * hardFile = *hardIter;
      
      (*hardFile) <<flush; hardFile->close();
      delete file;
      delete hardFile;

      ++iter; ++hardIter;
    }

  sotDEBUG(25) << "Clear the lists." << endl;
  files.clear();
  hardFiles.clear();

  sotDEBUGOUT(15);
}

void sotTracerRealTime::
trace( void )
{
  sotDEBUGIN(15);

  FileList::iterator iter = files.begin();
  HardFileList::iterator hardIter = hardFiles.begin();

  while( files.end()!=iter )
    {
      sotDEBUG(35) << "Next" << endl;
      std::ostream * os = *iter;
      if( NULL==os )
	{ SOT_THROW sotExceptionTraces( sotExceptionTraces::NOT_OPEN,
					"The buffer is null",""); }
      //std::stringstream & file = * dynamic_cast< stringstream* >(os);
      sotOutStringStream * file = dynamic_cast< sotOutStringStream* >(os); // segfault
      if( NULL==file )
	{ SOT_THROW sotExceptionTraces( sotExceptionTraces::NOT_OPEN,
					"The buffer is not open",""); }

      std::ofstream & hardFile = **hardIter;
      if(! hardFile.good() )
	{ SOT_THROW sotExceptionTraces( sotExceptionTraces::NOT_OPEN,
					"The file is not open",""); }

      if( (hardFile.good())&&(NULL!=file) )
	{
 
// 	  const unsigned int SIZE = 1024*8;
// 	  char buffer[SIZE]; 
// 	  streambuf * pbuf = file.rdbuf();
// 	  pbuf->pubseekpos(0);
// 	  const unsigned int NB_BYTE = pbuf->in_avail();
// 	  sotDEBUG(35) << "Bytes in buffer: " << NB_BYTE << endl;
// 	  //sotDEBUG(35) << "Copie" <<endl<<file.str()<< endl;

// 	  for( unsigned int index=0;index<NB_BYTE;index+=SIZE )
// 	    {
// 	      pbuf->pubseekpos( index );
// 	      int nget = pbuf->sgetn( buffer,SIZE );
// 	      sotDEBUG(35) << "Copie ["<<nget<<"] " <<buffer<<endl;
// 	      hardFile.write( buffer,nget );
// 	    }
	  //hardFile << file.str() << flush;
	  //file.seekp(0);

	  file->dump( hardFile );
	  file->empty();
	  hardFile.flush();

	  //file.str("");
	}

      ++iter; ++hardIter;
    }

  sotDEBUGOUT(15);
}

void sotTracerRealTime::
emptyBuffers( void )
{
  sotDEBUGIN(15);
  for( FileList::iterator iter = files.begin();files.end()!=iter;++iter )
    { 
      //std::stringstream & file = * dynamic_cast< stringstream* >(*iter);
      try {
	sotOutStringStream & file  = * dynamic_cast< sotOutStringStream* >(*iter); 
	file.empty();
	//file.str(""); 
      }
      catch( ... ) { SOT_THROW sotExceptionTraces( sotExceptionTraces::NOT_OPEN,
						   "The buffer is not open",""); }
    }
  sotDEBUGOUT(15);
}


// void sotTracerRealTime::
// emptyBuffer( std::stringstream & file )
// {
//   streambuf * pbuf = file.rdbuf();
//   pbuf->file.rdbuf() ->pubsetbuf( fileBuffer,10 );
  


// }
void sotTracerRealTime::
recordSignal( std::ostream& os,
	      const sotSignalAbstract<int>& sig )
{
  sotDEBUGIN(15);

  try {
    
    sotOutStringStream & file = dynamic_cast< sotOutStringStream& >(os);
    file.str("");
    sotDEBUG(45) << "Empty file [" << file.tellp() 
		 << "] <" << file.str().c_str() <<"> " <<endl;

    sotTracer::recordSignal( file,sig );
    file.addData( file.str().c_str(),file.tellp() );
    sotDEBUG(35) << "Write data [" << file.tellp() 
		 << "] <" << file.str().c_str() <<"> " <<endl;

  } catch( sotExceptionAbstract & exc ) { throw exc; }
  catch( ... ) {
    SOT_THROW sotExceptionTraces( sotExceptionTraces::NOT_OPEN,
				  "The buffer is not open",""); 
  }
  

  sotDEBUGOUT(15);
  return ;
}



/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

void sotTracerRealTime::
display( std::ostream& os ) const
{
  os << CLASS_NAME << " " << name << " [mode=" << (play?"play":"pause") 
     << "] : "<< endl
     << "  - Dep list: "<<endl;

  FileList::const_iterator iterFile = files.begin();
  for( SignalList::const_iterator iter = toTraceSignals.begin();
       toTraceSignals.end()!=iter;++iter )
    {
      sotDEBUG(35) << "Next" << endl;
      const sotOutStringStream * file = dynamic_cast< sotOutStringStream* >(*iterFile);
      os << "     -> "<<(*iter)->getName();
      if( file->givenname.length() ) os << " (in " << file->givenname << ")" ;
      os << "\t";
      if( file ) 
	{
	  const std::streamsize PRECISION = os.precision();
	  const unsigned int SIZE = file->index;
	  const unsigned int MSIZE = file->bufferSize;
	  unsigned int dec=0; std::string unit ="";
	  if( (SIZE>>30)||(MSIZE>>30) ) { dec = 30; unit="Go"; }
	  else if( (SIZE>>20)||(MSIZE>>20) ) { dec = 20; unit="Mo"; }
	  else if( (SIZE>>10)||(MSIZE>>10) ) { dec = 10; unit="Ko"; }
 	  os << "[" << std::setw(1)<<std::setprecision(1)
 	     << ((SIZE+0.0)/(1<<dec)) << unit << "/" 
 	     << std::setprecision(2)<<((MSIZE+0.0)/(1<<dec)) 
 	     << unit << "]\t";
	  if( file->full ) os << "(FULL)";
	  os.precision(PRECISION);
	}
      os<<endl;    
      iterFile++;
    }
  
}


std::ostream& operator<< ( std::ostream& os,const sotTracerRealTime& t )
{
  t.display(os);
  return os;
}


void sotTracerRealTime::
commandLine( const std::string& cmdLine
	     ,std::istringstream& cmdArgs
	     ,std::ostream& os )
{
  if( cmdLine=="help" )
    {
      os << "TracerRealTime: "<<endl
	 << "  - empty: trash the buffers." <<endl
	 << "  - bufferSize [<size>]: get/set the buffer size." <<endl;
	
      sotTracer::commandLine( cmdLine,cmdArgs,os );
    }
  else if( cmdLine=="empty" ) 
    {
      emptyBuffers();
    }
  else if( cmdLine=="bufferSize" ) 
    {
      cmdArgs>>ws;
      if( cmdArgs.good() ) 
	{
	  int s; cmdArgs>>s; 
	  setBufferSize( s );
	}
      else os << getBufferSize() << endl;
    }


  else  
    sotTracer::commandLine( cmdLine,cmdArgs,os );



}
