/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      sotTracer.h
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



#ifndef __SOT_TRACER_RT_H__
#define __SOT_TRACER_RT_H__

/* --------------------------------------------------------------------- */
/* --- INCLUDE --------------------------------------------------------- */
/* --------------------------------------------------------------------- */

/* STD */
#include <sstream>

/* SOT */
#include <sot/sotTracer.h>

/* --------------------------------------------------------------------- */
/* --- API ------------------------------------------------------------- */
/* --------------------------------------------------------------------- */

#if defined (WIN32) 
#  if defined (sotTracerRealTime_EXPORTS)
#    define SOTTRACERREALTIME_EXPORT __declspec(dllexport)
#  else  
#    define SOTTRACERREALTIME_EXPORT __declspec(dllimport)
#  endif 
#else
#  define SOTTRACERREALTIME_EXPORT
#endif

/* --------------------------------------------------------------------- */
/* --- TRACER ---------------------------------------------------------- */
/* --------------------------------------------------------------------- */

class SOTTRACERREALTIME_EXPORT sotOutStringStream
  : public std::ostringstream
{
public:
  char * buffer;
  unsigned int index;
  unsigned int bufferSize;
  bool full;
  std::string givenname;

public:

  sotOutStringStream( void );
  ~sotOutStringStream( void );    

  void resize( const unsigned int & size );
  bool addData( const char * data, const unsigned int & size );
  void dump( std::ostream& os );
  void empty( void );
  
};

class SOTTRACERREALTIME_EXPORT sotTracerRealTime
: public sotTracer
{
 public: 
  static const std::string CLASS_NAME;
  virtual const std::string& getClassName( void ) { return CLASS_NAME; }

 protected:
  int bufferSize;
  static const int BUFFER_SIZE_DEFAULT = 1048576; //  1Mo
  typedef std::list< std::ofstream* > HardFileList;
  HardFileList hardFiles;

 public:
  sotTracerRealTime( const std::string n );
  virtual ~sotTracerRealTime( void ){ }

 public:
  virtual void closeFiles( void );
  virtual void trace( void );
 protected:
  virtual void openFile( const sotSignalAbstract<int> & sig,
			 const std::string& filename );
  virtual void recordSignal( std::ostream& os,
			     const sotSignalAbstract<int>& sig );
  void emptyBuffers( void );

 public:
  void setBufferSize( const int & SIZE ) { bufferSize=SIZE; }
  const int & getBufferSize( void ) { return bufferSize; }

 public:

  /* --- DISPLAY ------------------------------------------------------------ */
  void display( std::ostream& os ) const;
  SOTTRACERREALTIME_EXPORT friend std::ostream& operator<< ( std::ostream& os,const sotTracer& t );

  /* --- PARAMS --- */
  virtual void commandLine( const std::string& cmdLine
			    ,std::istringstream& cmdArgs
			    ,std::ostream& os );
};





#endif /* #ifndef __SOT_TRACER_RT_H__ */


