/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      coshell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard from Torea Foissotte (test_corba_shell)
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *  Connection to a sotServerCommand, used as a relay to transmitt
 * shell instruction to the sotInterpretor running inside OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
#include <stdlib.h>

#include <string>
#include <sstream>
#include <iostream>
#include <vector>
#include <fstream>
#include <assert.h>

#include "CorbaConnection.h"
/* IDL automatically generated file. */
#include <ServerCommand.h>

#include <sot/sotDebug.h>

using namespace std;


/* -------------------------------------------------------------------------- */
/* --- OPTIONS -------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

class OptArgs 
{
public:
  std::string corbaServerName;
  std::string remoteServerName;
  std::string serverKind;

  OptArgs( void ) 
    : corbaServerName( "coshell" )
    ,remoteServerName("localhost")
    ,serverKind("")  
  {}

  static const char optlist [];

public:
  
  void parseOptions(  int argc,char** argv )
  {
    int ch;
    while( (ch=getopt( argc,argv,optlist ))!=-1 ) 
    {
      switch (ch) 
	{
	case 'c':
	  corbaServerName = optarg;
	  break;
	case 'r':
	  remoteServerName = optarg;
	  break;
	case 'k':
	  serverKind = optarg;
	  break;
	case '?':
	default:
	  usage();
	}
    }
  }
  void usage( void ) 
  { cout << "coshell [-c pluginName] [-r remoteServerName] [-k serverKind]"<<endl; }
};

const char OptArgs::optlist [] = "c:r:k:";

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */



int main( int argc,char ** argv )
{
  sotDEBUGIN(15);

  /* --- INIT --- */
  /* --- INIT --- */
  /* --- INIT --- */

  CorbaConnection * corbaConnectionHandler = NULL;
  SOT_Server_Command_ptr serverPtr = NULL;
  OptArgs args; args.parseOptions(argc,argv);

  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  /* --- CONNECTION --- */
  cout << " Corba Server Name : " << args.corbaServerName << endl
       << " Server Kind : " << args.serverKind << endl
       << " Remote Server Name : " << args.remoteServerName << endl;
  try
    {
      corbaConnectionHandler = new CorbaConnection( args.corbaServerName, 
						    args.serverKind,
						    args.remoteServerName );
      sotDEBUG(15) << "Connection ..." << endl;
      CORBA::Object_ptr corba_obj 
	= corbaConnectionHandler ->getObjectReference(); 
      
      sotDEBUG(15) << "Connection [OK]. Cast ... " << endl;
      serverPtr = SOT_Server_Command::_narrow( corba_obj );
      
      if(CORBA::is_nil( serverPtr ) )
	{
	  std::cout <<"Wrong server " <<std::endl;
	  exit(0);
	}
      sotDEBUG(1) << "Cast [OK]. Run the shell." << endl;
    }
  catch(...)
    {
      cerr<<"Exception when trying to connect to the corba server \"serverCorbaSot\"";
      cerr<<"the program continues but cannot communicate.."<<endl;
      serverPtr = NULL;
      exit(-1);
    }

  /* --- SHELL LOOP --- */
  /* --- SHELL LOOP --- */ 
  /* --- SHELL LOOP --- */

  std::cout << serverPtr << " " << CORBA::is_nil( serverPtr ) <<std::endl;
  SOT_Server_Command::CharSeq input;
  SOT_Server_Command::StringStreamer_var output;
  const unsigned int BUFFER_SIZE = 1024;
  char buffer [ BUFFER_SIZE ] ;

  while( 1 )
    //for( int loop=0;loop<15;++loop )
    {
      cout<<"> ";
      cin.getline( buffer,BUFFER_SIZE ); 
      unsigned int nbCar = cin.gcount();
      if( 1<nbCar )
	{
	  sotDEBUG(5) << nbCar << ": " << buffer << std::endl;
	  /* If "exit" is red, break the shell. */
	  if( ( nbCar>=sizeof("exit") )
	      &&( 0==strncmp(buffer,"exit",sizeof("exit")) ) )
	    {	      break; 	    }

	  /* Copy the buffer to the corba-friendly structure. */
	  input.length( nbCar-1 );
	  /* Take care not to send the last '\n' caracter. */
	  for( unsigned int i=0;i<nbCar-1;++i ){ input[i]=buffer[i]; }
	  
	  /* Send the command to the distant server and 
	   * print the result. */
	  if(! CORBA::is_nil(serverPtr) )
	    {try
	      {
		serverPtr->runAndRead( input,output );
		sotDEBUG(25) <<"server answered with ";
		for( unsigned int i=0;i<output->length();i++ )	
		  { cout << output[i]; }
		cout << endl;
	      }
	    catch(...)
	      { cerr<<"!!\tException catched when trying to communicate!"<<endl; }
	    }
	  else { cerr<< "!!\tCorba not plug. Nothing sent."<<endl; }
	}
    }

  cout<<"Shell done! (see you soon...)"<<endl;

  return 0;
}


