/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_corba_shell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard from Torea Foissotte (test_corba_shell)
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *  Connection to a sotServerCommand, used as a relay to transmitt
 * shell instruction to the sotInterpretor running inside OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <string>
#include <sstream>
#include <iostream>
#include <vector>
#include <fstream>

#include "CorbaConnection.h"
#include "ServerCommand.hh"
#include <sot/sotDebug.h>

using namespace std;


int tryCorbaConnection( CorbaConnection *g_aCC, SOT_Server_Command_ptr &serverPtr )
{
  CORBA::Object_ptr corba_obj = NULL;
	
  try {
    corba_obj = g_aCC->getObjectReference();
  }
  catch(...) {
    cerr << "sot Corba generic server not found" << endl;
    return 1;
  }

  try {
    serverPtr = SOT_Server_Command::_narrow( corba_obj );
  }
  catch(...) {
    cerr << "sot Corba generic server not narrowed" << endl;
    return 1;
  }

  if( !CORBA::is_nil( serverPtr ) )
    cerr << "Success!" << endl;
  else{
    cerr << "Failed..." << endl;
    return 1;
  }

  return 0;
}




int main(void)
{
  cout << "Wait for clearance (start corba server in OpenHRP then heat a touch)." << endl;
  { int i; cin >> i; }
  cout << "... get clearance, roger." << endl;

  string aServerName = "serverCorbaSot";
  string aServerKind = "";
  string aHostnameNameServer = "localhost";

  bool server_here=true;

  CorbaConnection *g_aCC=NULL;
  SOT_Server_Command_ptr serverPtr;

  cout<<"client begins!\n";

  try{
    g_aCC = new CorbaConnection( aServerName, aServerKind, aHostnameNameServer );
	
    tryCorbaConnection( g_aCC, serverPtr );
  }catch(...){
    cout<<"Exception when trying to connect to the corba server \"serverCorbaSot\"";
    cout<<"the program continues but cannot communicate..\n";
    server_here = false;
  }

  // communication with the SOT_Server_Command..

  SOT_Server_Command::CharSeq input;
  SOT_Server_Command::StringStreamer_var output;
  const unsigned int BUFFER_SIZE = 1024;
  char buffer [ BUFFER_SIZE ] ;

  //while( 1 )
  
  {
  input.length(1); input[0]='1'; 
  long int rank = serverPtr->createInputVectorSignal( input );
  { int waIt; cin >>waIt; }
    SOT_Server_Command::DoubleSeq_var outputDouble;
    serverPtr->readInputVectorSignal( rank,outputDouble );
    for( unsigned int i=0; i<outputDouble->length(); i++ )	
      { cout << " :" << outputDouble[i]; } cout<<std::endl;
  }
 

    for( int loop=0;loop<15;++loop )
    {
      cout<<"> ";
      cin.getline( buffer,BUFFER_SIZE ); 
      unsigned int nbCar = cin.gcount();
      if( 1<nbCar )
	{
	  sotDEBUG(5) <<buffer <<endl; 
	  cout << nbCar << ": " << buffer << endl;
	  input.length( nbCar-1 );
	  /* Take care not to send the last '\n' caracter. */
	  for( unsigned int i=0;i<nbCar-1;++i ) input[i]=buffer[i];
	  
	  if( server_here )
	    try
	      {
		//serverPtr->run( input );
		serverPtr->runAndRead( input,output );
		cout<<"server answered with ";
		for( unsigned int i=0; i<output->length(); i++ )	
		  cout<<output[i];
		cout << endl;
	      }
	    catch(...)
	      {
		cout<<"exception catched when trying to communicate!"<<endl;
	      }
	}
    }

  cout<<"client finishes!\n";

  return 0;
}


