#include <string>
#include "dynamicsJRLJapan/Joint.h"
#include "dynamicsJRLJapan/HumanoidDynamicMultiBody.h"
#include "robotDynamics/jrlRobotDynamicsObjectConstructor.h"
using namespace std;
using namespace dynamicsJRLJapan;

/* --- DISPLAY TREE --------------------------------------------------------- */
void RecursiveDisplayOfJoints(const CjrlJoint *aJoint)
{
  if( aJoint==0 )return;
  const Joint *a2Joint = dynamic_cast< const Joint* >( aJoint );
  if( a2Joint==0 )return;
  int NbChildren = aJoint->countChildJoints();
  cout << a2Joint->getName() 
       << " rank : " << a2Joint->rankInConfiguration() << endl;

  for(int i=0;i<NbChildren;i++)
    { RecursiveDisplayOfJoints(aJoint->childJoint(i));   }
}


void DisplayDynamicRobotInformation(CjrlDynamicRobot *aDynamicRobot)
{
  std::vector<CjrlJoint *> aVec = aDynamicRobot->jointVector();
  int r = aVec.size();
  cout << "Number of joints :" << r << endl;
  for(int i=0;i<r;i++)
    {
      Joint * aJoint = dynamic_cast<Joint *>(aVec[i]);
      cout << aJoint->getName();
    }	

  
}

void DisplayMatrix(MAL_MATRIX(,double) &aJ)
{
  for(unsigned int i=0;i<6;i++)
    {
      for(unsigned int j=0;j<MAL_MATRIX_NB_COLS(aJ);j++)
	{
	  if (aJ(i,j)==0.0)
	    printf("0 ");
	  else
	    printf("%10.5f ",aJ(i,j));
	}
      printf("\n");
    }

}

/* --- DISPLAY MASS PROPERTIES OF A CHAIN --- */
void GoDownTree(const CjrlJoint * startJoint)
{
  cout << "Mass-inertie property of joint ranked :" 
       << startJoint->rankInConfiguration() 
       << " (name :" << ((Joint *)startJoint)->getName() <<")"<< endl;
  cout << "Force on the related Body : " 
       << ((DynamicBody *)(startJoint->linkedBody()))->m_Force << endl;
  cout << "Torque on the related Body : " 
       << ((DynamicBody *)(startJoint->linkedBody()))->m_Torque << endl;
  cout << "Mass of the body: " 
       << ((DynamicBody *)(startJoint->linkedBody()))->getMasse() << endl;
  cout << "Name of the body: " 
       << ((DynamicBody *)(startJoint->linkedBody()))->getName() << endl;
  cout << "llimit: " 
       << ((Joint *)startJoint)->lowerBound(0)*180/M_PI
       << " ulimit: " << ((Joint *)startJoint)->upperBound(0)*180/M_PI << endl;
  cout << startJoint->currentTransformation() << endl;
  
  if (startJoint->countChildJoints()!=0)
    {
      const CjrlJoint * childJoint = startJoint->childJoint(0);
      GoDownTree(childJoint);
    }
}

/* --- MAIN ----------------------------------------------------------------- */
/* --- MAIN ----------------------------------------------------------------- */
/* --- MAIN ----------------------------------------------------------------- */
int main(int argc, char *argv[])
{
  if (argc!=4)
    {
      cerr << " This program takes 3 arguments: " << endl;
      cerr << "./TestHumanoidDynamicRobot PATH_TO_VRML_FILE "
	   << "VRML_FILE_NAME PATH_TO_SPECIFICITIES_XML" << endl;
      exit(-1);
    }	

  string aSpecificitiesFileName = argv[3];
  string aPath=argv[1];
  string aName=argv[2];

//   DynamicMultiBody * aDMB
//     = new DynamicMultiBody();
//   aDMB->parserVRML(aPath,aName,"");
//   HumanoidDynamicMultiBody *aHDMB 
//     = new HumanoidDynamicMultiBody(aDMB,aSpecificitiesFileName);

  /* ------------------------------------------------------------------------ */
  CjrlRobotDynamicsObjectConstructor<
    dynamicsJRLJapan::DynamicMultiBody, 
    dynamicsJRLJapan::HumanoidDynamicMultiBody, 
    dynamicsJRLJapan::JointFreeflyer, 
    dynamicsJRLJapan::JointRotation,
    dynamicsJRLJapan::JointTranslation,
    dynamicsJRLJapan::Body> aRobotDynamicsObjectConstructor;
  
  CjrlHumanoidDynamicRobot* aHDR 
    = aRobotDynamicsObjectConstructor.createhumanoidDynamicRobot();
  
  HumanoidDynamicMultiBody* aHDMB
    = dynamic_cast<dynamicsJRLJapan::HumanoidDynamicMultiBody *>(aHDR);
  if (aHDMB==0)
    { 
      cerr<< "Dynamic cast on HDR failed " << endl;
      exit(-1);
    }

  // DynamicMultiBody * aDMB
//     = (DynamicMultiBody *) aHDMB->getDynamicMultiBody();
  aHDMB->parserVRML(aPath,aName,
		   "/home/nmansard/src/PatternGeneratorJRL/src/data/HRP2LinkJointRank.xml");
  aHDMB->SetHumanoidSpecificitiesFile(aSpecificitiesFileName);
  cout << "-> Finished the initialization"<< endl;
  /* ------------------------------------------------------------------------ */
  
  // Display tree of the joints.
  CjrlJoint* rootJoint = aHDMB->rootJoint();  
  RecursiveDisplayOfJoints(rootJoint);

  // Test the computation of the jacobian.
  double dInitPos[40] = { 
    0.0, 0.0, -26.0, 50.0, -24.0, 0.0, 0.0, 0.0, -26.0, 50.0, -24.0, 0.0,  // legs

    0.0, 0.0, 0.0, 0.0, // chest and head

    15.0, -10.0, 0.0, -30.0, 0.0, 0.0, 10.0, // right arm
    15.0,  10.0, 0.0, -30.0, 0.0, 0.0, 10.0, // left arm 

    -20.0, 20.0, -20.0, 20.0, -20.0, // right hand
    -10.0, 10.0, -10.0, 10.0, -10.0  // left hand
  };

  int NbOfDofs = aHDMB->numberDof();
  cout << "NbOfDofs :" << NbOfDofs << endl;

  /* Set current conf to dInitPos. */
  MAL_VECTOR_DIM(aCurrentConf,double,NbOfDofs);
  for( int i=0;i<((NbOfDofs<46)?NbOfDofs:46);++i )
    if( i<6 ) aCurrentConf[i] = 0.0;
    else aCurrentConf[i] = dInitPos[i-6]*M_PI/180.0;
  aHDMB->currentConfiguration(aCurrentConf);

  /* Set current velocity to 0. */
  MAL_VECTOR_DIM(aCurrentVel,double,NbOfDofs); 
  for(int i=0;i<NbOfDofs;i++) aCurrentVel[i] = 0.0;
  aHDMB->currentVelocity(aCurrentVel);

  /* Compute ZMP and CoM */
  MAL_S3_VECTOR(ZMPval,double);
  aHDMB->setComputeZMP(true);
  aHDMB->computeForwardKinematics();
  ZMPval = aHDMB->zeroMomentumPoint();
  cout << "First value of ZMP : " << ZMPval <<endl;
  cout << "Should be equal to the CoM (on x-y): " 
       << aHDMB->positionCenterOfMass() << endl;

  /* Get Rhand joint. */

  cout << "****************************" << endl;
  cout << "Rank of the left hand "<< endl;
  cout << aHDMB->leftWrist()->rankInConfiguration() << endl;
  cout << ((Joint *)aHDMB->leftWrist())->getName() << endl;
  cout << ((Joint *)aHDMB->leftWrist())->getIDinVRML() << endl;

  aHDMB->LinkBetweenJointsAndEndEffectorSemantic();
  vector<CjrlJoint *> aVec = aHDMB->jointVector();
  Joint * aJoint = (Joint *)aVec[22]; // Try to get the hand.
  cout << "Joint [22] name (should be Rhand): "
       << aJoint->getName() << endl;  
  aJoint->computeJacobianJointWrtConfig();
  MAL_MATRIX(,double) aJ = aJoint->jacobianJointWrtConfig();
  DisplayMatrix(aJ);
  
  /* Get Waist joint. */
  cout << "****************************" << endl;
  cout << "Root: " << ((Joint *)rootJoint)->getName() << endl;
  rootJoint->computeJacobianJointWrtConfig();
  aJ = rootJoint->jacobianJointWrtConfig();  
  cout << "Rank of Root: " << rootJoint->rankInConfiguration() << endl;
  aJoint = (Joint *)aHDMB->waist();
  cout << "Name of the WAIST joint :" << aJoint->getName() << endl;

  /* Get CoM jacobian. */
  cout << "****************************" << endl;
  aHDMB->computeJacobianCenterOfMass();
  cout << "Value of the CoM's Jacobian:" << endl
       << aHDMB->jacobianCenterOfMass() << endl;

  /* Display the mass property of the leg. */
  cout << "****************************" << endl;
  GoDownTree(aHDMB->rootJoint());
  cout << "Mass of the robot " << aHDMB->getMasse() << endl;
  cout << "Force " << aHDMB->getMasse()*9.81 << endl;

  cout << "****************************" << endl;
  MAL_VECTOR_FILL(aCurrentVel,0.0);
  MAL_VECTOR_DIM(aCurrentAcc,double,NbOfDofs);
  MAL_VECTOR_FILL(aCurrentAcc,0.0);

  // This is mandatory for this implementation of computeForwardKinematics
  // to compute the derivative of the momentum.
  aHDMB->SetTimeStep(0.005);
  aHDMB->setComputeAcceleration(false);
  aHDMB->setComputeBackwardDynamics(false);
  aHDMB->setComputeZMP(true);
  for(int i=0;i<4;i++)
    {
      aHDMB->currentVelocity(aCurrentVel);
      aHDMB->currentAcceleration(aCurrentAcc);
      aHDMB->computeForwardKinematics();
      ZMPval = aHDMB->zeroMomentumPoint();
      cout << i << "-th value of ZMP : " << ZMPval <<endl;
      cout << "Should be equal to the CoM: " << aHDMB->positionCenterOfMass() << endl;
    }

  // Height of the foot. 
  cout << "****************************" << endl;
  cout << "Height foot: "<< aHDMB->footHeight() << endl;

  // The End!
  delete aHDMB;
  
}
