/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2008
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      LocalStackOfTasks.h
 * Project:   SOT
 * Author:    Francois Keith, Olivier Stasse
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * This file is largely inspired from the StackOfTasks in the 
 * plugin directory. It is intend to test the walking pattern generator
 * output without OpenHRP.
 * 
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifdef WIN32
#include <MatrixAbstractLayer/boost.h>
#include <Windows.h>

#include <float.h>
#define isnan _isnan

#else 
#include <dlfcn.h> 
#endif

#ifdef WIN32
	typedef HMODULE sotPluginKey;
#else
	typedef void* sotPluginKey;
#endif

#include <SimulatedStackOfTasks.h>

robot_state::robot_state()
{
  angle.length(SimulatedStackOfTasks::DOF);
  velocity.length(SimulatedStackOfTasks::DOF);
  torque.length(SimulatedStackOfTasks::DOF);

  angle.length(SimulatedStackOfTasks::DOF);
  waistPos.length(3);
  waistRpy.length(3);
  zmp.length(3);

  force.length(4);
  for(unsigned int i=0;i<4;i++)
    force[i].length(6);

  attitude.length(1);
  attitude[0].length(3);

}

robot_state::~robot_state()
{
}

motor_command::motor_command()
{
  angle.length(SimulatedStackOfTasks::DOF);
  waistPos.length(3);
  waistRpy.length(3);
  zmp.length(3);
}

motor_command::~motor_command()
{
}

static std::string HRPname = "OpenHRP";

static bool admissibleState( SimulatedStackOfTasks::sotPluginState state,char admis )
{
  return ( state&admis );
}


SimulatedStackOfTasks::SimulatedStackOfTasks(istringstream &strm)
  : sotEntity(HRPname)
   ,pluginState(sotJUST_BUILT)
    ,pluginLoader()
  ,iter(0)
  ,robotStatePrec(SimulatedStackOfTasks::DOF)
  ,robotStatePrecInit(false)
  ,reinitFromMc(true)
    
  ,controlSIN(NULL,"OpenHRP::input(vector40)::control")
  ,attitudeSIN(NULL,"OpenHRP::input(vector3)::attitudeIN")
  ,zmpSIN(NULL,"OpenHRP::input(vector3)::zmp")
  ,stateSOUT("OpenHRP::output(vector46)::state")
  ,attitudeSOUT("OpenHRP::output(vectorRPY)::attitude")
    
  ,pseudoTorqueSOUT("OpenHRP::output(vector)::ptorque")
  ,activatePseudoTorqueSignal(false)
  ,previousControlSOUT("OpenHRP::output(vector)::previousControl")
  ,activatePreviousControlSignal(false)
   
{


    if( sotDEBUG_ENABLE(1) ) { sotDebugTrace::openFile(); }
  sotDEBUGIN(5);


  /* --- FORCES --- */
  for( int i=0;i<4;++i ){ withForceSignals[i] = false; }
  forcesSOUT[0] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceRLEG");
  forcesSOUT[1] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceLLEG");
  forcesSOUT[2] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceRARM");
  forcesSOUT[3] =
    new sotSignal<ml::Vector, int>("OpenHRP::output(vector6)::forceLARM");

  signalRegistration( controlSIN
		      <<attitudeSIN
		      <<zmpSIN
		      <<stateSOUT
		      <<*forcesSOUT[0]
		      <<*forcesSOUT[1]
		      <<*forcesSOUT[2]
		      <<*forcesSOUT[3]
		      <<attitudeSOUT
		      //		      <<velocitySOUT
		      <<pseudoTorqueSOUT
		      <<previousControlSOUT);
  {
#ifndef WIN32
	sotPluginKey dlib = dlopen( "libsot-0.so",RTLD_NOW|RTLD_GLOBAL); 
#else
	sotPluginKey dlib = LoadLibrary ( "libsot-0.lib" ); 
#endif /*WIN32*/

    if( NULL==dlib ) 
      {
#ifndef WIN32
	sotDEBUG(5) << "Failure while loading: " <<dlerror() <<endl;
	SOT_THROW sotExceptionFactory( sotExceptionFactory::DYNAMIC_LOADING,
				       "Error while dlopen. ",dlerror() );
#else
		  // Retrieve the system error message for the last-error code
    LPTSTR pszMessage;
    DWORD dwLastError = GetLastError(); 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwLastError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&pszMessage,
        0, NULL );


	  sotDEBUG(5) << "Failure while loading: " << pszMessage <<endl;


	  SOT_THROW sotExceptionFactory( sotExceptionFactory::DYNAMIC_LOADING,
	  				 "Error while dlopen. ", pszMessage );

	  LocalFree(pszMessage);
#endif
      }
  }

  //std::cout << setprecision (9);

  sotDEBUGOUT(5);

}

SimulatedStackOfTasks::~SimulatedStackOfTasks(void)
{
  sotDEBUGIN(5);

  for( int i=0;i<4;++i ) { delete forcesSOUT[i]; }

  sotDEBUGOUT(5);
}

void SimulatedStackOfTasks::
m_script( istringstream &strm )
{
  sotDEBUGIN(5);

  string cmdLine;
  strm >>skipws>> cmdLine ;
  sotDEBUG(1) << "Cmd <" <<cmdLine<<">"<<endl;

  try {
    sotShell.cmd( cmdLine,strm, std::cout );
  } catch( const sotExceptionAbstract& e ) { std::cout << "!! " << e <<endl; }

  sotDEBUGOUT(5);
}

bool SimulatedStackOfTasks::setup(robot_state *rs,
				  motor_command *mc)
{
  sotDEBUG(25) << "Iter = "<<iter 
	       <<"   state = "<< pluginState << endl;
  sotDEBUGIN(5);

  if(! admissibleState( pluginState,sotINIT )) return false;
  ml::Vector mlrs(46);
  sotDEBUG(25) << endl;
  for( int i=0;i<40;++i )
    {mlrs(i+6) = rs->angle[i]; }
  stateSOUT.setConstant( mlrs );
  stateSOUT.setTime( iter );

  ml::Vector mlforces(6);
  for( int i=0;i<4;++i )
  {
    if( withForceSignals[i] == true )
    {
      for( int j=0;j<6;++j )
	mlforces(j) = rs->force[i][j];
      forcesSOUT[i]->setConstant( mlforces );
      forcesSOUT[i]->setTime( iter );
    }
  }

  sotDEBUG(25) << mlrs <<endl;

  pluginState = sotSETUP;
  sotDEBUGOUT(5);

  return true;
}

void SimulatedStackOfTasks::sotInit( void )
{

  sotShell.referencePluginLoader(&pluginLoader);

  iter = 0;
  pluginState = sotINIT;
}

void SimulatedStackOfTasks::control(robot_state *rs,
				    motor_command *mc)
{
  sotDEBUG(25) << "Iter = "<<iter 
	       <<"   state = "<< pluginState << endl;

  switch( pluginState )
    {
    case sotSETUP: pluginState = sotLOOP; // no break
    case sotLOOP: 
      sotControlLoop( rs,mc );
      break;
    case sotFINISHING:
    case sotHOLD:
      sotControlFinishing( rs,mc );
      break;
    default:
      break;
    }

  return;
}

bool SimulatedStackOfTasks::cleanup(robot_state *rs,
				    motor_command *mc)
{
  sotDEBUGIN(5);
  sotDEBUG(25) << "Iter = "<<iter 
	       <<"   state = "<< pluginState << endl;
  
  if( admissibleState( pluginState,sotLOOP ) ) 
    {
      pluginState = sotHOLD; 
      sotControlFinishing( rs,mc ); 
      return false; 
    }
  if( admissibleState( pluginState,sotFINISHING )) sotControlFinishing(rs,mc);
  if(! (admissibleState( pluginState,sotFINISHED )
	||admissibleState( pluginState,sotCLEANED_UP )) ) return false;
  pluginState = sotCLEANED_UP;


  sotDEBUGOUT(5);
  return true;
}

void SimulatedStackOfTasks::sotControlLoop( robot_state* rs, motor_command* mc )
{
  if( !admissibleState( pluginState,sotLOOP )) return;
  iter ++;
  sotDEBUG(15) << "--- Iter = "<<iter 
	       <<"-------------------------------------------------"<<endl;

  /* --- RS --- */
  ml::Vector mlrs(46);
  sotDEBUG(25) << endl;
  for( int i=0;i<6;++i )mlrs(i)=0.;
  for( int i=0;i<40;++i )
    {mlrs(i+6) = rs->angle[i]; }
  stateSOUT.setConstant( mlrs );
  stateSOUT.setTime( iter );

  sotDEBUG(25) << mlrs <<endl;
  if(! robotStatePrecInit )
    {
      sotDEBUG(25) << "Reinit RS." <<endl;
      for( int i=0;i<40;++i )
	if( reinitFromMc ) {  robotStatePrec(i) = mc->angle[i]; }
	else {  robotStatePrec(i) = mlrs(i+6); }
      robotStatePrecInit = true;
    }

  /* --- FORCES --- */
  ml::Vector mlforces(6);
  for( int i=0;i<4;++i )
  {
    if( withForceSignals[i] == true )
    {
      for( int j=0;j<6;++j )
	mlforces(j) = rs->force[i][j];
      forcesSOUT[i]->setConstant( mlforces );
      forcesSOUT[i]->setTime( iter );
    }
  }

  /* --- KF --- */
  sotVectorRollPitchYaw mlkf;  
  if( rs->attitude.length() )
    {
      sotDEBUG( 35 ) << "Get KF " <<endl;
      for( unsigned int i=0;i<3;++i ) mlkf(i)=rs->attitude[0][i];
    }
  attitudeSOUT.setConstant( mlkf );
  sotDEBUG(25) << mlkf <<endl;

  /* --- OTHER SIG --- */
  if( activatePseudoTorqueSignal&&rs->torque.length() )
    {
      unsigned int SIZE = rs->torque.length();
      ml::Vector torque(SIZE);
      for( unsigned int i=0;i<SIZE;++i ) torque(i)=rs->torque[i];
      pseudoTorqueSOUT.setConstant( torque );
      pseudoTorqueSOUT.setTime( iter );
    }
  if( activatePreviousControlSignal&&(robotStatePrec.size()>0) )
    {
      const double dtinv= 1./timestep;
      ml::Vector previousControl(robotStatePrec.size());
      robotStatePrec.opposite(previousControl);
      for( int i=0;i<40;++i ) 
	{
	  previousControl(i) += mc->angle[i];
	  previousControl(i) *= dtinv;
	}
      previousControlSOUT = previousControl;
    }
//   sotDEBUG(15) << "Velocity = " << rs->velocity.length()<<std::endl;
//   if( activateVelocitySignal&&rs->velocity.length() )
//     {
//       unsigned int SIZE = rs->velocity.length();
//       ml::Vector velocity(SIZE);
//       for( unsigned int i=0;i<SIZE;++i ) velocity(i)=rs->velocity[i];
//       velocitySOUT.setConstant( velocity );
//       velocitySOUT.setTime( iter );
//     }

  /* --- PERIODIC CALL --- */
  periodicCallBefore.run( iter );
  
  /* --- MC --- */
    ml::Vector mlmc(SimulatedStackOfTasks::DOF);

    mlmc.fill(.0);
    try{
      if(! suspend ) mlmc = controlSIN(iter);
    }catch( const sotExceptionAbstract& e ) {sotDEBUG(1) << e;}
    catch( ... ) { sotDEBUG(1) << "Unknown catched." <<endl; }
    sotDEBUG(25) << mlmc <<endl;

    for( int i=0;i<40;++i )
      if( isnan(mlmc(i)) ) 
	{	
	  sotDEBUG(1) << "!! Plug ->MC broken."<<endl;
	  controlSIN.unplug();
	  mlmc.fill(.0); break;
	}

    sotDEBUG(25) << robotStatePrec <<endl;
    robotStatePrec += (mlmc*0.005);
    for( int i=0;i<40;++i ) 
      {
	mc->angle[i] = robotStatePrec(i);
	sotDEBUG(45) << i << "*: "<<mc->angle[i] <<endl;
      }

  /* --- WAIST ATTITUDE --- */
  try {
    const ml::Vector & attitudemc = attitudeSIN(iter);
    for( int i=0;i<3;++i ) mc->waistRpy[i] = attitudemc(i);
  } catch (...) {}
  
  /* --- ZMP REF --- */
  try {
    const ml::Vector & zmpmc = zmpSIN(iter);
    sotDEBUG(25) << "Copy zmpref:"<< endl << zmpmc << endl;
    for( int i=0;i<3;++i ) mc->zmp[i] = zmpmc(i);
  } catch (...) {}
 
  /* --- PERIODIC CALL --- */
  periodicCallAfter.run( iter );
 
}

void SimulatedStackOfTasks::
sotControlFinishing(robot_state *rs, motor_command *mc)
{ 
  sotDEBUG(15) << "--- Iter = "<<iter 
	       <<"-------------------------------------------------"<<endl;
  if( admissibleState( pluginState,sotHOLD )) 
  {
    pluginState=sotLOOP;    
    sotControlLoop(rs,mc);

#if 0
    if( seqpluginPTR )
      {
	//seqpluginPTR->goActual();

	//	RobotState_var ref_state;	
	// 	for( int i=0;i<40;++i ) 
	// 	  {
	// 	    ref_state->angle[i] = robotStatePrec(i);
	// 	  }
	seqpluginPTR->setReferenceState(*mc,0.005);                              // send seq the next ref_state              
	seqpluginPTR->goHalfSitting(5.0);
      }
#endif
    for( unsigned int i=0;i<SimulatedStackOfTasks::DOF;i++)
      {
	//mc->angle[i]=rs->angle[i];
	 //mc->angle[i] = m_PrevMc[i];
      }
    pluginState=sotFINISHING;
    return ;
  } 

  if( admissibleState( pluginState,sotFINISHING )) 
  {
    //    if( seqpluginPTR->isEmpty() ) 
      pluginState=sotFINISHED;
  } 

  return ;
}

void SimulatedStackOfTasks::play( void )
{
  if( suspend )
    {
      suspend = false;
      //robotStatePrecInit=false;
    }
}


void SimulatedStackOfTasks::pause( void )
{
  if(!suspend) 
    {
      suspend = true;
      //robotStatePrecInit=false;
    }
}


void SimulatedStackOfTasks::commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,std::ostream& os )
{

  if( cmdLine =="help" )
  {
    os << "Plugin OpenHRP: " << std::endl
       << " - play/pause: \tStart/Stop writing the controlSIN"
       << " in the robot MC." << endl
       << " - reinit [from {MC|RS}]: \treinit the current pos value (default MC)" <<endl
       << " - withForces <i> \tCopy the force sensor data from sensor " 
       << "<i> in the corresponding SOUT." <<endl
       << " - whichForces: \tDisplay the activated force signals." <<endl
       << " - with{PseudoTorque|PreviousControl} [{true|false}]: "
       << "activate/inactivate the signal." << endl
//        << " - with{Velocity|PseudoTorque} [{true|false}]: "
//        << "activate/inactivate the signal." << endl
       << " - periodicCall{Before|After}:\tAccess to the periodic caller. "
       << "\'Before\' and \'After\' refeere to the MC computation." <<endl
       << " - timestep [<value>]: get/set the timestep (=0.005)." << std::endl;
    sotEntity::commandLine(cmdLine,cmdArgs,os);
  }
  else if( cmdLine == "print" ) 
    {
      os << getName() << " [mode=" << ((suspend)?"pause":"play") <<"]."<<endl;
    }
  else if( cmdLine == "pause" ) { pause(); }
  else if( cmdLine == "play" )  { play(); }
  else if( cmdLine == "reinit" )
    {
      std::string fromMc;
      cmdArgs>>ws>>fromMc>>ws;
      if( ("from"==fromMc)&&(cmdArgs.good()) )
	{
	  cmdArgs>>fromMc;
	  if( fromMc=="rs" ) { robotStatePrecInit=false; reinitFromMc=false; }
	  else if( fromMc=="mc" ) { robotStatePrecInit=false; reinitFromMc=true; }
	}
      else { robotStatePrecInit=false; reinitFromMc=true; }
    }
  else if( cmdLine == "withForces" )
  {
    int index;
    cmdArgs >> index;
    if((index >= 0) && (index < 4))
      cmdArgs >> withForceSignals[index];
  }
  else if( cmdLine == "whichForces" )
  {
    os << "Force signals: " << endl; 
    for( unsigned int i=0;i<4;++i )
      {
	os << "\t- Force " << i << ": ";
	if( withForceSignals[i] )
	  { os << "Active";	  } else { os << "Inactive"; } 
	os << endl;
      }
  }
//   else if( cmdLine == "withVelocity" )
//   {
//     cmdArgs >> std::ws; if( cmdArgs.good() )
//       {
// 	std::string val; cmdArgs>>val; 
// 	activateVelocitySignal = ( (val=="true")||(val=="1") );
//       } else {
// 	os << "withVelocity = " << activateVelocitySignal << std::endl;
//       }
//   }
  else if( cmdLine == "withPseudoTorque" )
  {
    cmdArgs >> std::ws; if( cmdArgs.good() )
      {
	std::string val; cmdArgs>>val; 
	activatePseudoTorqueSignal = ( (val=="true")||(val=="1") );
      } else {
	os << "withPseudoTorque = " << activatePseudoTorqueSignal << std::endl;
      }
  }
  else if( cmdLine == "withPreviousControl" )
  {
    cmdArgs >> std::ws; if( cmdArgs.good() )
      {
	std::string val; cmdArgs>>val; 
	activatePreviousControlSignal = ( (val=="true")||(val=="1") );
      } else {
	os << "withPreviousControl = " << activatePreviousControlSignal << std::endl;
      }
  }
  // Kept two tests for compatibility reason. 
  else if(( cmdLine == "periodicCall" )||( cmdLine == "periodicCallAfter" ))
    { 
      string cmd2; cmdArgs >> cmd2;
      periodicCallAfter .commandLine( cmd2,cmdArgs,os );
    }
  else if( cmdLine == "periodicCallBefore" )
    { 
      string cmd2; cmdArgs >> cmd2;
      periodicCallBefore .commandLine( cmd2,cmdArgs,os );
    }
  else if( cmdLine == "timestep" )
    { 
      cmdArgs >> std::ws;
      if( cmdArgs.good() ) { cmdArgs >> timestep; } 
      else { os << "timestep = " << timestep << std::endl; }
    }



  else sotEntity::commandLine(cmdLine,cmdArgs,os);

}


