/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-Japan, 2008
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      SimulatedStackOfTasks.h
 * Project:   SOT
 * Author:    Olivier Stasse
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 * This file is largely inspired from the StackOfTasks in the 
 * plugin directory. It is intend to test the walking pattern generator
 * output without OpenHRP.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef __SIMULATED_STACK_OF_TASKS_HWPG_H__
#define __SIMULATED_STACK_OF_TASKS_HWPG_H__

#include <iostream>
#include <sstream>
#include <vector>

#include <sot/sotInterpretor.h>
#include <sot/sotPluginLoader.h>
#include <sot/sotEntity.h>
#include <sot/sotSignal.h>
#include <sot/sotSignalPtr.h>
#include <sot/sotVectorRollPitchYaw.h>

#include <sot/sotPeriodicCall.h>

#include <MatrixAbstractLayer/boost.h>
namespace ml = maal::boost;


using namespace std;

template <class T> 
class _sequence
{
 private:
  vector<T> data;
  T anilT;
    
 public:
  _sequence(){};
  ~_sequence(){};

  unsigned int length()
  {return data.size();}
  
  void length(unsigned int i)
  {data.resize(i);}

  inline T& operator[] (unsigned int i)
  { if (i<data.size()) return data[i];
    return anilT;
  }
};

typedef _sequence<double> dsequence;
typedef _sequence<dsequence> sdsequence;

class robot_state
{
 public:

  dsequence angle;
  dsequence velocity;
  dsequence torque;

  sdsequence force;
  sdsequence attitude;

  dsequence zmp;
  dsequence waistPos;
  dsequence waistRpy;

  robot_state();
  ~robot_state();
};

class motor_command
{
 public:
  dsequence angle;				
  dsequence waistPos;
  dsequence waistRpy;
  dsequence zmp;
  motor_command();
  ~motor_command();
};
class SimulatedStackOfTasks: public sotEntity
{
 public:

  SimulatedStackOfTasks(istringstream &strm);
  virtual ~SimulatedStackOfTasks(void);
  bool setup( robot_state* rs, motor_command* mc );  
  void control( robot_state* rs, motor_command* mc );
  bool cleanup( robot_state* rs, motor_command* mc );

  static const  unsigned int DOF=40;

 public:

  enum sotPluginState
    {
      sotNO_STATE    = 0x00
      ,sotJUST_BUILT = 0x01  //! Just build, waiting for init
      ,sotINIT       = 0x02  //! Init done, waiting for setup
      ,sotSETUP      = 0x04  //! Setup done, ready to control
      ,sotLOOP       = 0x08  //! Setup done, already controlled once
      ,sotHOLD       = 0x10  //! Hold requiered
      ,sotFINISHING  = 0x20  //! Hold considered, control to finish
      ,sotFINISHED   = 0x40  //! Finish done, waiting for cleanup
      ,sotCLEANED_UP = 0x80  //! Cleanup done, ready for destruction
    };

 protected:
  sotPluginState pluginState;

  sotPluginLoader pluginLoader;
  bool suspend;
  
  static const double TIMESTEP_DEFAULT;
  double timestep;

 public:

  virtual const std::string& getClassName( void ) const { return CLASS_NAME; }

  void m_script(istringstream &strm);
  void commandLine( const std::string& cmdLine,std::istringstream& cmdArgs,
		    std::ostream& os );

  void play( void );
  void pause( void );

  void sotInit( void );
  void sotControlLoop( robot_state* rs, motor_command* mc );
  void sotControlFinishing( robot_state* rs, motor_command* mc );

  //! Specific fields of the 
  unsigned int iter;
  maal::boost::Vector robotStatePrec; 
  bool robotStatePrecInit;
  bool reinitFromMc;

  enum ForceSignalSource
  {
    FORCE_SIGNAL_RLEG,
    FORCE_SIGNAL_LLEG,
    FORCE_SIGNAL_RARM,
    FORCE_SIGNAL_LARM
  };
  bool withForceSignals[4];

  sotSignalPtr<ml::Vector,int> controlSIN;
  sotSignalPtr<ml::Vector,int> attitudeSIN;
  sotSignalPtr<ml::Vector,int> zmpSIN;

  sotSignal<ml::Vector,int> stateSOUT;
  sotSignal<ml::Vector,int>* forcesSOUT[4];
  sotSignal<sotVectorRollPitchYaw,int> attitudeSOUT;

  sotSignal<ml::Vector,int> pseudoTorqueSOUT;
  bool activatePseudoTorqueSignal;
  //sotSignal<ml::Vector,int> velocitySOUT;
  //bool activateVelocitySignal;
  sotSignal<ml::Vector,int> previousControlSOUT;
  bool activatePreviousControlSignal;

  /*! Peridic call that is called /before/ the MC computation
   * (ie for precomputation needed by the main graph, like 
   * event FSM-based computations). */
  sotPeriodicCall periodicCallBefore;
  /*! Peridic call that is called /after/ the MC computation (eg
   * for tracing). */
  sotPeriodicCall periodicCallAfter;

};

#endif /*  __LOCAL_STACK_OF_TASKS_HWPG_H__ */
