/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL 2008
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_hwpg.cpp
 * Project:   sot
 * Author:    Francois Keith, Olivier Stasse
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <iostream>

#include <sstream>

#include <sot/sotDebug.h>
#include <sot/sotPatternGenerator.h>

#include <SimulatedStackOfTasks.h>

using namespace std;

class EndEffectors
{
private:
  DynamicBody * m_LeftWrist, * m_RightWrist;
  DynamicBody * m_LeftFoot, * m_RightFoot;

public:

  EndEffectors()
  { 
    m_LeftWrist  = 0;
    m_RightWrist = 0;
    m_LeftFoot   = 0;
    m_RightFoot  = 0;
  }

  ~EndEffectors() 
  {};

  void ExtractBodiesFromHumanoid(HumanoidDynamicMultiBody *aHDMB);

  const DynamicBody *  LeftWrist() const
  { return m_LeftWrist;}

  const DynamicBody *  RightWrist() const
  { return m_RightWrist;}

  const DynamicBody *  LeftFoot() const
  { return m_LeftFoot;}

  const DynamicBody *  RightFoot() const
  { return m_RightFoot;}

};

void EndEffectors::ExtractBodiesFromHumanoid(HumanoidDynamicMultiBody *aHDMB)
{
  CjrlJoint * JointLeftWrist = aHDMB->leftWrist();
  if (JointLeftWrist!=0)
    m_LeftWrist = (DynamicBody *)JointLeftWrist->linkedBody();

  CjrlJoint * JointRightWrist = aHDMB->rightWrist();
  if (JointRightWrist!=0)
    m_RightWrist = (DynamicBody *)JointRightWrist->linkedBody();

  CjrlJoint * JointLeftFoot = aHDMB->leftFoot();
  if (JointLeftFoot!=0)
    m_LeftFoot = (DynamicBody *)JointLeftFoot->linkedBody();
    
  CjrlJoint * JointRightFoot = aHDMB->rightFoot();
  if (JointRightFoot!=0)
    m_RightFoot = (DynamicBody *)JointRightFoot->linkedBody();
 
}

void UpdateHumanoidStructure(HumanoidDynamicMultiBody *aHDMB,
			     MAL_VECTOR(,double) &PreviousConfiguration,
			     robot_state &aRobotState,
			     motor_command &aMotorCommand,
			     double lDeltaT,
			     EndEffectors &lEndEffectors)
{
  MAL_VECTOR_DIM(Configuration,double,SimulatedStackOfTasks::DOF+6) ;
  MAL_VECTOR_DIM(Velocity,double,SimulatedStackOfTasks::DOF+6);

  for(unsigned int i=0;i<3;i++)
    {
      Configuration[i] = 
	aRobotState.waistPos[i] = 
	aMotorCommand.waistPos[i];
      Configuration[i+3] = 
	aRobotState.waistRpy[i] = 
	aMotorCommand.waistRpy[i];
    }

  for(unsigned int i=0;i<SimulatedStackOfTasks::DOF;i++)
    {
      Configuration[i+6] = aMotorCommand.angle[i];
      aRobotState.angle[i] = aMotorCommand.angle[i];
    }
  
  for(unsigned int i=0;i<SimulatedStackOfTasks::DOF+6;i++)
    {
      Velocity[i] = (Configuration[i] - PreviousConfiguration[i])/lDeltaT;
    }
  
  aHDMB->currentConfiguration(Configuration);
  aHDMB->currentVelocity(Velocity);
  aHDMB->computeForwardKinematics();
  

  for(unsigned int i=0;i<3;i++)
    {
      aRobotState.force[0][i] = lEndEffectors.LeftFoot()->m_Force[i];
      aRobotState.force[1][i] = lEndEffectors.RightFoot()->m_Force[i];
      aRobotState.force[2][i] = lEndEffectors.LeftWrist()->m_Force[i];
      aRobotState.force[3][i] = lEndEffectors.RightWrist()->m_Force[i];

      aRobotState.force[0][i+3] = lEndEffectors.LeftFoot()->m_Torque[i];
      aRobotState.force[1][i+3] = lEndEffectors.RightFoot()->m_Torque[i];
      aRobotState.force[2][i+3] = lEndEffectors.LeftWrist()->m_Torque[i];
      aRobotState.force[3][i+3] = lEndEffectors.RightWrist()->m_Torque[i];

    }    
  
  MAL_MATRIX(,double) Torques;
  Torques = aHDMB->currentTorques();

  for(unsigned int i=0;i<SimulatedStackOfTasks::DOF;i++)
    {
      for(unsigned int k=0;k<3;k++)
	{
	  if (Torques(i+6,k)!=0.0)
	    aRobotState.torque[i] = 
	      Torques(i+6,k);
	}
    }
}

int main (void)
{

  string aPath("/home/stasse/src/OpenHRP/Controller/IOserver/robot/HRP2JRL/model/");
  string aName("HRP2JRLmain.wrl");
  string aMapFromJointToRank("/home/stasse/src/OpenHRP/Controller/IOserver/robot/HRP2JRL/etc/HRP2LinkJointRank.xml");  string SpecificitiesFile("/home/stasse/src/OpenHRP/Controller/IOserver/robot/HRP2JRL/etc/HRP2Specificities.xml");
  HumanoidDynamicMultiBody *aHDMB;
  robot_state aRobotState;
  motor_command aMotorCommand;
  EndEffectors lEndEffectors;
  
  double TimeOfSimulation=150.0; // in seconds.
  double CurrentTime=0.0;
  double DeltaT = 0.005;

  istringstream strm("");
  SimulatedStackOfTasks aSST(strm);

  aHDMB = new HumanoidDynamicMultiBody();

  aHDMB->parserVRML(aPath,aName,(char *)aMapFromJointToRank.c_str());
  aHDMB->SetHumanoidSpecificitiesFile(SpecificitiesFile);
  string inProperty="ComputeZMP"; string aValue="true";
  aHDMB->setProperty(inProperty,aValue);
  inProperty="ComputeBackwardDynamics"; aValue="true";
  aHDMB->setProperty(inProperty,aValue);
  aHDMB->computeForwardKinematics();

  lEndEffectors.ExtractBodiesFromHumanoid(aHDMB);

  double dInitPos[40] = { 
    0.0, 0.0, -26.0, 50.0, -24.0, 0.0, 
    0.0, 0.0, -26.0, 50.0, -24.0, 0.0,  // legs

    0.0, 0.0, 0.0, 0.0, // chest and head

    15.0, -10.0, 0.0, -30.0, 0.0, 0.0, 10.0, // right arm
    15.0,  10.0, 0.0, -30.0, 0.0, 0.0, 10.0, // left arm 

    -10.0, 10.0, -10.0, 10.0, -10.0  // right hand
    -10.0, 10.0, -10.0, 10.0, -10.0  // left hand
  };
  MAL_VECTOR_DIM(InitialPosition,double,40);
  for(unsigned int i=0;i<MAL_VECTOR_SIZE(InitialPosition);i++)
    InitialPosition(i) = dInitPos[i]*M_PI/180.0;
  
  MAL_VECTOR_DIM(PreviousConfiguration,double,46);
  for(int i=6;i<46;i++)
    {
      PreviousConfiguration[i] = InitialPosition[i-6];
    }


  aSST.sotInit();
  string lInitialization[12] = {
    "run /home/stasse/devel/openrobots/script/dyn",
    "run /home/stasse/devel/openrobots/script/coshell",
    "run /home/stasse/devel/openrobots/script/traces",
    "plug OpenHRP.state dyn.position",
    "plug OpenHRP.state dyn2.position",
    "plug OpenHRP.attitude posKF.attitudeIN",
    "plug OpenHRP.attitude attitudeSensor.in",
    "plug sot.control OpenHRP.control",
    "run /home/stasse/devel/openrobots/script/jointlimit",
    "run /home/stasse/devel/openrobots/script/hwpg",
    "OpenHRP.pause",
    "OpenHRP.play"
  };

   
  for(unsigned int i=0;i<12;i++)
    {
      string aS("script");
      istringstream aISS(lInitialization[i]);
      cout << "Command line "<< i << " " << lInitialization[i] << endl;
      aSST.m_script(aISS);
    }

  
  aSST.setup(&aRobotState,&aMotorCommand);
  for(CurrentTime=0.0;CurrentTime<TimeOfSimulation;CurrentTime+=DeltaT)
    {
      // Call the control of the robot.
      aSST.control(&aRobotState,&aMotorCommand);

      // Make a simple simulation of the robot using the dynamics library.
      UpdateHumanoidStructure(aHDMB,
			      PreviousConfiguration,
			      aRobotState,
			      aMotorCommand,
			      DeltaT,
			      lEndEffectors);
     
      if (((int)CurrentTime)%2==0)
	cout << "Time:" << CurrentTime << endl;
      
    }
  aSST.cleanup(&aRobotState,&aMotorCommand);
  
  delete aHDMB;
  return 0;
}

