/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_shell.cc
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <sot/sotContiifstream.h>
#include <sot/sotInterpretor.h>
#include <sot/sotPluginLoader.h>

#include <iostream>
using namespace std;

#include <sot/sotDebug.h>
#include <sot/sotDistantShell.h>

class OptArgs 
{
public:
  std::string filename;
  bool flagin, flagout;
  OptArgs( void ) : filename( "/tmp/nmansard/script.buffer" ),flagin(false), flagout(false) {}

  static const char optlist [];

public:
  void parseOptions(  int argc,char** argv )
  {
    int ch;
    while( (ch=getopt( argc,argv,optlist ))!=-1 ) 
    {
      switch (ch) 
	{
	case 'i':
	  flagin=true;
	  break;
	case 'o':
	  flagout=true;
	  break;
	case 'f':
	  filename = optarg;
	  break;
	case '?':
	default:
	  usage();
	}
    }
  }
  void usage( void ) { cout << "dshell [-o] [-i] [-f filename]"<<endl; }
};

const char OptArgs::optlist [] = "f:io";

int main( int argc,char** argv )
{
  OptArgs args; args.parseOptions(argc,argv);
  ofstream fileout( args.filename.c_str(),fstream::out|fstream::trunc );

  sotPluginLoader pl;
  sotShell.referencePluginLoader( &pl );
  
  if( args.flagout )
    {
      sotDistantShell dshell( args.filename,sotShell,cout );
      if( args.flagin ) dshell.loopInThread(); else dshell.loop(); 
    }

  if( args.flagin )
    {
      char buffer[1024];
      while(1)
	{
	  cout << "> ";
	  cin.getline( buffer,1024 ); 
	  sotDEBUG(5) <<buffer <<endl;
	  if( 1<cin.gcount() ) fileout << buffer <<endl;
	}
    }

  return 0;
}




