/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_factory.cc
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


#include <string>
#include <iostream>

#include <sot/sotFactory.h>
#include <sot/sotEntity.h>
#include <sot/sotFeatureVisualPoint.h>
#include <sot/sotExceptionFeature.h>
using namespace std;

#ifdef WIN32
#include <Windows.h>
#else 
#include <dlfcn.h> 
#endif

#ifdef WIN32
	typedef HMODULE sotPluginKey;
#else
	typedef void* sotPluginKey;
#endif

class TestFeature
  :public sotFeatureAbstract
{
public:
  TestFeature( void ) : sotFeatureAbstract("") {}
  virtual ~TestFeature( void ) {}
  virtual unsigned int& getDimension( unsigned int& res,int time ) {return res;}
  
  virtual ml::Vector& computeError( ml::Vector& res,int time ) {return res;} 
  virtual ml::Matrix& computeJacobian( ml::Matrix& res,int time ) {return res;}
  virtual ml::Vector& computeActivation( ml::Vector& res,int time ) {return res;}
};


int main()
{
  
  sotDEBUG(0) << "# In {"<<endl;
//   sotEntity test("");
//   sotExceptionFeature t2(sotExceptionFeature::BAD_INIT);
//   sotExceptionSignal t4(sotExceptionSignal::COPY_NOT_INITIALIZED);
//   sotFlags t3;
//   TestFeature t5;

#ifndef WIN32
  sotPluginKey dlib = dlopen("lib/plugin/sotFeatureVisualPoint.so", RTLD_NOW);
#else
  sotPluginKey dlib = LoadLibrary ( "lib/plugin/sotFeatureVisualPoint.lib"); 
#endif
if( NULL==dlib ) 
    {
      cerr << " Error dl"<<endl;
#ifndef WIN32
      cerr << dlerror() <<endl;
#else
    // Retrieve the system error message for the last-error code
    LPTSTR pszMessage;
    DWORD dwLastError = GetLastError(); 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwLastError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&pszMessage,
        0, NULL );


	  cerr << pszMessage <<endl;
	  LocalFree(pszMessage);
#endif

      exit(0);
    }


#ifndef WIN32
	dlib = dlopen("lib/plugin/sotGainAdaptative.so", RTLD_NOW);
#else
	dlib = LoadLibrary ("lib/plugin/sotGainAdaptative.lib"); 
#endif
  if( NULL==dlib ) 
    {
      cerr << " Error dl"<<endl;
#ifndef WIN32
      cerr << dlerror() <<endl;
#else
    // Retrieve the system error message for the last-error code
    LPTSTR pszMessage;
    DWORD dwLastError = GetLastError(); 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwLastError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&pszMessage,
        0, NULL );


	  cerr << pszMessage <<endl;
	  LocalFree(pszMessage);
#endif
      exit(0);
    }

  sotEntity* gain = sotFactory.newEntity("GainAdaptative","Gain");
  sotFeatureAbstract* point = sotFactory.newFeature("FeatureVisualPoint","DynamicTest.");
  
  gain->display(cout); cout << endl;
  cout <<gain->getClassName(); cout << endl;

  point->display(cout); cout << endl;
  cout <<point->getClassName(); cout << endl;


  sotDEBUG(0) << "# Out }"<<endl;
}

