/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet VISTA / IRISA, 2003
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_categorie.cc
 * Project:   Traces
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id: test_boost.cpp,v 1.1.1.1 2006-07-03 05:17:37 nmansard Exp $
 *
 * Description
 * ============
 *
 * Test la classe CategorieTrace.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include "vsplugin/vsDebug.h"      /* Macros de trace et debug.               */
INIT_TRACE_DEFAULT;
/* --- ViSP --- */

/* --- BOOST ---- */

#include <boost/bind.hpp>
#include <boost/function/function0.hpp>
#include <boost/function/function1.hpp>
using namespace boost;
using namespace std;
/* -------------------------------------------------------------------------- */
/* --- CONSTANTES ----------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */
/* --- CLASSE --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */



template <class ClasseGenerique>
class PointeurFonction
{
private:
  
  enum TypeFonctionUtilisee
    {
      FONCTION_STATIQUE,
      METHODE_CONSTANTE,
      METHODE_NON_CONSTANTE
    };


private:

  TypeFonctionUtilisee typeFonctionUtilisee;

  ClasseGenerique * refClasse;

  double (ClasseGenerique::* methode) (void);

  const ClasseGenerique * refClasseConst;

  double (ClasseGenerique::* methodeConst) (void) const;

  double (* fonction) (void);

  // Constructeur vide interdit.
  PointeurFonction (void);

public:

  PointeurFonction (double (* f) (void))
  {
    this ->fonction = f;
    this -> typeFonctionUtilisee = FONCTION_STATIQUE;
    return;
  }

  PointeurFonction (ClasseGenerique * ref, 
		    double (ClasseGenerique::* m) (void))
  {
    this ->refClasse = ref;
    this ->methode = m;
    this -> typeFonctionUtilisee = METHODE_NON_CONSTANTE;
    return ;
  }

  PointeurFonction (const ClasseGenerique * ref, 
		    double (ClasseGenerique::* m) (void) const)
  {
    this ->refClasseConst = ref;
    this ->methodeConst = m;
    this -> typeFonctionUtilisee = METHODE_CONSTANTE;
    return ;
  }


  double operator() (void)
  {
    double res = 0;

    switch (this ->typeFonctionUtilisee)
      {
      case FONCTION_STATIQUE:
	{
	  res = (* (this ->fonction)) ();
	  break;
	}
      case METHODE_CONSTANTE:
	{
	  const ClasseGenerique & refClasse = *this ->refClasseConst;
	  res = (refClasse .*this ->methodeConst) ();
	  break;
	}
      case METHODE_NON_CONSTANTE:
	{
	  ClasseGenerique & refClasse = *this ->refClasse;
	  res = (refClasse .*this ->methode) ();
	  break;
	}
      }
    
    return res;
  }
 

};






class Classe
{
public:
 
  Classe (void) { DEBUG_TRACE (1, "Constructeur vide."); }
  Classe (const Classe & c) { DEBUG_TRACE (1, "Constructeur par copie."); }
  ~Classe (void) { DEBUG_TRACE (1, "Destructeur."); }
  
  void voidFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return ;
  }
  
  int intFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return 46 ;
  }
  
  int intFooInt (int i)
  {
    TRACE ("Entree - Sortie.");
    return 46 + i ;
  }
  double doubleFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return 46 ;
  }
  
   double doubleFooDouble (double i)
  {
    TRACE ("Entree - Sortie.");
    return 46.0 + i ;
  }

 
};

ostream& operator<< (ostream &os, const Classe &c)
{
  os << "Flux sortie classe Classe." << endl;
  return os;
}

/* -------------------------------------------------------------------------- */
/* --- STATIC --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


// void lancementFunction0 (const function0<int> &f)
// {
//   cout << f() << endl;
// }

// void lancementFunction1 (const function1<int, int> &f)
// {
//   cout << f(18) << endl;
// }

/* --- FOO --- */

void voidFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return ;
}

int intFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return 46 ;
}

int intFooInt (int i)
{
  TRACE ("Entree - Sortie.");
  return 46 + i ;
}

int intFooIntInt (int i, int j)
{
  TRACE ("Entree - Sortie.");
  return 46 + i - j ;
}

double doubleFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return 46 ;
}


int foo (int i) 
{ 
  cout << "fooInt" << endl;
  return i;
}
int foo (int i, int j) 
{
  cout << "fooIntInt" << endl;
  return i+j; 
}


/* -------------------------------------------------------------------------- */
/* --- SANS BOOST ----------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


typedef int (* fonction1) (void);

//template<class C>
typedef int (Classe::* methode1) (void);

typedef Classe* pClasse;

//template<typename C>typedef C* pC;
// template<typename A, typename B> class X { /* ... */ };
// template<typename T> typedef X<T,int> Xi;

void lancementFunction0 (const fonction1 f)
{
  cout << (*f) () << endl;
}

template<class C>
void lancementFunction0 (Classe & refClasse, 
			 int (C::* f) (void))
{
  cout << (refClasse .*f) () << endl;
}



/* -------------------------------------------------------------------------- */
/* --- MAIN ----------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */



int
main(int argc, char ** argv)
{
  DEBUG_TRACE (9, "# Entree");

//   lancementFunction0 (intFooVoid);
//   lancementFunction1 (intFooInt);

  Classe c;
//   thread (c .voidFooVoid);

  //lancementFunction0 (c .intFooVoid);
  //lancementFunction1 (c .intFooInt);

//   lancementFunction0 (intFooVoid);
//   lancementFunction0<Classe> (c, &Classe::intFooVoid);

//   PointeurFonction<Classe> pF (&doubleFooVoid);
//   cout << pF () << endl;
//    PointeurFonction<Classe> pF2(&c, &Classe::doubleFooVoid);
//    cout << pF2 () << endl;


   ostream & (* op) (std::ostream &, const Classe &);
   op =  operator<<;
   function1<ostream &, ostream &> f = boost::bind (op , _1, c);
   f (cout);

//    int (* op) (int, int);
//    op =  foo;
//    cout <<  (*op)(1,5) << endl;


  DEBUG_TRACE (9, "# Sortie");
  return 0;
}
