/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet VISTA / IRISA, 2003
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_categorie.cc
 * Project:   Traces
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id: test_functions.cpp,v 1.2 2006-07-03 07:15:23 nmansard Exp $
 *
 * Description
 * ============
 *
 * Test la classe CategorieTrace.
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include "vsplugin/vsDebug.h"      /* Macros de trace et debug.               */
INIT_TRACE_DEFAULT;

#include "vsplugin/vsFunctions.h"  /* Classe PointeurFonction.                */
/* --- ViSP --- */
#include <visp/vpColVector.h>      /* Classe vpColVector.                     */


/* -------------------------------------------------------------------------- */
/* --- CONSTANTES ----------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */
/* --- CLASSE --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */


class Classe
{
public:
 
  Classe (void) { DEBUG_TRACE (1, "Constructeur vide."); }
  Classe (const Classe & c) { DEBUG_TRACE (1, "Constructeur par copie."); }
  ~Classe (void) { DEBUG_TRACE (1, "Destructeur."); }
  
  void voidFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return ;
  }
  
  int intFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return 46 ;
  }
  
  int intFooInt (int i)
  {
    TRACE ("Entree - Sortie.");
    return 46 + i ;
  }
  double doubleFooVoid (void)
  {
    TRACE ("Entree - Sortie.");
    return 46 ;
  }
  
  double doubleFooVoidConst (void) const
  {
    TRACE ("Entree - Sortie.");
    return 46 ;
  }
  

   double doubleFooDouble (double i)
  {
    TRACE ("Entree - Sortie.");
    return 46.0 + i ;
  }
 
  double operator() (void) { return 22; }
  double operator() (double arg) { return 22 + arg; }

};

/* -------------------------------------------------------------------------- */
/* --- STATIC --------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */



/* --- FOO --- */

void voidFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return ;
}

int intFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return 46 ;
}

int intFooInt (int i)
{
  TRACE ("Entree - Sortie.");
  return 46 + i ;
}

double doubleFooVoid (void)
{
  TRACE ("Entree - Sortie.");
  return 46 ;
}



/* -------------------------------------------------------------------------- */
/* --- MAIN ----------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */



int
main(int argc, char ** argv)
{
  DEBUG_TRACE (9, "# Entree");

  Classe c;
  c (); c (12);

  {
    FoncteurGenerique0<> pF (&doubleFooVoid);
    CTRACE << pF () << endl;
  }

  {
    FoncteurGenerique0<Classe> pF (&c, &Classe::doubleFooVoid);
    CTRACE << pF () << endl;
  }

  {
    FoncteurGenerique0<Classe> pF (&c, &Classe::doubleFooVoidConst);
    CTRACE << pF () << endl;
  }

  CTRACE << "--- 1 ARG -------------" << endl;
  
  {
    FoncteurGenerique1<ParDefault, int, int> pF (& intFooInt);
    CTRACE << pF (12) << endl;
  }

  {
    FoncteurGenerique1<Classe, int, int> pF (&c, & Classe::intFooInt);
    CTRACE << pF (12) << endl;
  }


  CTRACE << "--- OP () -------------" << endl;
  // Marche pas avec les op().
//   {
//     FoncteurGenerique0<Classe, double> pF (&c, & Classe::operator() (void));
//     ctrace << pF () << endl;
//   }




  DEBUG_TRACE (9, "# Sortie");
  return 0;
}
