/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL-JAPAN, Tsukuba, 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_chrono.cc
 * Project:   SOT
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* -------------------------------------------------------------------------- */
/* --- INCLUDES ------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

#include <sot/sotFactory.h>
#include <sot/sotEntity.h>
#include <sot/sotFeatureAbstract.h>

#include <sot/sotPluginLoader.h>
#include <sot/sotDebug.h>
#include <sot/sotDynamic.h>

#include <sot/sotInterpretor.h>
#include <sot/sotFeaturePoint6d.h>

#include <iostream>
using namespace std;

#include <sot/sotDebug.h>
#include <sot/sotInterpretor.h>
#include <sot/sotSOT.h>

#ifdef WIN32
#include <Winsock2.h>
#else /*WIN32*/
#include <sys/time.h>
#endif /*WIN32*/

#define sotCHRONO1 \
      gettimeofday(&t1,NULL);\
      dt = ( (t1.tv_sec-t0.tv_sec) * 1000.\
	     + (t1.tv_usec-t0.tv_usec+0.) / 1000. );\
      os << "dt: "<< dt 


#define sotCOUNTER(nbc1,nbc2) \
	  gettimeofday(&tpart##nbc2,NULL); \
	  dt##nbc2 += ( (tpart##nbc2.tv_sec-tpart##nbc1.tv_sec) * 1000. \
		   + (tpart##nbc2.tv_usec-tpart##nbc1.tv_usec+0.) / 1000. );
//#define sotCOUNTER(nbc1,nbc2)
#define sotINITCOUNTER(nbc1) \
   struct timeval tpart##nbc1; double dt##nbc1=0; 

void 
cmdTestChrono( const std::string& cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - tchrono"
	 << "\t\t\t\tRun the chrono." <<endl;
      return;
    }

  sotSOT &sot = dynamic_cast< sotSOT& > ( sotPool.getEntity( "sot" ) );
  sotDynamic &dyn = dynamic_cast< sotDynamic& > ( sotPool.getEntity( "dyn" ) );
  //sotFeaturePoint6d &p6 = dynamic_cast< sotFeaturePoint6d& > ( sotPool.getEntity( "p6" ) );
  
  istringstream iss( "OpenHRP.state");
  sotSignal< ml::Vector,int >& stateSIN 
    = dynamic_cast< sotSignal< ml::Vector,int >& >
    ( sotPool.getSignal( iss ) );
  ml::Vector state( stateSIN.accessCopy().size() );  state.fill(1);

  int nbIter=100; if( cmdArg.good() ) cmdArg >> nbIter;
  for( int iter=0;iter<20;++iter ) // Init processor
    {
      stateSIN = state;
      sot.controlSOUT( sot.controlSOUT.getTime()+1 );
    }

  const char * TASKS_NAMES[] = { "task" , "taskCom" , "taskTwofeet" };
  const int NB_TASKS=3;
  std::vector< sotTaskAbstract* > tasksList;
  for( int i=0;i<NB_TASKS;++i )
    {
       sotTaskAbstract &task 
	 = dynamic_cast< sotTaskAbstract& > ( sotPool.getEntity( TASKS_NAMES[i] ) );
       tasksList.push_back( &task );
    }
  
  os <<" ------------------------------------------------------- "<<endl
     <<" --- SOT.CONTROL : x" << nbIter << endl 
     <<" ------------------------------------------------------- "<<endl;
  struct timeval t0,t1,tpart0; double dt;
  sotINITCOUNTER(1); sotINITCOUNTER(2);  sotINITCOUNTER(3);
  sotINITCOUNTER(4); sotINITCOUNTER(5);  sotINITCOUNTER(task0);
  sotINITCOUNTER(task1); sotINITCOUNTER(task2);

  dt1=0;dt2=0;dt3=0;
  gettimeofday(&t0,NULL);
  for( int iter=0;iter<nbIter;++iter )
    {
      const int iterTime = sot.controlSOUT.getTime()+1;

      gettimeofday(&tpart0,NULL);
      stateSIN = state;
      dyn.comSOUT( iterTime );
      sotCOUNTER(0,1);
      dyn.JcomSOUT( iterTime );
      sotCOUNTER(1,2);
      sot.constraintSOUT( iterTime );
      sotCOUNTER(2,3);

      tasksList[0]->jacobianSOUT( iterTime );
      tasksList[0]->taskSOUT( iterTime );
      sotCOUNTER(3,task0);
      tasksList[1]->jacobianSOUT( iterTime );
      tasksList[1]->taskSOUT( iterTime );
      sotCOUNTER(task0,task1);
      tasksList[2]->jacobianSOUT( iterTime );
      tasksList[2]->taskSOUT( iterTime );
      sotCOUNTER(task1,task2);
      
      for( vector<sotTaskAbstract*>::iterator task=tasksList.begin();
	   task!=tasksList.end();++task )
	{ 
	  (*task)->jacobianSOUT( iterTime );
	  (*task)->taskSOUT( iterTime );
	}
      sotCOUNTER(task2,4);
	  
      sot.controlSOUT( iterTime );
      sotCOUNTER(4,5);
   }
  sotCHRONO1 <<endl;
  os <<" dynNE: " << dt1 << endl
     <<" Jcom: " << dt2 << endl
     <<" tasks: " << dttask0<<"+"<<dttask1<<"+"<<dttask2
     <<"="<< dttask0+dttask1+dttask2 << endl
     <<" constraint: " << dt3 << endl
     <<" dynamics: " << dt4 << endl 
     << " p+: "<<dt5 <<endl;



  os <<" ------------------------------------------------------- "<<endl
     <<" --- TASK PSEUDO INV : x" << nbIter << endl 
     <<" ------------------------------------------------------- "<<endl;
  sotINITCOUNTER(6);
  sotINITCOUNTER(7); sotINITCOUNTER(8);  sotINITCOUNTER(pinv1);
  sotINITCOUNTER(pinv2);
  dt1=0; dt2=0; dt3=0;dt4=0; dt5=0; dt6=0; dt7=0;  dt8=0;

  gettimeofday(&t0,NULL); 
  for( int iter=0;iter<nbIter;++iter )
    {
      const int iterTime = sot.controlSOUT.getTime()+1;
      sot.controlSOUT.setTime(iterTime);

      stateSIN = state;
      
      ml::Matrix P,Jp,JpJt;
      const double &th = sot.inversionThresholdSIN(iterTime);
      const ml::Matrix &K = sot.constraintSOUT(iterTime);
  
      for( vector<sotTaskAbstract*>::iterator taskIter=tasksList.begin();
	   taskIter!=tasksList.end();++taskIter )
	{ 
	  gettimeofday(&tpart0,NULL);
	
	  sotTaskAbstract & task = **taskIter;
	  
	  const ml::Matrix &JacRO = task.jacobianSOUT(iterTime);
	  const ml::Vector &err = task.taskSOUT(iterTime);
	  ml::Vector control;

	  sotCOUNTER(0,1); // Direct Dynamic

	  {
	    /* --- FIRST ALLOCS --- */
	    const unsigned int nJ = JacRO.nbRows(),mJ = K.nbCols();
	    const unsigned int FF_SIZE=JacRO.nbCols()-mJ;
	    ml::Matrix Jac(nJ,mJ); // = JacRO;
	    ml::Matrix Jff( nJ,FF_SIZE );
	    ml::Matrix Jact( nJ,mJ );
	    Jp.resize( mJ,nJ );
	    ml::Matrix Jt( nJ,mJ );
	    /***/sotCOUNTER(1,2); // first allocs 

	    /* --- COMPUTE JK --- */
	    for( unsigned int i=0;i<nJ;++i )
	      {
		for( unsigned int j=0;j<FF_SIZE;++j ) Jff(i,j)=JacRO(i,j);
		for( unsigned int j=FF_SIZE;j<JacRO.nbCols();++j ) 
		  Jact(i,j-FF_SIZE)=JacRO(i,j);
	      }
	    Jff.multiply(K,Jac);
	    Jac+=Jact;
   	    /***/sotCOUNTER(2,3); // compute JK

	    /* --- COMPUTE Jt --- */
	    if( mJ==P.nbCols() ) Jac.multiply(P,Jt);
	    else { Jt = Jac; P.resize( mJ,mJ ); JpJt.resize( mJ,mJ ); P.setIdentity(); } 
	    /***/sotCOUNTER(3,4); // compute Jt

	    /* --- PINV --- */
	    ml::Matrix U,V; ml::Vector S;
	    Jt.pseudoInverse( Jp,th,&U,&S,&V );
	    /***/sotCOUNTER(4,5); // PINV

	    /* --- COMPUTE QDOT AND P --- */
	    if( mJ!=control.size() ) { control.resize( mJ ); control.fill(.0); }
    	    control += Jp*(err - Jac*control);
 	    /***/sotCOUNTER(5,6); // QDOT 

	    unsigned int Jrank = 0; 
	    {
	      const unsigned int Jmax = S.size();
	      for( unsigned i=0;i<Jmax;++i ) { if( S(i)>th ) Jrank++; }
	    } 

// 	    ublas::matrix<double> &Pboost = P.matrix;
// 	    ublas::matrix<double> &Vboost = V.matrix;
// 	    for( unsigned int i=0;i<mJ;++i )
//  	      for( unsigned int j=0;j<mJ;++j )
//  		for( unsigned int k=0;k<Jrank;++k )
//  		{ Pboost(i,j) -= Vboost(i,k)*Vboost(j,k); } 

	    /***/sotCOUNTER(6,7); // Ppre

	    double *p,*v1,*v2,*vtmp1,*vtmp2;
	    p = traits::matrix_storage(P.matrix);
	    v1 = traits::matrix_storage(V.matrix);
	    v2 = traits::matrix_storage(V.matrix);
	    vtmp1 = traits::matrix_storage(V.matrix);

	    for( unsigned int i=0;i<mJ;++i )
	      {
		 vtmp2 = traits::matrix_storage(V.matrix);

		 for( unsigned int j=0;j<mJ;++j )
		   {
		     v1 = vtmp1;
		     v2 = vtmp2;
		     for( unsigned int k=0;k<Jrank;++k )
		       { 
			 (*p) -=( *v1) * (*v2); 
			 v2++;v1++;
		       } 
		     p++;
		     vtmp2 += mJ;
		   }

		 vtmp1 += mJ;

	       }
	     //os<<P<<endl;


	    /***/sotCOUNTER(7,8); // P

	    if(0)
	      {
	    ublas::matrix<double> Uboost = U.matrix;
	    ublas::matrix<double> Vboost = V.matrix;
	    ublas::vector<double> Sboost = S.accessToMotherLib();

	    ml::Matrix &invMatrix = Jp;
	    invMatrix.matrix.clear();
	    for( unsigned int i=0;i<Vboost.size2();++i )
	      for( unsigned int j=0;j<Uboost.size1();++j )
		for( unsigned int k=0;k<Jrank;++k )
		  invMatrix.matrix(i,j)+=Vboost(i,k)*Sboost(k)*Uboost(j,k);
	    /***/sotCOUNTER(8,pinv1); // PINV

	    invMatrix.matrix.clear();
	    double * pinv = traits::matrix_storage(invMatrix.matrix);
	    double * uptr;
	    double * vptr;
	    double * vptrRow = traits::matrix_storage(Vboost);

	    ublas::vector<double> sp( Jrank ); 
	    for( unsigned int i=0;i<Jrank;++i )		
	      if( fabs(S(i))>th ) sp(i)=1/S(i); else sp(i)=0.;		

	    double * spptr;

	    for( unsigned int i=0;i<Vboost.size2();++i )
	      {
		uptr = traits::matrix_storage(Uboost);
		for( unsigned int j=0;j<Uboost.size1();++j )
		  {
		    vptr = vptrRow; spptr = traits::vector_storage( sp );
		    for( unsigned int k=0;k<Jrank;++k )
		      {
			(*pinv) += (*vptr)* (*spptr) * (*uptr);
			uptr++; vptr++; spptr++;
		      }
		    pinv++; 
		    vptrRow += Vboost.size2();
		  }
	      }
	    /***/sotCOUNTER(pinv1,pinv2); // PINV
	      }

	  }

	}
    }
  sotCHRONO1 <<endl; 
  os<<" dynamics: " << dt1 << endl << " p+: "<<dt2+dt3+dt4+dt5+dt6+dt7+dt8 <<endl
    <<"   -> first allocs :" << dt2 << endl
    <<"   -> compute JK:" << dt3 << endl
    <<"   -> compute Jt:" << dt4 << endl
    <<"   -> PINV:" << dt5 << endl
    <<"   -> QDOT:" << dt6 << endl
    <<"   -> Ppre:" << dt7 << endl
    <<"   -> P:" << dt8 << endl;

  os<<endl   <<"   ->  Mult:" << dtpinv1 << endl;
  os<<endl   <<"   ->  Alamain:" << dtpinv2 << endl;

  os <<" ------------------------------------------------------- "<<endl
     <<" --- MULT : x" << nbIter << endl 
     <<" ------------------------------------------------------- "<<endl;

  ublas::matrix<double> P(40,40); 
  ublas::matrix<double> J(6,40);
  ublas::matrix<double> JK(6,40);
  for( unsigned int i=0;i<40;++i )
    for( unsigned int j=0;j<40;++j ) P(i,j) = (rand()+1.) / RAND_MAX;
  for( unsigned int i=0;i<J.size1();++i )
    for( unsigned int j=0;j<J.size2();++j ) J(i,j) = (rand()+1.) / RAND_MAX;

  gettimeofday(&t0,NULL); 
  for( int iter=0;iter<nbIter;++iter )
    {
      //J.multiply(P,JK);
      //prod(J.matrix,P.matrix,JK.matrix);
      prod(J,P,JK);
    }
  sotCHRONO1 <<endl; 


}

void 
cmdTestChronoDyn( const std::string& cmdLine, istringstream& cmdArg, std::ostream& os )
{
  if( cmdLine == "help" ) 
    {
      os << "  - tchronodyn"
	 << "\t\t\t\tRun the chrono of the dynamic." <<endl;
      return;
    }

  sotDynamic &dyn2 = dynamic_cast< sotDynamic& > ( sotPool.getEntity( "dyn2" ) );
  sotSignalTimeDependant< sotMatrixHomogeneous,int> & sigLleg 
    = dynamic_cast< sotSignalTimeDependant< sotMatrixHomogeneous,int>& >
    ( dyn2.getSignal( "lleg" ) );

  int nbIter=100; if( cmdArg.good() ) cmdArg >> nbIter;

  dyn2.comSOUT.setDependancyType( sotTimeDependancy<int>::ALWAYS_READY );
  dyn2.JcomSOUT.setDependancyType( sotTimeDependancy<int>::ALWAYS_READY );
  sigLleg.setDependancyType( sotTimeDependancy<int>::ALWAYS_READY );

  for( int iter=0;iter<20;++iter ) // Init processor
    {
      dyn2.newtonEulerSINTERN( 0 );
    }
   

  os <<" ------------------------------------------------------- "<<endl
     <<" --- DYNAMIC : x" << nbIter << endl 
     <<" ------------------------------------------------------- "<<endl;
  for( int iB=0;iB<10;++iB )
    {
      struct timeval t0,t1,tpart0; double dt;
      sotINITCOUNTER(1);      sotINITCOUNTER(2);      sotINITCOUNTER(3);
      sotINITCOUNTER(4);
      dt1=0;
      gettimeofday(&t0,NULL);
      for( int iter=0;iter<nbIter;++iter )
	{
	  //const int iterTime = sot.controlSOUT.getTime()+1;
	  
	  gettimeofday(&tpart0,NULL);
	  dyn2.newtonEulerSINTERN.setDependancyType
	    ( sotTimeDependancy<int>::ALWAYS_READY );
	  dyn2.newtonEulerSINTERN( 0 );
	  sotCOUNTER(0,1);

	  dyn2.newtonEulerSINTERN.setDependancyType
	    ( sotTimeDependancy<int>::BOOL_DEPENDANT );
	    dyn2.comSOUT( 0 );
	  sotCOUNTER(1,2);

	  dyn2.JcomSOUT( 0 );
	  sotCOUNTER(2,3);

	  sigLleg( 0 );
	  sotCOUNTER(3,4);
	}
      sotCHRONO1<<endl;
      os << "   NE: " << dt1 <<endl
	 << "   Com: " << dt2 <<endl
	 << "   JCom: " << dt3 <<endl
	 << "   lleg: " << dt4 <<endl<<endl;
	
    }
}
  



int main( int argc,char** argv )
{
  sotDEBUGIN(15);

  try
    {
      
      sotDEBUG(5) << " Loading..." << endl;
      sotPluginLoader pl;
      //pl.setDirectory( "lib/plugin" );
      //pl.loadPluginList( "plugin.txt" );
      //pl.loadPlugins();
      sotDEBUG(5) << " Loading done." << endl;
      

      sotShell.referencePluginLoader( &pl );
      sotShellFunctionRegisterer regFunTChrono
	( "tchrono",boost::bind(cmdTestChrono,_1,_2,_3) );
      sotShellFunctionRegisterer regFunTChrono2
	( "tchronodyn",boost::bind(cmdTestChronoDyn,_1,_2,_3) );

      for( int i=1;i<argc;++i )
	{
	  std::istringstream script( argv[i] );
	  cout << "Run "<< argv[i] << endl;
	  sotShell.cmd( "run",script,cout );
	}
      
 
      sotDEBUG(5) << "Run shell." << endl;
      sotShell.shell(cin,cout);
      sotDEBUG(5) << "Shell over." << endl;
    } 
  catch( sotExceptionAbstract& e )
    {
      cout << "!!  "  << e <<endl;
    }
  catch( ... ){ sotDEBUG(5) << "!! Unknown! " <<endl ; }
  
  sotDEBUGOUT(15);
  return 0;
}




