/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Copyright Projet JRL 2007
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *
 * File:      test_boost.cc
 * Project:   sot
 * Author:    Nicolas Mansard
 *
 * Version control
 * ===============
 *
 *  $Id$
 *
 * Description
 * ============
 *
 *
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#undef WITH_OPENHRP

#include <sys/time.h>
#include <iostream>
//#include <sot/sotDebug.h>
//#include <sot/sotMatrixHomogeneous.h>
//#include <sot/sotMatrixTwist.h>

#include <MatrixAbstractLayer/boost.h>

#include <unistd.h>
#include <list>


using namespace std;
namespace ml = maal::boost;


typedef  double FloatType;
typedef ublas::matrix<FloatType> InternalMatrix;
const double threshold = 1e-6;

#define sotCOUNTER(nbc1,nbc2) \
	  gettimeofday(&tpart##nbc2,NULL); \
	  dt##nbc2 += ( (tpart##nbc2.tv_sec-tpart##nbc1.tv_sec) * 1000. \
		   + (tpart##nbc2.tv_usec-tpart##nbc1.tv_usec+0.) / 1000. );
#define sotINITCOUNTER(nbc1) \
   struct timeval tpart##nbc1; double dt##nbc1=0; 



int
main( void )
{
  //unsigned int NR=36,NC=60;
  unsigned int NR=36,NC=16;
  //unsigned int NR=6,NC=30;
  ml::Matrix J(NR,NC);
  ml::Matrix UTref(NR,NR); 
  ml::Matrix Vref(NC,NC);

  for( unsigned int i=0;i<NR;++i )
    for( unsigned int j=0;j<NC;++j )
      J(i,j) = ((rand()+1.) / RAND_MAX*2-1) ;


  if(NR+NC<15)cout << ml::MATLAB <<"J="<< J << endl;

  if(NR+NC<15)cout << "Jinv=" << J.pseudoInverse() << endl;


  ublas::matrix<double> & matrix = J.matrix;
  ml::Matrix invMatrix(NC,NR);
  ublas::matrix<FloatType>*pmatrix = new ublas::matrix<double>;
  *pmatrix = matrix;
  
  ublas::matrix<FloatType,ublas::column_major> *Iptr 
    = new ublas::matrix<FloatType,ublas::column_major>( pmatrix->size1(),pmatrix->size2() );	
  ublas::vector<FloatType> s(std::min(matrix.size1(),matrix.size2()));		
  ublas::vector<double> w;//(lw)	

  for( int iter=0;iter<100;++iter ) J.pseudoInverse();

  const int ITER_MAX = 10000;
  
  for( int iterG=0; iterG<10;++iterG )
    {
      struct timeval tpartinit;
      sotINITCOUNTER(0);
      sotINITCOUNTER(1);sotINITCOUNTER(2);sotINITCOUNTER(3); 
      sotINITCOUNTER(4);sotINITCOUNTER(5); sotINITCOUNTER(6);
      sotINITCOUNTER(7);sotINITCOUNTER(8);
      for( int iter=0;iter<ITER_MAX;++iter )
	{			

	  gettimeofday(&tpartinit,NULL);
	  
  	  //ublas::matrix<FloatType,ublas::column_major> *Uptr
	  //= new ublas::matrix<FloatType,ublas::column_major>(pmatrix->size1(),pmatrix->size1()); 
  	  //ublas::matrix<FloatType,ublas::column_major> *VTptr
	  //= new ublas::matrix<FloatType,ublas::column_major>(pmatrix->size2(),pmatrix->size2());


	  ublas::matrix<FloatType,ublas::column_major> &I = *Iptr;	
 	  //ublas::matrix<FloatType,ublas::column_major> &U = *Uptr; 
 	  //ublas::matrix<FloatType,ublas::column_major> &VT = *VTptr;
	  ublas::matrix<FloatType> & U = UTref.matrix; 
	  ublas::matrix<FloatType> & VT = Vref.matrix;

	  sotCOUNTER(init,0);
	  
	  I=matrix; U.clear(); VT.clear();
	  char Jobu='A'; /* Compute complete U Matrix */	
	  char Jobvt='A'; /* Compute complete VT Matrix */	
	  char Lw; Lw='O'; /* Compute the optimal size for the working vector */ 

	  sotCOUNTER(0,1);

#ifdef WITH_OPENHRP
	  /* Presupposition: an external function jrlgesvd is defined
	   * and implemented in the calling library.
	   */

	  // get workspace size for svd
	  int lw=-1;
	  {
	    const int m = matrix.size1();     const int n = matrix.size2();
	    FloatType vw;                     int linfo;
	    int lda = std::max(m,n);
	    jrlgesvd_(&Jobu, &Jobvt, &m, &n,NULL, &lda, 
		      0, 0, &m, 0, &n, &vw, &lw, &linfo); 
	    lw = int(vw);
	  }
#else //#ifdef WITH_OPENHRP
	  int lw;
	  if( matrix.size1()>matrix.size2() )
	    {
	      ublas::matrix<FloatType,ublas::column_major> matrixtranspose;  
	      matrixtranspose = trans(matrix);
	      lw = lapack::gesvd_work(Lw,Jobu,Jobvt,matrixtranspose); 
	    } else {
	      lw = lapack::gesvd_work(Lw,Jobu,Jobvt,matrix); 
	    }
#endif //#ifdef WITH_OPENHRP
	  sotCOUNTER(1,2);

	  //ublas::vector<double> w(lw);		 
	  w.resize(lw);
	  lapack::gesvd(Jobu, Jobvt,I,s,U,VT,w);		

	  sotCOUNTER(2,3);

	  const unsigned int nsv = s.size();
	  ublas::vector<FloatType> sp(nsv);
	  unsigned int Jrank = 0; 
	  for( unsigned int i=0;i<nsv;++i )		
	    if( fabs(s(i))>threshold ) { sp(i)=1/s(i); Jrank++; }
	    else sp(i)=0.;		

	  sotCOUNTER(3,4);

	  invMatrix.matrix.clear();

	  if( 1 )
	    {
	      for( unsigned int i=0;i<VT.size2();++i )
		for( unsigned int j=0;j<U.size1();++j )
		  for( unsigned int k=0;k<nsv;++k )
		    invMatrix.matrix(i,j)+=VT(k,i)*sp(k)*U(j,k);//Std
	      //invMatrix.matrix(i,j)+=VT(i,k)*sp(k)*U(k,j);
	    }
	  else
	    {
	      double * pinv = traits::matrix_storage(invMatrix.matrix);
	      double * uptr;
	      double * uptrRow;
	      double * vptr;
	      double * vptrRow = traits::matrix_storage(VT);
	      
	      double * spptr;
	      
	      for( unsigned int i=0;i<VT.size2();++i )
		{
		  uptrRow = traits::matrix_storage(U);
		  for( unsigned int j=0;j<U.size1();++j )
		    {
		      uptr = uptrRow;
		      vptr = vptrRow; 
		      spptr = traits::vector_storage( sp );
		      for( unsigned int k=0;k<Jrank;++k )
			{
			  //(*pinv) += (*vptr) * (*spptr) * U(j,k);
			  (*pinv) += (*vptr) * (*spptr) * (*uptr);
			  uptr+=U.size1(); 
			  vptr++; spptr++;
			}
		      pinv++; 
		      uptrRow++; 
		    }
		  vptrRow += VT.size1();
		}
	    }

	  sotCOUNTER(4,5);
	  UTref.matrix = U;
	  Vref.matrix = VT;

	  sotCOUNTER(5,6);

	  //delete Uptr; delete VTptr;  delete Iptr;
	}

  
      cout << "Global time = " << dt0+dt1+dt2+dt3+dt4+dt5 <<endl
	   << "  alloc: \t" << dt0  << endl
	   << "  affect: \t" << dt1  << endl
	   << "  getsize: \t" << dt2  << endl
	   << "  svd: \t\t" << dt3  << endl
	   << "  rank: \t" << dt4  << endl
	   << "  VSU: \t\t" << dt5  << endl 
	   << "  Copy: \t" << dt6  << endl <<endl;
    }

  if(NR+NC<15)cout << "Jpinv=" << invMatrix << endl;
  if(NR+NC<15)cout << "UT=" << UTref << endl;
}
